
// ******************************************************************
//
//	I. Matchikhilian, S. Shuvalov
//
// 	Prototype for ECAL EP/MIP calibration on 4LT FARM
//      =================================================
//
//	10 May 2000 - preliminary version to estimate statistics
//                                         Initial revision;
//
//      15 May 2000 - Optimization of histogram dimensions; 
//                    Disk space is not needed anymore;
//               
//      16 May 2000 - RHP ntuples introducing
//  
//******************************************************************
//
//      07 Jul 2002 - cuts optimization after shutdown
//                    EP-ntuple format is changed
//                       
// ******************************************************************



#include <alca/AC_ecal_ifarm.h>

/***
 *   
 *   Global variables declaration:
 *
 ***/


static float                                 AC_ECAL_ihists_hits[AC_ECAL_N_IHIST];
static short                                 AC_ECAL_ihists_XY[AC_ECAL_N_XMOD][AC_ECAL_N_YMOD];
static short                                 AC_ECAL_ihists_XYPoolPointers[AC_ECAL_N_XMOD][AC_ECAL_N_YMOD];
static AC_ECAL_ihists_THINGS_TO_KNOW         AC_ECAL_ihistsThingsToKnow;


/*-----------       General functions:   --------------------*/


/***
 *    
 *   Should be called once during ARTE initialization:
 *
 ***/


void 
AC_ECAL_ihists_initialize()
{
  
  /*-------------------------------------------------------------*/

  int                                        i;
  int                                        k;
  int                                        rc;
  int                                        index;
  int                                        x_mod;
  int                                        y_mod;
  struct utsname                             AC_ECAL_ihistsUtsname;
  
  /*--------------------------------------------------------------*/


  /***
   *
   *  Initialization:
   *
   ***/

  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EPlow = AC_ECAL_ihists_EPlow_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EPup = AC_ECAL_ihists_EPup_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIPlow = AC_ECAL_ihists_MIPlow_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIPup = AC_ECAL_ihists_MIPup_default ;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_et_cut = AC_ECAL_ihists_et_cut_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ptrack_cut = AC_ECAL_ihists_ptrack_cut_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ptrack_low = AC_ECAL_ihists_ptrack_low_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ncells_betw_tr_cl = AC_ECAL_ihists_ncells_betw_tr_cl_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_isol_cut = AC_ECAL_ihists_isol_cut_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ztrack_cut = AC_ECAL_ihists_ztrack_cut_default;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_zcal = AC_ECAL_ihists_zcal_default;

  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EP_NTUPLE_ID = AC_ECAL_ihists_NTUPLE_RULE(AC_ECAL_ihists_EP_NTUPNUM);
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIP_NTUPLE_ID = AC_ECAL_ihists_NTUPLE_RULE(AC_ECAL_ihists_MIP_NTUPNUM);


  sprintf ( AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_hostname_prefix,
	    "%s",
	    AC_ECAL_ihists_hostname_prefix_default);
 
	 
  uname( & AC_ECAL_ihistsUtsname );

  if( strncmp( AC_ECAL_ihistsUtsname.nodename, 
	       AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_hostname_prefix,
	       strlen(AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_hostname_prefix))){

    AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do = 0;
    return;

  }
  else
    AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do = 1;
  
  /*------------------------*/

  // RESET ALL:
  
  memset((void *) AC_ECAL_ihists_XY,
	 0,
	 sizeof(AC_ECAL_ihists_XY));
  

  memset((void *) AC_ECAL_ihists_XYPoolPointers,
	 0,
	 sizeof(AC_ECAL_ihists_XYPoolPointers));

  /*------------------------*/
  
  // Initialize geometry:
	  
  
  // 1. Define xmin / ymin of ECAL( center - beam pipe )

  k = 0;
  for( ArteTable<GCAL>::iterator gcal = ArteTable<GCAL>::begin();
       gcal != ArteTable<GCAL>::end();
       gcal++ ) {

    if( gcal->cmp != 7 )
      continue;

    if( k == 0 ) {

      k = 1;
      AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell = gcal->xmax - gcal->xmin;
      AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell = gcal->ymax - gcal->ymin;
      AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin = gcal->xmin;
      AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin = gcal->ymin;

    }
    else {
     
      if( gcal->xmin < AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin )
	AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin = gcal->xmin;
      if( gcal->ymin < AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin )
	AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin = gcal->ymin;

    }
  

  }

  // 2. Fill AC_ECAL_ihists_XY - different ECAL sections id
 
  for( ArteTable<GCAL>::iterator gcal = ArteTable<GCAL>::begin();
       gcal != ArteTable<GCAL>::end();
       gcal++ ) {

    if( gcal->cmp != 7 )
      continue;

    x_mod = (int) (( gcal->xmin - AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin ) / (gcal->xmax - gcal->xmin ) + 0.5 );

    y_mod = (int) (( gcal->ymin - AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin ) / (gcal->ymax - gcal->ymin )  + 0.5 );

    if( x_mod < 0 || y_mod < 0 || x_mod >= AC_ECAL_N_XMOD || y_mod >= AC_ECAL_N_YMOD ) {

      // Smoething wrong with geometry:
      cout << "Wrong geometry xmod " << x_mod << " ymod " << y_mod << endl;
      AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do = 0;
      return;

    }

    AC_ECAL_ihists_XY[x_mod][y_mod] = gcal->ncl;
    
  }


  // 3. Fill AC_ECAL_ihists_XYPoolPointers - pointers to histogramms:

  index = 0;
  
  for( k = 0; k < AC_ECAL_N_YMOD; k++ ) 
    for( i = 0; i < AC_ECAL_N_XMOD; i++ ) {
      
      // Skip beam pipes:
      
      if( AC_ECAL_ihists_XY[i][k] == 0 ) 
	continue;

      AC_ECAL_ihists_XYPoolPointers[i][k] = index;
	 
      index += AC_ECAL_ihists_XY[i][k]*AC_ECAL_ihists_XY[i][k];
      
    }

  // Now one more initialization:

  AC_ECAL_ihistsThingsToKnow.AC_ECAL_product =  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ncells_betw_tr_cl * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell;

  // This is 1/4 of the inner-cell size

  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_minx = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell / 20;
  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_miny = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell / 20;


  /*------------------------------------------*/

  // RHP ntuples:

  rc = rhp_hbookn(AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EP_NTUPLE_ID,
		  "ECAL_CALIB:EP",
		  AC_ECAL_ihists_NTUPLE_NCOLUMNS_EP,
		  RHP_MAXCOL );

  if( rc != GRR_OK ) {

    cout << "Error in rhp_hbookn EP:" << rc << " " << grr_stript(rc) << endl;
    AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do = 0;
    return;

  }

  rc = rhp_hbookn(AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIP_NTUPLE_ID,
		  "ECAL_CALIB:MIP",
		  AC_ECAL_ihists_NTUPLE_NCOLUMNS_MIP,
		  RHP_MAXCOL );

  if( rc != GRR_OK ) {

    cout << "Error in rhp_hbookn:" << rc << " " << grr_stript(rc) << endl;
    AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do = 0;
    return;

  }
}



/***
 *
 *  Should be called once per each event:
 *  
 ***/


void 
AC_ECAL_ihists_event()
{

  /*-------------------------------------------------------------------------------------------*/
  
  int                                        tc_track;
  float                                      my_x = 0;
  float                                      my_y = 0;
  float                                      my_tx = 0;
  float                                      my_ty = 0;
  float                                      my_ze = 0;
  float                                      cur_dist;
  float                                      cur_dist1;
  float                                      min_dist;
  float                                      my_e = 0;
  short                                      xmod_ecal;
  short                                      ymod_ecal;
  short                                      chan;
  float                                      xcal;
  float                                      ycal;
  short                                      AC_ECAL_ihistnum;
  int                                        rc;
  static int                                 ECALihist_runnumber = -1;
  int                                        ECALihist_tmp_runnumber;
  float                                      ECALihist_ntup_varEP[AC_ECAL_ihists_NTUPLE_NCOLUMNS_EP];
  float                                      ECALihist_ntup_varMIP[AC_ECAL_ihists_NTUPLE_NCOLUMNS_MIP];
  float                                      my_p;
  float                                      fmy_p;
  float                                      ratio;

  /*---------------------------------------------------------------------------------------------*/

  if( !AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_do ) 
    return;
   
  ECALihist_tmp_runnumber = faDDAQrun_number();
  
  if( ECALihist_tmp_runnumber != ECALihist_runnumber ) {

    ECALihist_runnumber = ECALihist_tmp_runnumber;
  
    // Reset RHP ntuples:

    rhp_reset(AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EP_NTUPLE_ID);

    rhp_reset(AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIP_NTUPLE_ID);

  }
  
  memset((void *)AC_ECAL_ihists_hits,
	 0,
	 sizeof(AC_ECAL_ihists_hits));

  // 1. Store ECAL hits: 
  
  for( ArteTable<HITC>::iterator hitc = ArteTable<HITC>::begin();
       hitc != ArteTable<HITC>::end();
       hitc++ ) {

    // Store all becouse of isolaton cut

    rc = AC_ECAL_ihists_get_neigb( hitc->x,
				    hitc->y,
				    &xmod_ecal,
				    &ymod_ecal,
				    &chan);

    if( rc == -1 )
      continue;
	 
    AC_ECAL_ihistnum = AC_ECAL_ihists_XYPoolPointers[xmod_ecal][ymod_ecal] + chan;
    AC_ECAL_ihists_hits[AC_ECAL_ihistnum] = hitc->q;

  }

  // Loop over tracks:

  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end();
       rtra++ ) {

    if(rtra->fit != 0 )
      continue; 

    // rtra->pf is really 1/p

    if( fabs(rtra->pf) >= AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ptrack_low )
      continue;
  
    my_p = 1 / rtra->pf;
    fmy_p = fabs( my_p );
        
    /***
     *
     * Using tc-segments only:
     *
     ***/

    tc_track = 0;	    
    
    for( ArteRelationIterator<RSEG> patt1 = rtra->rseg.begin();
	 patt1 != rtra->rseg.end();
	 patt1++ ) {
	        
      if( ((*patt1)->cmp & Rsegc::trit ) && (*patt1)->fit == 0 ) {

	// I hope: 1track - 1 segment

 	tc_track = 1;
	my_x = (*patt1)->xe;
	my_y = (*patt1)->ye;
	my_tx = (*patt1)->txe;   
	my_ty = (*patt1)->tye;
	my_ze = (*patt1)->ze;

	break;

      }
      
    }

    if( tc_track == 0)
      continue;

    // Projection onto ECAL surface

    xcal = my_tx * (AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_zcal - my_ze) + my_x;
    ycal = my_ty * (AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_zcal - my_ze) + my_y;
    
    /***
     *
     *  Here E / P stuff:
     *
     ***/

    //    if( fmy_p <= AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ptrack_cut ) {

    min_dist = -1;
	
    for( ArteTable<RCCL>::iterator hitc = ArteTable<RCCL>::begin();
	 hitc != ArteTable<RCCL>::end();
	 hitc++ ) {
		
      
      cur_dist = (hitc->x - xcal) * (hitc->x - xcal ) +  (hitc->y - ycal) * (hitc->y - ycal );

      if( cur_dist > AC_ECAL_ihistsThingsToKnow.AC_ECAL_product )
	continue;
  
      cur_dist1 =  hitc->x * hitc->x + hitc->y * hitc->y;
	      
      // Simple et-cut

      if( hitc->esum * sqrt( cur_dist1 /( cur_dist1 + hitc->z * hitc->z)) < AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_et_cut ) 
	continue;

      if( min_dist == -1 || cur_dist < min_dist ) {
	  
 	min_dist = cur_dist;
	my_x = hitc->x;
	my_y = hitc->y;
	my_e = hitc->esum;
	min_dist = cur_dist;

      }
    }
	
    if( min_dist == -1 )
      continue;
 

    ratio = fabs(my_e * rtra->pf);

    if(  ratio  < AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EPlow || ratio >= AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EPup )
      continue;

    rc = AC_ECAL_ihists_get_neigb( my_x,
				   my_y,
				   &xmod_ecal,
				   &ymod_ecal,
				   &chan);

    if( rc == -1 )
      continue;
      
    if( min_dist > AC_ECAL_ihistsThingsToKnow.AC_ECAL_product / (AC_ECAL_ihists_XY[xmod_ecal][ymod_ecal] * AC_ECAL_ihists_XY[xmod_ecal][ymod_ecal]))
      continue;
    

    // Fill EP ntuple:
  	
    ECALihist_ntup_varEP[1] = my_e;
    ECALihist_ntup_varEP[2] = my_p;
    ECALihist_ntup_varEP[0] = (float) ( 10000 * xmod_ecal + 100 * ymod_ecal + chan );
 
    // Fill rhp ntuples:

    rc = rhp_hfn( AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_EP_NTUPLE_ID,
		  ECALihist_ntup_varEP);
      
    if( rc != GRR_OK ) 
      cout << "Error in rhp_hfn EP:" << rc << " " << grr_stript(rc) << endl;
      

    //    }

      
    // NOW: MIPS:
    
    rc = AC_ECAL_ihists_get_neigb(xcal,
				  ycal,
				  &xmod_ecal,
				  &ymod_ecal,
				  &chan);

    if( rc == -1 )
      continue;

    ECALihist_ntup_varMIP[1] = AC_ECAL_ihists_hits[AC_ECAL_ihists_XYPoolPointers[xmod_ecal][ymod_ecal] + chan];
      
    if(  ECALihist_ntup_varMIP[1] < AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIPlow || ECALihist_ntup_varMIP[1] >= AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIPup )
      continue;

    // Loop over all neighbours cells:

    if( !CheckNeighbCells(xcal,
			  ycal,
			  xmod_ecal,
			  ymod_ecal,
			  AC_ECAL_ihists_XY[xmod_ecal][ymod_ecal]) )
      continue;
    

    // Write RHP ntuples:

    ECALihist_ntup_varMIP[0] = (float) ( 10000 * xmod_ecal + 100 * ymod_ecal + chan );
 
    // Fill rhp ntuples:
    
    rc = rhp_hfn( AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_MIP_NTUPLE_ID,
		  ECALihist_ntup_varMIP);
    
    if( rc != GRR_OK ) 
      cout << "Error in rhp_hfn MIP:" << rc << " " << grr_stript(rc) << endl;
  }
  
}


/*--------------------    INTERNAL functions:        -----------------------*/


/***
 *
 * Calculates (xmod,ymod,chan) according to physical xcal,ycal
 * Returns -1, if xcal,ycal are out of range
 *          0, is success
 *
 ***/


int 
AC_ECAL_ihists_get_neigb(float xcal,
			 float ycal,
			 short *xmod_ecal,
			 short *ymod_ecal,
			 short *chan_ecal)
{
  
  /*----------------------------------------------------------------------*/
  
  float                                      low_x;
  float                                      low_y;
  int                                        tmp_x;
  int                                        tmp_y;
  
  /*-----------------------------------------------------------------------*/

 
  *xmod_ecal = (short)( ( xcal - AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin ) / AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell );

  *ymod_ecal = (short)( ( ycal - AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin ) / AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell );
  
  if(*xmod_ecal < 0 || *xmod_ecal >= AC_ECAL_N_XMOD || *ymod_ecal < 0 || *ymod_ecal >= AC_ECAL_N_YMOD) 
    return(-1);
  
  switch(AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal]) {

  case(1):
    *chan_ecal = 0;
    break;
  case(2):
  case(5):

    low_x = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin + *xmod_ecal * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell ;
    low_y = *ymod_ecal *  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell + AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin;
    
    tmp_x = (int) (( xcal - low_x ) / (AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell / AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal])  );
    tmp_y= (int) (( ycal - low_y ) / (AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell / AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal]) );
	
    if( tmp_x >= AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] || tmp_x < 0 ) 
      return(-1);
    
    if( tmp_y >= AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] || tmp_y < 0 ) 
      return(-1);
    
    *chan_ecal = AC_ECAL_ihists_XY[*xmod_ecal][*ymod_ecal] * tmp_y + tmp_x;
	      
    break;
  default:
    return(-1);

  };

  return(0);
  
}

/*-----------------------------------------------------------------------*/

/****
 *
 * Performs isolation cut, returns 0, if isolation cut is not passed
 *
 ****/


int
CheckNeighbCells( float xcal,
		  float ycal,
		  short xmod_ecal,
		  short ymod_ecal,
		  short sec_type)
{
  /*---------------------------------------------------------------------*/

  int                                        i;
  int                                        j;
  int                                        rc;
  float                                      xneighb;
  float                                      yneighb; 
  short                                      xmod_neighb;
  short                                      ymod_neighb;
  short                                      chan_neighb;
  float                                      xmin;
  float                                      xmax;
  float                                      ymin;
  float                                      ymax;
  int                                        ncells_x;
  int                                        ncells_y;
  float                                      current_ssize_x;
  float                                      current_ssize_y;
  short                                      prev_sec;
  float                                      incr;
  float                                      cut_val;
  float                                      cur_dist;

  /*---------------------------------------------------------------------*/

  current_ssize_x = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell / sec_type;
  current_ssize_y = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell / sec_type;
  cur_dist = xcal*xcal + ycal*ycal;
  cut_val = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_isol_cut * ( cur_dist + AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_zcal * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_zcal ) /  cur_dist;
  
  for( i = -1; i < 2; i++ )
    for( j = -1; j < 2; j++ ) {

      if( j == 0 && i == 0 ) // cell itself
	continue;

      // Low edges of module(!!! not cell):

      xmin = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xmin + xmod_ecal * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell ;
      ymin = AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ymin + ymod_ecal * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell ;      
      
      ncells_x = (int)((xcal - xmin) / current_ssize_x);

      ncells_y = (int)((ycal - ymin) / current_ssize_y);


      // Low/upper edges of cell:

      xmin += ncells_x * current_ssize_x;
      ymin += ncells_y * current_ssize_y;

      xmax = xmin + current_ssize_x;
      ymax = ymin + current_ssize_y;

      xneighb =( i < 0 ) ? xmin : xmax;
      yneighb =( j < 0 ) ? ymin : ymax;
      xneighb += i * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_minx;
      yneighb += j * AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_miny;

      if( i == 0 )
	xneighb = xcal;

      if( j == 0 )
	yneighb = ycal;
      
      rc = AC_ECAL_ihists_get_neigb(xneighb,
				    yneighb,
				    &xmod_neighb,
				    &ymod_neighb,
				    &chan_neighb);

      if( rc == -1 )
	continue;

      prev_sec = AC_ECAL_ihists_XY[xmod_neighb][ymod_neighb];

      if( (prev_sec == sec_type) || (i * j != 0) ) {

	if( AC_ECAL_ihists_hits[AC_ECAL_ihists_XYPoolPointers[xmod_neighb][ymod_neighb] + chan_neighb ] > cut_val ) 
	  return(0);
	else
	  continue;

      }

      // Now not corner cells, different type:

      if( i == 0 ) {

	incr =  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_xcell / prev_sec;
	for( xneighb = xmin + AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_minx; xneighb <= xmax; xneighb += incr ) {
		
	  rc = AC_ECAL_ihists_get_neigb(xneighb,
					yneighb,
					&xmod_neighb,
					&ymod_neighb,
					&chan_neighb);
	  if( rc == -1 )
	    continue;

	  if( AC_ECAL_ihists_hits[AC_ECAL_ihists_XYPoolPointers[xmod_neighb][ymod_neighb] + chan_neighb] > cut_val ) 
	    return(0);
	}
	
      }
      else {

	incr =  AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_ycell / prev_sec;
	for( yneighb = ymin + AC_ECAL_ihistsThingsToKnow.AC_ECAL_ihists_miny; yneighb <= ymax; yneighb += incr) {
	    
	  rc = AC_ECAL_ihists_get_neigb(xneighb,
					yneighb,
					&xmod_neighb,
					&ymod_neighb,
					&chan_neighb);
	  if( rc == -1 )
	    continue;

	  if( AC_ECAL_ihists_hits[AC_ECAL_ihists_XYPoolPointers[xmod_neighb][ymod_neighb] + chan_neighb] > cut_val ) 
	    return(0);
	  
	}
      }
     
    }

  return(1); 

}
