
// *****************************************************************************
//
// source:    RC_init.C
//
// type:      source code
//
// created:   23 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    RC_init()
//
// called by: acinit_()
//
// changes:
// 24 Nov 1999 (AG) improve print-out
//  1 Mar 2000 (CP) use PRCMHITMASK to steer HITB-building
// 21 Mar 2000 (AG) polish print-out
// 09 May 2000 (AG) use faComp() to check comonent
// 08 Mar 2002 (JH) Remove dangerous switching on of reco via DQ
// 08 Mar 2002 (JH) Include modification of reco flags according to reco and hit masks
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <strstream.h>
#include <stdlib.h>
#include <string.h>

#include "runcfg.h"
#include "grr.h"
#include "farm/farm.hh"
#include "dq/DQ.h"

// externals
extern int FADEB;

// reco flags
extern int recoTARGET;
extern int recoVDS;
extern int recoVDSmodule;
extern int recoHPT;
extern int recoOTR;
extern int recoITR;
extern int recoRICH;
extern int recoTRD;
extern int recoECAL;
extern int recoMUON;


//
// function
//
int
RC_init()
{
  // show up
  //cout << "RC_init()" << endl;

  int rc = 0;

  //
  // hit production flags
  //
  char *rhitmask = getenv( PRCMHITMASK );
  if( rhitmask ) {
    char txt[80];
    sprintf( txt, "RC_init(): HIT  env mask=0x%x", atoi(rhitmask) );
    ARTE_error( GRR_SINFO, txt, 0 );
  }

  //
  // reco flags. flag=0 reco off, flag=1 Hit level, flag=2 full reco level
  //
  char *rrecomask = getenv( PRCMRECOMASK );
  if( rrecomask ) {
    char txt[80];
    sprintf( txt, "RC_init(): RECO env mask=0x%x", atoi(rrecomask) );
    ARTE_error( GRR_SINFO, txt, 0 );
  }

  //
  // component flags
  //
  char *rcompmask = getenv( RC_Comp_Mask_ENV );
  if( rcompmask ) {
    char txt[80];
    sprintf( txt, "RC_init(): COMP env mask=0x%x", atoi(rcompmask) );
    ARTE_error( GRR_SINFO, txt, 0 );
  }

  if (rhitmask || rrecomask || rcompmask) {
    cout <<"Subdetector  RecoKumac  RecoMask  HitMask  ComponentMask" << endl;
    cout <<"--------------------------------------------------------" << endl;
    printf("%s          %d          %d        %d         %d\n", "VDS   ", recoVDS,  
	   faReco( COMP_M_SVD ),  faHit( COMP_M_SVD ),  faComp( COMP_M_SVD ));
    printf("%s          %d          %d        %d         %d\n", "ITR   ", recoITR,  
	   faReco( COMP_M_ITR ),  faHit( COMP_M_ITR ),  faComp( COMP_M_ITR ));
    printf("%s          %d          %d        %d         %d\n", "HPT   ", recoHPT,  
	   faReco( COMP_M_HIPT ), faHit( COMP_M_HIPT ), faComp( COMP_M_HIPT ));
    printf("%s          %d          %d        %d         %d\n", "OTR   ", recoOTR,  
	   faReco( COMP_M_OTR ),  faHit( COMP_M_OTR ),  faComp( COMP_M_OTR ));
    printf("%s          %d          %d        %d         %d\n", "RICH  ", recoRICH, 
	   faReco( COMP_M_RICH ), faHit( COMP_M_RICH ), faComp( COMP_M_RICH ));
    printf("%s          %d          %d        %d         %d\n", "TRD   ", recoTRD,  
	   faReco( COMP_M_TRD ),  faHit( COMP_M_TRD ),  faComp( COMP_M_TRD ));
    printf("%s          %d          %d        %d         %d\n", "ECAL  ", recoECAL, 
	   faReco( COMP_M_ECAL ), faHit( COMP_M_ECAL ), faComp( COMP_M_ECAL ));
    printf("%s          %d          %d        %d         %d\n", "MUON  ", recoMUON, 
	   faReco( COMP_M_MUON ), faHit( COMP_M_MUON ), faComp( COMP_M_MUON ));
    fflush(stdout);
  }

  //
  // modifify reco flags according to reco, hit and component masks
  //

  // in online mode, disable components (overide kumac settings) 
  // if bit in run control reco mask is off. 
  if (rrecomask) {
    //recoTARGET    = faReco( COMP_M_TARG  ) ? recoTARGET    : 0;
    recoVDS       = faReco( COMP_M_SVD  ) ? recoVDS       : 0;
    recoVDSmodule = faReco( COMP_M_SVD  ) ? recoVDSmodule : 0;
    recoITR       = faReco( COMP_M_ITR  ) ? recoITR       : 0;
    recoHPT       = faReco( COMP_M_HIPT ) ? recoHPT       : 0;
    recoOTR       = faReco( COMP_M_OTR  ) ? recoOTR       : 0;
    recoRICH      = faReco( COMP_M_RICH ) ? recoRICH      : 0;
    recoTRD       = faReco( COMP_M_TRD  ) ? recoTRD       : 0;
    recoECAL      = faReco( COMP_M_ECAL ) ? recoECAL      : 0;
    recoMUON      = faReco( COMP_M_MUON ) ? recoMUON      : 0;
  }

  // in online mode, if the reco flag for a given component is off but the 
  // hit flag is on, then override the kumac settings and switch ON the hit 
  // level (flag=1) for that component. If flag reco ON nothing is done. 
  if (rhitmask) {
    //if( faHit( COMP_M_TARG ) && !recoTARGET ) recoTARGET    = 1;
    if( faHit( COMP_M_SVD ) && !recoVDS    ) recoVDS       = 1;
    if( faHit( COMP_M_SVD ) && !recoVDS    ) recoVDSmodule = 1;
    if( faHit( COMP_M_ITR ) && !recoITR    ) recoITR       = 1;
    if( faHit( COMP_M_HIPT) && !recoHPT    ) recoHPT       = 1;
    if( faHit( COMP_M_OTR ) && !recoOTR    ) recoOTR       = 1;
    if( faHit( COMP_M_RICH) && !recoRICH   ) recoRICH      = 1;
    if( faHit( COMP_M_TRD ) && !recoTRD    ) recoTRD       = 1;
    if( faHit( COMP_M_ECAL) && !recoECAL   ) recoECAL      = 1;
    if( faHit( COMP_M_MUON) && !recoMUON   ) recoMUON      = 1;
  }

  // in online mode, disable components if not present in run 
  if (rcompmask) {
    //recoTARGET    = faComp( COMP_M_TARG  ) ? recoTARGET    : 0;
    recoVDS       = faComp( COMP_M_SVD  ) ? recoVDS       : 0;
    recoVDSmodule = faComp( COMP_M_SVD  ) ? recoVDSmodule : 0;
    recoITR       = faComp( COMP_M_ITR  ) ? recoITR       : 0;
    recoHPT       = faComp( COMP_M_HIPT ) ? recoHPT       : 0;
    recoOTR       = faComp( COMP_M_OTR  ) ? recoOTR       : 0;
    recoRICH      = faComp( COMP_M_RICH ) ? recoRICH      : 0;
    recoTRD       = faComp( COMP_M_TRD  ) ? recoTRD       : 0;
    recoECAL      = faComp( COMP_M_ECAL ) ? recoECAL      : 0;
    recoMUON      = faComp( COMP_M_MUON ) ? recoMUON      : 0;
  }

  // print final reco flags
  cout << "RC_init(): final RECO flags:" << endl;
  //cout << "              TARGET  " << recoTARGET    << endl;
  cout << "                 VDS  " << recoVDS       << endl;
  //cout << "           VDSmodule  " << recoVDSmodule << endl;
  cout << "                HIPT  " << recoHPT       << endl;
  cout << "                 ITR  " << recoITR       << endl;
  cout << "                 OTR  " << recoOTR       << endl;
  cout << "                RICH  " << recoRICH      << endl;
  cout << "                 TRD  " << recoTRD       << endl;
  cout << "                ECAL  " << recoECAL      << endl;
  cout << "                MUON  " << recoMUON      << endl;
  
  return rc;
}




