// ********************************************************************
//
//  Routine:      acRSEGinGlobalCoordinates(const int CMPnumber)
//                acRSEGinGlobalCoordinates(ArtePointer<RSEG>)
//
//  Author:       T.Oest
//
//  Description:  global alignment transformation for segments
//                
//
//  Modification:
//   -- RHITs are no longer transformated for VDS segments 
//                                                      20/09/99 T.Oest
//   -- RHIT transformation can be switched on/off, single segments
//      can be transformed                              29/10/99 T.Oest
//   -- RHIT transformation switched on                 07/01/00 T.Oest
//
// 08/12/00 (SKAG) - ACGA cmp compared to GEDE cmp
//                 - shift in z implemented
// ********************************************************************

// includes
#include <iostream.h>
#include <stdlib.h>
#include <math.h>
#include <algo.h>

#include "arte/arte.hh"

void acRSEGinGlobalCoordinates(ArtePointer<RSEG> rseg, int flag );


const int indexARTE[5][5] = { {0, 1,  3, 6,10},
                              {1, 2,  4, 7,11},
                              {3, 4,  5, 8,12},
                              {6, 7,  8, 9,13},
                              {10,11,12,13,14}};

//****************************************************************
// function
//****************************************************************
//
void acRSEGinGlobalCoordinates(const int CMPnumber){

  if (ArteTable<ACGA>::size() == 0 ) return;

//----------------------------------------------------------------
// loop over all segmets
//----------------------------------------------------------------

  int flag = 1;

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end(); ++rseg ) {

    if ((*rseg)->fit != 7 && (CMPnumber == (*rseg)->cmp)){
      acRSEGinGlobalCoordinates(*rseg, flag);
    }
  }
//----------------------------------------------------------------
// end of loop
//----------------------------------------------------------------
};




//****************************************************************
// function
//****************************************************************
//
void acRSEGinGlobalCoordinates(ArtePointer<RSEG> rseg, int flag){

  if (ArteTable<ACGA>::size() == 0 ) return;

//----------------------------------------------------------------
// if matched segment return
//----------------------------------------------------------------
  if (rseg->fit == 7) return;

//----------------------------------------------------------------
// loop over all ACGA rows
//----------------------------------------------------------------

  //  int CMPnumber = rseg->cmp;

  int ACGAcmp ;
  switch ( rseg->cmp )
    {
    case 0x1:
    case 0x10001: 
      ACGAcmp=0x1 ; //VDS
      break ;
    case 0x4000000:
      ACGAcmp=0x4000000 ; //RICH
      break ;
    default:
      return ;
    }
      
  for( ArteTable<ACGA>::iterator acga = ArteTable<ACGA>::begin();
       acga != ArteTable<ACGA>::end(); ++acga ) {

    if ((*acga)->cmp == ACGAcmp){

      float z0, dx, dy, dz, dtx, dty, cphi, sphi;
      float Vf[4][4], Ve[4][4],  Deriv[4][4];

      z0   = (*acga)->z;
      dx   = (*acga)->dx;
      dy   = (*acga)->dy;
      dz   = (*acga)->dz;
      dtx  = (*acga)->dtx;
      dty  = (*acga)->dty;
      cphi = (*acga)->rot[0];
      sphi = (*acga)->rot[3];

      for (int i=0 ; i<4 ; i++){
	for (int j = 0; j<4 ; j++){
	  Deriv[i][j] = 0.0;
	}
      }
      Deriv[0][0] =   cphi;
      Deriv[0][1] = - sphi;
      Deriv[1][0] =   sphi;
      Deriv[1][1] =   cphi;
      Deriv[2][2] =   cphi;
      Deriv[2][3] = - sphi;
      Deriv[3][2] =   sphi;
      Deriv[3][3] =   cphi;

//----------------------------------------------------------------
// transformation of segments
//----------------------------------------------------------------
      float dz0, x, y, z, tx, ty;

      if (rseg->cmp == 0x4000000){ //RICH

	rseg->txf = rseg->txf + dtx;
	rseg->tyf = rseg->tyf + dty;
       
      } else {

	dz0 = rseg->zf + dz - z0;
	x  = rseg->xf;
	y  = rseg->yf;
	z  = rseg->zf;
	tx = rseg->txf;
	ty = rseg->tyf;

	rseg->xf  =  x * cphi -  y * sphi + dz0 * dtx + dx ;
	rseg->yf  =  x * sphi +  y * cphi + dz0 * dty + dy ;
	rseg->zf  =  z + dz ;
	rseg->txf = tx * cphi - ty * sphi + dtx ;
	rseg->tyf = tx * sphi + ty * cphi + dty ;

	dz0 = rseg->ze + dz - z0;
	x  = rseg->xe;
	y  = rseg->ye;
	z  = rseg->ze;
	tx = rseg->txe;
	ty = rseg->tye;

	rseg->xe  =  x * cphi -  y * sphi + dz0 * dtx + dx ;
	rseg->ye  =  x * sphi +  y * cphi + dz0 * dty + dy ;
	rseg->ze  =  z + dz ;
	rseg->txe = tx * cphi - ty * sphi + dtx ;
	rseg->tye = tx * sphi + ty * cphi + dty ;

	for (int i=0; i<4; i++){
	  for (int j=0; j<=i; j++){

	    Vf[i][j] = 0.0;
	    Ve[i][j] = 0.0;

	    for (int k=0; k<4; k++){
	      for (int l=0; l<4; l++){

		Vf[i][j] =  Vf[i][j] +
		  Deriv[i][k] * rseg->cvf[indexARTE[k][l]] * Deriv[j][l];
		Ve[i][j] =  Ve[i][j] +
		  Deriv[i][k] * rseg->cve[indexARTE[k][l]] * Deriv[j][l];
	      }
	    }

	    Vf[j][i] = Vf[i][j];
	    Ve[j][i] = Ve[i][j];

	  }
	}

	for (int i=0; i<4; i++){
	  for (int j=0; j<4; j++){
	    rseg->cve[indexARTE[i][j]] = Ve[i][j];
	    rseg->cvf[indexARTE[i][j]] = Vf[i][j];
	  }
	}


//----------------------------------------------------------------
// transformation of RHITs (except for RICH)
//    this is obsolete
//----------------------------------------------------------------

	if (flag == 1){

	  for (ArteRelationIterator<RHIT> rhit = rseg->rhit.begin();
	       rhit != rseg->rhit.end(); ++rhit){

	    dz0 = (*rhit)->hitb->z + dz - z0;
	    x  = (*rhit)->x;
	    y  = (*rhit)->y;
	    tx = (*rhit)->tx;
	    ty = (*rhit)->ty;
      
	    (*rhit)->x  =  x * cphi -  y * sphi + dz0 * dtx + dx ;
	    (*rhit)->y  =  x * sphi +  y * cphi + dz0 * dty + dy ;
	    (*rhit)->tx = tx * cphi - ty * sphi + dtx ;
	    (*rhit)->ty = tx * sphi + ty * cphi + dty ;

	  }
	}
      }

//----------------------------------------------------------------
// end of loop
//----------------------------------------------------------------
    }
  }
}
