// ****************************************************************************
//
// Routine:      acReadTARGET
// Author:       T.Oest
//
// Description:  copy the target position to GTAR and
//               apply alignment corrections 
//               this was done before in rc99ATARGET
//
//           
//
// Modification:
// EVHD filled now in acread                                        08/03/00 TO
// fill DTA1 bank                                                   10/03/00 TO
// corrected include path bug                                       16/08/00 AG      
//
// including CNA mechanism : Target information DTAR
// is used with Alignment constants to fill GTAR table
// useable after run 18800                                          19/06/02 TZ
// corrected wire position correction bug                           23/07/02 TZ
// code polished and corrected                                      23/08/02 TZ
//
// ****************************************************************************
// ****************************************************************************
//
//  theses are the comments to the old routine rc99ATARGET
//
//  Routine:      rc99ATARGET
//  Author:       T.Oest/D.Samtleben
//
//  Description:  target rates, position
//
//  1998 data: 8 wires
//             1. GTAR table row  Ia    Al
//             2.            row  Ib    Ti
//             3.            row  Ii    Al
//             4.            row  Io    Al
//             5.            row IIa    Fe  (45 degree)
//             6.            row IIb    Al  (45 degree)
//             7.            row IIi    C
//             8.            row IIo    Ti
//
//
//  Modification:
//  -  dropping of GTAR is now included in rc99AStandAlone
//                                             01/07/99   Samtleben
//  -  date and time is only filled if DDAQ exist
//                                             08/08/99   T.Oest
//  -  return statement included to avoid warnings
//                                             24/08/99   T.Oest
//
// ****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include <fstream.h>

// ONLINE includes
#include "esp.h"

// ARTE includes
#include "database/keytable.h"
#include "database/table.h"

#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
static int Nrfile  = 0;
static char * DBfile[100];

int TARGET_ARTEDB_MAJ = -1;
int TARGET_ARTEDB_MIN = -1;

static int TARGETDBserver      =	-1;
static int TARGETDBtable_align =	-1;
static int TARGETDBtable_setup =	-1;

const int TARGET_ARTE_DEV_MAJ  = 18855;
const int TARGET_ARTE_DEV_MIN  = 1;

typedef struct {
  int mat;
  int shape;
  float dbdx;
  float dbdy;
  float dbdz;
  float x0;
  float lam;  
  int eff;
} Wire_info;

typedef struct {
  float dbx;
  float dby;
  float dbz;
  float dbtgts;
  float dbpos;
} Wire_align;

static Wire_info target_setupdb[8];
static Wire_align target_aligndb[8];
 
// prototypes
int acfillGTARfromDTAR();
int acTARGETapplyalignment();
int acTARGETapplysetup();

//*****************************************************************
// function
//*****************************************************************

int
acReadTARGET( int KTstatus )
{
  ARTE_mark( "acReadTARGET", 0 );

//==================================================================
// no action for MC
//==================================================================

  if (ArteTable<MTRA>::size() > 0 ) return 0;

 //===================================================================
// for consistency use old routine
//===================================================================

  if (ArtePointer<EVHD>(1)->run <=18800) { // runs before June 2002  

//===================================================================
// remove the GTAR for real data
//===================================================================

  ArteTable<GTAR>::clear();

//==================================================================
// 1. copy from DTA1 to GTAR 
//==================================================================

  
    if (ArteTable<DTA1>::size() > 0){

     for( ArteTable<DTA1>::iterator dta1 = ArteTable<DTA1>::begin();
          dta1 != ArteTable<DTA1>::end(); ++dta1 ) {

       GTAR gtar;

       gtar.cmp    = (*dta1)->cmp;
       gtar.wnr    = (*dta1)->wnr;
       gtar.dx     = (*dta1)->dx;
       gtar.dy     = (*dta1)->dy;
       gtar.dz     = (*dta1)->dz;
       gtar.x      = (*dta1)->x;
       gtar.y      = (*dta1)->y;
       gtar.z      = (*dta1)->z;
       gtar.ms     = (*dta1)->ms;
       gtar.x0bm   = (*dta1)->x0bm;
       gtar.y0bm   = (*dta1)->y0bm;
       gtar.sxbm   = (*dta1)->sxbm;
       gtar.sybm   = (*dta1)->sybm;
       gtar.shap   = (*dta1)->shap;
       gtar.name   = (*dta1)->name;
       gtar.eff    = (*dta1)->eff;

       for (int i = 0; i<=8 ; i++) gtar.rot[i]    = (*dta1)->rot[i];

       ArteTable<GTAR>::push_back( gtar );
     }
   }

   else { 

//==================================================================
// 2. copy from DTAR to GTAR/DTA1  (database access switched off)
//==================================================================


    if (ArtePointer<EVHD>(1)->date > 990200) {

      int flag = acfillGTARfromDTAR();

      DTA1 dta1;

      for( ArteTable<GTAR>::iterator tar = ArteTable<GTAR>::begin();
           tar != ArteTable<GTAR>::end(); ++tar ) {

         dta1.cmp    = (*tar)->cmp;
         dta1.wnr    = (*tar)->wnr;
         dta1.dx     = (*tar)->dx;
         dta1.dy     = (*tar)->dy;
         dta1.dz     = (*tar)->dz;
         dta1.x      = (*tar)->x;
         dta1.y      = (*tar)->y;
         dta1.z      = (*tar)->z;
         dta1.ms     = (*tar)->ms;
         dta1.x0bm   = (*tar)->x0bm;
         dta1.y0bm   = (*tar)->y0bm;
         dta1.sxbm   = (*tar)->sxbm;
         dta1.sybm   = (*tar)->sybm;
         dta1.shap   = (*tar)->shap;
         dta1.name   = (*tar)->name;
         dta1.eff    = (*tar)->eff;

         for (int i=0; i<=8 ; i++) dta1.rot[i]    = (*tar)->rot[i];

         ArteTable<DTA1>::push_back( dta1 );

      }
      return flag;
    }

    return -1;
   } // end of else

  } // end of exp <= 3

  else {

//==================================================================
//****************************************************************** 
// NEW : run 18801 or later : change GTAR from DTAR
//******************************************************************
//==================================================================

    static int firstcallofacreadtarget = 1;  

    if (firstcallofacreadtarget) {
      for (int i=0; i<TGTS_Nwires; i++) {
        target_setupdb[i].mat=0;
        target_setupdb[i].shape = 0;
        target_setupdb[i].dbdx  = 0.;
        target_setupdb[i].dbdy  = 0.;
        target_setupdb[i].dbdz  = 0.;
        target_setupdb[i].x0    = 0.;
        target_setupdb[i].lam   = 0.;
        target_setupdb[i].eff   = 0 ;
        
        target_aligndb[i].dbx   = 0.;
        target_aligndb[i].dby   = 0.;
        target_aligndb[i].dbz   = 0.;
        target_aligndb[i].dbtgts= 0.;
        target_aligndb[i].dbpos = 0.;
      }
    }

    // ---------------------------------------------------- 
    // Accessing DTAR table to fetch target position 
    // ----------------------------------------------------

    if( ArteTable<DTAR>::size() ){

      ArtePointer<DTAR> dtar( 2 ); //  pointer to DTAR
      char *p = (char *) &dtar->bits + sizeof( SLB_SUBHDR );  // pointer to data
      struct TGTS_RAW tgts; // target data structure
      ESP_get_int( (int *) p, (int *) &tgts, sizeof(TGTS_RAW)/sizeof( int ));

      for (int wire=0; wire<TGTS_Nwires; wire++) {
        // convert wire position to cm and correct sign if necessary
        if ( wire==1 || wire==5 || wire==3 || wire==7 )
          target_aligndb[wire].dbtgts = 0.1 * tgts.pos[wire];
        else
          target_aligndb[wire].dbtgts = -0.1 * tgts.pos[wire];

        target_setupdb[wire].eff = 0;
        if ((tgts.wactmask & (0x1<<(wire)))!=0) {
          target_setupdb[wire].eff  = 1;   // active wire
        }
      } // end of loop over target wires
    }
    else {
      cout << "acReadTARGET : No Target information available (DTAR empty)! " << endl;
      return -1;
    }

    // ---------------------------------------------------------------- 
    // Reading of setup and alignment constants from DB or file 
    // ----------------------------------------------------------------

     char default_targetserver[]   = "/TARGET_ARTE";
     char * dbserver;
     dbserver = default_targetserver;

     switch (KTstatus) {  // switch

       case 2: 

         // structure of keytable changed, so loop over the keytable and get 
         // your serverid and tableids 

         TARGETDBserver       = KeyFindServer(dbserver); 

         if ( TARGET_ARTEDB_MAJ != -1 && TARGET_ARTEDB_MIN != -1) {
	   cout << "acReadTARGET: === Version of TARGET Tables manually set to [" 
	        << TARGET_ARTEDB_MAJ << ", " << TARGET_ARTEDB_MIN << "]" << endl;
	   cout << "              === Ignoring key table entry" << endl;
 	   break; // use old method
         }

         if (TARGETDBserver == -1) { 
           cout<<"acReadTARGET: KeyTable Server " << dbserver << " not found!"<<endl; 
           break;  // use old method
         }

         cout << "acReadTARGET: Looking for Keytable..." << endl;
         TARGETDBtable_setup =	KeyFindTable(TARGETDBserver, "SETUP"); 
         TARGETDBtable_align =	KeyFindTable(TARGETDBserver, "ALIGN"); 

         if ((TARGETDBtable_align == -1) || (TARGETDBtable_setup == -1)) { 
           cout<<"acReadTARGET: Keytable not found!" << endl; 
	   break;  // use old method
         }
 
       // NO _BREAK_ HERE! 
      
       case 1:
 
        // an update happened, look if your server is involved 

        if (!KeyServerNew(TARGETDBserver)) {
          cout<<"acReadTARGET: no TARGET update necessary, server not involved"<<endl;
          break; // apply default update
        };

        // now for the tables - look whether they have changed 

        // first SETUP table
        if (KeyTableNew(TARGETDBtable_setup) && TARGET_ARTEDB_MAJ == -1) { 

          Tables x1=0;
          int t=0;
          cout << "acReadTARGET: keytable found: opening TARGET database "
               << TARGETDBserver <<endl; 

          x1 = OpenTables(KeyServerName(TARGETDBserver),TABLEreadonly);
          if (!x1) {
            ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Can not open database", -1);
          }

          t = BookTable(x1, KeyTableName(TARGETDBtable_setup), 0, 0);
          if (!t) {
            ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Cannot find table", -1);
          }
	
	  cout << "acReadTARGET: reading SETUP [" 
	       << KeyTableMajor (TARGETDBtable_setup) << ", " 
	       << KeyTableMinor (TARGETDBtable_setup) << "]" << endl;
	  KeyTableStamp(x1,KeyServerName(TARGETDBserver),
                        KeyTableName(TARGETDBtable_setup));
          if (!FetchTable(x1,t,0)) { 
            ARTE_exit( GRR_SERROR,
               " $$$ acReadTARGET: Cannot fatch a table for assigned key", -1);
          }
          // Now read the informations

          char *test[TGTS_Nwires];
          unsigned char wirenum[TGTS_Nwires];
          int mat[TGTS_Nwires];
          unsigned char shap[TGTS_Nwires];
          float dx[TGTS_Nwires], dy[TGTS_Nwires], dz[TGTS_Nwires], x0[TGTS_Nwires];
          float lamb[TGTS_Nwires];
             
          int size = TGTS_Nwires;
          test[0]=NULL;

          SkipTableField(x1,1);

          GetTableField(x1,TABLEstring,size,test);
          GetTableField(x1,TABLEubyte,size,wirenum);
          GetTableField(x1,TABLEint,size,mat);
          GetTableField(x1,TABLEubyte,size,shap);
          GetTableField(x1,TABLEfloat,size,dx);
          GetTableField(x1,TABLEfloat,size,dy);
          GetTableField(x1,TABLEfloat,size,dz);
          GetTableField(x1,TABLEfloat,size,x0);
          GetTableField(x1,TABLEfloat,size,lamb);
 
          for (int i=0; i<size; i++){
            target_setupdb[i].mat = mat[i];
            target_setupdb[i].shape = (int)shap[i];
            target_setupdb[i].dbdx  = dx[i];
            target_setupdb[i].dbdy  = dy[i];
            target_setupdb[i].dbdz  = dz[i];
            target_setupdb[i].x0    = x0[i];
            target_setupdb[i].lam   = lamb[i];
          }
          CloseTables(x1);          

          // apply the Target setup changes
          int rc = acTARGETapplysetup();
        }  

        // second ALIGN table        
        if (KeyTableNew(TARGETDBtable_align) && TARGET_ARTEDB_MAJ == -1) { 

          Tables x1=0;
          int t=0;
	  cout <<"acReadTARGET: keytable found: opening TARGET database "
               << TARGETDBserver <<endl; 

          x1 = OpenTables(KeyServerName(TARGETDBserver),TABLEreadonly);
          if (!x1) {
            ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Can not open database", -1);
          }	  

          t = BookTable(x1, KeyTableName(TARGETDBtable_align), 0, 0);
          if (!t) {
            ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Cannot find table", -1);
          }

	  cout << "acReadTARGET: reading ALIGN [" 
	       << KeyTableMajor (TARGETDBtable_align) << ", " 
	       << KeyTableMinor (TARGETDBtable_align) << "]" << endl;
	  KeyTableStamp(x1,KeyServerName(TARGETDBserver),
                        KeyTableName(TARGETDBtable_align));

          if (!FetchTable(x1,t,0)) { 
            ARTE_exit( GRR_SERROR,
               " $$$ acReadTARGET: Cannot fatch a table for assigned key", -1);
          }

          // Now read the informations 
          char *test[TGTS_Nwires];
          unsigned char wirenum[TGTS_Nwires];
          float x[TGTS_Nwires], y[TGTS_Nwires], z[TGTS_Nwires], pos[TGTS_Nwires];
             
          int size = TGTS_Nwires;     
          test[0]=NULL;

          SkipTableField(x1,1);

          GetTableField(x1,TABLEstring,size,test);
          GetTableField(x1,TABLEubyte,size,wirenum);
          GetTableField(x1,TABLEfloat,size,pos);
          GetTableField(x1,TABLEfloat,size,x);
          GetTableField(x1,TABLEfloat,size,y);
          GetTableField(x1,TABLEfloat,size,z);
 
          for (int i=0; i<size; i++){
            target_aligndb[i].dbx   = x[i];
            target_aligndb[i].dby   = y[i];
            target_aligndb[i].dbz   = z[i];
            target_aligndb[i].dbpos = pos[i];
          }
          CloseTables(x1);          
 
          int rc = acTARGETapplyalignment();

	  cout <<"acReadTARGET: TARGET tables loaded: closing TARGET database "
               << TARGETDBserver <<endl;
        }  

	firstcallofacreadtarget = 0;
        cout << "acReadTARGET: returning with rc = 1" << endl;
        ARTE_mark( "acReadTARGET 1", 0 );
        return 1; // update using keytable

     } // of switch 

     // =========================================================
     // default : apply alignment to GTAR : not for first event
     // =========================================================

     if (!firstcallofacreadtarget) {
       int rc = acTARGETapplysetup();
       rc = acTARGETapplyalignment();
       ARTE_mark( "acReadTARGET 1", 0 );
       return 0; 
     }

     // =========================================================
     // No keytable information available, using old method...
     // =========================================================

     firstcallofacreadtarget = 0;     

     cout << "acReadTARGET: Key NOT assigned --> using old method" << endl; 

     Tables x1=0;
     int t=0;
     cout << "acReadTARGET: opening TARGET database "
          << dbserver <<endl; 

     // first SETUP table

     x1 = OpenTables(dbserver,TABLEreadonly);
     if (!x1) {
       ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Can not open database", -1);
     }

     t = BookTable(x1, "SETUP", 0, 0);
     if (!t) {
       ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Cannot find table", -1);
     }
	
     if (TARGET_ARTEDB_MAJ == -1) {
       cout << "acReadTARGET: reading SETUP [" 
            << TARGET_ARTE_DEV_MAJ << ", " 
	    <<  TARGET_ARTE_DEV_MIN << "]" << endl;
       TableStamp(x1,TARGET_ARTE_DEV_MAJ,TARGET_ARTE_DEV_MIN);
     }
     else {
       cout << "acReadTARGET: reading SETUP [" 
            << TARGET_ARTEDB_MAJ << ", " 
	    <<  TARGET_ARTEDB_MIN << "]" << endl;
       TableStamp(x1,TARGET_ARTEDB_MAJ,TARGET_ARTEDB_MIN);
     }

     if (!FetchTable(x1,t,0)) { 
       ARTE_exit( GRR_SERROR,
         " $$$ acReadTARGET: Cannot fetch a table ", -1);
     }

     // Now read the informations
 
     char *test[TGTS_Nwires];
     unsigned char wirenum[TGTS_Nwires];
     int mat[TGTS_Nwires];
     unsigned char shap[TGTS_Nwires];
     float dx[TGTS_Nwires], dy[TGTS_Nwires], dz[TGTS_Nwires], x0[TGTS_Nwires];
     float lamb[TGTS_Nwires];
   
     int size = TGTS_Nwires;
     test[0]=NULL;

     SkipTableField(x1,1);

     GetTableField(x1,TABLEstring,size,test);
     GetTableField(x1,TABLEubyte,size,wirenum);
     GetTableField(x1,TABLEint,size,mat);
     GetTableField(x1,TABLEubyte,size,shap);
     GetTableField(x1,TABLEfloat,size,dx);
     GetTableField(x1,TABLEfloat,size,dy);
     GetTableField(x1,TABLEfloat,size,dz);
     GetTableField(x1,TABLEfloat,size,x0);
     GetTableField(x1,TABLEfloat,size,lamb);
 
     for (int i=0; i<size; i++){
       target_setupdb[i].mat = mat[i];
       target_setupdb[i].shape = (int)shap[i];
       target_setupdb[i].dbdx  = dx[i];
       target_setupdb[i].dbdy  = dy[i];
       target_setupdb[i].dbdz  = dz[i];
       target_setupdb[i].x0    = x0[i];
       target_setupdb[i].lam   = lamb[i];
     }

     CloseTables(x1);
     cout << " acReadTARGET : reading of  SETUP table finished. " << endl;

     // second ALIGN table

     x1 = OpenTables(dbserver,TABLEreadonly);
     if (!x1) {
       ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Can not open database", -1);
     }

     t = BookTable(x1, "ALIGN", 0, 0);
     if (!t) {
       ARTE_exit( GRR_SERROR," $$$ acReadTARGET: Cannot find table", -1);
     }
	
     if (TARGET_ARTEDB_MAJ == -1) {
       cout << "acReadTARGET: reading ALIGN [" 
            << TARGET_ARTE_DEV_MAJ << ", " 
	    <<  TARGET_ARTE_DEV_MIN << "]" << endl;
       TableStamp(x1,TARGET_ARTE_DEV_MAJ,TARGET_ARTE_DEV_MIN);
     }
     else {
       cout << "acReadTARGET: reading ALIGN [" 
            << TARGET_ARTEDB_MAJ << ", " 
	    <<  TARGET_ARTEDB_MIN << "]" << endl;
       TableStamp(x1,TARGET_ARTEDB_MAJ,TARGET_ARTEDB_MIN);
     }

     if (!FetchTable(x1,t,0)) { 
       ARTE_exit( GRR_SERROR,
         " $$$ acReadTARGET: Cannot fetch a table ", -1);
     }

     // Now read the informations
 
     float x[TGTS_Nwires], y[TGTS_Nwires], z[TGTS_Nwires], pos[TGTS_Nwires];

     test[0]=NULL;

     SkipTableField(x1,1);

     GetTableField(x1,TABLEstring,size,test);
     GetTableField(x1,TABLEubyte,size,wirenum);
     GetTableField(x1,TABLEfloat,size,pos);
     GetTableField(x1,TABLEfloat,size,x);
     GetTableField(x1,TABLEfloat,size,y);
     GetTableField(x1,TABLEfloat,size,z);

     for (int i=0; i<size; i++){
       target_aligndb[i].dbx   = x[i];
       target_aligndb[i].dby   = y[i];
       target_aligndb[i].dbz   = z[i];
       target_aligndb[i].dbpos = pos[i];
     }

     CloseTables(x1);
     cout << " acReadTARGET : reading of ALIGN table finished. " << endl;

     // ************************************************************************

     char * targetalignfile;
     char * targetsetupfile;
    
     targetsetupfile = getenv("TARGET_ARTE_SETUP_FILE");

     if (targetsetupfile) {

       cout << "acReadTARGET: === Setup read from file - overwriting DB ! " << endl;
       ifstream in;
       string name;
       int wnr;
       char buff[100]; 

       in.open(targetsetupfile, ios::in);

       cout << " *** setTARGETsetup_from_file : " << endl;

       while (in>>name) {
         if (name.at(0)!='W') { in.getline(buff,100); }
         else {
           if ((in>>wnr) >=0) 
             if(wnr < TGTS_Nwires) {
               in>>target_setupdb[wnr].mat
                 >>target_setupdb[wnr].shape
                 >>target_setupdb[wnr].dbdx
                 >>target_setupdb[wnr].dbdy
                 >>target_setupdb[wnr].dbdz
                 >>target_setupdb[wnr].x0
                 >>target_setupdb[wnr].lam; 
	     }
	  }
       }
       in.close();
     }

     targetalignfile = getenv("TARGET_ARTE_ALIGN_FILE");

     if (targetalignfile) {

       cout << "acReadTARGET: === Alignment read from file - overwriting DB ! " << endl;
       ifstream in;
       string name;
       int wnr;
       char buff[100]; 

       in.open(targetalignfile, ios::in);

       cout << " *** setTARGETalign_from_file : " << endl;
   
       while (in>>name) {
         if (name.at(0)!='W') { in.getline(buff,100); }
         else {
           if ((in>>wnr) >=0) 
             if(wnr < TGTS_Nwires) {
               in>>target_aligndb[wnr].dbpos
                 >>target_aligndb[wnr].dbx
                 >>target_aligndb[wnr].dby
                 >>target_aligndb[wnr].dbz;
	     }
	 }
       } 
       in.close();
     }
     // apply Setup & Alignment

     int rc = acTARGETapplysetup();
     rc = acTARGETapplyalignment();
     ARTE_mark( "acReadTARGET 1", 0 );
     return 0;
  }
  
  cout << "acReadTARGET : No Target information available ! " << endl;
  return -1;  
}

//=============================================================================


int acTARGETapplysetup() {

  // apply SETUP informations to GTAR table

  float rote[9] = { 1.0, 0.0, 0.0,    // unit matrix
	            0.0, 1.0, 0.0,
	            0.0, 0.0, 1.0};

  for(ArteTableIterator<GTAR> igtar; igtar.valid(); igtar.next()) {
    ArtePointer<GTAR> gtar(*igtar);
    int iwnr = gtar->wnr-1;

    gtar->rot = rote;

    // fill the wire names

    if ( iwnr == 0 ) std::strncpy( gtar->name, "1 a ", 4);
    if ( iwnr == 1 ) std::strncpy( gtar->name, "1 b ", 4);
    if ( iwnr == 2 ) std::strncpy( gtar->name, "1 i ", 4);
    if ( iwnr == 3 ) std::strncpy( gtar->name, "1 o ", 4);
    if ( iwnr == 4 ) std::strncpy( gtar->name, "2 a ", 4);
    if ( iwnr == 5 ) std::strncpy( gtar->name, "2 b ", 4);
    if ( iwnr == 6 ) std::strncpy( gtar->name, "2 i ", 4);
    if ( iwnr == 7 ) std::strncpy( gtar->name, "2 o ", 4);

    gtar->cmp  = target_setupdb[iwnr].mat;
    gtar->shap = target_setupdb[iwnr].shape;
    gtar->dx   = target_setupdb[iwnr].dbdx;
    gtar->dy   = target_setupdb[iwnr].dbdy;
    gtar->dz   = target_setupdb[iwnr].dbdz;     
    gtar->eff  = target_setupdb[iwnr].eff;
  }
  return 1;
}

//=============================================================================

int acTARGETapplyalignment() {

  // apply ALIGN informations to GTAR table

  for(ArteTableIterator<GTAR> igtar; igtar.valid(); igtar.next()) {
    ArtePointer<GTAR> gtar(*igtar);
    int iwnr = gtar->wnr-1;

    gtar->x = target_aligndb[iwnr].dbx;	
    gtar->y = target_aligndb[iwnr].dby;
    gtar->z = target_aligndb[iwnr].dbz;

    if (iwnr==0 || iwnr==4 || iwnr==1 || iwnr==5)  // above & below wires
      gtar->y = target_aligndb[iwnr].dbtgts- target_aligndb[iwnr].dbpos + 
                target_aligndb[iwnr].dby;
   
    if (iwnr==2 || iwnr==6 || iwnr==3 || iwnr==7)  // inner & outer wires
      gtar->x = target_aligndb[iwnr].dbtgts- target_aligndb[iwnr].dbpos + 
                target_aligndb[iwnr].dbx;
  }
  return 1;
}

//=============================================================================

/************************************************************************
* KATRGT  target database files
*
* called from  KUIP
***********************************************************************/
#include <stdio.h>
#include <string.h>
#include <ctype.h>
//#include "arte/Arte.hh"

extern "C" void kugetc_( char*, int *, int );
extern "C" void kugeti_( int* );

extern "C" void trgtdb_() {
  int number = -1;
  kugeti_( &number);
  TARGET_ARTEDB_MAJ=number;
  kugeti_( &number);
  TARGET_ARTEDB_MIN=number;
}

void katrgt_() {
  //extern "C" void katrgt_() {

//---------------------------------------------------------------
//   read in subdetector
//---------------------------------------------------------------
  if (Nrfile < 100){

   int len;
   int len1 = 100;
   char input[120];
   kugetc_(input, &len, len1);
   for (int i =0; i<len; i++) {input[i] = tolower(input[i]);};
   input[len] = 0;

   DBfile[Nrfile]  = (char *) malloc(strlen(input)+1);
   sprintf(DBfile[Nrfile],"%s", input);

   Nrfile++;

  } else {
    cout << " too many target database files " << endl;
  };

};
