
// *****************************************************************************
//
// source:    acinit.C
//
// type:      source code
//
// created:   22 Nov 1999
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// symbol:    acinit_()
//
// changes:
// 24 Apr 2000 (AG) use ARTE_mark()
//  2 Jun 2000 (AG) use ARTE_mark()
// 29 Oct 2001 (JH) added memory usage monitoring
// 05 Nov 2001 (JH) check that online kumac was read
// 04 Mar 2002 (JH) Do not make kumac check in online event display mode
//
// *****************************************************************************

// standard includes
#include <iostream.h>

// ARTE includes
#include "farm/farm.hh"
#include "arte/arte.hh"

// prototypes
int RC_init();
int DQ_init();
int AC_init();

extern int FAMEM;
extern int FAON;
extern int DISPLAY;
extern int NOBLOCK;

//
// program code
//
extern "C" void
acinit_()
{
  ARTE_mark( "acinit_ 0", 0 );

  // check that in online mode the kumac was read properly by checking 
  // that the DST_OUT stream was set

  if (FAON && !DISPLAY && !NOBLOCK) {
    int OK = 0;
    for( ArteTable<MEIO>::iterator ptr = ArteTable<MEIO>::begin();
	 ptr != ArteTable<MEIO>::end();++ptr ){
      if (!strncmp(ptr->stream, "DST_OUT", 7) || !strncmp(ptr->stream, "MINI_OUT", 8)) {
	if (ptr->flag == 1) OK = 1;
      }
    }

    if (!OK) ARTE_exit(GRR_SFATAL, "Online Farm mode but neither DST_OUT nor "
		       "MINI_OUT stream were set in kumac! Exiting!", 0);
  }

  // inits
  int rc = 0;
  int in[2], out[2];
  static int mem_ac = 0;
  static int mem_rc = 0;
  static int mem_dq = 0;
  static int mem_before = 0;

  vm_size(in);
  mem_before = in[0];

  // RC initialiation
  ARTE_mark( "RC_init", 0 );
  vm_size(in);
  rc = RC_init();
  vm_size(out);
  mem_rc += (out[0]-in[0]);
  
  // DQ initialization
  ARTE_mark( "DQ_init", 0 );
  vm_size(in);
  rc = DQ_init();
  vm_size(out);
  mem_dq += (out[0]-in[0]);

  // AC initialization
  ARTE_mark( "AC_init", 0 );
  vm_size(in);
  rc = AC_init();
  vm_size(out);
  mem_ac += (out[0]-in[0]);

  if (FAMEM) printf("\033[31m Memory usage (KB) : Before_init: %d, RC_init:%d, "
		    "DQ_init:%d, AC_init:%d\033[0m\n",
		    mem_before/1024, mem_rc/1024, mem_dq/1024, mem_ac/1024);

  ARTE_mark( "acinit_ 1", 0 );
}
