// ********************************************************************
//
//  Routine:	alca/actaxxPrep.C
//  Author:		Dmitry Golubkov
//
//  Description:	Restores TAHB --> HITB,
//				called from a user routine
//
//  Modifications:
//   -- none yet					06/12/00 DG
//   -- added ARTE_mark					14/12/00 DG
//   -- restore ITR hits				19/12/00 TZ
//   -- fix precision loss of ITR resolution		18/01/01 DG
//   -- added VDS					02/02/01 DG
//   -- no OTR hit preparation if OtrI not initialized	15/11/01 DG
//
// ********************************************************************

#include <iostream.h>
#include <math.h>
#include <stdio.h>
#include <stdlib.h>
#include <fstream.h>
#include <iomanip.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "rang/zfextse.hh"
#include "rang/zfparext.hh"
#include "rang/Fit.hh"
#include "reco/rcflags.hh"
#include "Otr/OtrI.h"

#include "Otr/OtrEvent.h"


// ONLINE includes
#include "farm/farm.hh"
#include "table.h"
#include "esp.h"

#include <sys/time.h>

#include "alca/actaxxComm.hh"
#include "alca/actaxxFLG.hh"

#include "silarte/silarte.hh"




// Make hit preparation input for OtrEventUpdate() from TAHB bank

int actaxxPrep( int jTaTr =-1 )
{
	ARTE_mark( "actaxxPrep 0", GRR_OK );


	// Complain loud switching recoXXX flags
	{
		static char *format = {	" *** actaxxPrep: according to AC/TAXX/HITP, "
					"%s hitPreparation done now by TAXX\n"
					"\t\t[/RECO/SUBD bla-bla-bla settings may be rewritten]\n"
				      };

		int nSubd = 0;
		if ( actaxx_Command.flg & TACMD::hp_vds )
		{
			nSubd++;
			static int count = 0;
			if ( !count ) printf(format,"VDS");
			count = 1;
			recoVDS &= ~0x01;
		}

		if ( actaxx_Command.flg & TACMD::hp_itr )
		{
			nSubd++;
			static int count = 0;
			if ( !count ) printf(format,"ITR");
			count = 1;
			recoITR &= ~0x01;
		}

		if ( actaxx_Command.flg & TACMD::hp_otr )
		{
			nSubd++;
			static int count = 0;
			if ( !count ) printf(format,"OTR");
			count = 1;
			recoOTR &= ~0x01;
			if ( !OtrDecodeGet() )
			{
				printf("\n\n *** actaxxPrep: OtrI is not initialized, no TAXX OTR hit preparation done!\n\n\n");
				actaxx_Command.flg &= ~TACMD::hp_otr;
			}
		}


		if ( !nSubd )
		{
			static unsigned int count = 0;
			if ( count++ < 20 ) printf(" *** actaxxPrep: AC/TAXX/HITP are all off!\n\t\t[no TAXX hit preparation done]\n");

			return 0;
		}
	}


	taTATRSeedTATRGetIndex(1);

	// We_should_avoid_possibility_of_counting_the_same_hit_twice !
	// Check if any hit preparation has been done already

	for (	ArteTableIterator<HITB> hitb = ArteTable<HITB>::begin();
		hitb != ArteTable<HITB>::end(); ++hitb )
	{
		if (	hitb->gede->cmp != 1
			&& hitb->gede->cmp != 2
			&& hitb->gede->cmp != 3
		   ) continue;

		cout << " *** actaxxPrep: found an VDS/OTR/ITR hit "
			<< "(#"<< hitb.index()<< ")\n"
			<< "\t\t[TAHB --> HITB preparation skipped]\n";

		taTATRSeedTATRGetNumberOf(0);

		ARTE_mark( "actaxxPrep 1", GRR_OK );
		return 0;
	}


	// Remove everything that can remind about previous hit preparation
	for ( ArteTableIterator<GEDE> gede = ArteTable<GEDE>::begin();
		gede != ArteTable<GEDE>::end(); ++gede )
	{
		if ( (gede->cmp != 1) && (gede->cmp != 2) && (gede->cmp != 3) ) continue;

		gede->hit1 = 0;
		gede->hitl =-1;
	}

	// Clear tables before next event (usually done in rcStandAlone)
	ArteTable<RREL>::clear();
	ArteTable<RPNT>::clear();
	ArteTable<RVER>::clear();
	ArteTable<RHIT>::clear();
	ArteTable<RSEG>::clear();
	ArteTable<RTRA>::clear();
	ArteTable<RCCL>::clear();


	// Note, this is needed for TATR seeding only
	taTATRSeedTATRGetNumberOf(ArteTable<TATR>::size());

	if ( !ArteTable<TAHB>::size() )
	{
		ARTE_mark( "actaxxPrep 1", GRR_OK );
		return 0;
	}


	if ( jTaTr == 0 )
		taTATRSeedTATRGetNumberOf(0); // No seeding
	else if ( jTaTr > 0 )
	{
		taTATRSeedTATRGetNumberOf(1);
		taTATRSeedTATRGetIndex(jTaTr);
	}




	int nHits = 0;
	HITB hitb;

	for (	ArteTableIterator<TAHB> tahb = ArteTable<TAHB>::begin();
		tahb != ArteTable<TAHB>::end(); ++tahb )
	{
		// FITONLY flagging if hit was used to fit the track
		if (	(actaxx_Command.flg & TACMD::fitonly)
			&& !(tahb->flg&TAFLG::fit)
		   ) continue;

		if ( jTaTr >= 0 )
		{
			// For TA event split into single tracks
			if ( (int)(tahb->flg&TAFLG::tatr) != jTaTr ) continue;
		}
		else
		{
			// Do not count the same hit twice
			if ( tahb.index() > 1 )
			{
				ArtePointer<TAHB> tahbOld(tahb.index()-1);

				if ( (tahbOld->flg & tahb->flg) & (TAFLG::itr|TAFLG::otr) )
				{
					if	( tahb->flg & TAFLG::otr )
					{
						if ( tahb->channel == tahbOld->channel ) continue;
					}
					else if	( tahb->flg & TAFLG::itr )
					{
						if (	tahb->dt == tahbOld->dt
							&& tahb->gede == tahbOld->gede
						   ) continue;
					}
					else if ( tahb->flg & TAFLG::vds )
					{
						if (	tahb->dt == tahbOld->dt
							&& tahb->gede == tahbOld->gede
						   ) continue;
					}
				}
			}
		}

		// Restore HITB
		if ( tahb->flg&TAFLG::itr )
		{
		  // Inner tracker, Torsten Zeuner 18/12/2000

                  if ( !(actaxx_Command.flg & TACMD::hp_itr) ) continue;

                  if ( tahb->gede->cmp!=3 ) {
                    cout << "ERROR : NOT AN ITR HIT " << endl;
                    continue;
                  }

                  ArtePointer<GEDE> gede, gede1;
                   
                   if ( tahb->gede->sec == 2 || tahb->gede->sec == 4 ) {
                     gede1 = tahb->gede;
                     gede  = ArtePointer<GEDE>(gede1.index()+1);
                   }
                   if ( tahb->gede->sec == 5 || tahb->gede->sec == 7 ) {
                     gede = tahb->gede;
                     gede1  = ArtePointer<GEDE>(gede.index()+1);
                   }
                   if ( tahb->gede->sec == 1 || tahb->gede->sec == 3 ) {
                     gede = tahb->gede;
                     gede1 = ArtePointer<GEDE>(gede.index()-1);
                   }
                   if ( tahb->gede->sec == 6 || tahb->gede->sec == 8 ) {
                     gede1 = tahb->gede;
                     gede = ArtePointer<GEDE>(gede1.index()-1);
                   }
                   
                   float relpos=0;
                   float gasgap = gede->zmax-gede->zmin;

                   hitb.z    = gede->zfw[0]+gasgap/2;

                   if ( gede->gesl->nsl>8 && gede->sec > 4 ) {
                     // -x side Typ III 
                     relpos = gede->nwi[0]+gede1->nwi[0]+4-tahb->dt;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl>8 && gede->sec < 5 ) {
                     // +x side Typ III
                     relpos = tahb->dt-5;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl<9 && gede->sec > 4 ) {
                     // -x side Typ I+II
                     relpos = gede->nwi[0]+gede1->nwi[0]+7-tahb->dt;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   if ( gede->gesl->nsl<9 && gede->sec < 5 ) {
                     // +x side Typ I+II
                     relpos = tahb->dt-8;
                     hitb.x    = relpos*gede->pit + gede->xfw[0];
                   }

                   hitb.r = float(tahb->channel)/1000000.;

                   hitb.gede = tahb->gede;


		}
		else if ( tahb->flg & TAFLG::otr )
		{
			// Outer tracker

			if ( !(actaxx_Command.flg & TACMD::hp_otr) ) continue;

			unsigned char d = (unsigned char) (int) tahb->dt;

			int ci		= OtrIFromId(tahb->channel);
			if ( ci == OTRIInvalid )
			{
				printf("actaxxPrep: *** problems with tahb->channel\n");
				continue;
			}


                        // Ask OtrI's current opinion on this channel (Bad = Noisy or Dead)
                        if ( OtrIIsBad(ci) ) continue;


			// Note, OtrEventUpdate will think, Wax_Was_Used (as in MC)
			// In Wax hitb->chid = wireId + 128*(chId + 16*asdId),
			// where wireId = wire+32*mono

			int mono	= OtrIMonoLayer(ci);

			if ( taCommOTR.flg & TACommOTR::OtrI_st )
			{
				int wire	= OtrIWire(ci);
				int ch		= OtrIAsdChannel(ci);
				int asd		= OtrIAsdIndex(ci);
				int waxid	= wire + (mono<<5) + ((ch + (asd<<4))<<7);
				hitb.chid	= waxid;
			}
			else
				hitb.chid	= tahb->channel;



			hitb.y		= (int) d;

			if ( taCommOTR.flg & TACommOTR::OtrI_gd )
				hitb.gede	= ArtePointer<GEDE>
						  (
							OtrGeoGet( OtrIGeoIndex(ci) ).index()
						  );
			else
			{
				if ( tahb->gede.index() == 0 )
				{
					cout<<"acatxxPrep: *** OTR gede = 0 found,"
						<<" try /AC/TAXX/OTR/OPT gede=n";

					ARTE_mark( "actaxxPrep 1", GRR_FISHY );
					return -1;
				}

				hitb.gede	= tahb->gede;
			}


			hitb.x		= hitb.gede->xfw[mono]
						+ OtrIWire(ci)*OtrIWirePitch(ci);
			hitb.z		= hitb.gede->zfw[mono];

			float t		= d - OtrITdcT0(ci);
			hitb.q		= OtrRtRadius(ci,t);
			hitb.r		= OtrRtRadiusSigma(ci,t);
		}
		else if ( tahb->flg & TAFLG::vds )
		{
			// Vertex detector

			if ( !(actaxx_Command.flg & TACMD::hp_vds) ) continue;

			// compressed side id (superlayer nameing scheme)
			/* sideid = ((superlayer<<4) | (quadrant<<2) | view) */

			int sideid	= tahb->gede.index();

			int sl		= ((sideid&0x70)>>4) + 1;
			int sec		= ((sideid&0x0c)>>2) + 1;
			int lay		= (sideid&0x03) + 1;

			char slname[4];
			sprintf(slname,"**%d",sl);

			// VDS gede index
			int ind = silarte_returnGEDE( slname, lay, sec );

			if ( !ind )
			{
				static unsigned int count = 0;
				if ( (count++) < 20 )
					printf(" *** actaxxPrep: VDS plane{sl=%d, lay=%d, sec=%d} has no matching gede!\n\t\t[failed to restore a hit]\n", sl, lay, sec);

				continue;
			}

			ArtePointer<GEDE> gede(ind);

			hitb.z		= gede->zfw[0];
			hitb.x		= tahb->dt * gede->pit + gede->xfw[0];
			hitb.q		= float(tahb->channel)/1.e3;
			hitb.r		= silarte_iol.hit_resolution.value;
			hitb.gede	= gede;
		}
		else
			continue;


		ArteTable<HITB>::push_back(hitb);




		int hitbInd		= ArteTable<HITB>::size();
		ArtePointer<GEDE> gede	= hitb.gede;

		// TAHBs are sorted already at write time
		// struct GEDE { ... ArteTableIterator<HITB> hit1; ... }

		if ( !(*gede->hit1).index() )
		{
			gede->hit1 = hitbInd;
			gede->hitl = hitbInd;
		}
		else
			gede->hitl = hitbInd;




		nHits++;
	}

	if ( !nHits ) taTATRSeedTATRGetNumberOf(0);

	ARTE_mark( "actaxxPrep 1", GRR_OK );
	return nHits;
}





// Store/(by default return) index of first TATR for ranger seed
int taTATRSeedTATRGetNumberOf(int a)
{
	static int b;
	if ( a !=-1 ) b = a;

	return b;
}





// Store/(by default return) number of TATRs for ranger to seed with
int taTATRSeedTATRGetIndex(int a) 
{
	static int b;
	if ( a !=-1 ) b = a;

	return b;
}
