//                              -*- Mode: C++ -*-
//: ChronoManager.C
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 3/4/97
// Last Update ........
//
#undef TESTCHRONOMANAGER


#include <iostream>
#include "arte/ChronoManager.hh"
#include "arte/Chrono.hh"


///////// -- Chrono Manager Class ----------------------

///////// ChronoManager static data initialization

int ChronoManager::numberOfChronoManagers = 0;
std::list<Chrono*> * ChronoManager::plist = NULL;

// Constructor
ChronoManager::ChronoManager(){
  if(numberOfChronoManagers==0){
    instance();
  } else {
    numberOfChronoManagers++;
  }
}

// Destructor
ChronoManager::~ChronoManager(){
  if(numberOfChronoManagers>0){
    numberOfChronoManagers--;
    if(numberOfChronoManagers==0){
      delete plist;
      plist = NULL;
    }
  } 
}

//--
//  Initialization of the ChronoManager is done calling this
//  function
void ChronoManager::instance(){
  if(numberOfChronoManagers==0){
    numberOfChronoManagers = 1;
    plist = new std::list<Chrono*>;
  } else {
    numberOfChronoManagers++;
  }
}

//--
//  Delete the ChronoManager static data
void ChronoManager::deleteInstance(){
  if(numberOfChronoManagers>0){
    delete plist;
    plist = NULL;
    numberOfChronoManagers = 0;
  }
}



// Return a brand new Chrono
Chrono* ChronoManager::createChrono(char *s){
  Chrono* nChp= new Chrono(s);
  plist->push_back(nChp);
  return nChp;
}
/*
// Return a brand new Chrono
Chrono* ChronoManager::createChrono(string &s){
  Chrono* nChp= new Chrono(s);
  plist->push_back(nChp);
  return nChp;
} 
*/

// Delete the current chrono. It is assumed that this chrono is in the list
void  ChronoManager::deleteChrono(Chrono* ch){
  plist->remove(ch);
  delete ch;
}


// Print informations for all stored Chronos
void ChronoManager::printInformations(){
  if(plist->begin()!=plist->end()) {
    std::cout<<"\t\t\t   Measured time (s)\t   Average  time (s)"<<std::endl;
    std::cout<<"Chrono Name   Measures  Total\tUser \tSystem"
        <<"\tTotal \tUser \tSystem"<<std::endl; 
    for(std::list<Chrono*>::const_iterator p = plist->begin();
		p!=plist->end(); p++)
      (*p)->printLine();
  } else {
    std::cout<<" ChronoManager: no active chrono !!!!! "<<std::endl;
  }
}




#ifdef TESTCHRONOMANAGER
main(){
  ChronoManager CM;
  Chrono*  outer = CM.createChrono("Outer loop");
  Chrono*  medium = CM.createChrono("Medium loop");
  Chrono*  inner = CM.createChrono("Inner loop");
 
  int i,j;
  outer->start();
  for(i=0; i<4000; i++){
     medium->start();
     for(j=0; j<400; j++){
       inner->start();
       inner->stop();
     }
     medium->stop();
  }
  outer->stop();

  CM.printInformations();

}
#endif

