>NAME LUNDKUIP

>MENU /Generator
>GUIDANCE
Lund event generators: Jetset, Pythia, Fritiof.

>COMMAND LUND
>GUIDANCE
Define Lund event generator configuration.
Examples:
 LUND inel
 lund cc
 lund jpsi 
 LUND bb
 LUND jpsi -Jpsi
 lund jpsi -chi_c0
 lund upsilon -Upsilon
 lund upsilon -chi_b2
 lund jpsi -eta_c inel 4 
.
The last example means: generate events with standard charmonium production,
with a eta_c in the final state decay chain. In addition the event
contains N inelastic interactions where N is Poisson-distributed with a
mean value of 4. Replacing 4 by -4 leads to a generation of
exactly 4 inelastic events.
.
Attention: If any of a group of particles is required in the final state,
use the PARTICLE command. If for example the charmonium triplet of
P states is to be selected in the JPSI mode, use the command
PARTICLE 'chi_c0' 'chi_c1' 'chi_c2'.
.
Suggestion: in most cases it is by far more efficient to avoid
event mixing after the Lund step. In this case the mixing can be performed
after the Geant step by reading from different input files (see command
IOMIX).
.
The command LUND and any specification of an event input file are
mutually exclusive.
>ACTION KALUND
>PARAMETER
PHYS1 'bb/cc/inelastic/Jpsi/Upsilon' C D=' '
+
PHYS2 'inelastic' C D=' '
MIX 'number of inelastic events to be mixed to cc/bb' R D=4.
OPTION 'requested particle' C D=' '

>COMMAND PARTICLE
>GUIDANCE
Select (up to 5) particles, at least one of which is requested in the 
final state. For hard scattering processes, the particle are
required in the hard process (PYTHIA) ignoring the underlying
event (FRITIOF).
Examples:
  PARTICLE eta_c
  particle Jpsi
  PARTICLE 'B.' 'Lam_b.' 'pi+'
  particle 'chi_b0' 'chi_b1' 'chi_b2'
.
>ACTION KASELE
>PARAMETER
PAR1 'requested particle (1)' C D=' '
+
PAR2 'requested particle (2)' C D=' '
PAR3 'requested particle (3)' C D=' '
PAR4 'requested particle (4)' C D=' '
PAR5 'requested particle (5)' C D=' '

>COMMAND LEPTON
>GUIDANCE
Define lepton trigger conditions on generator level.
>ACTION KALEPT
>PARAMETER
PAR1 'Number of leptons (1 or 2)' I D=2
PAR2 'PT cut of lepton 1' R D=0.8
+
PAR3 'PT cut of lepton 2' R D=0.8
PAR4 'Invariant mass cut for lepton pair' R D=2.

>MENU PRETRIG
>GUIDANCE
Define pretrigger conditions and acceptances on generator level.
The conditions set by the LEPTON command are used in addition.

>COMMAND MULT
>GUIDANCE
Minimum numbers of electrons and muons required.
>ACTION KALMUL
>PARAMETER
PAR1 'N_el_min' I D=0
PAR2 'N_mu_min' I D=0

>MENU CUT
>GUIDANCE
Activate an angular pretrigger region for the trigger leptons
and define a minimum pT cut for this region. Up to 20 regions 
can be activated by repeated application of the command. If no
pretrigger region is explicitly activated, no angular or pT cuts
are applied. Exception: The cuts defined by the LEPTON command
are applied independently.

>COMMAND ELECTRON
>GUIDANCE
Activate a pretrigger window for electrons.
>ACTION KAECUT
>PARAMETER
PAR1 'pT_min' R D=0.
+
PAR2 'Theta_x_min' R D=-3.1415
PAR3 'Theta_y_min' R D=-3.1415
PAR4 'Theta_x_max' R D=3.1415
PAR5 'Theta_y_max' R D=3.1415

>COMMAND MUON
>GUIDANCE
Activate a pretrigger window for muons.
>ACTION KAMCUT
>PARAMETER
PAR1 'pT_min' R D=0.
+
PAR2 'Theta_x_min' R D=-3.1415
PAR3 'Theta_y_min' R D=-3.1415
PAR4 'Theta_x_max' R D=3.1415
PAR5 'Theta_y_max' R D=3.1415

>MENU /Generator
>Command PHASESPACE
>Guidance
Generates phase space particles
>Action KAPHSP
>Parameter
CPART 'Arte name of the particle' C
PMIN  'minimal momentum' R D=10.
PMAX  'maximal momentum' r D=10.
THMIN 'minimal polar angle' R D=0.010
THMAX 'maximal polar angle' R D=0.250
PHIMIN 'minimal azimuth angle' R D=0.
PHIMAX 'maximal azimuth angle' R D=6.283185307

>COMMAND SEEDLUND
>GUIDANCE
Set the seed for the Lund random number generator.
The generator is RANECU (CERNLIB V114) which needs exactly 2 positive
integer numbers as seed. Arte generates an automatical seed if
both numbers or the whole SEEDLUND command are omitted.
>ACTION KASEED
>PARAMETER
+
SEED1 '1st random seed' I D=0
SEED2 '2nd random seed' I D=0

>COMMAND EBEAM
>GUIDANCE
Set the energy of the HERA proton beam [GeV].
>ACTION KAEBEA
>PARAMETER
EBEAM 'beam energy' R D=920.

>COMMAND TARGET
>GUIDANCE
Define target material for Lund.
Only a few materials are known to Lund (see Lund manual).
.
IMPORTANT: this does not affect the target material in GEANT !!!
>ACTION KATARG
>PARAMETER
MAT 'target material' C D='AL'

>COMMAND LUGIVE
>GUIDANCE
Modifiy the value of parameters in LUJETS, LUDAT1...4, LUDATR,
PYSUBS, PYPARS, PYINT1...7 of the Lund program.
Equivalent to the Fortran statement CALL LUGIVE ('LUVAR=VALUE').
.
Example:
 LUGIVE "MSTJ(26)=0"
switches B-Bbar mixing off (c.f. Pythia/Jetset manual).
>ACTION KALUGI
>PARAMETER
ASSIGN 'assignment statement enclosed in apostrophes' C D=' '

>COMMAND EVENTNO
>GUIDANCE
Set event/run(/experiment) number for MC events.
The experiment number must be negative.
>ACTION KAEVEN
>PARAMETER
EVENT 'event number' I D=1
RUN 'run number' I D=0
+
EXP 'experiment number' I D=0

>MENU MODE
>GUIDANCE
Run modes for the subprocess generators.

>COMMAND INELASTIC
>GUIDANCE
Special generator modes for inelastic events.
Possibilities:
   MB       all events are accepted with weight 1.
   PTTHRES  importance sampling; events with maximum final particle pT
            below a threshold PT0 are statistically suppressed; 
            compensated by event weight.
   PTCUT    biased events with weight 1; Only hard scatterings with
            a minimum pT>pT0 of at least one final state particle.
>ACTION KAINEL
>PARAMETER
INEMOD 'mode MB/PTTHRES/PTCUT' C D='MB'
+
PT0 'pT threshold/cut' R D=0.

>COMMAND PSI
>GUIDANCE
Special generator modes for J/Psi and chi_c production.
Possibilities:
    PTCUT   Require minimum parton level pT>pT0 in partonic final state.
            Leads to a soft pT cutoff for the charmonium. Weight=1.
    REWEI   use default pT cut-off. Introduce event weight such that
            inclusive charmonium spectra fit the data.
>ACTION KAJPSI
>PARAMETER
PSIMOD 'mode PTCUT/REWEI' C D='REWEI'
+
pT0 'pT cut (partons)' R D=1.

>COMMAND UPSILON
>GUIDANCE
Special generator modes for Upsilon and chi_b production.
Possibilities:
    PTCUT   Require minimum parton level pT>pT0 in partonic final state.
            Leads to a soft pT cutoff for the bottomonium. Weight=1.
    REWEI   use default pT cut-off. Introduce event weight such that
            inclusive bottomonium spectra fit the data.
>ACTION KAUPSI
>PARAMETER
UPSMOD 'mode PTCUT/REWEI' C D='REWEI'
+
pT0 'pT cut (partons)' R D=1.

>COMMAND QQ
>GUIDANCE
Special fragmentation options for open charm and beauty production.
Defines how PYTHIA and FRITIOF are combined. 
None of the methods is fully consistent.
Possibilities:
    MODE 1:  Keep only single resonance/particle originating directly
             from the heavy (anti)quark; rest of 4-mom transferred to
             underlying event (FRITIOF).
    MODE 2:  Keep full clusters and half-strings connected to heavy quarks;
             rest of 4-mom transferred to underlying event (FRITIOF).
    MODE 3:  Keep jets from heavy quarks as generated by independent
             jet fragmentation performed in the cms frame of the
             colliding partons; rest of 4-mom transferred to underlying 
             event (FRITIOF).
    MODE 4:  Keep full clusters and full colour strings connected to
             heavy quarks; rest of 4-mom (if any) transferred to underlying
             event (FRITIOF).
    MODE 5:  Keep full hard pN interaction (PYTHIA) in which heavy quarks
             are produced; ignore nuclear energy loss and multiple 
             scattering (FRITIOF).
>ACTION KAFRAG
>PARAMETER
mode 'fragmentation mode (1-5)' I D=2

>MENU /Generator
>COMMAND REWEIGHT
>GUIDANCE
Adjust the multiplicity for heavy flavour production. 
There are four reweighting methods available:
    mode 0:  No reweighting (default).
    mode 1:  Reweighting with the fraction of proton-nucleon sub-collisions.
	     An importance sampling is carried out in with the quantity
	     N/A (N number of sub-collisions, A atomic number of target 
	     nucleus).
    mode 2:  Reweighting with the fraction of proton-nucleon sub-collisions.
	     A reweighting with (N/A)**2 reproduces the multiplicity in data
	     better than N/A in mode 1.
    mode 3:  Reweighting with the number of accepted charged tracks, defined
 	     as stable charged tracks in decay chains from the primary 
	     interaction with slopes 10 mrad < px/pz < 220 mrad and
	     10 mrad < py/pz < 220 mrad. 
    mode 4:  Reweighting with the number of gluons in the interaction.
>ACTION KAFRIT
>PARAMETER
mode 'reweighting mode (0-4)' I D=3






