/*-- Author :    Serguei Moukhine   26/04/98
**
** Changes :
** 24/05/02	use KUPROC to avoid problems with tty settings	(DG)
*/

#include <arte/mecCount.h>
#include "arte/ArteFlag.h"
#include "arte/MEPack.h"

extern void chttoh_ (char *t, int *h, int *n);
extern void qnexte_ ();
extern void meallpc_ (int *buf);

int mecErrorCount   (int *id, char *text)
/************************************************************************
*
* Description: error message; count for final statistics
*
* Called from: anywhere (from C programs)
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does
*                  not matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_INFINITE_COUNT,1,0);
return(0);
}

int mecWarningCount (int *id, char *text)
/************************************************************************
*
* Description: warning; count number of occurencies
*
* Called from: anywhere
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does not
*                  matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_WARNING_COUNT,MEC_INFINITE_COUNT,0);
return(0);
}

int mecNextCount    (int *id, char *text)
/************************************************************************
*
* Description: warning; count number of occurencies; skip current event
*
* Called from: anywhere
*
* External references: mecMessageCount
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does not
*                  matter: important is its address.
*  TEXT    in      message text
*
************************************************************************/
{
mecMessageCount(id,text,MEC_WARNING_COUNT,MEC_INFINITE_COUNT,1);
return(0);
}

int mecMessageCount (int *id, char *text, int np, int nf, int lnext)
/************************************************************************
*
* Description: list, count, handle an error
*
* Called from: anywhere
*
* External references: mecAllPrint, mmbook_, atstopcc_, qnexte_
*
* Argument in/out  description
*  ID      pointer identifier. The content of this parameter does
*                  not matter: important is its address.
*  NP              print the text NP times.
*  NF              fatal error if it occurs more than NF times.
*  LNEXT           if 1 then skip current event.
*  TEXT    in      message text
*
************************************************************************/
{
/*
** Negative NP and NF mean INFINITE
*/

if (np < 0)
   np = MEC_INFINITE_COUNT;

if (nf < 0)
   nf = MEC_INFINITE_COUNT;

memesc_(id, &np, &nf, text, strlen(text));

/*
** Skip to next event
*/
if (lnext)
   {
   if (arteflag_.progstat == 0) /* Event processing */
      {
      static int errPtr;
      mecWarningCount(&errPtr, "*** Skip to next event!");
      qnexte_();
      }
   else
      {
      if (arteflag_.ibatch != 0) /* Not interactive */
         {
         char *s = "=== Program may not be properly"
                   " terminated: STOP immediately.";
         meallp_(s, strlen(s));
         exit(0);
         }
      }
   }


return(0);
}

int mecAllPrint (char *text)
/************************************************************************
*
* Description: print message
*
* Called from: mecMessageCount
*
* External references: meallpc_
*
* Argument in/out  description
*  TEXT    in      message text
*
************************************************************************/
{
int  var;
int  buf[20];

var = 20;
chttoh_ (text, buf, &var);
meallpc_(buf);
return(0);
}

