
// *****************************************************************************
//
// source:    utstage.C
//
// created:    1 Jun 1999
//
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    utstage_
//
// called by: IFOPEN
//
// changes:
// 09 Sep 1999 (AG) replace cerr by cout
// 31 Mar 2000 (AG) fix new defines for g++
// 15 Sep 2000 (AG) added l4aFileF_()
// 06 Oct 2000 (AG) polished printout
// 29 Nov 2000 (AG) polished printout
// 05 Aug 2001 (DG) kuip entry for dCache steering
// 18 Dec 2001 (DG) now utstage is called for ZEBRA files only
// 25 Apr 2002 (IR,DG) DCflag bug fixed, io_fpath_() added
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <string.h>

// ONLINE includes
#include "l4arch/stage.h"

// ARTE includes
#include "farm/farm.hh"
#include "farm/fa.hh"

// externals
extern int  FADEB;
extern struct {
  char ctape[256];
  char cdisk[256];
} stage_;

//
// program code
//
extern "C" void
utstage_()
{
  ARTE_mark( "utstage 0", 0 );

  // handle names
  char *ptr = strtok( stage_.ctape, " " ); // find end of tape name
  char tmp[256];                           // temporary tape name
  strcpy( tmp, ptr );                      // copy tape name
  memset( stage_.cdisk, 0, 256 );          // reset disk file name
  memset( stage_.ctape, 0, 256 );          // reset tape file name
  strcpy( stage_.ctape, tmp );             // copy clean tape name
  cout << "FILE: Tape: " << stage_.ctape << endl;


  // --------------------------------------------------------
  // Search for disk file, note that ZEBRA is not usable with
  // dCache, thus we will have to do staging silently
  // --------------------------------------------------------

  int rc = ut_fpath(stage_.ctape, stage_.cdisk, sizeof(stage_.cdisk),
						FADEB > 1 ? FADEB : 1 );
 
  if( FADEB ) cout << "FILE: Disk: " << stage_.cdisk << endl;
  if ( rc != L4A_OK || !strlen(stage_.cdisk) ){
	printf("%s L%d:\tfile finder reported error"
		" \"%s\"\n", __FILE__, __LINE__,
		rc == L4A_FAILURE ? "L4A_FAILURE"
		: rc == L4A_NO_ACS_FILE ? "L4A_NO_ACS_FILE"
		: rc == L4A_NAMETOOLONG ? "L4A_NAMETOOLONG" : "???"
	      );
	ARTE_mark( "utstage 1", 0 );
	return;
  }

  if (ut_getDCflag()) return;

  if( strncmp(stage_.cdisk, ACS_PATH, strlen(ACS_PATH)) ){
    cout << "FILE: Disk: " << stage_.cdisk << endl;
    ARTE_mark( "utstage 1", 0 );  
    return;
  }


  // staging implemented for Linux, Sun, SGI (((?)))
  strcpy(stage_.ctape, stage_.cdisk);
  rc = ut_stage(stage_.ctape, stage_.cdisk, sizeof(stage_.cdisk),
					FADEB > 1 ? FADEB : 1 );

  if ( rc != L4A_OK || !strlen(stage_.cdisk) )
  {
	printf("%s L%d:\tstaging reported error \"%s\"\n",
		__FILE__, __LINE__,
		rc == L4A_FAILURE ? "L4A_FAILURE"
		: rc == L4A_NO_ACS_FILE ? "L4A_NO_ACS_FILE" : "???"
	      );
	ARTE_mark( "utstage 1", 0 );
	return;
  }

  cout << "STAGE: Disk: " << stage_.cdisk << endl;

  ARTE_mark( "utstage 1", 0 );  
}




// ------------------------------
// Kuip entry for dCache steering
// ------------------------------

extern "C" {
	int	ku_npar();   // return the number of parameters 
	int	ku_geti();   // read an integer
	double	ku_getr();  // read a float
	char*	ku_gets();   // read a string
	char*	ku_getc();   // read an uppercase string
	char*	ku_getf();   // read a filename
	char*	ku_getl();   // read a comma separated list

	void	kugeti_(int*);
	void	kugetr_(float*);
}





extern "C"
void io_dcache_()
{
	if ( ku_npar() != 0 )
	{
		char * swt = ku_getc();

		if ( !strcmp(swt, "ON" ) )	ut_setDCflag(1);
		if ( !strcmp(swt, "OFF") )	ut_setDCflag(0);
	}

	printf("The dCache is %s\n", ut_getDCflag() ? "enabled" : "disabled");
}


extern "C"
void io_fpath_()
{
	if ( ku_npar() != 0 )	ut_setfpath(ku_gets());
	printf("The data file search path is '%s'\n", ut_setfpath(NULL));
}
