//                              -*- Mode: C++ -*-
//: ESLightYieldCor.C
//
//
// Author: Mauro Villa   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//
// Created 14/4/97
// Last Update ........
//

#include <math.h>
#include "care/ESLightYieldCor.hh"
#include "arte/ArteMessage.hh"
#include "care/caredata.hh"
#include "care/EMHitCell.hh"

/////////////////////// Static data initialization
/* old values
float ESLightYieldCor::corrPolin
	[ESLightYieldCor::ncals][ESLightYieldCor::maxdegree] = 
    {{1.0, 8.134e-3, -0.2166, 0.2683, 0.2866, -0.5923, 0.07765,0.,-0.03473},
     {1.0,0.0316,0.1941,-3.1,7.42,-5.47,0.,0.,0.},
     {1.0,0.0,0.0,0.,0.,0.,0.,0.,0.}};

unsigned int ESLightYieldCor::degree[ESLightYieldCor::ncals] = {8,5,2};

ESLightYieldCor::lightcorrtype ESLightYieldCor::cortype[ESLightYieldCor::ncals]
= {radial, radial , radial};
*/


float ESLightYieldCor::corrPolin
	[ESLightYieldCor::ncals][ESLightYieldCor::maxdegree] = 
    {{1.0, 0.2717,  0.2122, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
     {1.0, 0.2780, -0.2575, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0},
     {1.0, 0.3691, -0.0819, 0.0, 0.0, 0.0, 0.0, 0.0, 0.0}};

unsigned int ESLightYieldCor::degree[ESLightYieldCor::ncals] = {2,2,2};

ESLightYieldCor::lightcorrtype ESLightYieldCor::cortype[ESLightYieldCor::ncals]
= {coord2, coord2, coord2};

////////////////////// Static function definitions
//--
///  Filling the static data for corrections (light)
void ESLightYieldCor::fill(SubCal subCal, unsigned int cortyp, 
			   unsigned int num, float* data){
  if( num>maxdegree ) num=maxdegree;
  int cal= subCal-1;
  if( cal>=0 && cal<3 && data!=0){
    degree[cal] = num;
    cortype[cal] = (ESLightYieldCor::lightcorrtype) cortyp;
    for(unsigned int i=0; i<=num; ++i){
      corrPolin[cal][i] = data[i];
    }
  }
  dump();
}

//--
///  Print informations on static data
void ESLightYieldCor::dump() {
  arteMsg<<" Light Yield correction factors ";
  arteMsg.allp();
  arteMsg<<" Calorim, type, degree, coefficients";
  arteMsg.allp();
  for(int cal=0; cal<3; cal++){
    arteMsg<<cal<<((cortype[cal]==0)? " radial " : " coord ")
	   <<degree[cal]<<" ";
    for(unsigned int i=0; i<=degree[cal]; ++i){
      arteMsg<<corrPolin[cal][i]<<" ";
    }
    arteMsg.allp();
  }
}

// methods

/// Constructor
ESLightYieldCor::ESLightYieldCor() : ESEnCorStrategy(){}
/// destructor
ESLightYieldCor::~ESLightYieldCor(){}

Energy_t ESLightYieldCor::evaluateEnergy(const EMCluster* clus, 
					 ESPosition & positionData){
  ESEnCorStrategy::evaluateEnergy(clus, positionData);

  const EMHitCell* cell = positionData.cell();
  int ncl = cell->ncl();
  int cal = (ncl==5)? 0: ((ncl==2)? 1 : 2) ;
  float widthx = ECalInfo.xMsize/ncl;
  float widthy = ECalInfo.yMsize/ncl;
  float x = fabs( positionData.x()-cell->x())/widthx;
  float y = fabs( positionData.y()-cell->y())/widthy;
  float r = 0.0;
  switch(cortype[cal]){
  case radial:
    r = sqrt(x*x+y*y);
    corrfact = corr(r, degree[cal], corrPolin[cal]);
    break;
  case coord:
    corrfact = corr(x,y,degree[cal], corrPolin[cal]);
    break;
  case radial2:
    r = x*x+y*y;
    corrfact = corr(r, degree[cal], corrPolin[cal]);
    break;
  case coord2:
    corrfact = corr(x*x,y*y,degree[cal], corrPolin[cal]);
    break;
  default:
    break;
  }
  //cout<<"LY Correction: "<<corrfact<<" Initial energy: "<<energy<<endl;
  energyVariance *= corrfact*corrfact;
  energy *=corrfact;
  positionData.setEnergy(energy);
  positionData.setSqEnErr(energyVariance);
  return energy;
}

float ESLightYieldCor::corr(float x, float y, 
			    unsigned int n, float * array) const{
  return corr(x,n,array)*corr(y,n,array);
}

float ESLightYieldCor::corr(float r, unsigned int n, float * array) const{
  float tmp = array[0];
  float x1 = r;
  for(unsigned int i=1; i<=n; i++, x1 *=r)
    tmp += x1*array[i];
/*  cout<<"Dumping values: r="<<r<<" n="<<n<<" ";
  for(unsigned int i=0; i<=n; i++)
   cout<<array[i]<< "  ";
 cout<<endl; */
  return tmp;
}

void ESLightYieldCor::doAnalysis(const class EMClusterMember *){}
void ESLightYieldCor::doAnalysis(const class EMHitCell *){}
void ESLightYieldCor::doAnalysis(const class EMCluster *){}

