//                              -*- Mode: C++ -*-
//: ESPIDEnergy3cell.C
//
//
// Author: roberto spighi   
// ECAL group 
// HERA-B experiment, Desy
// e-mail: spighi@bo.infn.it
//
// Created 27/10/97
// Last Update ........
//

#include "care/EMCluster.hh"
#include "care/EMHitCell.hh"
#include "care/ESPIDEnergy3cell.hh"
#include "arte/ArteMessage.hh"
#include <math.h>
/* 
CLASS
  ESPIDEnergy3cell
   Class for PID (pion/electron discrimination) analysis: 
*/

// parameter of likelihood initializzation

float ESPIDEnergy3cell::eparinner[8]= {0.58, 0.529, 0., 0., 0., 0., 0., 0.};

float ESPIDEnergy3cell::pparinner[8]= {0.42, -0.529, 0., 0., 0., 0., 0., 0.}; 

float ESPIDEnergy3cell::eparmiddle[8]={0.42, 0.757, 0., 0., 0., 0., 0., 0.}; 

float ESPIDEnergy3cell::pparmiddle[8]={0.58, -0.757, 0., 0., 0., 0., 0., 0.}; 

float ESPIDEnergy3cell::eparouter[8]={0.58, 0.386, 0., 0., 0., 0., 0., 0.};
 
float ESPIDEnergy3cell::pparouter[8]={0.42, -0.386, 0., 0., 0., 0., 0., 0.};

ESPIDEnergy3cell::ESPIDEnergy3cell() : ESPIDStrategy() {}

ESPIDEnergy3cell::~ESPIDEnergy3cell(){}    

//--
// TrackInTower analysis
//void ESPIDEnergy::doAnalysis(TrackOnEcal *){}
 
//--
//  Start a cluster analysis
void ESPIDEnergy3cell::beginAnalysis(const EMCluster* clus, 
			   ArtePointer<RCCL> prccl,const EMHitCell* phitc){

  ESPIDStrategy::beginAnalysis(clus, prccl, phitc);


  energyTot = 0.;
  energyfraz = 0.;
  contcell = 0;


  frac_cell = 3;
  for(int i=0;i<frac_cell;i++) enmax[i]=0.;

  doAnalysis(clus);
  for(int i=0;i<frac_cell;i++) energyfraz +=enmax[i];
  if(energyTot>0.) {
    RPID = energyfraz/energyTot;
  }
  else{
    RPID = -1.;
  }
  // Evaluation likelihoods for electron and hadron

  likeElectron();
  likeHadron();
  updatebank(rccl);

}

//--
// HitCell Analyzer
void ESPIDEnergy3cell::doAnalysis(const EMHitCell* tower){

  // for a tower add it's energy
  Energy_t  ener = tower->energy();
  float enmin;
  float temp;
  int cellmin=0;
  contcell++;
  energyTot += ener;

  //  arteMsg<<" Energy  "<<ener;
  //  arteMsg.allp();

  if(contcell<=frac_cell) {
    enmax[contcell-1] =ener;
  }
  else{
    // find the minimum of enmax[]
    enmin=enmax[0];
    for(int i=0;i<frac_cell;i++){
      temp = enmax[i];
      if(temp<enmin){
        enmin = temp;
        cellmin = i;
      }
    }
    // substitute the minimun
    if(ener>enmin){
      enmax[cellmin] = ener;
    }
  }

}

//--
// Cluster Analyzer
void ESPIDEnergy3cell::doAnalysis(const EMCluster * cluster){

  // loop on clusters
  
  for(EMClusterMember* child = cluster->begin(); 
      child!=0;  child = child->next() ){
    child->analyzeMe(*this);
  }

}
void ESPIDEnergy3cell::likeElectron(){
  //  function to calculate the likelihood for electron hypotesis
  float x;
  float par[8];

  if(typecal==5) 
    for(int i=0; i<8; i++)
      par[i] = eparinner[i];

  if(typecal==2)
    for(int i=0; i<8; i++)
       par[i] = eparmiddle[i];
  if(typecal==1) 
    for(int i=0; i<8; i++)
       par[i] = eparouter[i];

  x=RPID;
  
  if( x < 0.3 ) likeEle = par[0];
  if( x <= 0.99){
    likeEle = par[0] + par[1]*(x-0.3);     
  } 
  if( x > 0.99) likeEle = 0.5;
}

void ESPIDEnergy3cell::likeHadron(){
  //  function to calculate the likelihood for electron hypotesis
  float x;
  float par[8];

  if(typecal==5) 
    for(int i=0; i<8; i++)
      par[i] = pparinner[i];

  if(typecal==2)
    for(int i=0; i<8; i++)
      par[i] = pparmiddle[i];

  if(typecal==1) 
    for(int i=0; i<8; i++)
      par[i] = pparouter[i];

  x=RPID;

  if( x < 0.3 ) likeHad = par[0];
  if( x <= 0.99){
    likeHad = par[0] + par[1]*(x-0.3);     
  } 

  if( x > 0.99) likeHad = 0.5;

}


void ESPIDEnergy3cell::setparlikeEnergy3cell(int cal,int particle,float 
					likeEnergy[]){
  //  set the parameter for likelihood curves of electron and pion 
  if(cal==1 && particle==1) 
    for(int i=0; i<8; i++ ) 
      eparinner[i] = likeEnergy[i];

  if(cal==2 && particle==1) 
    for(int i=0; i<8; i++ ) 
      eparmiddle[i] = likeEnergy[i];

  if(cal==3 && particle==1) 
    for(int i=0; i<8; i++ ) 
      eparouter[i] = likeEnergy[i];


  if(cal==1 && particle==2) 
    for(int i=0; i<8; i++ ) 
      pparinner[i] = likeEnergy[i];

  if(cal==2 && particle==2) 
    for(int i=0; i<9; i++ ) 
      pparmiddle[i] = likeEnergy[i];

  if(cal==3 && particle==2) 
    for(int i=0; i<8; i++ ) 
      pparouter[i] = likeEnergy[i];

}










































