//                              -*- Mode: C++ -*-
//:ecdbnoise.hh
//
//
// Author: Mauro Villa
// ECAL group
// HERA-B experiment, Desy
// e-mail: villa@bo.infn.it
//

#ifndef ecdbnoise_hh
#define ecdbnoise_hh

extern "C" {
#include "database/table.h"
};

#include <vector>
#include <stdio.h>
#include <string>
#include <fstream>
#include <ctime>

struct ECFedCor {
  int fed;
  float corr;
  ECFedCor(int fd=0, float r=0.0) : fed(fd), corr(r) {};
  ECFedCor(const ECFedCor & afedcor) : fed(afedcor.fed), corr(afedcor.corr) {};
  const ECFedCor& operator=(const ECFedCor & afedcor){
    fed  = afedcor.fed;
    corr = afedcor.corr;
    return *this;
  }; 
};


class ECDbNoise {
private:
  //
  // The main vector of noise data
  //
  vector<float> chancorr;
  vector<float> boardsigma;
  vector< vector<ECFedCor> > fedcorrs;

  // DB specific entries
  int major;
  int minor;
  int tab;
  Tables x;
  // db strings
  static char* comment;
  static char* fieldDesc;
  static const int fieldType[];
  static const int fieldDim[];

public:
  enum {ECALCHAN=5956,ECALBOARDS=226, FEDMIN=16}; // number of ECAL RO boards

  ECDbNoise() : chancorr(), boardsigma(), fedcorrs(), 
    major(0), minor(0), tab(0), x(0){init();};
  ~ECDbNoise(){}
  void init();

  // Database access functions
  // Return codes: rc=0 OK! rc>0 error
  int openDB(const char * dbname, const char *tablename, int access=0);
  int writeToDB(int major, int minor, int access);
  int readFromDB(int major, int minor, int entries);

  // Return codes: rc=0 OK! rc>0 error
  int closeDB();

  // access functions
  Tables db() const { return x;}
  int    table() const {return tab;}
  int    majorTag() const { return major;}
  int    minorTag() const { return minor;}

  //
  // Access/set
  //
  inline int store(int fed, float bsigma, 
		   const vector<ECFedCor>  & afedcorr){
    int rc=0;
    if( size()!=ECALBOARDS ){
      init();
    }
    if( fed>=FEDMIN && fed<ECALBOARDS+FEDMIN){ 
      boardsigma[fed-FEDMIN] = bsigma;
      fedcorrs[fed-FEDMIN] = afedcorr; rc =1;
    }
    return rc;
  }

  inline int store(int gcce, float achancorr){
    int rc=0;
    if( size()!=ECALBOARDS ){
      init();
    }
    if( gcce>0 && gcce<=ECALCHAN){
      chancorr[gcce-1] = achancorr; rc =1;
    }
    return rc;
  }

  inline float chanCorr(int gcce) const{
    return chancorr[gcce-1];
  }
 
  inline float boardSigma(int fed) const {
    return boardsigma[fed-FEDMIN];
  }

  inline const vector<ECFedCor>  & fedCorrs(int fed) const {
    return fedcorrs[fed-FEDMIN];
  }

  //
  // Size of the array ( should be 226 or 0)
  //
  inline unsigned int size() const { return boardsigma.size();}

};

// *********************************************


#endif
