//  Authors:  Dmitri Emeliyanov                Created:  01-Jun-00 ver. 0.01
//            Ivan Kisel                       Modified: 01-Nov-00 ver. 1.01 
//  Email:    emelian@mail.desy.de                       03-Feb-01 ver. 1.03
//            ikisel@mppmu.mpg.de                        15-Feb-01 ver. 1.04
//            HERA-B Collaboration                       11-Mar-01 ver. 1.05
//                                                       19-Mar-01 ver. 1.06
//                                                       16-Apr-01 ver. 1.07
//                                                       26-Apr-01 ver. 1.08
//                                                       20-May-01 ver. 1.09
//                                                       20-May-01 ver. 2.01
//                                                       30-May-01 ver. 2.02
//                                                       12-Dec-01 ver. 3.00
//                                                       31-Jan-02 ver. 3.01

using namespace std;

#include "cats/OtrCats.hh"
#include "cats/higz.h"
#include "cats/hbook.h"
#include <math.h>
#include <map>
#include "arte/RSEG_cmp.hh"

#include "arte/Chrono.hh"
Chrono zeit_otr_total    ("OtrCats TOTAL ");
Chrono zeit_otr_start    ("Otr START     ");
Chrono zeit_otr_cluster  ("Otr CLUSTER   ");
Chrono zeit_otr_sector   ("Otr SECTOR    ");
Chrono zeit_otr_sp       ("Otr SPOINTS   ");
Chrono zeit_otr_neighbors("Otr NEIGHBORS ");
Chrono zeit_otr_collect  ("Otr COLLECT   ");
Chrono zeit_otr_gather   ("Otr GATHER    ");
Chrono zeit_otr_refit    ("Otr REFIT     ");
Chrono zeit_otr_rseg     ("Otr RSEG      ");

Chrono zeit_ranger("Ranger");

extern "C"{
  float prob_(const float & chi2, const int & ndeg);
}


int ItrOtrGraphInitialized = 0;

static int OtrCatsRun = 0;
static int PC01shift=0;
static const double B    =0.65945;
static const double zmagn=450.0;

//=======================
// steering parameters

int OTR_PERFORMANCE = 0;
int OTR_MC          = 0;
int OTR_DRAW        = 0;
int OTR_HISTO       = 0;

//========================

static map<int, ArtePointer<GESL> >            geslmap;
static map<int, ArtePointer<GEDE> >            gedemap;

typedef struct{
  float sin, cos, alpha, sxmn, sxmx, symn, symx, szmn, szmx;
} GEDE_SIZE, *pGEDE_SIZE;

static map<int, GEDE_SIZE >                    gedesizemap;


typedef struct{
  int index;
  float alpha;
  float sxmn, sxmx, symn, symx, szmn, szmx;
  vector<ArtePointer<GEDE> > vgede;
} SECTOR, *pSECTOR;

static vector<map<int, SECTOR> > vsectormap;
static map<ArtePointer<GESL>, int >  geslsectormap;

static map<ArtePointer<GESL>, int >        geslsupergedemap;   // pointers to the following vector
static vector<vector<int> >                vgeslsupergede;     // list of supergede acc. to gesl
static vector<vector<ArtePointer<GEDE> > > vsupergede;         // list of gede to access clusters
static vector<float>                       vsupergede_z;       // mean z positions

vector<TRACK> vtrack;
map<ArtePointer<GEDE>, int >  gedeclustermap; // contains vector number 
vector<vector<CLUSTER> >     vgedecluster;
map<ArtePointer<GESL>, int >  geslspmap;
vector<vector<SPACEPOINT> >  vgeslsp;

vector<vector<pSPACEPOINT> > vpneighbor;
vector<vector<float> >       vchi2neighbor;

typedef struct{
  pSPACEPOINT psp;
  int propagated;           // real sp or propagated parameters?
  float xm, ym, dxm, dym;
} PROPSPOINT, *pPROPSPOINT;


vector<HIT > vhit;              // all hitbs are stored here
map<int, int> hit1map, hitlmap; // "pointers" to the first and the last hits of a gede

bool OtrGedeCompareZ(const ArtePointer<GEDE> gede1, const ArtePointer<GEDE> gede2){
  //  return (gede1->zfw[0] < gede2->zfw[0]);
  return (gede1->zmin < gede2->zmin);
}

bool OtrTrackCompareLength(const TRACK &a, const TRACK &b){

  int a_nsp = (a.nclu/7)*7;
  int b_nsp = (b.nclu/7)*7;
  if (a_nsp != b_nsp)
    return (a.nclu > b.nclu);
  else
    return (a.pf < b.pf);

    /*
  if ((a.nsp == 1)&&(b.nsp == 1)){

    if (a.nclu != b.nclu)
      return (a.nclu > b.nclu);

    if (int(10.0*a.prob) != int(10.0*b.prob))
      return (a.prob > b.prob);

    return (fabs(a.tyf) < fabs(b.tyf));
  }  

  if (a.nclu != b.nclu)
    return (a.nclu > b.nclu);
  else
    return (a.prob > b.prob);
    */
}

bool OtrTrackCompareChi2(const TRACK &a, const TRACK &b){
  return (a.chi2 < b.chi2);
}

bool OtrTrackCompareProb(const TRACK &a, const TRACK &b){
  return (a.prob > b.prob);
}

bool OtrPropSPointCompareX(const PROPSPOINT &a, const PROPSPOINT &b){
  float ax, bx;
  if (!a.propagated) ax = a.psp->xm-3.0*sqrt(a.psp->cvm[0][0]);
  else               ax = a.xm-a.dxm;
  if (!b.propagated) bx = b.psp->xm-3.0*sqrt(b.psp->cvm[0][0]);
  else               bx = b.xm-b.dxm;
  return (ax < bx);
}


bool OtrTrueClusterCompareX(const CLUSTER a, const CLUSTER b){
  return (a.x < b.x);
}

bool OtrClusterCompareX(const pCLUSTER a, const pCLUSTER b){
  return (a->x < b->x);
}

bool OtrClusterCompareZ(const pCLUSTER a, const pCLUSTER b){
  return (a->z < b->z);
}

bool OtrHitCompareZ(const pHIT a, const pHIT b){
  return (a->z < b->z);
}

bool OtrHitCompareX(const HIT a, const HIT b){
  return (a.x < b.x);
}

bool OtrHitbCompareZ(const ArtePointer<HITB> a, const ArtePointer<HITB> b){
  return (a->z < b->z);
}

bool OtrRhitCompareZ(const ArtePointer<RHIT> a, const ArtePointer<RHIT> b){
  return ((a->hitb)->z < (b->hitb)->z);
}

extern void ItrCatsEvent(void);
 
void  vier_dim_inv(double a[4][4]);
void  OtrResolveLR(pTRACK);
void  OtrFitSPointNew(SPACEPOINT &sp);
void  OtrFastDriftRefitTrack(pTRACK);
void  OtrDriftRefitTrack(pTRACK);
void  OtrRefitTrack(pTRACK);
float OtrNeighborsChi2(const pSPACEPOINT, const pSPACEPOINT);
void  OtrFindTrack(const pSPACEPOINT , TRACK&, TRACK);
void  OtrGraphIni(void);
void  OtrGraphClose(void);
void  OtrRotatePoint(float&, float&, float&, const ArtePointer<GEDE>);
void  OtrGedeMinMax (float&, float&, float&, float&, float&, float&, const ArtePointer<GEDE>);
void  OtrPerformance(vector<TRACK>);
//float Chi2DistanceRSEG(const ArtePointer<RSEG> rseg1, const ArtePointer<RSEG> rseg2);
//void  OtrDrawMTRA(void);


static int Black =  1, 
    DarkGray     =  2, 
    Blue         =  3, 
    Green        =  4, 
    Cyan         =  5, 
    Red          =  6, 
    Magenta      =  7, 
    Brown        =  8, 
    LightBlue    =  9, 
    LightGreen   = 10, 
    LightCyan    = 11, 
    LightRed     = 12, 
    LightMagenta = 13, 
    Yellow       = 14, 
    LightGray    = 15, 
    White        =  0;

static float xpl[100], ypl[100], zpl[100], xpm[100], zpm[100];// ypm[100]
static float x_shift = -250.0;  //OTR 
static float y_shift =  250.0;  //OTR


static int first_call = 1;

map< ArtePointer<GESL>, int >           geslsignmap;
map< ArtePointer<GESL>, int >::iterator geslsignmapIt;


static float z_geslf = 10000.0, z_gesle = 0.0;

void OtrCatsEvent(){
  ItrCatsEvent();
  zeit_otr_total.start();

  int OtrCatsFillGeo = 0;
  ArtePointer<EVHD> evhd = *(ArteTable<EVHD>::begin());

  if (OtrCatsRun == 0){
    OtrCatsRun = evhd->run;
    OtrCatsFillGeo = 1;
  }

  if (evhd->run != OtrCatsRun){
    OtrCatsRun = evhd->run;
    OtrCatsFillGeo = 1;
  }

  if (first_call){
    OtrCatsFillGeo = 1;
  }

  if (first_call){
    first_call = 0;

    cout<<"OtrCats  version 3.00 started"<<endl;

    if (OTR_DRAW)
      OtrGraphIni();

    if (ArteTable<MCIN>::booked())
      OTR_MC = 1;
    else
      OTR_MC = 0;
  }
  
  zeit_otr_start.start();

  if (OTR_DRAW){
    isfaci(White); isplci(Red);
    igbox(700.00, 860.00,-250.00+x_shift, 250.00+x_shift);  //OTR
    igbox(700.00, 860.00,-250.00+y_shift, 250.00+y_shift);  //OTR

    /* draw layers and mimps */
    
    isplci(Black); ispmci(Red);
    ismk(3); ismksc(0.5);
  }

  // fill gede map for the clusterizing
  
  if (OtrCatsFillGeo){

    //cout<<"---------------------->"<<endl;
    //cout<<"---------------------->  *** OtrCats new geo filled for run "<<OtrCatsRun<<" ***"<<endl;
    //cout<<"---------------------->"<<endl;

    geslmap.clear();
    gedemap.clear();
    gedesizemap.clear();
    geslsupergedemap.clear();

    geslsectormap.clear();
    vsectormap.clear();
    int vsector_index = 0;
    map<int, SECTOR> sectormap;
    vector<ArtePointer<GEDE> > vgede;
    int vsupergede_index = 0;
    int vgeslsupergede_index = 0;
    vgeslsupergede.clear();
    vsupergede.clear();
    vsupergede_z.clear();


    char msfi[5],strtmp[5];
    PC01shift=0;
    strcpy(msfi,"PC01");
    
    for ( ArteTable<GESL>::iterator geslIt = ArteTable<GESL>::begin();
	  geslIt != ArteTable<GESL>::end(); geslIt++ ){
      if ((*geslIt)->cmp!=2) continue; // not OTR
      if ((*geslIt)->nsl>8) continue;
      strtmp[0]=(*geslIt)->name[0];
      strtmp[1]=(*geslIt)->name[1];
      strtmp[2]=(*geslIt)->name[2];
      strtmp[3]=(*geslIt)->name[3];
      strtmp[4]=msfi[4];   
      if(!strcmp(strtmp,msfi)){
	if((*geslIt)->nsl==8) PC01shift=0;
	if((*geslIt)->nsl<8)  PC01shift=6;
      }
    }

    //    cout<<PC01shift<<" shift "<<endl;
    for ( ArteTable<GESL>::iterator geslIt = ArteTable<GESL>::begin();
	  geslIt != ArteTable<GESL>::end(); geslIt++ ) {//gesl
      
      if ((*geslIt)->cmp!=2) continue; // not OTR
      if (((*geslIt)->nsl<(8-PC01shift))||((*geslIt)->nsl>(11-PC01shift))) continue; // not PC

      sectormap.clear();                 
      
      geslmap.insert(pair<int, ArtePointer<GESL> >((*geslIt).index(), *geslIt));

      vgede.clear();

      for ( ArteRelationIterator<GEDE> gedeIt = (*geslIt)->gede.begin();
	    gedeIt != (*geslIt)->gede.end(); gedeIt++ ) {//gede

	if ((*gedeIt)->zmin < z_geslf) z_geslf = (*gedeIt)->zmin;
	if ((*gedeIt)->zmax > z_gesle) z_gesle = (*gedeIt)->zmax;
	
	vgede.push_back(*gedeIt);
	gedemap.insert(pair<int, ArtePointer<GEDE> >((*gedeIt).index(), *gedeIt));

	float cos = (*gedeIt)->cos*(*gedeIt)->rot[0] - (*gedeIt)->sin*(*gedeIt)->rot[1];
	float sin = (*gedeIt)->sin*(*gedeIt)->rot[4] - (*gedeIt)->cos*(*gedeIt)->rot[3];
      
	float alpha = 180.0/3.1415926*atan2(sin, cos); if (alpha<-10.0) alpha += 360.0;
      
	float sxmn, sxmx, symn, symx, szmn, szmx;
	OtrGedeMinMax(sxmn, sxmx, symn, symx, szmn, szmx, (*gedeIt));
      
	GEDE_SIZE gede_size;
	gede_size.alpha = alpha;
	gede_size.sin   = sin;   gede_size.cos   = cos;
	gede_size.sxmn  = sxmn;  gede_size.sxmx  = sxmx;
	gede_size.symn  = symn;  gede_size.symx  = symx;
	gede_size.szmn  = szmn;  gede_size.szmx  = szmx;

	gedesizemap.insert(pair<int, GEDE_SIZE >((*gedeIt).index(), gede_size));

	int sec = (*gedeIt)->sec;
	if(sectormap.find(sec) == sectormap.end()){
	  SECTOR sector;      sector.index = sec;
	  sector.alpha = alpha;
	  sector.sxmn = sxmn; sector.sxmx  = sxmx;
	  sector.symn = symn; sector.symx  = symx;
	  sector.szmn = szmn; sector.szmx  = szmx;
	  sector.vgede.push_back(*gedeIt);
	  sectormap.insert(pair<int, SECTOR>(sec, sector));
	}
	else{ // make wider ?
	  if (sxmn < sectormap[sec].sxmn) sectormap[sec].sxmn = sxmn;
	  if (sxmx > sectormap[sec].sxmx) sectormap[sec].sxmx = sxmx;
	  if (symn < sectormap[sec].symn) sectormap[sec].symn = symn;
	  if (symx > sectormap[sec].symx) sectormap[sec].symx = symx;
	  if (szmn < sectormap[sec].szmn) sectormap[sec].szmn = szmn;
	  if (szmx > sectormap[sec].szmx) sectormap[sec].szmx = szmx;
	  sectormap[sec].vgede.push_back(*gedeIt);
	}
      } //gede

      sort(vgede.begin(), vgede.end(), OtrGedeCompareZ);

      vector<int> vgeslsgede; vgeslsgede.clear();

      ArtePointer<GEDE> gede = *(vgede.begin());
      GEDE_SIZE gede_size = gedesizemap[gede.index()];
      float alpha = gede_size.alpha;

      vector<ArtePointer<GEDE> > vsgede; vsgede.clear();
      float z_sgede_min = 100000.0, z_sgede_max = -100000.0;
      for ( vector<ArtePointer<GEDE> >::iterator gedeIt = vgede.begin();
	    gedeIt != vgede.end(); gedeIt++){

	GEDE_SIZE gede_size = gedesizemap[(*gedeIt).index()];
	float alph = gede_size.alpha;

	if (alph == alpha){
	  vsgede.push_back(*gedeIt);
	  if ((*gedeIt)->zmin < z_sgede_min) z_sgede_min = (*gedeIt)->zmin;
	  if ((*gedeIt)->zmax > z_sgede_max) z_sgede_max = (*gedeIt)->zmax;
	}
	else{
	  vsupergede.push_back(vsgede);
	  vgeslsgede.push_back(vsupergede_index++);
	  vsupergede_z.push_back(0.5*(z_sgede_min+z_sgede_max));
	  vsgede.clear();
	  vsgede.push_back(*gedeIt);
	  alpha = alph;
	  z_sgede_min = (*gedeIt)->zmin;
	  z_sgede_max = (*gedeIt)->zmax;
	}
      }
      if (distance(vsgede.begin(), vsgede.end()) != 0){
	vsupergede.push_back(vsgede);
	vgeslsgede.push_back(vsupergede_index++);
	vsupergede_z.push_back(0.5*(z_sgede_min+z_sgede_max));
	vsgede.clear();
      }
      vgeslsupergede.push_back(vgeslsgede);
      geslsupergedemap.insert(pair<ArtePointer<GESL>, int >
			   (*geslIt, vgeslsupergede_index++));



      vsectormap.push_back(sectormap);
      geslsectormap.insert(pair<ArtePointer<GESL>, int >
			   (*geslIt, vsector_index++));
    }

  } //first call

  //make clusterization
  
  zeit_otr_cluster.start();

  vhit.clear();
  hit1map.clear();  hitlmap.clear();

  for (map<int, ArtePointer<GEDE> >::iterator gedemIt = gedemap.begin();
       gedemIt != gedemap.end(); ++gedemIt){

    ArtePointer<GEDE> gede = gedemIt->second;

    int hit1 = distance(vhit.begin(), vhit.end()); // number of hits stored already
    int hitl = -1;                                 // for a case of empty gede
    
    ArteTableIterator<HITB> hitbItf = gede->hit1;
    ArteTableIterator<HITB> hitbIte = gede->hitl;
    if (distance(hitbItf, hitbIte)+1 == 0){ // no hitbs? => strange storage for STL
      hit1map.insert(pair<int, int >(gede.index(), hit1));
      hitlmap.insert(pair<int, int >(gede.index(), hitl));
      continue;
    }
    hitbIte++;
    
    int gede_index = gede.index();
    hitl = 0;
    for (ArteTableIterator<HITB> hitbIt = hitbItf; 
	 hitbIt != hitbIte; ++hitbIt ) {
      if ((*hitbIt)->z < 0.0){
	(*hitbIt)->z = -(*hitbIt)->z;
	continue;
      }
      HIT hit;
      hit.gede_index = gede_index;
      hit.hitb = *hitbIt;
      hit.x    = (*hitbIt)->x;
      hit.q    = fabs((*hitbIt)->q);
      hit.z    = (*hitbIt)->z;
      hit.r    = (*hitbIt)->r;
      hit.mask = 1.0;
      hit.lr   = 0.0;
      
      vhit.push_back(hit);
      hitl++;
    }
    hitl += hit1-1;

    hit1map.insert(pair<int, int >(gede.index(), hit1));
    hitlmap.insert(pair<int, int >(gede.index(), hitl));

    sort(vhit.begin()+hit1, vhit.end(), OtrHitCompareX);
  }

  vector<CLUSTER> vcluster;
  vector<pCLUSTER> vzpcluster, vspcluster;
  //vzpcluster.reserve(256); 
  //vspcluster.reserve(256);
  gedeclustermap.clear();
  vgedecluster.clear();
  int vgedecluster_index = 0;

  for (map<int, ArtePointer<GEDE> >::iterator gedemIt = gedemap.begin();
       gedemIt != gedemap.end(); ++gedemIt){

    ArtePointer<GEDE> gede = gedemIt->second;

    vcluster.clear();
    OtrClusters(vcluster, gede);
    sort(vcluster.begin(), vcluster.end(), OtrTrueClusterCompareX);
    vgedecluster.push_back(vcluster);
    gedeclustermap.insert(pair<ArtePointer<GEDE>, int >
			  (gede, vgedecluster_index++));
  } // gedemIt
  zeit_otr_cluster.stop();

  //clusters are stored according the gede planes

  // collect space points for each sector

  zeit_otr_sector.start();

  geslspmap.clear();
  vgeslsp.clear();
  int vgeslsp_index = 0;

  for (map<int, ArtePointer<GESL> >::iterator geslmIt = geslmap.begin();
       geslmIt != geslmap.end(); ++geslmIt){//geslit

    ArtePointer<GESL> gesl = geslmIt->second;

    vector<SPACEPOINT> vspoint;

    int gesl_index = geslsectormap[gesl];
    for (map <int, SECTOR>::iterator secIt = vsectormap[gesl_index].begin();
	 secIt != vsectormap[gesl_index].end(); ++secIt){//secit

      if (OTR_DRAW){
	for (vector< ArtePointer<GEDE> >::iterator gedeIt = (secIt->second).vgede.begin();
	     gedeIt != (secIt->second).vgede.end(); ++gedeIt){
	  
	  GEDE_SIZE gede_size = gedesizemap[(*gedeIt).index()];

	  if (gede_size.alpha < -2.0)                             isplci(Magenta);
	  if ((gede_size.alpha>=-2.0)&&(gede_size.alpha <= 2.0))  isplci(Black); 
	  if (gede_size.alpha >  2.0)                             isplci(Brown);
	  
	  zpl[0] = 0.5*((*gedeIt)->zmin + (*gedeIt)->zmax); 
	  xpl[0] = (secIt->second).sxmn + x_shift; 
	  ypl[0] = (secIt->second).symn + y_shift;
	  zpl[1] = 0.5*((*gedeIt)->zmin + (*gedeIt)->zmax); 
	  xpl[1] = (secIt->second).sxmx + x_shift; 
	  ypl[1] = (secIt->second).symx + y_shift;
	  ipl(2,zpl,xpl); ipl(2,zpl,ypl);
	}
      }

      
      vzpcluster.clear(); vspcluster.clear();
      for (vector< ArtePointer<GEDE> >::iterator gedeIt = (secIt->second).vgede.begin();
	   gedeIt != (secIt->second).vgede.end(); ++gedeIt){

	GEDE_SIZE gede_size = gedesizemap[(*gedeIt).index()];

	int gede_index = gedeclustermap[*gedeIt];
	if ((gede_size.alpha > -2.0)&&(gede_size.alpha < 2.0)){ // add to the zero projection
	  for (vector<CLUSTER>::iterator cluIt = vgedecluster[gede_index].begin();
	       cluIt != vgedecluster[gede_index].end(); ++cluIt){
	    vzpcluster.push_back(cluIt);
	  }
	}
	else{                               // add to the stereo projection
	  for (vector<CLUSTER>::iterator cluIt = vgedecluster[gede_index].begin();
	       cluIt != vgedecluster[gede_index].end(); ++cluIt){
	    vspcluster.push_back(cluIt);
	  }
	}
      } // gedeIt


      // sort clusters
      sort(vzpcluster.begin(), vzpcluster.end(), OtrClusterCompareX);
      sort(vspcluster.begin(), vspcluster.end(), OtrClusterCompareX);

      if (OTR_DRAW){
	switch (secIt->first){
	case 1:  {ispmci(Black     ); break;}
	case 2:  {ispmci(LightGray ); break;}
	case 3:  {ispmci(Blue      ); break;}
	case 4:  {ispmci(Red       ); break;}
	case 5:  {ispmci(Green     ); break;}
	case 6:  {ispmci(Magenta   ); break;}
	case 7:  {ispmci(Cyan      ); break;}
	case 8:  {ispmci(Brown     ); break;}
	case 9:  {ispmci(LightBlue ); break;}
	case 10: {ispmci(LightGreen); break;}
	default:  ispmci(DarkGray  );
	}
	
	for (vector<pCLUSTER>::iterator pcluIt = vzpcluster.begin();
	     pcluIt != vzpcluster.end(); ++pcluIt){
	  zpm[0] = (*pcluIt)->z; xpm[0] = (*pcluIt)->x+x_shift; 
	  ipm(1, zpm, xpm);
	}
	
	for (vector<pCLUSTER>::iterator pcluIt = vspcluster.begin();
	     pcluIt != vspcluster.end(); ++pcluIt){
	  zpm[0] = (*pcluIt)->z; xpm[0] = (*pcluIt)->x+x_shift; 
	  ipm(1, zpm, xpm);
	}
      }

      // collect space points
      zeit_otr_sp.start();
      OtrSPoints(vzpcluster, vspcluster, vspoint, 
		 (secIt->second).symn, (secIt->second).symx);
#ifdef XXX
      for (vector<pCLUSTER>::iterator pcluIt = vzpcluster.begin();
	   pcluIt != vzpcluster.end(); ++pcluIt){
	(*pcluIt)->used = 0;
      }
#endif
      //#ifdef XXX
      for (vector<pCLUSTER>::iterator pcluIt = vspcluster.begin();
	   pcluIt != vspcluster.end(); ++pcluIt){
	(*pcluIt)->used = 0;
      }
      //#endif
      zeit_otr_sp.stop();
    } // secIt

    vgeslsp.push_back(vspoint);
    geslspmap.insert(pair<ArtePointer<GESL>, int >(gesl, vgeslsp_index++));
  } // geslIt
  zeit_otr_sector.stop();

  zeit_otr_start.stop();

  zeit_otr_neighbors.start();

  vpneighbor.clear();
  vchi2neighbor.clear();
  int neighborindex = 0;


  for (map<int, ArtePointer<GESL> >::iterator rgeslIt = geslmap.begin();
	rgeslIt != geslmap.end(); rgeslIt++ ) {

    if (OTR_DRAW){//draw space points

      for(vector<SPACEPOINT>::iterator spIt  = vgeslsp[geslspmap[rgeslIt->first]].begin();
	  spIt != vgeslsp[geslspmap[rgeslIt->first]].end(); ++spIt ){
	
	zpl[0] = spIt->zf;
	xpl[0] = spIt->xf + x_shift; 
	ypl[0] = spIt->yf + y_shift; 
	zpl[1] = spIt->ze;
	xpl[1] = spIt->xe + x_shift; 
	ypl[1] = spIt->ye + y_shift; 

	float momentum = 1.0/spIt->pf;
	if (momentum>1.0) isplci(Red);
	else              isplci(Blue);
	//ipl(2,zpl,xpl); ipl(2,zpl,ypl);
      }
    }


    if (rgeslIt == geslmap.begin()) continue; // start from the second gesl

    map<int, ArtePointer<GESL> >::iterator lgeslIt = rgeslIt;
    lgeslIt--;

    ArtePointer<GESL> lgesl = lgeslIt->second;
    float lgesl_zm = (lgesl->z1+lgesl->z2)/2.0;

    vector<PROPSPOINT> vpropspoint;

    for(vector<SPACEPOINT>::iterator rspIt  = vgeslsp[geslspmap[rgeslIt->first]].begin();
	rspIt != vgeslsp[geslspmap[rgeslIt->first]].end(); ++rspIt ){
      
      PROPSPOINT propspoint;
      propspoint.psp = rspIt;
      propspoint.propagated = 1; // propagate to the left
      propspoint.xm  = rspIt->xm + rspIt->txm*(lgesl_zm - rspIt->zm);
      propspoint.ym  = rspIt->ym + rspIt->tym*(lgesl_zm - rspIt->zm);
      propspoint.dxm = 3.0*sqrt(rspIt->cvm[0][0]+(lgesl_zm-rspIt->zm)*(lgesl_zm-rspIt->zm)*rspIt->cvm[2][2]);
      propspoint.dym = 3.0*sqrt(rspIt->cvm[1][1]+(lgesl_zm-rspIt->zm)*(lgesl_zm-rspIt->zm)*rspIt->cvm[3][3]);

      vpropspoint.push_back(propspoint);
    }

    ArtePointer<GESL> rgesl = rgeslIt->second;
    float rgesl_zm = (rgesl->z1+rgesl->z2)/2.0;

    for(vector<SPACEPOINT>::iterator lspIt  = vgeslsp[geslspmap[lgeslIt->first]].begin();
	lspIt != vgeslsp[geslspmap[lgeslIt->first]].end(); ++lspIt ){
      
      PROPSPOINT propspoint;
      propspoint.psp = lspIt;
      propspoint.propagated = 0; // real sp, but propagate to the right gesl
      propspoint.xm  = lspIt->xm + lspIt->txm*(rgesl_zm - lspIt->zm);
      propspoint.ym  = lspIt->ym + lspIt->tym*(rgesl_zm - lspIt->zm);
      propspoint.dxm = 3.0*sqrt(lspIt->cvm[0][0]+(rgesl_zm-lspIt->zm)*(rgesl_zm-lspIt->zm)*lspIt->cvm[2][2]);
      propspoint.dym = 3.0*sqrt(lspIt->cvm[1][1]+(rgesl_zm-lspIt->zm)*(rgesl_zm-lspIt->zm)*lspIt->cvm[3][3]);

      vpropspoint.push_back(propspoint);
    }
    
    sort(vpropspoint.begin(), vpropspoint.end(), OtrPropSPointCompareX);
    
    for(vector<PROPSPOINT>::iterator propspIt  = vpropspoint.begin();
	propspIt != vpropspoint.end(); ++propspIt ){
      
      // find propagate sp
      if (!propspIt->propagated) continue;
      
      float x_top = propspIt->xm+propspIt->dxm;
      
      vector<pSPACEPOINT> vpnei;
      vector<float>       vchi2nei;
      int neifound = 0;

      vector<PROPSPOINT>::iterator vpropspoint_begin = propspIt;
      vpropspoint_begin++;
      
      for(vector<PROPSPOINT>::iterator neiIt  = vpropspoint_begin;
	  neiIt != vpropspoint.end(); ++neiIt ){
	
	// top region reached?
	/*
	float x;
	if (neiIt->propagated) x = neiIt->xm-neiIt->dxm;
	else                   x = neiIt->psp->xm;
	if (x>x_top) break;
	*/

	if (!neiIt->propagated) 
	  if ((neiIt->psp->xm+3.0*sqrt(neiIt->psp->cvm[0][0])) > x_top) break;;

	// skip if propagated sp
	if (neiIt->propagated) continue;

	//float const_dx = 1.0, const_dy = 13.0;
	float const_dx = 0.0, const_dy = 0.0;

	if (fabs(propspIt->xm - neiIt->psp->xm) > 
	    (propspIt->dxm+3.0*sqrt(neiIt->psp->cvm[0][0])+const_dx)) continue; 
	    // out at the left gesl
	if (fabs(propspIt->psp->xm - neiIt->xm) > 
	    (neiIt->dxm+3.0*sqrt(propspIt->psp->cvm[0][0])+const_dx)) continue;    
	    // out at the right gesl
	if (fabs(propspIt->ym - neiIt->psp->ym) > 
	    (propspIt->dym+3.0*sqrt(neiIt->psp->cvm[1][1])+const_dy)) continue; 
	    // out at the left gesl
	if (fabs(propspIt->psp->ym - neiIt->ym) > 
	    (neiIt->dym+3.0*sqrt(propspIt->psp->cvm[1][1])+const_dy)) continue;    
	    // out at the right gesl

	//ok, probably both are neighbors -> calculate chi2


	SPACEPOINT sp;

	sp.xf  = neiIt->psp->xf;      sp.txf = neiIt->psp->txf;
	sp.yf  = neiIt->psp->yf;      sp.tyf = neiIt->psp->tyf; 
	sp.zf  = neiIt->psp->zf;
	sp.xe  = propspIt->psp->xe;   sp.txe = propspIt->psp->txe;
	sp.ye  = propspIt->psp->ye;   sp.tye = propspIt->psp->tye;
	sp.ze  = propspIt->psp->ze;
	sp.used = 0; sp.level = 0; sp.neighborindex = -1;

	sp.vpclu.clear();
	for (vector<pCLUSTER>::iterator cluIt = neiIt->psp->vpclu.begin();
	     cluIt != neiIt->psp->vpclu.end(); ++cluIt){
	  sp.vpclu.push_back(*cluIt);
	}
	for (vector<pCLUSTER>::iterator cluIt = propspIt->psp->vpclu.begin();
	     cluIt != propspIt->psp->vpclu.end(); ++cluIt){
	  sp.vpclu.push_back(*cluIt);
	}
	//sort(sp.vpclu.begin(), sp.vpclu.end(), OtrClusterCompareZ);

	OtrFitSPoint(sp);

	if (sp.prob > 0.05){
	  if (neiIt->psp->level == 0)                     neiIt->psp->level++;
	  if (neiIt->psp->level+1 > propspIt->psp->level) propspIt->psp->level = neiIt->psp->level+1;
	  
	  neifound = 1;
	  vpnei.push_back(neiIt->psp);
	  vchi2nei.push_back(sp.chi2);
	}
      }
      if (neifound){
	propspIt->psp->neighborindex = neighborindex++;
	vpneighbor.push_back(vpnei);
	vchi2neighbor.push_back(vchi2nei);
      }
    }
  }

  zeit_otr_neighbors.stop();

  // collect track candidates

  zeit_otr_collect.start();

  vector<TRACK> vtracktmp;

  for ( int level = 4; level >= 0; level-- ) {
    vector<TRACK> vtrackcandidate;

    for (map<int, ArtePointer<GESL> >::iterator mgeslIt = geslmap.begin();
	 mgeslIt != geslmap.end(); mgeslIt++ ) {
      for(vector<SPACEPOINT>::iterator spIt  = vgeslsp[geslspmap[mgeslIt->first]].begin();
	  spIt != vgeslsp[geslspmap[mgeslIt->first]].end(); ++spIt ){

	if ((spIt->used)&&(spIt->pf>1.0/*0.2*/)) continue; // slow track - remove
	//if (spIt->used) continue; // slow track - remove
	
	if (spIt->level == level){

	  int lay1 = 0; int lay3 = 0; int lay5 = 0; // check zero layers
	  int lay2 = 0; int lay4 = 0; int lay6 = 0; // check stereo layers
	  for (vector<pCLUSTER>::iterator pcluIt = spIt->vpclu.begin();
	       pcluIt != spIt->vpclu.end(); ++pcluIt){
	    if (!(*pcluIt)->used){
	      switch ((*pcluIt)->lay){
	      case 1: {lay1 = 1; break;}
	      case 2: {lay2 = 1; break;}
	      case 3: {lay3 = 1; break;}
	      case 4: {lay4 = 1; break;}
	      case 5: {lay5 = 1; break;}
	      case 6: {lay6 = 1; break;}
	      }
	    }
	  }
	  if (lay2+lay4+lay6 <= 1){  // used space point
	    spIt->used = 1;
	  }
	  if (lay1+lay3+lay5 <= 1){  // used space point
	    spIt->used = 1;
	  }
	  
	  if ((spIt->used)&&(spIt->pf>1.0/*0.2*/)) continue; // again slow track - remove
	  //if (spIt->used) continue; // slow track - remove
	
	  TRACK newtrack, currenttrack;
	  newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
				  (mgeslIt->first, spIt));
	  newtrack.chi2 = spIt->chi2;
	  newtrack.prob = spIt->prob;
	  newtrack.nsp  = 1;
	  newtrack.nclu = distance(spIt->vpclu.begin(),spIt->vpclu.end()); 
	  newtrack.good = 1;

	  if (level>1){
	    currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
					   (mgeslIt->first, spIt));
	    currenttrack.chi2 = spIt->chi2;
	    currenttrack.prob = spIt->prob;
	    currenttrack.nsp  = 1;
	    currenttrack.nclu = distance(spIt->vpclu.begin(),spIt->vpclu.end()); 
	    currenttrack.good = 1;
	    
	    OtrFindTrack(spIt, newtrack, currenttrack);
	  }

	  if (newtrack.good){
	    
	    {
	      float pinv=1.0/(B*zmagn*sqrt(1.0+newtrack.txf*newtrack.txf+
					   newtrack.tyf*newtrack.tyf)/
			      ((newtrack.xf-newtrack.zf*newtrack.txf)*
			       sqrt(1.0+newtrack.txf*newtrack.txf)));
	      int sign1;
	      if(pinv<0.0) sign1=-1;else sign1=1;
	      pinv = fabs(pinv);
	      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	      newtrack.pf=newtrack.pe=pinv*sign1;
	    }
	    
	    vtrackcandidate.push_back(newtrack); //store track
	    
	  }
	}
      }
    }


    // 1. sort tracks
    //sort(vtrackcandidate.begin(), vtrackcandidate.end(), OtrTrackCompareProb);
    sort(vtrackcandidate.begin(), vtrackcandidate.end(), OtrTrackCompareLength);


    // 2. remove used hits
    
    for (vector<TRACK>::iterator trackIt = vtrackcandidate.begin();
	 trackIt != vtrackcandidate.end(); trackIt++){ //trackIt


      if ((!trackIt->good)&&(trackIt->pf>1.0/*0.2*/)) continue; // slow track - remove
      //if (!trackIt->good) continue; // slow track - remove
      
      // check the track

      int length = distance(trackIt->pspointmap.begin(), trackIt->pspointmap.end());

      for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator spIt = trackIt->pspointmap.begin();
	   spIt != trackIt->pspointmap.end(); spIt++){
	
	if ((spIt->second)->used){ //check space points
	  trackIt->good = 0;
	  break;
	}
	
	int lay1 = 0; int lay3 = 0; int lay5 = 0; // check zero layers
	int lay2 = 0; int lay4 = 0; int lay6 = 0; // check stereo layers
	for (vector<pCLUSTER>::iterator pcluIt = (spIt->second)->vpclu.begin();
	     pcluIt != (spIt->second)->vpclu.end(); ++pcluIt){
	  if (!(*pcluIt)->used){
	    switch ((*pcluIt)->lay){
	    case 1: {lay1 = 1; break;}
	    case 2: {lay2 = 1; break;}
	    case 3: {lay3 = 1; break;}
	    case 4: {lay4 = 1; break;}
	    case 5: {lay5 = 1; break;}
	    case 6: {lay6 = 1; break;}
	    }
	  }
	}
	if (lay2+lay4+lay6 <= 1){  // used space point
	  (spIt->second)->used = 1;
	  trackIt->good = 0;
	  break;
	}
	if (length == 1){           // require for short tracks to be independent
	  if (lay1+lay3+lay5 <= 1){  // used space point
	    (spIt->second)->used = 1;
	    trackIt->good = 0;
	    break;
	  }
	}
      }

      if ((!trackIt->good)&&(trackIt->pf>1.0/*0.2*/)) continue; // keep fast tracks, even bad
      //if (!trackIt->good) continue; // keep fast tracks, even bad
      
      // checking the track finished
      
      TRACK  track;
      pTRACK ptrack = &track;
      
      ptrack->pspointmap.clear();
      for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator pos = trackIt->pspointmap.begin();
	   pos != trackIt->pspointmap.end(); pos++){
	ptrack->pspointmap.insert(pair<ArtePointer<GESL>, pSPACEPOINT >
				  (pos->first, pos->second ));
      }
      ptrack->good = 1;//trackIt->good;

      SPACEPOINT sp;
      pSPACEPOINT psp, pspf, pspe;
      psp  = &sp;
      pspf = (trackIt->pspointmap.begin())->second;
      pspe = (trackIt->pspointmap.begin())->second;

      psp->vpclu.clear();
      for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator spIt = trackIt->pspointmap.begin();
	   spIt != trackIt->pspointmap.end(); spIt++){
	
	if ((spIt->second)->zf < pspf->zf) pspf = spIt->second;
	if ((spIt->second)->ze > pspe->ze) pspe = spIt->second;

	(spIt->second)->used = 1; // flag clusters as used

	for (vector<pCLUSTER>::iterator pcluIt = (spIt->second)->vpclu.begin();
	     pcluIt != (spIt->second)->vpclu.end(); ++pcluIt){

	  (*pcluIt)->used = 1;

	  psp->vpclu.push_back(*pcluIt);
	  for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	       phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	    ptrack->vphit.push_back(*phitIt);
	  }
	}
      }
	
      psp->xf  = pspf->xf;   psp->txf = pspf->txf;
      psp->yf  = pspf->yf;   psp->tyf = pspf->tyf; 
      psp->zf  = pspf->zf;
      psp->xe  = pspe->xe;   psp->txe = pspe->txe;
      psp->ye  = pspe->ye;   psp->tye = pspe->tye;
      psp->ze  = pspe->ze;
      psp->used = 0; psp->level = 0; psp->neighborindex = -1;
      
      sort(psp->vpclu.begin(), psp->vpclu.end(), OtrClusterCompareZ);
      
      OtrFitSPoint(*psp);

      ptrack->xf   = psp->xf;        ptrack->yf  = psp->yf;
      ptrack->txf  = psp->txf;       ptrack->tyf = psp->tyf;
      ptrack->zf   = psp->zf;        ptrack->ze  = psp->ze;
      ptrack->xe   = psp->xe;        ptrack->ye  = psp->ye;
      ptrack->txe  = psp->txe;       ptrack->tye = psp->tye;
      ptrack->chi2 = psp->chi2;      ptrack->prob = psp->prob;
      ptrack->nsp  = trackIt->nsp;   ptrack->nclu  = trackIt->nclu;

      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  ptrack->cvf[j][i] = psp->cvf[j][i];
	  ptrack->cve[j][i] = psp->cve[j][i];
	}

      //ptrack->pf = ptrack->pe = 4.0*sqrt(ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf);
      {
	float pinv=1.0/(B*zmagn*
			sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/
			((ptrack->xf-ptrack->zf*ptrack->txf)*
			 sqrt(1.0+ptrack->txf*ptrack->txf)));
	pinv = fabs(pinv);
	if (pinv > 0.5) pinv = 2.0*pinv*pinv;
	ptrack->pf=ptrack->pe=pinv;
      }

      for (vector<pCLUSTER >::iterator pcluIt = psp->vpclu.begin();
	   pcluIt != psp->vpclu.end(); ++pcluIt){
	ptrack->vpcluster.push_back(*pcluIt);
      }

      sort(ptrack->vphit.begin(), ptrack->vphit.end(), OtrHitCompareZ);


      //OtrRefitTrack(ptrack);

      vtracktmp.push_back(*ptrack); // store track

      if (OTR_DRAW){//draw track candidate
	
	zpl[0] = ptrack->zf;
	xpl[0] = ptrack->xf + x_shift; 
	ypl[0] = ptrack->yf + y_shift; 
	zpl[1] = ptrack->ze;
	xpl[1] = ptrack->xe + x_shift; 
	ypl[1] = ptrack->ye + y_shift; 
	
	float momentum = 1.0/ptrack->pf;

	if (momentum>1.0) isplci(Red);
	else              isplci(Blue);
	//ipl(2,zpl,xpl); ipl(2,zpl,ypl);
      }



    }
  }

  //sort(vtracktmp.begin(),vtracktmp.end(), OtrTrackCompareLength);
  
  zeit_otr_collect.stop();

  // gather clusters in supergedes
  vector<vector<pCLUSTER> > vsupergedevpcluster;
  for ( vector<vector<ArtePointer<GEDE> > >::iterator vsupergedeIt = vsupergede.begin();
	vsupergedeIt != vsupergede.end(); vsupergedeIt++){

    vector<pCLUSTER> vpcluster;
    for ( vector<ArtePointer<GEDE> >::iterator vgedeIt = (*vsupergedeIt).begin();
	  vgedeIt != (*vsupergedeIt).end(); vgedeIt++){

      int gede_index = gedeclustermap[*vgedeIt];
      for (vector<CLUSTER>::iterator cluIt = vgedecluster[gede_index].begin();
	   cluIt != vgedecluster[gede_index].end(); ++cluIt){
	cluIt->used = 0;      // clear cluster as unused
	vpcluster.push_back(cluIt);
      }
    }

    sort(vpcluster.begin(), vpcluster.end(), OtrClusterCompareX);
    vsupergedevpcluster.push_back(vpcluster);
  }

  map <ArtePointer<GESL>, int > geslusedmap;
  for (map<int, ArtePointer<GESL> >::iterator geslmIt = geslmap.begin();
       geslmIt != geslmap.end(); ++geslmIt){
    ArtePointer<GESL> gesl = geslmIt->second;
    geslusedmap.insert(pair< ArtePointer<GESL>, int>(gesl, 0));
  }

  //==================       start of hits gathering    =================

  zeit_otr_gather.start();

  vtrack.clear();

  for (vector<TRACK>::iterator trackIt = vtracktmp.begin();//trackIt
       trackIt != vtracktmp.end(); ++trackIt){ // trackIt

    // is the track good ?
    int nclu_total = 0;
    int nclu_free  = 0;
    for (vector<pCLUSTER>::iterator pcluIt = trackIt->vpcluster.begin();
	 pcluIt != trackIt->vpcluster.end(); ++pcluIt){
      nclu_total++;
      if (!(*pcluIt)->used) nclu_free++;
    }
    
    //if (float(nclu_free)/float(nclu_total) < 0.7) continue; // track is used
    
    //TRACK  track;
    //pTRACK ptrack = &track;
      
    //create temporary track as a space point

    SPACEPOINT sp;

    sp.xf  = trackIt->xf;   sp.txf = trackIt->txf;
    sp.yf  = trackIt->yf;   sp.tyf = trackIt->tyf; 
    sp.zf  = trackIt->zf;
    sp.xe  = trackIt->xe;   sp.txe = trackIt->txe;
    sp.ye  = trackIt->ye;   sp.tye = trackIt->tye;
    sp.ze  = trackIt->ze;
    sp.used = 0; sp.level = 0; sp.neighborindex = -1;
    sp.nsp = trackIt->nsp;  sp.nclu = trackIt->nclu;
    
    sp.vpclu.clear();
    for (vector<pCLUSTER >::iterator pcluIt = trackIt->vpcluster.begin();
	 pcluIt != trackIt->vpcluster.end(); ++pcluIt){
      sp.vpclu.push_back(*pcluIt);
    }

    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
	sp.cvf[j][i] = trackIt->cvf[j][i];
	sp.cve[j][i] = trackIt->cve[j][i];
      }

    OtrFitSPoint(sp);
    
    //    if (sp.prob < 0.05) continue; // bad track

    // detect an empty superlayer inside 
    for (map<int, ArtePointer<GESL> >::iterator geslmIt = geslmap.begin();
	 geslmIt != geslmap.end(); ++geslmIt){
      ArtePointer<GESL> gesl = geslmIt->second;
      geslusedmap[gesl] = 0;
    }

    for (vector<pCLUSTER>::iterator pcluIt = sp.vpclu.begin();
	 pcluIt != sp.vpclu.end(); pcluIt++){
      ArtePointer<GESL> gesl = (*pcluIt)->gede->gesl;
      geslusedmap[gesl]++;
    }

    int start = -1, end = -1;
    int geslusedarray[4] = {0, 0, 0, 0};
    ArtePointer<GESL> geslarray[4] = {0, 0, 0, 0};
    int gesl_index = -1; // to start from 0
    for (map<ArtePointer<GESL>, int >::iterator geslIt = geslusedmap.begin();
	   geslIt != geslusedmap.end(); geslIt++){
      gesl_index++;
      geslarray[gesl_index] = geslIt->first;
      if (geslIt->second == 0) continue;
      geslusedarray[gesl_index] = geslIt->second;
      if (start == -1) start = gesl_index;
      end = gesl_index;
    }

    for (int i = start-1; i >= 0; i--){ // start

      int supergede_index = geslsupergedemap[geslarray[i]];

      vector<pCLUSTER> vpclu;

      for (vector<int>::iterator sgedeIt = vgeslsupergede[supergede_index].begin();
	   sgedeIt != vgeslsupergede[supergede_index].end(); sgedeIt++){// sgede
	int sgede_index = *sgedeIt;

	int nhits = distance(vsupergedevpcluster[sgede_index].begin(),
			     vsupergedevpcluster[sgede_index].end());

	if (nhits == 0)	continue;

	// propagate track to z of the supergede
	float z_sgede = vsupergede_z[sgede_index];

	float x = sp.xf+sp.txf*(z_sgede-sp.zf);
	float y = sp.yf+sp.tyf*(z_sgede-sp.zf);

	float errx2  = sp.cvf[0][0]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[2][2];
	float erry2  = sp.cvf[1][1]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[3][3];
	float errxy2 = sp.cvf[0][1]+(z_sgede-sp.zf)*(z_sgede-sp.zf)*sp.cvf[2][3];
	float erry = sqrt(erry2); //erry *= 3.5;

	// find hit, closest to track position

	vector<pCLUSTER>::iterator pcluIt_f =  
	  vsupergedevpcluster[sgede_index].begin();

	// transform into plane coordinate system
	float cosa = (*pcluIt_f)->cosa; float sina = (*pcluIt_f)->sina;
	float u = x*cosa - y*sina;

	float erru = sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			    erry2*sina*sina);
	//	erru *= 0.8;
	erru += 0.7; // plus cluster error
  	
	int kl = 0, kr = nhits-1, k0 = 0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) k0 = kr;
	else                                                   k0 = kl;
	
	if (fabs(u-(*(pcluIt_f+k0))->x) < erru) { // hit found
	  vector<pCLUSTER>::iterator pcluIt_0 = pcluIt_f+k0; 
	  int hit_ok = 1;
	  //inside gede?
	  GEDE_SIZE gede_size = gedesizemap[(*pcluIt_0)->gede.index()];
	  if ((y-erry > gede_size.symx) || (y+erry < gede_size.symn))
	    hit_ok = 0;
	  if (hit_ok){
	    vpclu.push_back(*pcluIt_0);
	  }
	}
      }


      //cout<<"Gesl** "<<geslarray[i]<<" found "<<distance(vpclu.begin(), vpclu.end())<<" clusters"<<endl;
      if (distance(vpclu.begin(), vpclu.end())<=2) break;//2//-1

      float zf = 10000.0;
      float ze = 0.0;
      for (vector<pCLUSTER>::iterator pcluIt = vpclu.begin();
	   pcluIt != vpclu.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }

      //refit the track

      SPACEPOINT sptmp;

      sptmp.xf  = sp.xf+sp.txf*(zf-sp.zf);   sptmp.txf = sp.txf;
      sptmp.yf  = sp.yf+sp.tyf*(zf-sp.zf);   sptmp.tyf = sp.tyf; 
      sptmp.zf  = zf;
      sptmp.xe  = sp.xe;   sptmp.txe = sp.txe;
      sptmp.ye  = sp.ye;   sptmp.tye = sp.tye;
      sptmp.ze  = sp.ze;
      sptmp.used = 0; sptmp.level = 0; sptmp.neighborindex = -1;
      
      sptmp.vpclu.clear();
      for (vector<pCLUSTER >::iterator pcluIt = sp.vpclu.begin();
	   pcluIt != sp.vpclu.end(); ++pcluIt){
	sptmp.vpclu.push_back(*pcluIt);
      }
      for (vector<pCLUSTER>::iterator pcluIt = vpclu.begin();
	   pcluIt != vpclu.end(); ++pcluIt){
	sptmp.vpclu.push_back(*pcluIt);
      }      
      sort(sptmp.vpclu.begin(), sptmp.vpclu.end(), OtrClusterCompareZ);

      //      OtrFitSPoint(sptmp,1);
      OtrFitSPointNew(sptmp);

      if (sptmp.prob < 0.05) break; // bad combination

      // track is good -> remove clusters and store

      sp.xf  = sptmp.xf;   sp.txf = sptmp.txf;
      sp.yf  = sptmp.yf;   sp.tyf = sptmp.tyf; 
      sp.zf  = sptmp.zf;
      sp.xe  = sptmp.xe;   sp.txe = sptmp.txe;
      sp.ye  = sptmp.ye;   sp.tye = sptmp.tye;
      sp.ze  = sptmp.ze;
      sp.used = 0; sp.level = 0; sp.neighborindex = -1; 
      sp.chi2 = sptmp.chi2; sp.prob = sptmp.prob;

      sp.vpclu.clear();
      for (vector<pCLUSTER >::iterator pcluIt = sptmp.vpclu.begin();
	   pcluIt != sptmp.vpclu.end(); ++pcluIt){
	//(*pcluIt)->used = 1;
	sp.vpclu.push_back(*pcluIt);
      }

      sp.nsp++; sp.nclu = distance(sp.vpclu.begin(), sp.vpclu.end());

           
      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  sp.cvf[j][i] = sptmp.cvf[j][i];
	  sp.cve[j][i] = sptmp.cve[j][i];
	}
      

    }


    for (int i = end+1; i < 4; i++){ // end

      int supergede_index = geslsupergedemap[geslarray[i]];

      vector<pCLUSTER> vpclu;

      //cout<<"gesl "<<*geslIt<<" supergede_index "<<supergede_index;
      for (vector<int>::iterator sgedeIt = vgeslsupergede[supergede_index].begin();
	   sgedeIt != vgeslsupergede[supergede_index].end(); sgedeIt++){// sgede
	int sgede_index = *sgedeIt;

	int nhits = distance(vsupergedevpcluster[sgede_index].begin(),
			     vsupergedevpcluster[sgede_index].end());
	if (nhits == 0) continue;

	// propagate track to z of the supergede
	float z_sgede = vsupergede_z[sgede_index];

	float x = sp.xe+sp.txe*(z_sgede-sp.ze);
	float y = sp.ye+sp.tye*(z_sgede-sp.ze);

	float errx2  = sp.cve[0][0]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[2][2];
	float erry2  = sp.cve[1][1]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[3][3];
	float errxy2 = sp.cve[0][1]+(z_sgede-sp.ze)*(z_sgede-sp.ze)*sp.cve[2][3];
	float erry = sqrt(erry2); //erry *= 3.5;

	// find hit, closest to track position

	vector<pCLUSTER>::iterator pcluIt_f =  
	  vsupergedevpcluster[sgede_index].begin();

	// transform into plane coordinate system
	float cosa = (*pcluIt_f)->cosa; float sina = (*pcluIt_f)->sina;
	float u    = x*cosa - y*sina;

	float erru = sqrt(errx2*cosa*cosa - 2.0*errxy2*cosa*sina + 
			    erry2*sina*sina);
	//	erru *= 0.8;
	erru += 0.7; // plus cluster error
	  	
	int kl = 0, kr = nhits-1, k0 = 0;
	while((kr-kl) > 1){
	  k0=(kl+kr)>>1;
	  if(u <= (*(pcluIt_f+k0))->x) kr = k0;
	  else                         kl = k0;
	}
	if (((*(pcluIt_f+kr))->x-u) < (u-(*(pcluIt_f+kl))->x)) k0 = kr;
	else                                                   k0 = kl;

	if (fabs(u-(*(pcluIt_f+k0))->x) < erru) { // hit found
	  vector<pCLUSTER>::iterator pcluIt_0 = pcluIt_f+k0; 
	  int hit_ok = 1;
	  //inside gede?
	  GEDE_SIZE gede_size = gedesizemap[(*pcluIt_0)->gede.index()];
	  if ((y-erry > gede_size.symx) || (y+erry < gede_size.symn))
	    hit_ok = 0;
	  if (hit_ok){
	    vpclu.push_back(*pcluIt_0);
	  }
	}
      }

      //cout<<"**Gesl "<<geslarray[i]<<" found "<<distance(vpclu.begin(), vpclu.end())<<" clusters"<<endl;
      if (distance(vpclu.begin(), vpclu.end())<=2) break;//2//-1

      float zf = 10000.0;
      float ze = 0.0;
      for (vector<pCLUSTER>::iterator pcluIt = vpclu.begin();
	   pcluIt != vpclu.end(); pcluIt++){
	if ((*pcluIt)->z < zf) zf = (*pcluIt)->z;
	if ((*pcluIt)->z > ze) ze = (*pcluIt)->z;
      }

      //refit the track

      SPACEPOINT sptmp;

      sptmp.xf  = sp.xf;   sptmp.txf = sp.txf;
      sptmp.yf  = sp.yf;   sptmp.tyf = sp.tyf; 
      sptmp.zf  = sp.zf;
      sptmp.xe  = sp.xe+sp.txe*(ze-sp.ze);   sptmp.txe = sp.txe;
      sptmp.ye  = sp.ye+sp.tye*(ze-sp.ze);   sptmp.tye = sp.tye;
      sptmp.ze  = ze;
      sptmp.used = 0; sptmp.level = 0; sptmp.neighborindex = -1;
      
      sptmp.vpclu.clear();
      for (vector<pCLUSTER >::iterator pcluIt = sp.vpclu.begin();
	   pcluIt != sp.vpclu.end(); ++pcluIt){
	sptmp.vpclu.push_back(*pcluIt);
      }
      for (vector<pCLUSTER>::iterator pcluIt = vpclu.begin();
	   pcluIt != vpclu.end(); ++pcluIt){
	sptmp.vpclu.push_back(*pcluIt);
      }      
      sort(sptmp.vpclu.begin(), sptmp.vpclu.end(), OtrClusterCompareZ);

      //      OtrFitSPoint(sptmp,1);
      OtrFitSPointNew(sptmp);

      if (sptmp.prob < 0.05) break; // bad combination

      // track is good -> remove clusters and store

      sp.xf  = sptmp.xf;   sp.txf = sptmp.txf;
      sp.yf  = sptmp.yf;   sp.tyf = sptmp.tyf; 
      sp.zf  = sptmp.zf;
      sp.xe  = sptmp.xe;   sp.txe = sptmp.txe;
      sp.ye  = sptmp.ye;   sp.tye = sptmp.tye;
      sp.ze  = sptmp.ze;
      sp.used = 0; sp.level = 0; sp.neighborindex = -1; 
      sp.chi2 = sptmp.chi2; sp.prob = sptmp.prob;

      sp.vpclu.clear();
      for (vector<pCLUSTER >::iterator pcluIt = sptmp.vpclu.begin();
	   pcluIt != sptmp.vpclu.end(); ++pcluIt){
	//(*pcluIt)->used = 1;
	sp.vpclu.push_back(*pcluIt);
      }
      sp.nsp++; sp.nclu = distance(sp.vpclu.begin(), sp.vpclu.end());

            
      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  sp.cvf[j][i] = sptmp.cvf[j][i];
	  sp.cve[j][i] = sptmp.cve[j][i];
	}
      
    }

    // store the track -> as a copy of the space point
    
    OtrFitSPoint(sp);
    //  OtrFitSPointNew(sp);


    TRACK  track;
    pTRACK ptrack = &track;
    
    // no sence to keep space points
    ptrack->pspointmap.clear();
    ptrack->good = 1;
    
    ptrack->xf   = sp.xf;        ptrack->yf   = sp.yf;
    ptrack->txf  = sp.txf;       ptrack->tyf  = sp.tyf;
    ptrack->zf   = sp.zf;        ptrack->ze   = sp.ze;
    ptrack->xe   = sp.xe;        ptrack->ye   = sp.ye;
    ptrack->txe  = sp.txe;       ptrack->tye  = sp.tye;
    ptrack->chi2 = sp.chi2;      ptrack->prob = sp.prob;
    ptrack->nsp  = sp.nsp;       ptrack->nclu = sp.nclu;    
    
    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
 	ptrack->cvf[j][i] = sp.cvf[j][i];
	ptrack->cve[j][i] = sp.cve[j][i];
      }
    
    //ptrack->pf = ptrack->pe = 4.0*sqrt(ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf);
    {
      float pinv=1.0/(B*zmagn*
		      sqrt(1.0+ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf)/
		      ((ptrack->xf-ptrack->zf*ptrack->txf)*
		       sqrt(1.0+ptrack->txf*ptrack->txf)));
      pinv = fabs(pinv);
      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
      ptrack->pf=ptrack->pe=pinv;
    }

    for (vector<pCLUSTER >::iterator pcluIt = sp.vpclu.begin();
	 pcluIt != sp.vpclu.end(); ++pcluIt){
      ptrack->vpcluster.push_back(*pcluIt);
      for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	   phitIt != (*pcluIt)->vphit.end(); ++phitIt){
 	ptrack->vphit.push_back(*phitIt);
      }
    }
    sort(ptrack->vphit.begin(), ptrack->vphit.end(), OtrHitCompareZ);

    //OtrRefitTrack(ptrack);

    vtrack.push_back(*ptrack);

    if (OTR_DRAW){//draw track candidate
      
      zpl[0] = ptrack->zf;
      xpl[0] = ptrack->xf + x_shift; 
      ypl[0] = ptrack->yf + y_shift; 
      zpl[1] = ptrack->ze;
      xpl[1] = ptrack->xe + x_shift; 
      ypl[1] = ptrack->ye + y_shift; 
      
      float momentum = 1.0/ptrack->pf;
      
      if (momentum>1.0) isplci(Red);
      else              isplci(Blue);
      //ipl(2,zpl,xpl); ipl(2,zpl,ypl);
    }
    

  } 
  zeit_otr_gather.stop();

  zeit_otr_refit.start();

  sort(vtrack.begin(), vtrack.end(), OtrTrackCompareLength);

  // clear cluster usage
  for (map<int, ArtePointer<GEDE> >::iterator gedemIt = gedemap.begin();
       gedemIt != gedemap.end(); ++gedemIt){
    int gede_index = gedeclustermap[gedemIt->second];
    for (vector<CLUSTER>::iterator cluIt = vgedecluster[gede_index].begin();
	 cluIt != vgedecluster[gede_index].end(); ++cluIt){
      cluIt->used = 0;      // clear cluster as unused
    }
  }

  vector<TRACK> vtrackgood;
  int good_tracks = 0;
  for (vector<TRACK>::iterator trackIt = vtrack.begin();
     trackIt != vtrack.end(); ++trackIt){// trackIt

    // is the track good ?
    int nclu_total = 0;
    int nclu_free  = 0;
    vector<pCLUSTER> vpclu;
    vector<pCLUSTER> vpclu_track;
    for (vector<pCLUSTER>::iterator pcluIt = trackIt->vpcluster.begin();
	 pcluIt != trackIt->vpcluster.end(); ++pcluIt){ // over clusters
      nclu_total++;
      if (!(*pcluIt)->used){ 
	vpclu_track.push_back(*pcluIt);
	nclu_free++;
	continue;
      }

      int clu_found = 0;
      pCLUSTER pclu;
      // else try to find another free cluster at THE SAME gede plane !!!
      //==========================================================
	// propagate track to z of the supergede

      float z = (*pcluIt)->z;

      float x = trackIt->xf+trackIt->txf*(z-trackIt->zf);
      float y = trackIt->yf+trackIt->tyf*(z-trackIt->zf);

      // find hit, closest to track position
      int gede_index = gedeclustermap[(*pcluIt)->gede];
      vector<CLUSTER>::iterator cluIt_f =  
	vgedecluster[gede_index].begin();
      vector<CLUSTER>::reverse_iterator rcluIt_f =  
	vgedecluster[gede_index].rbegin();

      // transform into plane coordinate system
      float cosa = (*pcluIt)->cosa; float sina = (*pcluIt)->sina;
      float u      = x*cosa      - y*sina;

      float erru_f = sqrt(trackIt->cvf[0][0]*cosa*cosa - 2.0*trackIt->cvf[0][1]*cosa*sina + 
			  trackIt->cvf[1][1]*sina*sina);
      float erru_e = sqrt(trackIt->cve[0][0]*cosa*cosa - 2.0*trackIt->cve[0][1]*cosa*sina + 
			  trackIt->cve[1][1]*sina*sina);
      float erru = erru_f + (erru_e-erru_f)*(z-trackIt->zf)/(trackIt->ze-trackIt->zf);
      erru *= 3.5;
      erru += 1.0; // plus cluster error
	  	
      int nhits = distance(vgedecluster[gede_index].begin(), vgedecluster[gede_index].end());
      int k0 = distance(cluIt_f,*pcluIt);

      vector<CLUSTER>::iterator cluIt_0 = *pcluIt; 
      vector<CLUSTER>::reverse_iterator rcluIt_0 = rcluIt_f+(nhits-k0-1); 

      pCLUSTER pclu_best;
      float    pclu_dist = 1000.0;
      
      for (vector<CLUSTER>::iterator cluIt =  (cluIt_0+1);
	   cluIt != vgedecluster[gede_index].end(); cluIt++){
	if ((*cluIt).used) continue;
	float dist = fabs((*cluIt).x - u);
	if (dist > erru) break;
	pclu_best = cluIt;
	pclu_dist = fabs((*cluIt).x - u);
	break;
      }
      for (vector<CLUSTER>::reverse_iterator cluIt =  (rcluIt_0+1);
	   cluIt != vgedecluster[gede_index].rend(); cluIt++){
	if ((*cluIt).used) continue;
	float dist = fabs((*cluIt).x - u);
	if (dist > erru) break;
	if (fabs((*cluIt).x - u) < pclu_dist){
	  pclu_best = &(*cluIt);
	  pclu_dist = fabs((*cluIt).x - u);
	}
	break;
      }
      
      if (pclu_dist<1000.0){
	vpclu.push_back(pclu_best);
	pclu = pclu_best;
	clu_found = 1;
	nclu_free++;
      }
      
      if (clu_found){
	vpclu_track.push_back(pclu);
      }
      else
	vpclu_track.push_back(*pcluIt); //which is used ;-(

    }
    
    //    cout<<"Track : (replaced "<<distance(vpclu.begin(), vpclu.end())<<") "<<nclu_free<<"/"<<nclu_total<<" -> "<<float(nclu_free)/float(nclu_total)<<endl;

    //    cout<<"compare dist old and new : "<<distance(trackIt->vpcluster.begin(),trackIt->vpcluster.end())
    //	<<"/"<<distance(vpclu_track.begin(), vpclu_track.end())<<endl;

    //if (float(nclu_free)/float(nclu_total) < 0.7){

    if ((nclu_free <= 3) || (float(nclu_free)/float(nclu_total) < 0.7)){
      trackIt->good = 0;
      continue; // track is used
    }

    if (distance(vpclu.begin(), vpclu.end()) == 0){ // track not changed

      OtrResolveLR(trackIt);

      vector<pHIT> vphit;

      int indexf = -1;
      int indexe = 0;
      int index = 0;
      float zcluf = 0.0, zclue = 0.0;
      for (vector<pHIT>::iterator phitIt = trackIt->vphit.begin();
	   phitIt != trackIt->vphit.end(); ++phitIt){
	if ((*phitIt)->mask>0.0){
	  if (indexf == -1){
	    indexf = index;
	    zcluf = (*phitIt)->z;
	  }
	  indexe = index;
	  zclue = (*phitIt)->z;

	}
	index++;
      }

      index = 0;
      int index_tmp = 0;
      for (vector<pHIT>::iterator phitIt = trackIt->vphit.begin();
	   phitIt != trackIt->vphit.end(); ++phitIt){
	if ((index_tmp>=indexf)&&(index_tmp<=indexe)){
	  vphit.push_back(*phitIt);
	  index++;
	}
	index_tmp++;
      }

      if (index == 0) continue;// no usefull hits;


      // create new track

      TRACK track;
      pTRACK ptrack = &track;

      // no sense to keep space points
      ptrack->pspointmap.clear();
      ptrack->good = 1;

      ptrack->zf = vphit[0]->z;
      ptrack->xf = trackIt->xf + trackIt->txf*(ptrack->zf-trackIt->zf);
      ptrack->yf = trackIt->yf + trackIt->tyf*(ptrack->zf-trackIt->zf);
      ptrack->ze = vphit[--index]->z;
      ptrack->xe = trackIt->xe + trackIt->txe*(ptrack->ze-trackIt->ze);
      ptrack->ye = trackIt->ye + trackIt->tye*(ptrack->ze-trackIt->ze);
    
      //ptrack->xf   = trackIt->xf;        ptrack->yf  = trackIt->yf;
      ptrack->txf  = trackIt->txf;       ptrack->tyf = trackIt->tyf;
      //ptrack->zf   = trackIt->zf;        ptrack->ze  = trackIt->ze;
      //ptrack->xe   = trackIt->xe;        ptrack->ye  = trackIt->ye;
      ptrack->txe  = trackIt->txe;       ptrack->tye = trackIt->tye;
      ptrack->chi2 = trackIt->chi2;      ptrack->nsp  = trackIt->nsp;  

      for (int i=0; i<4; i++) 
	for (int j=0; j<=i; j++){
	  ptrack->cvf[j][i] = trackIt->cvf[j][i];
	  ptrack->cve[j][i] = trackIt->cve[j][i];
	}
      
      ptrack->pf = ptrack->pe = 4.0*sqrt(ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf);


      ptrack->vpcluster.clear();
      for (vector<pCLUSTER>::iterator pcluIt = trackIt->vpcluster.begin();
	   pcluIt != trackIt->vpcluster.end(); ++pcluIt){
	/*
	int used = 0;
	for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	     phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	  //if ((*phitIt)->mask>0.0)
	  if (((*phitIt)->z >= zcluf)&&((*phitIt)->z <= zclue))
	    used = 1;
	}
	if (used)  //keep only if used
	*/
	ptrack->vpcluster.push_back(*pcluIt);
      }

      ptrack->nclu = distance(ptrack->vpcluster.begin(), ptrack->vpcluster.end());

      ptrack->vphit.clear();
      index = 0;
      for (vector<pHIT>::iterator phitIt = vphit.begin();
	   phitIt != vphit.end(); ++phitIt){
	ptrack->vphit.push_back(*phitIt);
	index++;
      }

      ptrack->nonmask = 1;

      OtrFastDriftRefitTrack(ptrack);

      if (ptrack->nonmask != 0){
	vtrackgood.push_back(*ptrack);

	// flag used clusters 
	int nclu_found = 0;
	for (vector<pCLUSTER >::iterator pcluIt = ptrack->vpcluster.begin();
	     pcluIt != ptrack->vpcluster.end(); ++pcluIt){
	  
	  int used = 0;
	  for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	       phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	    if ((*phitIt)->mask > 0.0)
	      used = 1;
	  }
	  //(*pcluIt)->used = used;
	  if (used) nclu_found++;

	  (*pcluIt)->used = 1;
	}
	ptrack->nclu = nclu_found;

	good_tracks++;
      }
      continue;
    }

    // else create new track

    TRACK track;
    pTRACK ptrack = &track;

    // no sense to keep space points
    ptrack->pspointmap.clear();
    ptrack->good = 1;
    
    ptrack->xf   = trackIt->xf;        ptrack->yf  = trackIt->yf;
    ptrack->txf  = trackIt->txf;       ptrack->tyf = trackIt->tyf;
    ptrack->zf   = trackIt->zf;        ptrack->ze  = trackIt->ze;
    ptrack->xe   = trackIt->xe;        ptrack->ye  = trackIt->ye;
    ptrack->txe  = trackIt->txe;       ptrack->tye = trackIt->tye;
    ptrack->chi2 = trackIt->chi2;
    ptrack->nsp  = trackIt->nsp;       ptrack->nclu= trackIt->nclu;    

    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
	ptrack->cvf[j][i] = trackIt->cvf[j][i];
	ptrack->cve[j][i] = trackIt->cve[j][i];
      }
    
    ptrack->pf = ptrack->pe = 4.0*sqrt(ptrack->txf*ptrack->txf+ptrack->tyf*ptrack->tyf);

    ptrack->vpcluster.clear();
    ptrack->vphit.clear();
    for (vector<pCLUSTER >::iterator pcluIt = vpclu_track.begin();
	 pcluIt != vpclu_track.end(); ++pcluIt){
      ptrack->vpcluster.push_back(*pcluIt);
      //(*pcluIt)->used = 1;      // flag used clusters 
      for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	   phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	ptrack->vphit.push_back(*phitIt);
      }
    }
    sort(ptrack->vphit.begin(), ptrack->vphit.end(), OtrHitCompareZ);
    ptrack->nclu = distance(vpclu_track.begin(), vpclu_track.end());    

    OtrResolveLR(ptrack);

    vector<pHIT> vphit;

    int indexf = -1;
    int indexe = 0;
    int index = 0;
    float zcluf = 0.0, zclue = 0.0;
    for (vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
	 phitIt != ptrack->vphit.end(); ++phitIt){
      if ((*phitIt)->mask>0.0){
	if (indexf == -1){
	  indexf = index;
	  zcluf = (*phitIt)->z;
	}
	indexe = index;
	zclue = (*phitIt)->z;
	
      }
      index++;
    }
    
    index = 0;
    int index_tmp = 0;
    for (vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
	 phitIt != ptrack->vphit.end(); ++phitIt){
      if ((index_tmp>=indexf)&&(index_tmp<=indexe)){
	vphit.push_back(*phitIt);
	index++;
      }
      index_tmp++;
    }
    
    if (index == 0) continue;// no usefull hits;
    
    // create new track
    
    TRACK track_new;
    pTRACK ptrack_new = &track_new;
    
    // no sense to keep space points
    ptrack_new->pspointmap.clear();
    ptrack_new->good = 1;

    ptrack_new->zf = vphit[0]->z;
    ptrack_new->xf = ptrack->xf + ptrack->txf*(ptrack_new->zf-ptrack->zf);
    ptrack_new->yf = ptrack->yf + ptrack->tyf*(ptrack_new->zf-ptrack->zf);
    ptrack_new->ze = vphit[--index]->z;
    ptrack_new->xe = ptrack->xe + ptrack->txe*(ptrack_new->ze-ptrack->ze);
    ptrack_new->ye = ptrack->ye + ptrack->tye*(ptrack_new->ze-ptrack->ze);
    
    ptrack_new->txf  = ptrack->txf;       ptrack_new->tyf = ptrack->tyf;
    ptrack_new->txe  = ptrack->txe;       ptrack_new->tye = ptrack->tye;
    ptrack_new->chi2 = ptrack->chi2;      ptrack_new->nsp  = ptrack->nsp;


    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++){
	ptrack_new->cvf[j][i] = ptrack->cvf[j][i];
	ptrack_new->cve[j][i] = ptrack->cve[j][i];
      }
    
    ptrack_new->pf = ptrack_new->pe = 4.0*sqrt(ptrack_new->txf*ptrack_new->txf+
    					       ptrack_new->tyf*ptrack_new->tyf);


    ptrack_new->vpcluster.clear();
    for (vector<pCLUSTER>::iterator pcluIt = ptrack->vpcluster.begin();
	 pcluIt != ptrack->vpcluster.end(); ++pcluIt){
      /*
      int used = 0;
      for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	   phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	//if ((*phitIt)->mask>0.0)
	if (((*phitIt)->z >= zcluf)&&((*phitIt)->z <= zclue))
	  used = 1;
      }
      if (used)  //keep only if used
      */
      ptrack_new->vpcluster.push_back(*pcluIt);
    }
    ptrack_new->nclu = distance(ptrack_new->vpcluster.begin(), ptrack_new->vpcluster.end());

    ptrack_new->vphit.clear();
    index = 0;
    for (vector<pHIT>::iterator phitIt = vphit.begin();
	 phitIt != vphit.end(); ++phitIt){
      ptrack_new->vphit.push_back(*phitIt);
      index++;
    }

    ptrack_new->nonmask = 1;
      
    OtrFastDriftRefitTrack(ptrack_new);
    
    if (ptrack_new->nonmask != 0){
      vtrackgood.push_back(*ptrack_new);
      good_tracks++;

      int nclu_found = 0;
      for (vector<pCLUSTER >::iterator pcluIt = ptrack_new->vpcluster.begin();
	   pcluIt != ptrack_new->vpcluster.end(); ++pcluIt){

	int used = 0;
	for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	     phitIt != (*pcluIt)->vphit.end(); ++phitIt){
 	  if ((*phitIt)->mask > 0.0)
	    used = 1;
	}
	//(*pcluIt)->used = used;
	if (used) nclu_found++;

	(*pcluIt)->used = 1;
      }
      ptrack_new->nclu = nclu_found;

    }
  }

  //cout<<endl;

  zeit_otr_refit.stop();

  //#ifdef XXX
  //find clones

  int n_clone = 0;
  for (vector<TRACK>::iterator ptrack = vtrackgood.begin();
       ptrack != vtrackgood.end(); ++ptrack){

    n_clone++;
    if (ptrack->nonmask == 0) continue;

    //    cout<<"Clones for a track "<<n_clone<<endl;

  int n_clone2 = n_clone;

    vector<TRACK>::iterator ptrack_begin = ptrack;
    ptrack_begin++;
    if (ptrack_begin == vtrackgood.end()) continue;

    for (vector<TRACK>::iterator ptrack2 = ptrack_begin;
	 ptrack2 != vtrackgood.end(); ++ptrack2){
      n_clone2++;
      if (ptrack2->nonmask == 0) continue;

      // check overlaping
      int ok = ((ptrack->zf-ptrack2->ze)>0.0) || ((ptrack2->zf-ptrack->ze)>0.0);
      if (!ok) continue;

      vector<TRACK>::iterator ptrack_f, ptrack_e;
      ptrack_f = ptrack;
      ptrack_e = ptrack2;
      if (ptrack->zf > ptrack2->ze){
	ptrack_f = ptrack2;
	ptrack_e = ptrack;
      }

      float x1=0.0, dx1=0.0;
      
      x1  = ptrack_f->xe + ptrack_f->txe*(ptrack_e->zf - ptrack_f->ze);
      dx1 = 3.5*(sqrt(ptrack_f->cve[0][0]+(ptrack_e->zf-ptrack_f->ze)*(ptrack_e->zf-ptrack_f->ze)*
		     ptrack_f->cve[2][2]) + sqrt(ptrack_e->cvf[0][0])) + 1.0;
      
      float x2=0.0, dx2=0.0;

      x2  = ptrack_e->xf + ptrack_e->txf*(ptrack_f->ze - ptrack_e->zf);
      dx2 = 3.5*(sqrt(ptrack_e->cvf[0][0]+(ptrack_f->ze-ptrack_e->zf)*(ptrack_f->ze-ptrack_e->zf)*
		     ptrack_e->cvf[2][2]) + sqrt(ptrack_f->cve[0][0])) + 1.0;

      if ((fabs((ptrack_e->xf-x1)/dx1)>1.0)||(fabs((ptrack_f->xe-x2)/dx2)>1.0)) continue;

      //clone found - remove worth

      //if (ptrack->prob > ptrack2->prob)
      if (ptrack->nclu > ptrack2->nclu)
	ptrack2->nonmask = 0;
      else 
	ptrack->nonmask = 0;

    }
  }
  //#endif //XXX

  zeit_otr_rseg.start();

  for (vector<TRACK>::iterator ptrack = vtrackgood.begin();
       ptrack != vtrackgood.end(); ++ptrack)
    //for (vector<TRACK>::iterator ptrack = vtrack.begin();
    // ptrack != vtrack.end(); ++ptrack)
    //for (vector<TRACK>::iterator ptrack = vtracktmp.begin();
    // ptrack != vtracktmp.end(); ++ptrack)


    if (ptrack->nonmask != 0)
      {
	RSEG r;
	RHIT rh;
	RPNT rp;

	r.xf  = ptrack->xf;  r.yf  = ptrack->yf;  r.zf  = ptrack->zf; 
	r.txf = ptrack->txf; r.tyf = ptrack->tyf;
	r.xe  = ptrack->xe;  r.ye  = ptrack->ye;  r.ze  = ptrack->ze; 
	r.txe = ptrack->txe; r.tye = ptrack->tye;
    
	r.cvf[ 0] = ptrack->cvf[0][0];      r.cvf[ 1] = ptrack->cvf[0][1];
	r.cvf[ 2] = ptrack->cvf[1][1];      r.cvf[ 3] = ptrack->cvf[0][2];
	r.cvf[ 4] = ptrack->cvf[1][2];      r.cvf[ 5] = ptrack->cvf[2][2];
	r.cvf[ 6] = ptrack->cvf[0][3];      r.cvf[ 7] = ptrack->cvf[1][3];
	r.cvf[ 8] = ptrack->cvf[2][3];      r.cvf[ 9] = ptrack->cvf[3][3];

	r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
	r.cvf[14] = 100.0;

	r.cve[ 0] = ptrack->cve[0][0];      r.cve[ 1] = ptrack->cve[0][1];
	r.cve[ 2] = ptrack->cve[1][1];      r.cve[ 3] = ptrack->cve[0][2];
	r.cve[ 4] = ptrack->cve[1][2];      r.cve[ 5] = ptrack->cve[2][2];
	r.cve[ 6] = ptrack->cve[0][3];      r.cve[ 7] = ptrack->cve[1][3];
	r.cve[ 8] = ptrack->cve[2][3];      r.cve[ 9] = ptrack->cve[3][3];

	r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
	r.cve[14] = 100.0;
	
	r.pf = r.pe = ptrack->pf;
	
	r.time = 0.0; r.dtim = 0.0;
	r.chi  = ptrack->chi2;

	r.fit  = 0;
	r.cmp  = Rsegc::patt | Rsegc::bit16 | Rsegc::wch;
    
	ArteTable<RSEG>::push_back( r );
	ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );

#ifdef XXX
	for (vector<pCLUSTER>::iterator pcluIt = ptrack->vpcluster.begin();
	     pcluIt != ptrack->vpcluster.end(); ++pcluIt){
	  for (vector<pHIT>::iterator phitIt = (*pcluIt)->vphit.begin();
	       phitIt != (*pcluIt)->vphit.end(); ++phitIt){
	    //if ((*phitIt)->mask>0.0)
	      createRelation( (*phitIt)->hitb, rseg, &RSEG::hitb );
	  }
	}
#endif
	
        int number_of_rhits = 0;
        for (vector<EST>::reverse_iterator estIt = ptrack->vest.rbegin();
             estIt != ptrack->vest.rend(); ++estIt){
	  createRelation((*estIt).phit->hitb, rseg, &RSEG::hitb );
	  ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back(rh);
	  createRelation((*estIt).phit->hitb, rhit, &HITB::rhit );
	  number_of_rhits++;
	  createRelation( rseg, rhit, &RSEG::rhit );
	  rhit->tx = (*estIt).tx;
	  rhit->ty = (*estIt).ty;
	  rhit->x  = (*estIt).x0;
	  rhit->y  = (*estIt).y0;
	  rhit->p  = rseg->pf;
	  rhit->q  = (*estIt).q;
	}
	rseg->nhit = number_of_rhits; //!!!

	//rp.rseg = rseg;
	rp.xf  = rseg->xf;  rp.yf  = rseg->yf;  rp.zf = rseg->zf; 
	rp.txf = rseg->txf; rp.tyf = rseg->tyf; rp.pf = rseg->pf;
	for (int i=0; i<15; i++) rp.cvf[i] = rseg->cvf[i];
    
	ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rp );
	createRelation( rseg, rpntf, &RSEG::rpnt );

	//rp.rseg = rseg;
	rp.xf  = rseg->xe;  rp.yf  = rseg->ye;  rp.zf = rseg->ze; 
	rp.txf = rseg->txe; rp.tyf = rseg->tye; rp.pf = rseg->pe;
	for (int i=0; i<15; i++) rp.cvf[i] = rseg->cve[i];
    
	ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rp );
	createRelation( rseg, rpnte, &RSEG::rpnt ); 
      }

  zeit_otr_rseg.stop();


  //#ifdef XXX

  
  vector<ArteTable<RSEG>::iterator > vecrsegIt;
  vector<vector<ArteTable<RSEG>::iterator > > vecclones;

  for (ArteTable<RSEG>::iterator rsegIt1 = ArteTable<RSEG>::begin();
       rsegIt1 != ArteTable<RSEG>::end(); rsegIt1++){ 

    if (!((*rsegIt1)->cmp & Rsegc::patt)) continue;
    if (!((*rsegIt1)->cmp & Rsegc::bit16)) continue; //not cats
    if ((*rsegIt1)->cmp & Rsegc::bitsign) continue;  // killed
    if ((*rsegIt1)->cmp & Rsegc::slt) continue;

    ArteTable<RSEG>::iterator clone = 0;
    int nclu_clone = 0;
    ArteTable<RSEG>::iterator rseg_begin = rsegIt1; rseg_begin++;
    for (ArteTable<RSEG>::iterator rsegIt2 = rseg_begin;
	 rsegIt2 != ArteTable<RSEG>::end(); rsegIt2++){ 
      if (!((*rsegIt2)->cmp & Rsegc::patt)) continue;
      if (!((*rsegIt2)->cmp & Rsegc::bit16)) continue; //not cats
      if ((*rsegIt2)->cmp & Rsegc::bitsign) continue;  // killed
      if ((*rsegIt2)->cmp & Rsegc::slt) continue;

#ifdef XXX      
      if  ((((*rsegIt1)->cmp & Rsegc::wch) && ((*rsegIt2)->cmp & Rsegc::wch)) ||
	   (((*rsegIt1)->cmp & Rsegc::msg) && ((*rsegIt2)->cmp & Rsegc::msg))) continue;
#endif //XXX
      //#ifdef XXX
      int ok = 0;

      if (( ((*rsegIt1)->cmp & Rsegc::msg)&&((*rsegIt1)->cmp & Rsegc::wch)) &&       // prolongated
	  (!((*rsegIt2)->cmp & Rsegc::msg)&&((*rsegIt2)->cmp & Rsegc::wch))) ok = 1; // wch

      if (( ((*rsegIt1)->cmp & Rsegc::msg)&&!((*rsegIt1)->cmp & Rsegc::wch)) &&       // msg
	  (!((*rsegIt2)->cmp & Rsegc::msg)&& ((*rsegIt2)->cmp & Rsegc::wch))) ok = 1; // wch

      if ((!((*rsegIt1)->cmp & Rsegc::msg)&&((*rsegIt1)->cmp & Rsegc::wch)) &&       // wch
	  ( ((*rsegIt2)->cmp & Rsegc::msg)&&((*rsegIt2)->cmp & Rsegc::wch))) ok = 1; // prolongated
      
      if ((!((*rsegIt1)->cmp & Rsegc::msg)&& ((*rsegIt1)->cmp & Rsegc::wch)) &&       // wch
	  ( ((*rsegIt2)->cmp & Rsegc::msg)&&!((*rsegIt2)->cmp & Rsegc::wch))) ok = 1; // msg
      
      if (!ok) continue;
      //#endif //XXX

      // check overlaping
      //int ok = (((*rsegIt1)->zf-(*rsegIt2)->ze)>0.0) || (((*rsegIt2)->zf-(*rsegIt1)->ze)>0.0);
      //if (!ok) continue;

      ArtePointer<RSEG> rseg_f = *rsegIt1;
      if ((*rsegIt2)->zf < rseg_f->zf)
	rseg_f = *rsegIt2;
      ArtePointer<RSEG> rseg_e = *rsegIt1;
      if ((*rsegIt2)->ze > rseg_e->ze)
	rseg_e = *rsegIt2;
      
      float ax = (rseg_e->xe-rseg_f->xf)/(rseg_e->ze-rseg_f->zf);
      //float dx = 5.0*1.0;//0.0200;
      float ay = (rseg_e->ye-rseg_f->yf)/(rseg_e->ze-rseg_f->zf);
      //float dy = 5.0*1.0;//0.0200;
      float coeff = 20.0;

      
      if ((fabs(rseg_f->xf+ax*((*rsegIt1)->zf-rseg_f->zf) - (*rsegIt1)->xf)/sqrt((*rsegIt1)->cvf[0]) < coeff ) &&
	  (fabs(rseg_f->xf+ax*((*rsegIt1)->ze-rseg_f->zf) - (*rsegIt1)->xe)/sqrt((*rsegIt1)->cve[0]) < coeff ) && 
	  (fabs(rseg_f->xf+ax*((*rsegIt2)->zf-rseg_f->zf) - (*rsegIt2)->xf)/sqrt((*rsegIt2)->cvf[0]) < coeff ) &&
	  (fabs(rseg_f->xf+ax*((*rsegIt2)->ze-rseg_f->zf) - (*rsegIt2)->xe)/sqrt((*rsegIt2)->cve[0]) < coeff ) &&
	  (fabs(rseg_f->yf+ay*((*rsegIt1)->zf-rseg_f->zf) - (*rsegIt1)->yf)/sqrt((*rsegIt1)->cvf[2]) < coeff ) &&
	  (fabs(rseg_f->yf+ay*((*rsegIt1)->ze-rseg_f->zf) - (*rsegIt1)->ye)/sqrt((*rsegIt1)->cve[2]) < coeff ) &&
	  (fabs(rseg_f->yf+ay*((*rsegIt2)->zf-rseg_f->zf) - (*rsegIt2)->yf)/sqrt((*rsegIt2)->cvf[2]) < coeff ) &&
	  (fabs(rseg_f->yf+ay*((*rsegIt2)->ze-rseg_f->zf) - (*rsegIt2)->ye)/sqrt((*rsegIt2)->cve[2]) < coeff )){
  
  
	//float chi2dist;
	//chi2dist=Chi2DistanceRSEG((*rsegIt1),(*rsegIt2));
  //  if(chi2dist<15.0)
  // {
	(*rsegIt1)->cmp = (*rsegIt1)->cmp | Rsegc::bitsign; // kill first
	(*rsegIt2)->cmp = (*rsegIt2)->cmp | Rsegc::bitsign; // kill second
	int nclu2 = distance((*rsegIt2)->hitb.begin(),(*rsegIt2)->hitb.end());
	if (nclu2 > nclu_clone){
	  clone = rsegIt2;
	  nclu_clone = nclu2;
	}
      }
    }
    if (nclu_clone > 0){
      vecrsegIt.clear();
      vecrsegIt.push_back(rsegIt1);
      vecrsegIt.push_back(clone);
      vecclones.push_back(vecrsegIt);
    }
  }

  for ( vector<vector<ArteTable<RSEG>::iterator> >::iterator cloneIt = vecclones.begin();
	cloneIt != vecclones.end(); cloneIt++){
    ArteTable<RSEG>::iterator rsegIt1 = (*cloneIt)[0];
    ArteTable<RSEG>::iterator rsegIt2 = (*cloneIt)[1];

    RSEG r;
    RHIT rh;
    RPNT rp;
    
    ArtePointer<RSEG> rseg1 = *rsegIt1;
    if ((*rsegIt2)->zf < rseg1->zf){
      rseg1 = *rsegIt2;
    }

    r.xf  = rseg1->xf;  r.yf  = rseg1->yf;  r.zf  = rseg1->zf; 
    r.txf = rseg1->txf; r.tyf = rseg1->tyf;
    
    r.cvf[ 0] = rseg1->cvf[ 0];      r.cvf[ 1] = rseg1->cvf[ 1];
    r.cvf[ 2] = rseg1->cvf[ 2];      r.cvf[ 3] = rseg1->cvf[ 3];
    r.cvf[ 4] = rseg1->cvf[ 4];      r.cvf[ 5] = rseg1->cvf[ 5];
    r.cvf[ 6] = rseg1->cvf[ 6];      r.cvf[ 7] = rseg1->cvf[ 7];
    r.cvf[ 8] = rseg1->cvf[ 8];      r.cvf[ 9] = rseg1->cvf[ 9];
    
    r.cvf[10] = r.cvf[11] = r.cvf[12] = r.cvf[13] = 0.0;
    r.cvf[14] = 100.0;
    
    ArtePointer<RSEG> rseg2 = *rsegIt2;
    if ((*rsegIt1)->ze > rseg2->ze){
      rseg2 = *rsegIt1;
    }

    r.xe  = rseg2->xe;  r.ye  = rseg2->ye;  r.ze  = rseg2->ze; 
    r.txe = rseg2->txe; r.tye = rseg2->tye;

    r.cve[ 0] = rseg2->cve[ 0];      r.cve[ 1] = rseg2->cve[ 1];
    r.cve[ 2] = rseg2->cve[ 2];      r.cve[ 3] = rseg2->cve[ 3];
    r.cve[ 4] = rseg2->cve[ 4];      r.cve[ 5] = rseg2->cve[ 5];
    r.cve[ 6] = rseg2->cve[ 6];      r.cve[ 7] = rseg2->cve[ 7];
    r.cve[ 8] = rseg2->cve[ 8];      r.cve[ 9] = rseg2->cve[ 9];
    
    r.cve[10] = r.cve[11] = r.cve[12] = r.cve[13] = 0.0;
    r.cve[14] = 100.0;
    
    rseg1 = *rsegIt1;
    rseg2 = *rsegIt2;

    r.pf = r.pe = rseg1->pf;
    
    r.time = 0.0; r.dtim = 0.0;
    r.chi  = rseg1->chi+rseg2->chi;
    
    r.fit  = 0;
    r.cmp  = Rsegc::patt | Rsegc::bit16 | Rsegc::wch | Rsegc::msg;
    
    ArteTable<RSEG>::push_back( r );
    ArtePointer<RSEG> rseg( ArteTable<RSEG>::size() );
    
    vector<ArtePointer<HITB> > vhitb;
    for (ArteRelationIterator<HITB> hitbIt = rseg1->hitb.begin();
	 hitbIt != rseg1->hitb.end(); hitbIt++){
      vhitb.push_back(*hitbIt);
    }
    for (ArteRelationIterator<HITB> hitbIt = rseg2->hitb.begin();
	 hitbIt != rseg2->hitb.end(); hitbIt++){
      vhitb.push_back(*hitbIt);
    }
    sort(vhitb.begin(), vhitb.end(), OtrHitbCompareZ);

    for (vector<ArtePointer<HITB> >::reverse_iterator hitbIt = vhitb.rbegin();
	 hitbIt != vhitb.rend(); hitbIt++){
      createRelation(*hitbIt, rseg, &RSEG::hitb );
    }

    vector<ArtePointer<RHIT> > vrhit;
    for (ArteRelationIterator<RHIT> rhitIt = rseg1->rhit.begin();
	 rhitIt != rseg1->rhit.end(); rhitIt++){
      vrhit.push_back(*rhitIt);
    }
    for (ArteRelationIterator<RHIT> rhitIt = rseg2->rhit.begin();
	 rhitIt != rseg2->rhit.end(); rhitIt++){
      vrhit.push_back(*rhitIt);
    }
    sort(vrhit.begin(), vrhit.end(), OtrRhitCompareZ);

    for (vector<ArtePointer<RHIT> >::reverse_iterator rhitIt = vrhit.rbegin();
	 rhitIt != vrhit.rend(); rhitIt++){
      ArtePointer<RHIT> rhit = ArteTable<RHIT>::push_back(rh);
      createRelation((*rhitIt)->hitb, rhit, &HITB::rhit );
      createRelation( rseg, rhit, &RSEG::rhit );
      rhit->tx = (*rhitIt)->tx;
      rhit->ty = (*rhitIt)->ty;
      rhit->x  = (*rhitIt)->x;
      rhit->y  = (*rhitIt)->y;
      rhit->p  = rseg->pf;
      rhit->q  = (*rhitIt)->q;
    }

    rseg->nhit = rseg1->nhit+rseg2->nhit;

    //rp.rseg = rseg;
    rp.xf  = rseg->xf;  rp.yf  = rseg->yf;  rp.zf = rseg->zf; 
    rp.txf = rseg->txf; rp.tyf = rseg->tyf; rp.pf = rseg->pf;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cvf[i];
    
    ArtePointer<RPNT> rpntf = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpntf, &RSEG::rpnt );
    
    //rp.rseg = rseg;
    rp.xf  = rseg->xe;  rp.yf  = rseg->ye;  rp.zf = rseg->ze; 
    rp.txf = rseg->txe; rp.tyf = rseg->tye; rp.pf = rseg->pe;
    for (int i=0; i<15; i++) rp.cvf[i] = rseg->cve[i];
    
    ArtePointer<RPNT> rpnte = ArteTable<RPNT>::push_back( rp );
    createRelation( rseg, rpnte, &RSEG::rpnt ); 
  }

  //#endif


  if (OTR_DRAW){
    for (vector<TRACK>::iterator trackIt = vtrackgood.begin();
	 trackIt != vtrackgood.end(); ++trackIt){
      
      xpl[0] = trackIt->xf+x_shift; ypl[0] = trackIt->yf+y_shift; zpl[0] = trackIt->zf;
      xpl[1] = trackIt->xe+x_shift; ypl[1] = trackIt->ye+y_shift; zpl[1] = trackIt->ze;
      
      isplci(Red);
      //ipl(2, zpl, xpl);
      //ipl(2, zpl, ypl);
    }
  }

  if (OTR_DRAW)
    iuwk(1,1);
  zeit_otr_total.stop();
}


//========================================================

void OtrFindTrack(const pSPACEPOINT psp, TRACK& newtrack, TRACK currenttrack){

  if (psp->level == 1){ // the last sp -> check and store

    if ((currenttrack.nclu > newtrack.nclu)||
	((currenttrack.nclu == newtrack.nclu)&&(currenttrack.prob > newtrack.prob))){

      newtrack.pspointmap.clear();
      for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator pos = currenttrack.pspointmap.begin();
	   pos != currenttrack.pspointmap.end(); pos++){
	newtrack.pspointmap.insert(pair<ArtePointer<GESL>, pSPACEPOINT >
				   (pos->first, pos->second ));
      }
      newtrack.chi2 = currenttrack.chi2;
      newtrack.prob = currenttrack.prob;
      newtrack.nsp  = currenttrack.nsp;
      newtrack.nclu = currenttrack.nclu;
      newtrack.good = currenttrack.good;
    } // store track
  } // hit.level == 1
  else {
    vector<float>::iterator lchiIt = vchi2neighbor[psp->neighborindex].begin();
    for (vector<pSPACEPOINT>::iterator plIt = vpneighbor[psp->neighborindex].begin();
	 plIt != vpneighbor[psp->neighborindex].end(); ++plIt){

      pSPACEPOINT pspl = *plIt;

      int lay1 = 0; int lay3 = 0; int lay5 = 0; // check zero layers
      int lay2 = 0; int lay4 = 0; int lay6 = 0; // check stereo layers
      for (vector<pCLUSTER>::iterator pcluIt = pspl->vpclu.begin();
	   pcluIt != pspl->vpclu.end(); ++pcluIt){
	if (!(*pcluIt)->used){
	  switch ((*pcluIt)->lay){
	  case 1: {lay1 = 1; break;}
	  case 2: {lay2 = 1; break;}
	  case 3: {lay3 = 1; break;}
	  case 4: {lay4 = 1; break;}
	  case 5: {lay5 = 1; break;}
	  case 6: {lay6 = 1; break;}
	  }
	}
      }
      if (
	  (lay2+lay4+lay6 <= 1)
	  //||(lay1+lay3+lay5 < 2)
	  ){  // used space point

	pspl->used = 1;
	//compare with the original track:
	if ((currenttrack.nclu > newtrack.nclu)||
	    ((currenttrack.nclu == newtrack.nclu)&&(currenttrack.prob > newtrack.prob))){
	  newtrack.pspointmap.clear();
	  for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator pos = currenttrack.pspointmap.begin();
	       pos != currenttrack.pspointmap.end(); pos++){
	    newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
				       (pos->first, pos->second ));
	  }
	  newtrack.chi2 = currenttrack.chi2;
	  newtrack.prob = currenttrack.prob;
	  newtrack.nsp  = currenttrack.nsp;
	  newtrack.nclu = currenttrack.nclu;
	  newtrack.good = 1;
	} // store track
      }
      else{//else

	// store the current track for a case of bad prolongation
	TRACK oldtrack;
	oldtrack.pspointmap.clear();
	for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator pos = currenttrack.pspointmap.begin();
	     pos != currenttrack.pspointmap.end(); pos++){
	  oldtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
				     (pos->first, pos->second ));
	}
	oldtrack.chi2 = currenttrack.chi2;
	oldtrack.prob = currenttrack.prob;
	oldtrack.nsp  = currenttrack.nsp;
	oldtrack.nclu = currenttrack.nclu;
	oldtrack.good = 1;
	// stored
	
      	currenttrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
				       ((*(pspl->vpclu.begin()))->gede->gesl, pspl));
	//currenttrack.chi2 += (*lchiIt);
	
	// fit current track as space point to calculate chi2 and prob      
	SPACEPOINT sp;
	pSPACEPOINT psp, pspf, pspe;
	psp  = &sp;
	pspf = (currenttrack.pspointmap.begin())->second;
	pspe = (currenttrack.pspointmap.begin())->second;
	
	psp->vpclu.clear();
	for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator spIt = currenttrack.pspointmap.begin();
	     spIt != currenttrack.pspointmap.end(); spIt++){
	  
	  if ((spIt->second)->zf < pspf->zf) pspf = spIt->second;
	  if ((spIt->second)->ze > pspe->ze) pspe = spIt->second;
	  
	  for (vector<pCLUSTER>::iterator pcluIt = (spIt->second)->vpclu.begin();
	       pcluIt != (spIt->second)->vpclu.end(); ++pcluIt){
	    psp->vpclu.push_back(*pcluIt);
	  }
	}
	
	psp->xf  = pspf->xf;   psp->txf = pspf->txf;
	psp->yf  = pspf->yf;   psp->tyf = pspf->tyf; 
	psp->zf  = pspf->zf;
	psp->xe  = pspe->xe;   psp->txe = pspe->txe;
	psp->ye  = pspe->ye;   psp->tye = pspe->tye;
	psp->ze  = pspe->ze;
	psp->used = 0; psp->level = 0; psp->neighborindex = -1;
	
	sort(psp->vpclu.begin(), psp->vpclu.end(), OtrClusterCompareZ);
	
	OtrFitSPoint(*psp);
	// fitted
	
	currenttrack.chi2 = psp->chi2;
	currenttrack.prob = psp->prob;
	currenttrack.nsp  = distance(currenttrack.pspointmap.begin(), currenttrack.pspointmap.end());
	currenttrack.nclu = distance(psp->vpclu.begin(), psp->vpclu.end());
	
	if (psp->prob > 0.1){
	  OtrFindTrack(pspl, newtrack, currenttrack);
	}
	else{// compare and store old track as new one
	  if ((oldtrack.nclu > newtrack.nclu)||
	      ((oldtrack.nclu == newtrack.nclu)&&(oldtrack.prob > newtrack.prob))){
	    newtrack.pspointmap.clear();
	    for (map<ArtePointer<GESL>, pSPACEPOINT >::iterator pos = oldtrack.pspointmap.begin();
		 pos != oldtrack.pspointmap.end(); pos++){
	      newtrack.pspointmap.insert(pair< ArtePointer<GESL>, pSPACEPOINT >
					 (pos->first, pos->second ));
	    }
	    newtrack.chi2 = oldtrack.chi2;
	    newtrack.prob = oldtrack.prob;
	    newtrack.nsp  = oldtrack.nsp;
	    newtrack.nclu = oldtrack.nclu;
	    newtrack.good = 1;
	  }
	}
      }
      
      ++lchiIt;
    }
  }
  return;
}


//========================================================

float OtrNeighborsChi2(const pSPACEPOINT sp1, const pSPACEPOINT sp2){
  float ax, ay, bx, by;
  float G[4][4];
  float chi2 = 0.0;
  float sum  = sp1->zf + sp2->zf;
  float sum2 = sum*sum;

  ax = 0.5*(sp1->txf + sp2->txf); ay = 0.5*(sp1->tyf + sp2->tyf);
  bx = 0.5*(sp1->xf  + sp2->xf) - 0.5*ax*sum;
  by = 0.5*(sp1->yf  + sp2->yf) - 0.5*ay*sum;
  G[2][2] = 0.25*(sp1->cvf[2][2] + sp2->cvf[2][2]);
  G[3][3] = 0.25*(sp1->cvf[3][3] + sp2->cvf[3][3]);
  G[0][0] = 0.25*(sp1->cvf[0][0] + sp2->cvf[0][0]) + 0.25*sum2*G[2][2];
  G[1][1] = 0.25*(sp1->cvf[1][1] + sp2->cvf[1][1]) + 0.25*sum2*G[3][3];
  chi2 += (ax - sp1->txf)*(ax - sp1->txf)/(G[2][2] + sp1->cvf[2][2]);
  chi2 += (ax - sp2->txf)*(ax - sp2->txf)/(G[2][2] + sp2->cvf[2][2]);
  chi2 += (ay - sp1->tyf)*(ay - sp1->tyf)/(G[3][3] + sp1->cvf[3][3]);
  chi2 += (ay - sp2->tyf)*(ay - sp2->tyf)/(G[3][3] + sp2->cvf[3][3]);
  chi2 += (bx + ax*sp1->zf - sp1->xf)*(bx + ax*sp1->zf - sp1->xf)/(G[0][0] + sp1->cvf[0][0]);
  chi2 += (bx + ax*sp2->zf - sp2->xf)*(bx + ax*sp2->zf - sp2->xf)/(G[0][0] + sp2->cvf[0][0]);
  chi2 += (by + ay*sp1->zf - sp1->yf)*(by + ay*sp1->zf - sp1->yf)/(G[1][1] + sp1->cvf[1][1]);
  chi2 += (by + ay*sp2->zf - sp2->yf)*(by + ay*sp2->zf - sp2->yf)/(G[1][1] + sp2->cvf[1][1]);

  return chi2;
}

//========================================================

void vier_dim_inv(double a[4][4])
{
  /**** Gaussian algorithm for 4x4 matrix inversion ****/
  int i,j,k,l;
  double factor;
  double temp[4];
  double b[4][4];

  // Set b to I
  for(i=0;i<4;i++) for(j=0;j<4;j++)
    if(i==j) b[i][j]=1.0; else b[i][j]=0.0;

  for(i=0;i<4;i++){
    for(j=i+1;j<4;j++)
      if (fabs(a[i][i])<fabs(a[j][i])){
        for(l=0;l<4;l++) temp[l]=a[i][l];
        for(l=0;l<4;l++) a[i][l]=a[j][l];
        for(l=0;l<4;l++) a[j][l]=temp[l];
        for(l=0;l<4;l++) temp[l]=b[i][l];
        for(l=0;l<4;l++) b[i][l]=b[j][l];
        for(l=0;l<4;l++) b[j][l]=temp[l];
      }
    factor=1.0/a[i][i];
    for(j=4-1;j>-1;j--) {
      b[i][j]*=factor;a[i][j]*=factor;
    }
    for(j=i+1;j<4;j++) {
      factor=-a[j][i];
      for(k=0;k<4;k++){
        a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
      }
    }
  } 
  for(i=3;i>0;i--)
    {
      for(j=i-1;j>-1;j--)
	{
	  factor=-a[j][i];
	  for(k=0;k<4;k++)
	    {
	      a[j][k]+=a[i][k]*factor;b[j][k]+=b[i][k]*factor;
	    }
	}
    }
  for(i=0;i<4;i++) for(j=0;j<4;j++) a[i][j]=b[i][j];
}

typedef struct{
  float x,z,q,sina,cosa,sigma_m2,chi2,maxdrift;
  double Gk[4][4];
  double Rk[4];
  int mask,LR;
  vector<pHIT>::iterator ihit;
} OTRHIT, *pOTRHIT;

void OtrFastDriftRefitTrack(pTRACK ptrack){

  const double chi2cut=10.0;

  double h[4],Rk[4],Gk[4][4],Kk[4],L[4][4],CF[4][4],L_1[4][4],A[4][4],dG[4][4];
  int index,numhits,nonmask,nchanges,newLR,nstereo=0,nzero=0,n0;
  double zprev, Z, sina, cosa, zeta_k, D_z, f_k, theta_k, drift_dist,tu_k,Du_k;
  double chi2t=0.0,l2sqrt,q33,q34,q44,pinv,coeff;
  double sigma_ms = 0.9e-6; // 1.2 multiple scattering corrections 

  OTRHIT ar;
  OTRHIT arr[200];
  map<int, int> zerolayermap, stereolayermap;

  ar.z=0.0;ar.x=0.0;ar.sina=0.707;ar.cosa=sqrt(1.0-ar.sina*ar.sina);
  ar.q=0.0;ar.mask=10;ar.maxdrift=10000.0;
  ar.ihit=NULL;ar.LR=1;ar.sigma_m2=0.25;
  arr[0]=ar;
  ar.sina=-ar.sina;
  arr[1]=ar;

  index=2;nonmask=0;nstereo=0;nzero=0;
  for(vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
      phitIt != ptrack->vphit.end(); ++phitIt){
   
    ArtePointer<GEDE> gede = (*phitIt)->hitb->gede;
    ar.sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
    ar.cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
    ar.z=(*phitIt)->z;ar.x=(*phitIt)->x;ar.q=(*phitIt)->q;
    ar.maxdrift=gede->pit/(2.0*sqrt(3.0))+3.0*(*phitIt)->r;
    if(gede->cmp==3) ar.maxdrift=10000.0;
    ar.ihit=phitIt;
    if ((*phitIt)->mask>0.0){ 
      ar.mask=1;
      nonmask++;
      if(fabs(ar.sina)<0.03) nzero++;else nstereo++;
    }
    else 
      ar.mask=0;
    if ((*phitIt)->lr> 0.0) ar.LR=1;
    if ((*phitIt)->lr<=0.0) ar.LR=-1;
    ar.sigma_m2 = (*phitIt)->r*(*phitIt)->r;
    arr[index]=ar;
    index++;
  }
  numhits = index;
 
  nchanges=0;

  int nzerolayers = 0, nstereolayers = 0;
  zerolayermap.clear(); stereolayermap.clear();

  if(nstereo<3) n0=0;
  else n0=2;
  arr[n0].Rk[0] = ptrack->xf+ptrack->txf*(arr[n0].z-ptrack->zf); 
  arr[n0].Rk[1] = ptrack->yf+ptrack->tyf*(arr[n0].z-ptrack->zf);
  arr[n0].Rk[2] = ptrack->txf;
  arr[n0].Rk[3] = ptrack->tyf;

  if(nonmask>4) 
    for(int iter=0;iter<2;iter++){
      for(int i=0;i<4;i++){
	Rk[i]=arr[n0].Rk[i];
	for (int j=0;j<=i;j++)
	  Gk[j][i]=0.0;
      }
      
      Gk[0][0]=4.0;Gk[1][1]=25.0;Gk[2][2]=0.1;Gk[3][3]=1.0;

      pinv=1.0/(B*zmagn*sqrt(1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3])/
		((Rk[0]-ptrack->zf*Rk[2])*sqrt(1.0+Rk[2]*Rk[2])));
      int sign1;
      if(pinv<0.0) sign1=-1;else sign1=1;
      pinv = fabs(pinv);
      if (pinv > 0.5) pinv = 2.0*pinv*pinv;
      coeff=(1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
      l2sqrt=1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3]; 
      l2sqrt=l2sqrt*sqrt(l2sqrt); ptrack->pf=ptrack->pe=pinv*sign1;
      
      q33=sigma_ms*coeff*l2sqrt*(1.0+Rk[2]*Rk[2]); 
      q44=sigma_ms*coeff*l2sqrt*(1.0+Rk[3]*Rk[3]); 
      q34=sigma_ms*coeff*l2sqrt*Rk[2]*Rk[3];
      
      zprev=arr[n0].z; // start from the first hit
      for(int k=n0; k<numhits; k++){
	sina  = arr[k].sina; 
	cosa  = arr[k].cosa;
	D_z   = arr[k].z-zprev;
	zprev = arr[k].z;
	Rk[0]+=D_z*Rk[2];
	Rk[1]+=D_z*Rk[3];
	Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
	Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
	Gk[0][2]+=D_z*Gk[2][2];
	Gk[0][3]+=D_z*Gk[2][3];
	Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
	Gk[1][2]+=D_z*Gk[2][3];
	Gk[1][3]+=D_z*Gk[3][3];
	if(arr[k].mask>0.0){
	  tu_k=Rk[2]*cosa-Rk[3]*sina;
	  theta_k=1.0*arr[k].LR/sqrt(1.0+tu_k*tu_k);
	  Du_k=Rk[0]*cosa-Rk[1]*sina-arr[k].x;
	  f_k=Du_k*tu_k*theta_k*theta_k;
	  h[0]= cosa*theta_k;
	  h[1]=-sina*theta_k;
	  h[2]=-h[0]*f_k;
	  h[3]= h[1]*f_k;
	  Kk[0]=h[0]*Gk[0][0]+h[1]*Gk[0][1]+h[2]*Gk[0][2]+h[3]*Gk[0][3];
	  Kk[1]=h[0]*Gk[0][1]+h[1]*Gk[1][1]+h[2]*Gk[1][2]+h[3]*Gk[1][3];
	  Kk[2]=h[0]*Gk[0][2]+h[1]*Gk[1][2]+h[2]*Gk[2][2]+h[3]*Gk[2][3];
	  Kk[3]=h[0]*Gk[0][3]+h[1]*Gk[1][3]+h[2]*Gk[2][3]+h[3]*Gk[3][3];
	  Z=arr[k].sigma_m2;for(int i=0;i<4;i++) Z+=h[i]*Kk[i];
	  zeta_k=arr[k].q-Du_k*theta_k;
	  Z=1.0/Z;zeta_k=Z*zeta_k;
	  for(int i=0;i<4;i++){
	    Rk[i]+=Kk[i]*zeta_k;
	    for(int j=0;j<=i;j++) 
	      Gk[j][i]-=Z*Kk[j]*Kk[i];
	  }
	}
	for(int i=0;i<4;i++){
	  arr[k].Rk[i]=Rk[i];
	  for(int j=0;j<=i;j++) 
	    arr[k].Gk[j][i]=Gk[j][i];
	}
	Gk[2][2]+=q33;Gk[3][3]+=q44;Gk[2][3]+=q34;
      }
      zprev=arr[numhits-1].z;chi2t=0.0;nonmask=0;
      nstereo=0;nzero=0;
      zerolayermap.clear(); stereolayermap.clear();

      for(int k=numhits-1;k>=n0;k--){
	D_z=zprev-arr[k].z;
	if(k<numhits-1){
	  L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][1]+
				       D_z*arr[k].Gk[1][1]);
	  L[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][1];L[1][0]=L[0][1];
	  L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][2]+
				       arr[k].Gk[2][2]*D_z);
	  L[0][1]=arr[k].Gk[0][1]+D_z*(arr[k].Gk[0][3]+arr[k].Gk[1][2]+
				       D_z*arr[k].Gk[2][3]);L[1][0]=L[0][1];
	  L[0][2]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];L[2][0]=L[0][2];
	  L[0][3]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];L[3][0]=L[0][3];
	  L[1][1]=arr[k].Gk[1][1]+D_z*(2.0*arr[k].Gk[1][3]+arr[k].Gk[3][3]*D_z);
	  L[1][2]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];L[2][1]=L[1][2];
	  L[1][3]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];L[3][1]=L[1][3];
	  
	  L[2][2]=arr[k].Gk[2][2]+q33;
	  L[3][3]=arr[k].Gk[3][3]+q44;
	  L[2][3]=arr[k].Gk[2][3]+q34;L[3][2]=L[2][3];
	  
	  for(int v=0;v<4;v++) 
	    for(int vv=0;vv<4;vv++) 
	      L_1[v][vv]=L[v][vv];

	  vier_dim_inv(L_1);
	  
	  CF[0][0]=arr[k].Gk[0][0]+D_z*arr[k].Gk[0][2];
	  CF[1][0]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][2];
	  CF[2][0]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];
	  CF[3][0]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];
	  CF[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[0][3];
	  CF[1][1]=arr[k].Gk[1][1]+D_z*arr[k].Gk[1][3];
	  CF[2][1]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];
	  CF[3][1]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];
	  CF[0][2]=arr[k].Gk[0][2];CF[0][3]=arr[k].Gk[0][3];
	  CF[1][2]=arr[k].Gk[1][2];CF[1][3]=arr[k].Gk[1][3];
	  CF[2][2]=arr[k].Gk[2][2];CF[2][3]=arr[k].Gk[2][3];
	  CF[3][2]=arr[k].Gk[2][3];CF[3][3]=arr[k].Gk[3][3];
	  for(int i=0;i<4;i++) 
	    for(int j=0;j<4;j++){
	      A[i][j]=0.0;
	      for(int m=0;m<4;m++) 
		A[i][j]+=CF[i][m]*L_1[m][j];
	    }
	  h[0]=arr[k+1].Rk[0]-arr[k].Rk[0]-D_z*arr[k].Rk[2];
	  h[1]=arr[k+1].Rk[1]-arr[k].Rk[1]-D_z*arr[k].Rk[3];
	  h[2]=arr[k+1].Rk[2]-arr[k].Rk[2];
	  h[3]=arr[k+1].Rk[3]-arr[k].Rk[3];
	  for(int i=0;i<4;i++)
	    for(int j=0;j<4;j++) 
	      arr[k].Rk[i]+=A[i][j]*h[j];
	  
	  for(int i=0;i<4;i++) 
	    for(int j=0;j<=i;j++) 
	      dG[j][i]=arr[k+1].Gk[j][i]-L[j][i];
	      
	  CF[0][0]=A[0][0]*dG[0][0]+A[0][1]*dG[0][1]+A[0][2]*dG[0][2]+A[0][3]*dG[0][3];
	  CF[0][1]=A[0][0]*dG[0][1]+A[0][1]*dG[1][1]+A[0][2]*dG[1][2]+A[0][3]*dG[1][3];
	  CF[0][2]=A[0][0]*dG[0][2]+A[0][1]*dG[1][2]+A[0][2]*dG[2][2]+A[0][3]*dG[2][3];
	  CF[0][3]=A[0][0]*dG[0][3]+A[0][1]*dG[1][3]+A[0][2]*dG[2][3]+A[0][3]*dG[3][3];
	  CF[1][0]=A[1][0]*dG[0][0]+A[1][1]*dG[0][1]+A[1][2]*dG[0][2]+A[1][3]*dG[0][3];
	  CF[1][1]=A[1][0]*dG[0][1]+A[1][1]*dG[1][1]+A[1][2]*dG[1][2]+A[1][3]*dG[1][3];
	  CF[1][2]=A[1][0]*dG[0][2]+A[1][1]*dG[1][2]+A[1][2]*dG[2][2]+A[1][3]*dG[2][3];
	  CF[1][3]=A[1][0]*dG[0][3]+A[1][1]*dG[1][3]+A[1][2]*dG[2][3]+A[1][3]*dG[3][3];
	  CF[2][0]=A[2][0]*dG[0][0]+A[2][1]*dG[0][1]+A[2][2]*dG[0][2]+A[2][3]*dG[0][3];
	  CF[2][1]=A[2][0]*dG[0][1]+A[2][1]*dG[1][1]+A[2][2]*dG[1][2]+A[2][3]*dG[1][3];
	  CF[2][2]=A[2][0]*dG[0][2]+A[2][1]*dG[1][2]+A[2][2]*dG[2][2]+A[2][3]*dG[2][3];
	  CF[2][3]=A[2][0]*dG[0][3]+A[2][1]*dG[1][3]+A[2][2]*dG[2][3]+A[2][3]*dG[3][3];
	  CF[3][0]=A[3][0]*dG[0][0]+A[3][1]*dG[0][1]+A[3][2]*dG[0][2]+A[3][3]*dG[0][3];
	  CF[3][1]=A[3][0]*dG[0][1]+A[3][1]*dG[1][1]+A[3][2]*dG[1][2]+A[3][3]*dG[1][3];
	  CF[3][2]=A[3][0]*dG[0][2]+A[3][1]*dG[1][2]+A[3][2]*dG[2][2]+A[3][3]*dG[2][3];
	  CF[3][3]=A[3][0]*dG[0][3]+A[3][1]*dG[1][3]+A[3][2]*dG[2][3]+A[3][3]*dG[3][3];
	  for(int i=0;i<4;i++) 
	    for(int j=0;j<=i;j++)
	      for(int m=0;m<4;m++) 
		arr[k].Gk[j][i]+=CF[j][m]*A[i][m];
	}
	tu_k=(arr[k].cosa*arr[k].Rk[2]-arr[k].sina*arr[k].Rk[3]);      
	Du_k=arr[k].cosa*arr[k].Rk[0]-arr[k].sina*arr[k].Rk[1]-arr[k].x;
	theta_k=1.0/sqrt(1.0+tu_k*tu_k);
	drift_dist=Du_k*theta_k;
	newLR=(drift_dist<0.0)?-1:1;
	if(newLR!=arr[k].LR){
	  nchanges++;
	  arr[k].LR=newLR;
	}
	zeta_k=arr[k].q*arr[k].LR-drift_dist;
	theta_k*=arr[k].LR;
	f_k=tu_k*Du_k*theta_k*theta_k;
	h[0]= theta_k*arr[k].cosa;
	h[1]=-theta_k*arr[k].sina;
	h[2]=-h[0]*f_k;
	h[3]= h[1]*f_k;
	Kk[0]=h[0]*arr[k].Gk[0][0]+h[1]*arr[k].Gk[0][1]+h[2]*arr[k].Gk[0][2]+
	  h[3]*arr[k].Gk[0][3];
	Kk[1]=h[0]*arr[k].Gk[0][1]+h[1]*arr[k].Gk[1][1]+h[2]*arr[k].Gk[1][2]+
	  h[3]*arr[k].Gk[1][3];
	Kk[2]=h[0]*arr[k].Gk[0][2]+h[1]*arr[k].Gk[1][2]+h[2]*arr[k].Gk[2][2]+
	  h[3]*arr[k].Gk[2][3];
	Kk[3]=h[0]*arr[k].Gk[0][3]+h[1]*arr[k].Gk[1][3]+h[2]*arr[k].Gk[2][3]+
	  h[3]*arr[k].Gk[3][3];
	Z=arr[k].sigma_m2;for(int i=0;i<4;i++) Z+=h[i]*Kk[i];
	arr[k].chi2=zeta_k*zeta_k/Z;

	if(arr[k].mask!=10) {
	if ((arr[k].chi2<chi2cut)&&(fabs(drift_dist)<arr[k].maxdrift)){
	  chi2t+=arr[k].chi2;
	  
	  (*arr[k].ihit)->mask=1;
	  if(!arr[k].mask) nchanges++;
	  arr[k].mask=1;
	  (*arr[k].ihit)->x0=arr[k].Rk[0];
	  (*arr[k].ihit)->y0=arr[k].Rk[1];
	  (*arr[k].ihit)->tx=arr[k].Rk[2];
	  (*arr[k].ihit)->ty=arr[k].Rk[3];
	  (*arr[k].ihit)->q=arr[k].q*arr[k].LR;
	  nonmask++;
	  if(fabs(arr[k].sina)<0.001){ 
	    nzero++;
	    int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+
	      (*arr[k].ihit)->hitb->gede->lay;
	    zerolayermap.insert(pair<int, int>(lay, 1));
	  }
	  else{
	    nstereo++;
	    int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+
	      (*arr[k].ihit)->hitb->gede->lay;
	    stereolayermap.insert(pair<int, int>(lay, 1));
	  }
	}
	else{
	  if(arr[k].mask) nchanges++;
	  arr[k].mask=0;(*arr[k].ihit)->mask=0;
	}
	zprev=arr[k].z;
	}
      }
      if(!nchanges) break;
    }

  ptrack->nonmask =  0;
  int first_index =  0;
  int last_index  = -1;

  nzerolayers   = distance(zerolayermap.begin(), zerolayermap.end());
  nstereolayers = distance(stereolayermap.begin(), stereolayermap.end());
  zerolayermap.clear(); stereolayermap.clear();
  if( (nzerolayers>=3) && (nstereolayers>=3) ){ //3x2
    ptrack->nonmask=nonmask;
    
    int first_flag=1;
    for (int k=n0; k<numhits; k++){
      if ((arr[k].mask == 0) || (arr[k].mask==10)) continue;
      
      last_index=k;
      if(first_flag){
	first_flag  = 0;
	first_index  = k;
	ptrack->xf  = arr[k].Rk[0];          
	ptrack->yf  = arr[k].Rk[1];
	ptrack->zf  = arr[k].z;
	ptrack->txf = arr[k].Rk[2];          
	ptrack->tyf = arr[k].Rk[3];
	for(int i=0;i<4;i++) 
	  for(int j=0;j<=i;j++) 
	    ptrack->cvf[j][i] = arr[k].Gk[j][i];    
      }
    }
    if (last_index != -1){
      ptrack->xe  = arr[last_index].Rk[0];          
      ptrack->ye  = arr[last_index].Rk[1];
      ptrack->ze  = arr[last_index].z;
      ptrack->txe = arr[last_index].Rk[2];          
      ptrack->tye = arr[last_index].Rk[3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  ptrack->cve[j][i] = arr[last_index].Gk[j][i]; 
      
    }

    chi2t = 0.0;nzero=0;nstereo=0;
    for (int k=first_index; k<=last_index; k++){
      if (arr[k].mask==0) continue;
      if (arr[k].mask==10) continue;
      chi2t+=arr[k].chi2; 
      if(fabs(arr[k].sina)<0.01){ 
	nzero++;
	int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+
	  (*arr[k].ihit)->hitb->gede->lay;
	zerolayermap.insert(pair<int, int>(lay, 1));
      }
      else{
	nstereo++;
	int lay = 10*((*arr[k].ihit)->hitb->gede->gesl.index())+
	  (*arr[k].ihit)->hitb->gede->lay;
	stereolayermap.insert(pair<int, int>(lay, 1));
      }
    }
        
    nzerolayers   = distance(zerolayermap.begin(), zerolayermap.end());
    nstereolayers = distance(stereolayermap.begin(), stereolayermap.end());
    if( (nzerolayers<2) || (nstereolayers<2) ) ptrack->nonmask = 0;

    ptrack->chi2 = chi2t;
    if (nzero + nstereo > 4)
      ptrack->prob = prob_(chi2t,(nzero + nstereo - 4));
    else
      ptrack->prob = 0.001;
    
    //   if (((nzerolayers+nstereolayers<=6)||(ptrack->pf>1.0))&&(ptrack->prob<0.002)) ptrack->nonmask = 0;
}
  
  if (ptrack->nonmask != 0)
    {
      if ((ptrack->cvf[0][0]< 0.0)||(ptrack->cvf[1][1]< 0.0)||
	  (ptrack->cvf[2][2]< 0.0)||(ptrack->cvf[3][3]< 0.0)){
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------> OtrCats bad cvf matrix "<<ptrack->cvf[0][0]<<" "<<
	  ptrack->cvf[1][1]<<" "<<ptrack->cvf[2][2]<<" "<<ptrack->cvf[3][3]<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
      }
      if ((ptrack->cve[0][0]< 0.0)||(ptrack->cve[1][1]< 0.0)||
	  (ptrack->cve[2][2]< 0.0)||(ptrack->cve[3][3]< 0.0)){
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------> OtrCats bad cve matrix "<<ptrack->cve[0][0]<<" "<<
	  ptrack->cve[1][1]<<" "<<ptrack->cve[2][2]<<" "<<ptrack->cve[3][3]<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
      }

      EST est;
      ptrack->vest.clear();
      vector<pHIT> htmp;
      htmp.clear();
      for (int k=first_index; k<=last_index; k++){
	if ((arr[k].mask == 0) || (arr[k].mask==10)) continue;
	est.q=(*arr[k].ihit)->q;est.phit=(*arr[k].ihit);
	est.x0=(*arr[k].ihit)->x0;est.y0=(*arr[k].ihit)->y0;
	est.tx=(*arr[k].ihit)->tx;est.ty=(*arr[k].ihit)->ty;
	ptrack->vest.push_back(est);
	htmp.push_back((*arr[k].ihit));
      }
      ptrack->vphit.clear();
      for(vector<pHIT>::iterator phitIt=htmp.begin();phitIt!=htmp.end(); ++phitIt)
	ptrack->vphit.push_back((*phitIt));
    }
  else{
    for(vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
	phitIt != ptrack->vphit.end(); ++phitIt)
      (*phitIt)->mask = 0.0;
  }
  
  return;   
}

//========================================================

typedef struct{
  float x,z,sina,cosa,sigma_m2,chi2;
  double Gk[4][4];
  double Rk[4];
  int mask;
} FHIT, *pFHIT;

void OtrKalmanFilterSmoother(FHIT arr[], int numhits, float sigma_ms)
{
  double D_z,zprev,sina,cosa,l2sqrt,coeff,q33,q34,q44,zeta_k,Z,pinv;
  double h[4],Rk[4],Gk[4][4],Kk[4],L[4][4],CF[4][4],L_1[4][4],A[4][4],dG[4][4];

  for(int i=0;i<4;i++){
    Rk[i]=arr[0].Rk[i];
    for (int j=0;j<4;j++)
      Gk[j][i]=0.0;
  }
  Gk[0][0]=1.0;Gk[1][1]=4.0;Gk[2][2]=0.1;Gk[3][3]=0.1;
  zprev=arr[0].z; // start from the first hit
  pinv=1.0/(B*zmagn*sqrt(1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3])/
	    ((Rk[0]-zprev*Rk[2])*sqrt(1.0+Rk[2]*Rk[2])));
  pinv = fabs(pinv);
  if (pinv > 0.5) pinv = 2.0*pinv*pinv;

  coeff=(1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
  l2sqrt=1.0+Rk[2]*Rk[2]+Rk[3]*Rk[3]; 
  l2sqrt=l2sqrt*sqrt(l2sqrt);
  q33=sigma_ms*coeff*l2sqrt*(1.0+Rk[2]*Rk[2]); 
  q44=sigma_ms*coeff*l2sqrt*(1.0+Rk[3]*Rk[3]); 
  q34=sigma_ms*coeff*l2sqrt*Rk[2]*Rk[3];
  for(int k=0; k<numhits; k++){
    sina  = arr[k].sina; 
    cosa  = arr[k].cosa;
    D_z   = arr[k].z-zprev;
    zprev = arr[k].z;
    Rk[0]+=D_z*Rk[2];
    Rk[1]+=D_z*Rk[3];
    Gk[0][0]+=D_z*(2.0*Gk[0][2]+D_z*Gk[2][2]);
    Gk[0][1]+=D_z*(Gk[1][2]+Gk[0][3]+D_z*Gk[2][3]);
    Gk[0][2]+=D_z*Gk[2][2];
    Gk[0][3]+=D_z*Gk[2][3];
    Gk[1][1]+=D_z*(2.0*Gk[1][3]+D_z*Gk[3][3]);
    Gk[1][2]+=D_z*Gk[2][3];
    Gk[1][3]+=D_z*Gk[3][3];
    if(arr[k].mask>0){
      Z=arr[k].sigma_m2+cosa*cosa*Gk[0][0]-2.0*sina*cosa*Gk[0][1]+sina*sina*Gk[1][1];
      zeta_k=arr[k].x-cosa*Rk[0]+sina*Rk[1];
      Kk[0]=cosa*Gk[0][0]-sina*Gk[0][1];
      Kk[1]=cosa*Gk[0][1]-sina*Gk[1][1];
      Kk[2]=cosa*Gk[0][2]-sina*Gk[1][2];
      Kk[3]=cosa*Gk[0][3]-sina*Gk[1][3];
      zeta_k=zeta_k/Z;
      for(int i=0;i<4;i++){
	Rk[i]+=Kk[i]*zeta_k;
	for(int j=0;j<=i;j++)
	  Gk[j][i]-=Kk[i]*Kk[j]/Z;
      }
    }
    //	cout<<k<<" ";
    for(int i=0;i<4;i++){
      arr[k].Rk[i]=Rk[i];
      //	  cout<<Rk[i]<<" ";
      
      for(int j=0;j<=i;j++) 
	arr[k].Gk[j][i]=Gk[j][i];
    }
    //	cout<<" mask="<<arr[k].mask<<endl;
    Gk[2][2]+=q33;Gk[3][3]+=q44;Gk[2][3]+=q34;    
  }
  zprev=arr[numhits-1].z;
  
  for(int k=numhits-1;k>=0;k--){
    D_z=zprev-arr[k].z;
    if(k<numhits-1){
      L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][1]+
				   D_z*arr[k].Gk[1][1]);
      L[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][1];L[1][0]=L[0][1];
      L[0][0]=arr[k].Gk[0][0]+D_z*(2.0*arr[k].Gk[0][2]+
				   arr[k].Gk[2][2]*D_z);
      L[0][1]=arr[k].Gk[0][1]+D_z*(arr[k].Gk[0][3]+arr[k].Gk[1][2]+
				   D_z*arr[k].Gk[2][3]);L[1][0]=L[0][1];
      L[0][2]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];L[2][0]=L[0][2];
      L[0][3]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];L[3][0]=L[0][3];
      L[1][1]=arr[k].Gk[1][1]+D_z*(2.0*arr[k].Gk[1][3]+arr[k].Gk[3][3]*D_z);
      L[1][2]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];L[2][1]=L[1][2];
      L[1][3]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];L[3][1]=L[1][3];
      
      L[2][2]=arr[k].Gk[2][2]+q33;
      L[3][3]=arr[k].Gk[3][3]+q44;
      L[2][3]=arr[k].Gk[2][3]+q34;L[3][2]=L[2][3];
	  
      for(int v=0;v<4;v++) 
	for(int vv=0;vv<4;vv++) 
	  L_1[v][vv]=L[v][vv];
      
      vier_dim_inv(L_1);
      
      CF[0][0]=arr[k].Gk[0][0]+D_z*arr[k].Gk[0][2];
      CF[1][0]=arr[k].Gk[0][1]+D_z*arr[k].Gk[1][2];
      CF[2][0]=arr[k].Gk[0][2]+D_z*arr[k].Gk[2][2];
      CF[3][0]=arr[k].Gk[0][3]+D_z*arr[k].Gk[2][3];
      CF[0][1]=arr[k].Gk[0][1]+D_z*arr[k].Gk[0][3];
      CF[1][1]=arr[k].Gk[1][1]+D_z*arr[k].Gk[1][3];
      CF[2][1]=arr[k].Gk[1][2]+D_z*arr[k].Gk[2][3];
      CF[3][1]=arr[k].Gk[1][3]+D_z*arr[k].Gk[3][3];
      CF[0][2]=arr[k].Gk[0][2];CF[0][3]=arr[k].Gk[0][3];
      CF[1][2]=arr[k].Gk[1][2];CF[1][3]=arr[k].Gk[1][3];
      CF[2][2]=arr[k].Gk[2][2];CF[2][3]=arr[k].Gk[2][3];
      CF[3][2]=arr[k].Gk[2][3];CF[3][3]=arr[k].Gk[3][3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<4;j++){
	  A[i][j]=0.0;
	  for(int m=0;m<4;m++) 
		A[i][j]+=CF[i][m]*L_1[m][j];
	}
      h[0]=arr[k+1].Rk[0]-arr[k].Rk[0]-D_z*arr[k].Rk[2];
      h[1]=arr[k+1].Rk[1]-arr[k].Rk[1]-D_z*arr[k].Rk[3];
      h[2]=arr[k+1].Rk[2]-arr[k].Rk[2];
      h[3]=arr[k+1].Rk[3]-arr[k].Rk[3];
      for(int i=0;i<4;i++)
	for(int j=0;j<4;j++) 
	  arr[k].Rk[i]+=A[i][j]*h[j];
      
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  dG[j][i]=arr[k+1].Gk[j][i]-L[j][i];
      
      CF[0][0]=A[0][0]*dG[0][0]+A[0][1]*dG[0][1]+A[0][2]*dG[0][2]+A[0][3]*dG[0][3];
      CF[0][1]=A[0][0]*dG[0][1]+A[0][1]*dG[1][1]+A[0][2]*dG[1][2]+A[0][3]*dG[1][3];
      CF[0][2]=A[0][0]*dG[0][2]+A[0][1]*dG[1][2]+A[0][2]*dG[2][2]+A[0][3]*dG[2][3];
      CF[0][3]=A[0][0]*dG[0][3]+A[0][1]*dG[1][3]+A[0][2]*dG[2][3]+A[0][3]*dG[3][3];
      CF[1][0]=A[1][0]*dG[0][0]+A[1][1]*dG[0][1]+A[1][2]*dG[0][2]+A[1][3]*dG[0][3];
      CF[1][1]=A[1][0]*dG[0][1]+A[1][1]*dG[1][1]+A[1][2]*dG[1][2]+A[1][3]*dG[1][3];
      CF[1][2]=A[1][0]*dG[0][2]+A[1][1]*dG[1][2]+A[1][2]*dG[2][2]+A[1][3]*dG[2][3];
      CF[1][3]=A[1][0]*dG[0][3]+A[1][1]*dG[1][3]+A[1][2]*dG[2][3]+A[1][3]*dG[3][3];
      CF[2][0]=A[2][0]*dG[0][0]+A[2][1]*dG[0][1]+A[2][2]*dG[0][2]+A[2][3]*dG[0][3];
      CF[2][1]=A[2][0]*dG[0][1]+A[2][1]*dG[1][1]+A[2][2]*dG[1][2]+A[2][3]*dG[1][3];
      CF[2][2]=A[2][0]*dG[0][2]+A[2][1]*dG[1][2]+A[2][2]*dG[2][2]+A[2][3]*dG[2][3];
      CF[2][3]=A[2][0]*dG[0][3]+A[2][1]*dG[1][3]+A[2][2]*dG[2][3]+A[2][3]*dG[3][3];
      CF[3][0]=A[3][0]*dG[0][0]+A[3][1]*dG[0][1]+A[3][2]*dG[0][2]+A[3][3]*dG[0][3];
      CF[3][1]=A[3][0]*dG[0][1]+A[3][1]*dG[1][1]+A[3][2]*dG[1][2]+A[3][3]*dG[1][3];
      CF[3][2]=A[3][0]*dG[0][2]+A[3][1]*dG[1][2]+A[3][2]*dG[2][2]+A[3][3]*dG[2][3];
      CF[3][3]=A[3][0]*dG[0][3]+A[3][1]*dG[1][3]+A[3][2]*dG[2][3]+A[3][3]*dG[3][3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++)
	  for(int m=0;m<4;m++) 
	    arr[k].Gk[j][i]+=CF[j][m]*A[i][m];
    }
    Z=arr[k].sigma_m2+arr[k].cosa*arr[k].cosa*arr[k].Gk[0][0]-
      2.0*arr[k].sina*arr[k].cosa*arr[k].Gk[0][1]+
      arr[k].sina*arr[k].sina*arr[k].Gk[1][1];
    zeta_k=(arr[k].x-arr[k].cosa*arr[k].Rk[0]+arr[k].sina*arr[k].Rk[1]);
    arr[k].chi2=zeta_k*zeta_k/Z;
    zprev=arr[k].z;
  }
  return;
}

int CompareFHIT(const void* pv1,const void* pv2)
{
  pFHIT p1=(pFHIT) pv1;
  pFHIT p2=(pFHIT) pv2;

  if(p2->z>p1->z) return -1;
  else return 1;
}

void OtrFitSPointNew(SPACEPOINT& sp)
{ 
  const double chi2cut=2.0;
 
  int index,numhits,nstereo=0,nzero=0,nchanges;
  double chi2t=0.0,zf,pf;
  double sigma_ms = 2.0e-6;// multiple scattering corrections 
  FHIT arr[300];

  numhits=0;index=0;

  for (vector<pCLUSTER>::iterator cluIt = sp.vpclu.begin();
       cluIt != sp.vpclu.end(); ++cluIt)
    {
      pCLUSTER pclu = (*cluIt);
      arr[index].sina=pclu->sina;
      arr[index].cosa=pclu->cosa;
      arr[index].z=pclu->z;
      arr[index].x=pclu->x;
      arr[index].mask=1;
      arr[index].sigma_m2 = pclu->sigma*pclu->sigma;
      index++;
    }
  numhits = index;
  qsort(arr,index,sizeof(FHIT),CompareFHIT);
  arr[0].Rk[0] = sp.xf; arr[0].Rk[1] = sp.yf;
  arr[0].Rk[2] = sp.txf;arr[0].Rk[3] = sp.tyf;
  zf=arr[0].z;pf=sp.pf;
  for(int iter=1;iter<=2;iter++)
    {
      nchanges=0;
      OtrKalmanFilterSmoother(arr,numhits,sigma_ms);
      chi2t=0.0;nstereo=0;nzero=0;
      for(int k=0;k<numhits;k++)
	{
	  //	  cout<<k<<" chi2="<<arr[k].chi2<<endl;
	  if(arr[k].chi2<chi2cut)
	    {
	      chi2t+=arr[k].chi2;
	      if(arr[k].mask!=1) nchanges++;
	      arr[k].mask=1;
	      if(fabs(arr[k].sina)<0.03) nzero++;
	      else nstereo++;
	    }
	else 
	  {
	    if(arr[k].mask==1) nchanges++;
	    arr[k].mask=0;
	  }
	}
           if(!nchanges) break;
    }
  if((nstereo>=2)&&(nzero>=2))
    {
      for(int iter=1;iter<5;iter++)
	{
	  float best_chi2=100000.0;
	  int best_ind=-1;
	  for(int k=0;k<numhits;k++) if(arr[k].mask<0.1)
	    {
	      //  cout<<k<<" chi2="<<arr[k].chi2<<endl;
	      if(arr[k].chi2<best_chi2) 
		{
		  best_ind=k;
		  best_chi2=arr[k].chi2;
		}
	    }
	  //	  cout<<"best outlier = "<<best_ind<<endl;
	  if(best_ind!=-1) 
	    {
	      arr[best_ind].mask=1;
	      OtrKalmanFilterSmoother(arr,numhits,sigma_ms);
	      chi2t=0.0;nstereo=0;nzero=0;
	      for(int k=0;k<numhits;k++)
		{
		  if(arr[k].chi2<chi2cut)
		    {
		      chi2t+=arr[k].chi2;
		      arr[k].mask=1;
		      if(fabs(arr[k].sina)<0.03) nzero++;
		      else nstereo++;
		    }
		  else arr[k].mask=0;
		}
	      if(arr[best_ind].mask==0) break;
	    }
	}
    }
  sp.chi2=chi2t;
  int  nclu=nzero + nstereo;
  if (nzero + nstereo > 4)
    sp.prob = prob_(chi2t,(nzero + nstereo - 4));
  else
    sp.prob = 0.0001;
  if (nzero + nstereo > 4)
    {
      int kmin=1000;
      int kmax=-1;
      float zmin=10000.0;
      float zmax=-10000.0;     
      for(int k=0; k<numhits; k++) 
	if(arr[k].mask)
	  {
	    if(arr[k].z>zmax)
	      {
		zmax=arr[k].z;kmax=k;
	      }
	    if(arr[k].z<zmin)
	      {
		zmin=arr[k].z;kmin=k;
	      }
	  }
      sp.xf  = arr[kmin].Rk[0];          
      sp.yf  = arr[kmin].Rk[1];
      sp.zf  = arr[kmin].z;
      sp.txf = arr[kmin].Rk[2];          
      sp.tyf = arr[kmin].Rk[3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  sp.cvf[j][i] = arr[kmin].Gk[j][i];
      sp.xe  = arr[kmax].Rk[0];          
      sp.ye  = arr[kmax].Rk[1];
      sp.ze  = arr[kmax].z;
      sp.txe = arr[kmax].Rk[2];          
      sp.tye = arr[kmax].Rk[3];
      for(int i=0;i<4;i++) 
	for(int j=0;j<=i;j++) 
	  sp.cve[j][i] = arr[kmax].Gk[j][i]; 
      
      //      cout<<"Kmin="<<kmin<<" Kmax="<<kmax<<endl;

      vector<pCLUSTER> ctmp;
      ctmp.clear();index=0;
      for(vector<pCLUSTER>::iterator pcluIt = sp.vpclu.begin();
	  pcluIt != sp.vpclu.end(); ++pcluIt)
	{
	  if(arr[index].mask!=0)
	    {
	      ctmp.push_back((*pcluIt));
	    }
	  index++;
	}
      sp.vpclu.clear();
      for(vector<pCLUSTER>::iterator pcluIt=ctmp.begin();pcluIt!=ctmp.end(); ++pcluIt)
	sp.vpclu.push_back((*pcluIt));
      
      if ((sp.cvf[0][0]< 0.0)||(sp.cvf[1][1]< 0.0)||
	  (sp.cvf[2][2]< 0.0)||(sp.cvf[3][3]< 0.0)){
	
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------> OtrCats bad cvf matrix ("<<nclu<<" hits) "
	    <<sp.cvf[0][0]<<" "<<sp.cvf[1][1]<<" "<<sp.cvf[2][2]<<" "<<sp.cvf[3][3]<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
      }
      if ((sp.cve[0][0]< 0.0)||(sp.cve[1][1]< 0.0)||
	  (sp.cve[2][2]< 0.0)||(sp.cve[3][3]< 0.0)){
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------> OtrCats bad cve matrix ("<<nclu<<" hits) "
	    <<sp.cve[0][0]<<" "<<sp.cve[1][1]<<" "<<sp.cve[2][2]<<" "<<sp.cve[3][3]<<endl;
	cout<<"------------------>"<<endl;
	cout<<"------------------>"<<endl;
      }
    }
  return;
}


//================================================================================================

void OtrRefitTrack(pTRACK ptrack){
  const float sigma_ms = 0.0003; 
  float Rk[4], Gk[4][4], Kk[4], Bk[4];
  float Sk, zprev, Dk, Dk2, q33, q44, q34, sq, Z, zeta;
  
  float pinv = ptrack->pf;

  //  coeff = (1.0+txy2)*(1.0+txy2)*(1.0+0.01*pinv*pinv)*pinv*pinv;
  float coeff = (1.0+0.105*0.105*pinv*pinv)*pinv*pinv; // muon mass correction
  float l2sqrt = 1.0 + ptrack->txf*ptrack->txf + ptrack->tyf*ptrack->tyf; l2sqrt = l2sqrt*sqrt(l2sqrt);

  q33 = sigma_ms*sigma_ms*coeff*l2sqrt*(1.0+ptrack->txf*ptrack->txf); 
  q44 = sigma_ms*sigma_ms*coeff*l2sqrt*(1.0+ptrack->tyf*ptrack->tyf); 
  q34 = sigma_ms*sigma_ms*coeff*l2sqrt*ptrack->txf*ptrack->tyf;

  /**** Fit for the last point: starting from the first one ****/

  for (int i=0; i<4; i++) 
    for (int j=0; j<=i; j++) 
      Gk[j][i] = ptrack->cvf[j][i];
  ///
  for (int i=0; i<4; i++) 
    for (int j=0; j<4; j++) 
      Gk[i][j] = 0.0;
  //Gk[0][0] = 1.0; Gk[1][1] = 50.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  Gk[0][0] = 1.0; Gk[1][1] = 10.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  ///
  Rk[0] = ptrack->xf; Rk[1] = ptrack->yf; Rk[2] = ptrack->txf; Rk[3] = ptrack->tyf;
  zprev = ptrack->zf;
  //for (vector<ArtePointer<HITB> >::iterator hitbIt = ptrack->vhitb.begin();
  //     hitbIt != ptrack->vhitb.end(); ++hitbIt){
  for (vector<pCLUSTER >::iterator pcluIt = ptrack->vpcluster.begin();
       pcluIt != ptrack->vpcluster.end(); ++pcluIt){
    /**** One-step extrapolation ****/

    //ArtePointer<GEDE> gede = (*hitbIt)->gede;
    ArtePointer<GEDE> gede = (*pcluIt)->gede;
    float cosa  = gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
    float sina  = gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
    float sigma = gede->pit/(1.6*3.0);
    //sigma = fabs((*hitbIt)->q);
    sigma = (*pcluIt)->sigma;


    //Dk = (*hitbIt)->z - zprev; Dk2 = Dk*Dk; zprev = (*hitbIt)->z;
    Dk = (*pcluIt)->z - zprev; Dk2 = Dk*Dk; zprev = (*pcluIt)->z;
    Rk[0] += Rk[2]*Dk; Rk[1] += Rk[3]*Dk;
    Gk[0][0] += 2*Dk*Gk[0][2] + Dk2*Gk[2][2];
    Gk[1][1] += 2*Dk*Gk[1][3] + Dk2*Gk[3][3];
    Gk[0][1] += Dk*(Gk[1][2]  + Gk[0][3]) + Dk2*Gk[2][3];
    Gk[0][2] += Dk*Gk[2][2];
    Gk[0][3] += Dk*Gk[2][3];
    Gk[1][2] += Dk*Gk[2][3];
    Gk[1][3] += Dk*Gk[3][3];
    
    /**** Multiple scattering corrections ****/
    
    Gk[2][2] += q33; Gk[2][3] += q34; Gk[3][3] += q44;
      
    /**** Sigma^2 of pseudolinear measurement ****/
    
    sq = Rk[2]*cosa - Rk[3]*sina; sq=sq*sq;
    Sk = sigma*sigma*(1.0+sq);
    
    /**** Kalman gain ****/
    
    Z = Gk[0][0]*cosa*cosa - 2.0*Gk[0][1]*cosa*sina + Gk[1][1]*sina*sina + Sk; 
    Z = 1.0/Z;
    
    Bk[0] = Gk[0][0]*cosa - Gk[0][1]*sina;
    Bk[1] = Gk[0][1]*cosa - Gk[1][1]*sina;
    Bk[2] = Gk[0][2]*cosa - Gk[1][2]*sina;
    Bk[3] = Gk[0][3]*cosa - Gk[1][3]*sina;
    for (int i=0; i<4; i++) 
      Kk[i] = Z*Bk[i];
    
    /**** Updated estimates ****/
    
    //zeta = 0.0 - Rk[0]*cosa + Rk[1]*sina + (*hitbIt)->x;
    zeta = 0.0 - Rk[0]*cosa + Rk[1]*sina + (*pcluIt)->x;
    for (int i=0; i<4; i++) 
      Rk[i] += Kk[i]*zeta;
    
    /**** Updated covariance matrix ****/
    
    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Bk[i]*Kk[j];
  }

  ptrack->xe = Rk[0]; ptrack->ye = Rk[1]; ptrack->txe = Rk[2]; ptrack->tye = Rk[3];
  for (int i=0; i<4; i++) 
    for (int j=0; j<=i; j++) 
      ptrack->cve[j][i] = Gk[j][i];

  /**** Fit for the first point: starting from the last one ****/

  Dk = ptrack->ze - ptrack->zf; Dk2 = Dk*Dk;
  Gk[0][0] = ptrack->cvf[0][0] + 2*Dk*ptrack->cvf[0][2] + Dk2*ptrack->cvf[2][2];
  Gk[1][1] = ptrack->cvf[1][1] + 2*Dk*ptrack->cvf[1][3] + Dk2*ptrack->cvf[3][3];
  Gk[0][1] = ptrack->cvf[0][1] + Dk*(ptrack->cvf[1][2]  + ptrack->cvf[0][3]) + Dk2*ptrack->cvf[2][3];
  Gk[0][2] = ptrack->cvf[0][2] + Dk*ptrack->cvf[2][2];
  Gk[0][3] = ptrack->cvf[0][3] + Dk*ptrack->cvf[2][3];
  Gk[1][2] = ptrack->cvf[1][2] + Dk*ptrack->cvf[2][3];
  Gk[1][3] = ptrack->cvf[1][3] + Dk*ptrack->cvf[3][3];
  Gk[2][2] = ptrack->cvf[2][2];
  Gk[2][3] = ptrack->cvf[2][3];
  Gk[3][3] = ptrack->cvf[3][3];
  zprev = ptrack->ze;

  ///
  for (int i=0; i<4; i++) 
    for (int j=0; j<4; j++) 
      Gk[i][j] = 0.0;
  //Gk[0][0] = 1.0; Gk[1][1] = 50.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  Gk[0][0] = 1.0; Gk[1][1] = 10.0; Gk[2][2] = 1.0; Gk[3][3] = 1.0;
  ///



  //for (vector<ArtePointer<HITB> >::reverse_iterator hitbIt = ptrack->vhitb.rbegin();
  //     hitbIt != ptrack->vhitb.rend(); ++hitbIt){
  for (vector<pCLUSTER >::reverse_iterator pcluIt = ptrack->vpcluster.rbegin();
       pcluIt != ptrack->vpcluster.rend(); ++pcluIt){
    /**** One-step extrapolation ****/

    //ArtePointer<GEDE> gede = (*hitbIt)->gede;
    ArtePointer<GEDE> gede = (*pcluIt)->gede;
    float cosa  = gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
    float sina  = gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
    float sigma = gede->pit/(1.6*3.0);
    //sigma = fabs((*hitbIt)->q);
    sigma = (*pcluIt)->sigma;

    //Dk = (*hitbIt)->z - zprev; Dk2 = Dk*Dk; zprev = (*hitbIt)->z;
    Dk = (*pcluIt)->z - zprev; Dk2 = Dk*Dk; zprev = (*pcluIt)->z;
    Rk[0] += Rk[2]*Dk; Rk[1] += Rk[3]*Dk;
    Gk[0][0] += 2*Dk*Gk[0][2] + Dk2*Gk[2][2];
    Gk[1][1] += 2*Dk*Gk[1][3] + Dk2*Gk[3][3];
    Gk[0][1] += Dk*(Gk[1][2]  + Gk[0][3]) + Dk2*Gk[2][3];
    Gk[0][2] += Dk*Gk[2][2];
    Gk[0][3] += Dk*Gk[2][3];
    Gk[1][2] += Dk*Gk[2][3];
    Gk[1][3] += Dk*Gk[3][3];
    
    /**** Multiple scattering corrections ****/
      
    Gk[2][2] += q33; Gk[2][3] += q34; Gk[3][3] += q44;
    
    /**** Sigma^2 of pseudolinear measurement ****/
    
    sq = Rk[2]*cosa - Rk[3]*sina; sq = sq*sq;
    Sk = sigma*sigma*(1.0 + sq);
    
    /**** Kalman gain ****/
    
    Z = Gk[0][0]*cosa*cosa - 2.0*Gk[0][1]*cosa*sina +
      Gk[1][1]*sina*sina + Sk;
    Z = 1.0/Z;
    
    Bk[0] = Gk[0][0]*cosa - Gk[0][1]*sina;
    Bk[1] = Gk[0][1]*cosa - Gk[1][1]*sina;
    Bk[2] = Gk[0][2]*cosa - Gk[1][2]*sina;
    Bk[3] = Gk[0][3]*cosa - Gk[1][3]*sina;
    for (int i=0; i<4; i++) 
      Kk[i] = Z*Bk[i];
    
    /**** Updated estimates ****/
    
    //zeta = 0.0 - Rk[0]*cosa + Rk[1]*sina + (*hitbIt)->x;
    zeta = 0.0 - Rk[0]*cosa + Rk[1]*sina + (*pcluIt)->x;
    for (int i=0; i<4; i++) 
      Rk[i] += Kk[i]*zeta;
    
    /**** Updated covariance matrix ****/
    
    for (int i=0; i<4; i++) 
      for (int j=0; j<=i; j++) 
	Gk[j][i] -= Bk[i]*Kk[j];
  }
  
  ptrack->xf = Rk[0]; ptrack->yf = Rk[1]; ptrack->txf = Rk[2]; ptrack->tyf = Rk[3];
  for (int i=0; i<4; i++) 
    for (int j=0; j<=i; j++) 
      ptrack->cvf[j][i] = Gk[j][i];
}

//====================================================================================

void OtrResolveLR(pTRACK ptrack){
  
  float ax = (ptrack->xe-ptrack->xf)/(ptrack->ze-ptrack->zf);
  float bx = ptrack->xf - ax*ptrack->zf;
  float errfx = 0.0500+1.5*3.5*(sqrt(ptrack->cvf[0][0]));
  float errex = 0.0500+1.5*3.5*(sqrt(ptrack->cve[0][0]));
  
  float ay = (ptrack->ye-ptrack->yf)/(ptrack->ze-ptrack->zf);
  float by = ptrack->yf - ay*ptrack->zf;
  //ay = ptrack->yf/ptrack->zf;
  //by = 0.0;
  float errfy = 1.5*3.5*sqrt(ptrack->cvf[1][1]);
  float errey = 1.5*3.5*sqrt(ptrack->cve[1][1]);
  

  int maxiter = 5;//3;//5;
  for (int iter = 0; iter < maxiter; ++iter){ // iter
    
    double S = 0.0, Sx = 0.0, Sy = 0.0, Sxx = 0.0, Sxy = 0.0;
    double s = 0.0, sx = 0.0, sy = 0.0, sxx = 0.0, sxy = 0.0; // y copy
    
    float zmin_zero   = 1000000.0, zmax_zero   = -1000000.0;
    float zmin_stereo = 1000000.0, zmax_stereo = -1000000.0;

    // attract to the measurements
    //loop over hits of the track

    for (vector<pHIT>::iterator phitIt = ptrack->vphit.begin();
    	 phitIt != ptrack->vphit.end(); ++phitIt){ // phit
          
      ArtePointer<GEDE> gede = (*phitIt)->hitb->gede;      
     
      float sina=gede->sin*gede->rot[4] - gede->cos*gede->rot[3];
      float cosa=gede->cos*gede->rot[0] - gede->sin*gede->rot[1];
      float alpha = 180.0/3.1415926*atan2(sina, cosa); if (alpha<-10.0) alpha += 360.0;
     
      float hitbIt_x = (*phitIt)->x;
      float hitbIt_q = (*phitIt)->q;
      float hitbIt_r = (*phitIt)->r;
      float hitbIt_z = (*phitIt)->z;

      if ((alpha > -2.0)&&(alpha < 2.0)){ // zero projection
	
	//straight line
	float x = ax*hitbIt_z + bx;
	float cutx = errfx + ((errex-errfx)/(ptrack->ze-ptrack->zf))*
	  (hitbIt_z-ptrack->zf);
	int sidex = 0;
	if (x >= hitbIt_x){
	  float lr = 1.0;
	  if ((x < hitbIt_x+hitbIt_q)&&(hitbIt_q>hitbIt_r)){ //inside a cell -> reduce lr weight
	    lr = (x - hitbIt_x)/(hitbIt_q+0.000001);
	    if (lr > 1.0) lr = 1.0; //just for a case
	  }
	  (*phitIt)->lr = 1.0;//lr;
	  
	  float hitb_x = hitbIt_x+hitbIt_q;
	  float w = 0.0;
	  float dist2 = (hitb_x-x)*(hitb_x-x);
	  float cut2 = cutx*cutx;
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	    sidex = 1; // top

	    if (hitbIt_z < zmin_zero) zmin_zero = hitbIt_z;
	    if (hitbIt_z > zmax_zero) zmax_zero = hitbIt_z;

	  }
	  (*phitIt)->mask = w;

	  S   += w;
	  Sx  += w*hitbIt_z;
	  Sy  += w*hitb_x;
	  Sxx += w*hitbIt_z*hitbIt_z;
	  Sxy += w*hitb_x*hitbIt_z;

	  // include also opposite side
	  hitb_x = hitbIt_x-hitbIt_q;
	  w = 0.0;
	  dist2 = (hitb_x-x)*(hitb_x-x);
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	  }
	  w *= 0.3*(maxiter-iter)/float(maxiter); // make weaker attraction
	  S   += w;
	  Sx  += w*hitbIt_z;
	  Sy  += w*hitb_x;
	  Sxx += w*hitbIt_z*hitbIt_z;
	  Sxy += w*hitb_x*hitbIt_z;
	}
	else{
	  float lr = -1.0;
	  if ((x > hitbIt_x-hitbIt_q)&&(hitbIt_q>hitbIt_r)){ //inside a cell -> reduce lr weight
	    lr = (x - hitbIt_x)/(hitbIt_q+0.000001);
	    if (lr < -1.0) lr = -1.0; //just for a case
	  }
	  (*phitIt)->lr = -1.0;//lr;
	  
	  float hitb_x = hitbIt_x-hitbIt_q;
	  float w = 0.0;
	  float dist2 = (hitb_x-x)*(hitb_x-x);
	  float cut2 = cutx*cutx;
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	    sidex = -1; // bottom

	    if (hitbIt_z < zmin_zero) zmin_zero = hitbIt_z;
	    if (hitbIt_z > zmax_zero) zmax_zero = hitbIt_z;

	  }
	  (*phitIt)->mask = w;

	  S   += w;
	  Sx  += w*hitbIt_z;
	  Sy  += w*hitb_x;
	  Sxx += w*hitbIt_z*hitbIt_z;
	  Sxy += w*hitb_x*hitbIt_z;

	  // include also opposite side
	  hitb_x = hitbIt_x+hitbIt_q;
	  w = 0.0;
	  dist2 = (hitb_x-x)*(hitb_x-x);
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	  }
	  w *= 0.3*(maxiter-iter)/float(maxiter); // make weaker attraction
	  S   += w;
	  Sx  += w*hitbIt_z;
	  Sy  += w*hitb_x;
	  Sxx += w*hitbIt_z*hitbIt_z;
	  Sxy += w*hitb_x*hitbIt_z;
	}
      } // zero projection
      else{
	//straight line
	float y = ay*hitbIt_z + by;
	float cuty = errfy + ((errey-errfy)/(ptrack->ze-ptrack->zf))*(hitbIt_z-ptrack->zf);
	int sidey = 0;
	if (((ax*hitbIt_z + bx)*cosa - y*sina) >= hitbIt_x){
	  
	  float xx = (ax*hitbIt_z + bx)*cosa - y*sina;
	  float lr = 1.0;
	  if ((xx < hitbIt_x+hitbIt_q)&&(hitbIt_q>hitbIt_r)){ //inside a cell -> reduce lr weight
	    lr = (xx - hitbIt_x)/(hitbIt_q+0.000001);
	    if (lr > 1.0) lr = 1.0; //just for a case
	  }
	  (*phitIt)->lr = 1.0;//lr;
	  
	  float hitb_y = ((ax*hitbIt_z + bx)*cosa - (hitbIt_x + hitbIt_q))/sina;
	  float w = 0.0;
	  float dist2 = (hitb_y-y)*(hitb_y-y);
	  float cut2 = cuty*cuty;
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	    sidey = 1; // top

	    if (hitbIt_z < zmin_stereo) zmin_stereo = hitbIt_z;
	    if (hitbIt_z > zmax_stereo) zmax_stereo = hitbIt_z;

	  }
	  (*phitIt)->mask = w;

	  s   += w;
	  sx  += w*hitbIt_z;
	  sy  += w*hitb_y;
	  sxx += w*hitbIt_z*hitbIt_z;
	  sxy += w*hitb_y*hitbIt_z;

	  // include also opposite side
	  hitb_y = ((ax*hitbIt_z + bx)*cosa - (hitbIt_x - hitbIt_q))/sina;
	  w = 0.0;
	  dist2 = (hitb_y-y)*(hitb_y-y);
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	  }
	  w *= 0.3*(maxiter-iter)/float(maxiter); // make weaker attraction
	  s   += w;
	  sx  += w*hitbIt_z;
	  sy  += w*hitb_y;
	  sxx += w*hitbIt_z*hitbIt_z;
	  sxy += w*hitb_y*hitbIt_z;

	}
	else{
	  
	  float xx = (ax*hitbIt_z + bx)*cosa - y*sina;
	  float lr = -1.0;
	  if ((xx > hitbIt_x-hitbIt_q)&&(hitbIt_q>hitbIt_r)){ //inside a cell -> reduce lr weight
	    lr = (xx - hitbIt_x)/(hitbIt_q+0.000001);
	    if (lr < -1.0) lr = -1.0; //just for a case
	  }
	  (*phitIt)->lr = -1.0;//lr;
	  
	  float hitb_y = ((ax*hitbIt_z + bx)*cosa - (hitbIt_x - hitbIt_q))/sina;
	  float w = 0.0;
	  float dist2 = (hitb_y-y)*(hitb_y-y);
	  float cut2 = cuty*cuty;
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	    sidey = -1; // bottom

	    if (hitbIt_z < zmin_stereo) zmin_stereo = hitbIt_z;
	    if (hitbIt_z > zmax_stereo) zmax_stereo = hitbIt_z;

	  }
	  (*phitIt)->mask = w;

	  s   += w;
	  sx  += w*hitbIt_z;
	  sy  += w*hitb_y;
	  sxx += w*hitbIt_z*hitbIt_z;
	  sxy += w*hitb_y*hitbIt_z;

	  // include also opposite side
	  hitb_y = ((ax*hitbIt_z + bx)*cosa - (hitbIt_x + hitbIt_q))/sina;
	  w = 0.0;
	  dist2 = (hitb_y-y)*(hitb_y-y);
	  if (dist2<cut2){
	    w = (1.0-(dist2/cut2))*(1.0-(dist2/cut2));// bi-weight Teukey
	  }
	  w *= 0.3*(maxiter-iter)/float(maxiter); // make weaker attraction
	  s   += w;
	  sx  += w*hitbIt_z;
	  sy  += w*hitb_y;
	  sxx += w*hitbIt_z*hitbIt_z;
	  sxy += w*hitb_y*hitbIt_z;

	}
      } // zero 
    } // phit
    
    /*{
      float w = 50.0;

      s   += w;
      sx  += w*0.0;
      sy  += w*0.0;
      sxx += w*0.0*0.0;
      sxy += w*0.0*0.0;
      }*/


    if ( (zmax_zero-zmin_zero) > 3.0){ 
      double D = S*Sxx-Sx*Sx;
      ax = (S*Sxy-Sx*Sy)/D;
      bx = (Sxx*Sy-Sx*Sxy)/D;
    }
    else{
      errfx *= 2.0; errex *= 2.0;
    }
    
    if ( (zmax_stereo-zmin_stereo) > 3.0){ 
      double d = s*sxx-sx*sx;
      ay = (s*sxy-sx*sy)/d;
      by = (sxx*sy-sx*sxy)/d;
    }      
    else{
      errfy *= 2.0; errey *= 2.0;
    }

  } // iter
  
  ptrack->xf = ax*ptrack->zf+bx; ptrack->xe = ax*ptrack->ze+bx; 
  ptrack->txf = ptrack->txe = ax;
  ptrack->yf = ay*ptrack->zf+by; ptrack->ye = ay*ptrack->ze+by; 
  ptrack->tyf = ptrack->tye = ay;

}


//========================================================



void OtrGraphIni(){

  if (ItrOtrGraphInitialized == 1) return; 
  ItrOtrGraphInitialized = 1;

  iginit(0);
  igsse(0, 6);


  iscr(1, Black       ,0.0000, 0.0000, 0.0000);
  iscr(1, DarkGray    ,0.5000, 0.5400, 0.5300);
  iscr(1, Blue        ,0.0000, 0.0000, 1.0000);
  iscr(1, Green       ,0.1333, 0.5451, 0.1333);
  iscr(1, Cyan        ,0.0000, 0.8078, 0.8196);
  iscr(1, Red         ,1.0000, 0.0000, 0.0000);
  iscr(1, Magenta     ,1.0000, 0.0000, 1.0000);
  iscr(1, Brown       ,0.5000, 0.1647, 0.1647);
  iscr(1, LightBlue   ,0.6784, 0.8471, 0.9020);
  iscr(1, LightGreen  ,0.5961, 0.9843, 0.5961);
  iscr(1, LightCyan   ,0.4980, 1.0000, 0.8314);
  iscr(1, LightRed    ,0.9804, 0.5020, 0.4471);
  iscr(1, LightMagenta,0.8549, 0.4392, 0.8392);
  iscr(1, Yellow      ,1.0000, 1.0000, 0.0000);
  iscr(1, LightGray   ,0.8275, 0.8275, 0.8275);
  iscr(1, White       ,1.0000, 1.0000, 1.0000);



  iswn(2, 700.00, 860.00,-500.00,500.00);  //OTR
#ifdef XXX
  iswn(2, 680.00, 830.00,-50.00, 50.00);  //ITR
#endif
  isvp(2,0.0,1.0,0.0,1.0);
  iselnt(2);

  isfais(1);

  isfaci(LightGray);
  isplci(Red);

  igbox(700.00, 860.00,-500.00,500.00);  //OTR
#ifdef XXX 
  igbox(680.00, 830.00,-0.00, 50.00);  //ITR
#endif
  iuwk(1,1);

}

//========================================================

void OtrGraphClose(){
  igend();
}

//========================================================

void OtrRotatePoint(float& x, float& y, float& z, const ArtePointer<GEDE> gede) {

  float xx, yy, zz; 
  xx = x; yy = y; zz = z;
  
  // transform from plane coordinate system to Arte coordinate system
  x = xx*gede->rot[0] + yy*gede->rot[1] + zz*gede->rot[2];
  y = xx*gede->rot[3] + yy*gede->rot[4] + zz*gede->rot[5];
  z = xx*gede->rot[6] + yy*gede->rot[7] + zz*gede->rot[8];
}


//========================================================

void OtrGedeMinMax(float& sxmn,float& sxmx, float& symn,float& symx, float& szmn,float& szmx, 
		   const ArtePointer<GEDE> gede){
 
  float x, y, z;

  x = gede->sxmn; y = gede->symn; z = gede->zmin;
  OtrRotatePoint(x, y, z, gede);
  sxmn = x; sxmx = x; 
  symn = y; symx = y; 
  szmn = z; szmx = z;

  x = gede->sxmx; y = gede->symn; z = gede->zmin;
  OtrRotatePoint(x, y, z,gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmn; y = gede->symx; z = gede->zmin;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmx; y = gede->symx; z = gede->zmin;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmn; y = gede->symn; z = gede->zmax;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmx; y = gede->symn; z = gede->zmax;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmn; y = gede->symx; z = gede->zmax;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

  x = gede->sxmx; y = gede->symx; z = gede->zmax;
  OtrRotatePoint(x, y, z, gede);
  if (x < sxmn) sxmn = x; if (x > sxmx) sxmx = x;
  if (y < symn) symn = y; if (y > symx) symx = y;
  if (z < szmn) szmn = z; if (z > szmx) szmx = z;

}

//========================================================

typedef struct
{
  ArtePointer<RSEG> otr;
  ArtePointer<RSEG> itr;
  float F;
} RSEG_PAIR, *pRSEG_PAIR;

bool MatchCompareF(const pRSEG_PAIR p1, const pRSEG_PAIR p2)
{
  return (p1->F<p2->F);
}

int QuickMatch(ArtePointer<RSEG> otr, ArtePointer<RSEG> itr)
{
  const float delta_max=9.0;

  int res=1;
  float xp,yp,delta,dx,dy;

  if(otr->zf<itr->ze) res=0;
  else
    {
      xp=itr->xf+(otr->zf-itr->zf)*(itr->xe-itr->xf)/(itr->ze-itr->zf);
      yp=itr->yf+(otr->zf-itr->zf)*(itr->ye-itr->yf)/(itr->ze-itr->zf);
      dx=xp-otr->xf;dy=yp-otr->yf;
      delta=dx*dx+dy*dy;
      if(delta>delta_max) res=0;
    }
  return res;
}


float Chi2DistanceRSEG(const ArtePointer<RSEG> rseg1, const ArtePointer<RSEG> rseg2)
{
  float dchi2=0.0;

  float x[4],y[4],z[4],alfa13,alfa23;
  double gx[4],gy[4],Dx,Dy;
  int i;

  x[0]=rseg1->xf;y[0]=rseg1->yf;z[0]=rseg1->zf;gx[0]=rseg1->cvf[0];gy[0]=rseg1->cvf[2];
  x[1]=rseg1->xe;y[1]=rseg1->ye;z[1]=rseg1->ze;gx[1]=rseg1->cve[0];gy[1]=rseg1->cve[2];
  x[2]=rseg2->xf;y[2]=rseg2->yf;z[2]=rseg2->zf;gx[2]=rseg2->cvf[0];gy[2]=rseg2->cvf[2];
  x[3]=rseg2->xe;y[3]=rseg2->ye;z[3]=rseg2->ze;gx[3]=rseg2->cve[0];gy[3]=rseg2->cve[2];

  for(i=1;i<3;i++)
    {
      alfa13=(z[3]-z[i])/(z[3]-z[0]);alfa23=(z[i]-z[0])/(z[3]-z[0]);
      Dx=x[i]-alfa13*x[0]-alfa23*x[3];Dy=y[i]-alfa13*y[0]-alfa23*y[3];

            dchi2+=(Dx*Dx+Dy*Dy)/(gx[i]+gy[i]+alfa13*alfa13*(gx[0]+gy[0])+alfa23*alfa23*(gx[3]+gy[3]));
	    //  dchi2+=Dx*Dx/(gx[i]+alfa13*alfa13*gx[0]+alfa23*alfa23*gx[3]);
    }

  return dchi2;



}


float Chi2Distance(pRSEG_PAIR p)
{
  float dchi2=0.0;

  float x[4],y[4],z[4],alfa13,alfa23;
  double gx[4],gy[4],Dx,Dy;
  int i;

  x[0]=p->itr->xf;y[0]=p->itr->yf;z[0]=p->itr->zf;gx[0]=p->itr->cvf[0];gy[0]=p->itr->cvf[2];
  x[1]=p->itr->xe;y[1]=p->itr->ye;z[1]=p->itr->ze;gx[1]=p->itr->cve[0];gy[1]=p->itr->cve[2];
  x[2]=p->otr->xf;y[2]=p->otr->yf;z[2]=p->otr->zf;gx[2]=p->otr->cvf[0];gy[2]=p->otr->cvf[2];
  x[3]=p->otr->xe;y[3]=p->otr->ye;z[3]=p->otr->ze;gx[3]=p->otr->cve[0];gy[3]=p->otr->cve[2];

  for(i=1;i<3;i++)
    {
      alfa13=(z[3]-z[i])/(z[3]-z[0]);alfa23=(z[i]-z[0])/(z[3]-z[0]);
      Dx=x[i]-alfa13*x[0]-alfa23*x[3];Dy=y[i]-alfa13*y[0]-alfa23*y[3];
      dchi2+=(Dx*Dx+Dy*Dy)/(gx[i]+gy[i]+alfa13*alfa13*(gx[0]+gy[0])+alfa23*alfa23*(gx[3]+gy[3]));
    }
  p->F=dchi2;
  return dchi2;
}

void OtrItrMatch()
{
  
  ArtePointer<RSEG> rotr;
  ArtePointer<RSEG> ritr;
  int flag,notr=0,nitr=0,npair=0;
  vector<RSEG_PAIR> vp; vp.clear();
  RSEG_PAIR pair;

  flag=2;
  for( ArteTable<RSEG>::iterator rsegIt = ArteTable<RSEG>::begin();
       rsegIt != ArteTable<RSEG>::end(); ++rsegIt ) 
    if(((*rsegIt)->fit >= 0)&&((*rsegIt)->cmp & Rsegc::patt)&&!((*rsegIt)->cmp & Rsegc::bitsign))
      {      
	if((*rsegIt)->cmp & Rsegc::wch) notr++;
	if((*rsegIt)->cmp & Rsegc::msg) nitr++;
      }
  for( ArteTable<RSEG>::iterator rsegIt = ArteTable<RSEG>::begin();
       rsegIt != ArteTable<RSEG>::end(); ++rsegIt ) 
    if(((*rsegIt)->fit >= 0)&&((*rsegIt)->cmp & Rsegc::patt)&&!((*rsegIt)->cmp & Rsegc::bitsign))
      {      
	if((*rsegIt)->cmp & Rsegc::wch)
	  {
	    flag=0;
	    rotr=(*rsegIt);
	  }
	if((*rsegIt)->cmp & Rsegc::msg)
	  {
	    flag=1;
	    ritr=(*rsegIt);
	  }

	for(ArteTable<RSEG>::iterator rsegIt2 = rsegIt;rsegIt2 != ArteTable<RSEG>::end(); ++rsegIt2) 
	  if(((*rsegIt2)->fit >= 0)&&((*rsegIt2)->cmp & Rsegc::patt)&&!((*rsegIt2)->cmp & Rsegc::bitsign))
	    {
	      if((flag==1)&&((*rsegIt2)->cmp & Rsegc::msg)) continue;
	      if((flag==0)&&((*rsegIt2)->cmp & Rsegc::wch)) continue;

	      if((flag==1)&&((*rsegIt2)->cmp & Rsegc::wch)) rotr=(*rsegIt2);
	      if((flag==0)&&((*rsegIt2)->cmp & Rsegc::msg)) ritr=(*rsegIt2);
	      if(QuickMatch(rotr,ritr))
		{
		  npair++;
		  pair.otr=rotr;
		  pair.itr=ritr;
		  Chi2Distance(&pair);
		  vp.push_back(pair);
		}
	    }
      }
  for(vector<RSEG_PAIR>::iterator pairIt = vp.begin();pairIt!=vp.end(); ++pairIt) 
    {
      cout<<"Otr:"<<(*pairIt).otr<<" Itr:"<<(*pairIt).itr<<" F="<<(*pairIt).F<<endl; 
    }
  cout<<"**** MatchOtrItr: Statistics ****"<<endl;
  cout<<notr<<" segs in OTR "<<nitr<<" segs in ITR "<<npair<<" pairs created"<<endl;


}
//===================================================

