// ********************************************************************
//
// Routine:      rc99Bremsstrahlung
// Author:       T. Oest
//
// Description:  this routine returns the Bremsstrahlung
//               for a given track
//
// Modification:
// 13/11/00 (DS) radius for Bremsstrahlung search fixed to 4 cm
//               (before old momentum dependent approximation)
// 28/11/00 (BG) radius for brem search is given by the cluster cell width
// 30/11/00 (AG) some polishing                 
// 16/07/01 (TG) return Pointer to Bremsstrahlungs cluster
//
// ********************************************************************

// includes
#include <iostream>

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

//
// function
//
bool findBremsstrahlung(const ArtePointer<RTRA>& rtra, ArtePointer<RCCL>& brems)
{
  ARTE_mark( "rc99Bremsstrahlung 0", 0 );

  //-----------------------------------------------------------------
  //   select two body decays
  //-----------------------------------------------------------------
  
  //  float energy    = 0;
  float minImpact = 100000;
  float ycut      = 0;
  static const float dimcell   = 11.15;

  if(distance(rtra->rccl.begin(), rtra->rccl.end()) == 0)
    return false;

  const ArtePointer<RCCL> rcclTrack  = *(rtra->rccl.begin());

  const ArteTableIterator<RCCL> end = ArteTable<RCCL>::end();
  for( ArteTable<RCCL>::iterator rcit = ArteTable<RCCL>::begin();
       rcit != end; ++rcit ) {
    const ArtePointer<RCCL> rccl = *rcit;

    if (rccl == rcclTrack) continue;
      
    const float yimpact  = rtra->yf + rtra->tyf * (rccl->z - rtra->zf) - rccl->y;
    const float ximpact  = rtra->xf + rtra->txf * (rccl->z - rtra->zf) - rccl->x;
    const float rimpact  = sqrt (ximpact*ximpact + yimpact*yimpact);
    if (rimpact < minImpact) {
      minImpact = rimpact;
      brems     = rccl;
      //	energy    = (*rccl)->esum;
      ycut = dimcell/static_cast<float>(rccl->flag & 0x7);
    }
  }
  
  // float ycut = (1.0 + 3. * (1. - energy / 100.))/sqrt(2);
  
  ARTE_mark( "findBremsstrahlung 1", 0 );

  if (minImpact > ycut) return false;

  return true;
}



