#ifndef __TABLEFILL_BASE_HH
#define __TABLEFILL_BASE_HH
// *****************************************************************************
//
// source:
//
// type:      header file
//
// created:   02. Jan 2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: abstract class to fill BEE ARTE tables
//
// changes:
// 02 Jan 2001 (TG) creation
// 03 Jan 2001 (TG) removed all member functions except Fill()
// 10 Sep 2001 (TG) bug fix in doc++ comments
//
// *****************************************************************************

/** Abstract class which defines the cluearte interface for user defined
    filling procedures. You can define your own BEE event filling routines by
    inheriting a class from \texttt{TableFillBase} and overloading the pure
    virtual functions of the base class.

    @memo BEE event filling interface.
    @author T. Glebe
 */
//============================================================================
// TableFillBase
//============================================================================
class TableFillBase {
public:
  /// returns false in case event should be skipped
  virtual bool EventSelect() const = 0;
  /// fill all tables, called in clueWriteEvent()
  virtual bool Fill()        const = 0;
  /// fill from RSEG table
  virtual bool FillRSEG()    const = 0;
}; // class TableFillBase
#endif
