#ifndef __CHECKPID_HH
#define __CHECKPID_HH
// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   16.07.2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Particle ID routine for RTRAS.
//
// changes:
// 27 Sep 2001 (TG) creation
// 28 Sep 2001 (TG) added PID_init() function
//
// **********************************************************************

/**
   helper class to keep PID information
   @author T. Glebe
 */
//==============================================================================
// PID_data
//==============================================================================
class PID_data {
public:
  /** @name --- Constructors --- */
  ///
  PID_data() :
    PID      (0),
    Lelectron(0.),
    Lmuon    (0.),
    Lpion    (0.),
    Lkaon    (0.),
    Lproton  (0.)
    {}

  /** @name --- Class Members --- */
  /// set all data to 0
  void clear() {
    PID=0;
    Lelectron = Lmuon = Lpion = Lkaon = Lproton = 0.;
  }

  /** @name global member data */
  /// particle identification flag
  int   PID;
  /// global electron likelihood
  float Lelectron;
  /// global muon likelihood
  float Lmuon;
  /// global pion likelihood
  float Lpion;
  /// global kaon likelihood
  float Lkaon;
  /// global proton likelihood
  float Lproton;
}; // PID_data

/** @name PID functions
 */
//@{
/** 
    Initialize HBOOK histos for PID monitoring. This function should be called in
    the ARTE usinit() function.

    @author T. Glebe
 */
//==============================================================================
// PID_init
//==============================================================================
void PID_init();

/** 
    Strategy: Basic PID-device is the RICH detector. If
    information from RISE is present, it will be used.
    Otherwise likelihoods from RITER are used. In case
    neither leads to a positive identification despite the
    fact that a ring is associated to the track, some
    private code is used. (The code in many cases gives
    results similar to the output of RISE.) Rare light
    particles ($e$,$\mu$) need a RICH likelihood above $0.99$
    to be identified. Pions are identified (essentially by
    default) if the likelihood exceeds $0.3$. Kaons are
    identified if the likelihood is larger than $0.6$
    and protons are required to have a likelihood larger
    than $0.3$. If hits in the muon system are present,
    then a particle with muon-likelihood above $0.0$ is
    identified as muon. Also, if no RICH information is
    present, a track with hits in the muon system is
    identified as a muon.
    
    @author Michael Schmelling
 */
//==============================================================================
// checkPID
//==============================================================================
const PID_data& checkPID(const ArtePointer<RTRA>& rtra);
//@}
#endif
