#ifndef __PID_DATA_HH
#define __PID_DATA_HH
// **********************************************************************
//
// source:
//
// type:      source code
//
// created:   27.09.2001
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: Helper class to keep PID information
//
// changes:
// 27 Sep 2001 (TG) creation
//
// **********************************************************************

/**
   helper class to keep PID information
   @author T. Glebe
 */
//==============================================================================
// PID_data
//==============================================================================
class PID_data {
public:
  /** @name --- Constructors --- */
  ///
  PID_data() :
    PID      (0),
    Lelectron(0.),
    Lmuon    (0.),
    Lpion    (0.),
    Lkaon    (0.),
    Lproton  (0.)
    {}

  /** @name --- Class Members --- */
  /// set all data to 0
  void clear() {
    PID=0;
    Lelectron = Lmuon = Lpion = Lkaon = Lproton = 0.;
  }

  /** @name global member data */
  /// particle identification flag
  int   PID;
  /// global electron likelihood
  float Lelectron;
  /// global muon likelihood
  float Lmuon;
  /// global pion likelihood
  float Lpion;
  /// global kaon likelihood
  float Lkaon;
  /// global proton likelihood
  float Lproton;
};
#endif
