// *****************************************************************************
//
// source:    daq_sparse_if.C
//
// type:      source code
//
// created:   13 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   functions for sparsified components
//            called by daqif.C
//
// changes:   
//
// *****************************************************************************
#include "arte/ArteMessage.hh"

extern "C" {
#include "esp.h"
	   }

#include "daqif/daq_sparse_if.hh"

/***
   *
   * This asks for the FED ids, then allocates them to one SLB
   *
   ***/

void DAQ_SPARSE_IF::setSLBs(void)
{
  setSLBsCalled = 1;

  unsigned short ids[MAX_FEDS];
  unsigned int nFEDs = getFED_IDs(ids, MAX_FEDS);

  fedIDs = new unsigned short[nFEDs];
  SLB = new SLB_struct[1];

  if (fedIDs == 0 || SLB == 0) 
    {
      arteMsg << "DAQ_SPARSE_IF::setSLBs failed, component: " << compid ;
      arteMsg.warn();
      return; // effectively ignoring this component
    }

  for(unsigned int i = 0; i < nFEDs; i++)
    fedIDs[i] = ids[i];

  SLB[0].first = fedIDs;
  SLB[0].nFEDs = nFEDs;

  nSLBs = 1; // no limit in SLB size for sparsified data

}


int DAQ_SPARSE_IF::prepare(void)
{
  if (setSLBsCalled == -1) {
    delete SLB;
    delete fedIDs;
    setSLBsCalled = 0;
  }
  if (setSLBsCalled == 0)
    setSLBs();

  int status = prepareEvent();

  if (status < 0) return status;

  //The return value of prepareEvent steers whether FED_IDs have to be re-read (status=1)

  if (status==1) setSLBsCalled = -1;

  return 0;
}

DAQ_SPARSE_IF::DAQ_SPARSE_IF(int component_id) : DAQ_MC_IF(component_id) {;}
