// *****************************************************************************
//
// source:    daq_svd_if.C
//
// type:      source code
//
// created:   14 Mar 2001
//
// author:    Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   functions to deliver the content of component SVD to the class
//            DAQ_SVD_IF
//            called by DAQ_MC_IF::buildEvent
//
// changes:   
//
// *****************************************************************************
#include "arte/ArteMessage.hh"
#include "daqif/daq_svd_if.hh"
#include "arte/arte.hh"

#include "marvin.h"

extern "C" {
#include "esp.h"
	   }
/***
   * The component SVD needs to override buildComponent
   * because its record has an entirely different structure 
   * from that of ordinary components
   *
   ***/

unsigned int DAQ_SVD_IF::buildComponent(char *buf, unsigned int maxSize)
{
  unsigned int nBytes = 0;
  static Marvin_Sevt_t  *sevt;

  if (fillSEVTfromHITB(0,0) != 0) 
   {
     arteMsg << "DAQ_SVD_IF::buildComponent: fillSEVTfromHITB not successful" ;
     arteMsg.warn();
     return nBytes;
   }

  sevt = marvin_fetch_sevt();

  Marvin_Header_t *Marvin_Header = (Marvin_Header_t*) sevt->data;

  int format = Marvin_Header->format;
  ESP_get_int(&format, &format, 1);
  if (format!=3)
   {
     arteMsg << "DAQ_SVD_IF::buildComponent: format " << format
             << " not supported." ;
     arteMsg.warn();
     return nBytes;
   }

  SLB_HDR *hdr = (SLB_HDR*) buf;
  nBytes = sizeof(SLB_HDR);

  maxSize = (nBytes < maxSize)? maxSize-nBytes : 0;

  int ssize = sizeof(short);
  int isize = sizeof(int);
  int csize = sizeof(char);

  int ncl = Marvin_Header->ncl;
  ESP_get_int(&ncl, &ncl, 1);

  unsigned int datasize = sizeof(Marvin_Header_t) + ncl*(4*csize+isize);
  unsigned int length = ssize+isize+ssize+datasize;

  if (length > 0x7FFF)
   {
     arteMsg << "DAQ_SVD_IF::buildComponent: length " << length
             << " is too much for sparse format." << endl;
     arteMsg.warn();
     return nBytes;
   }

  if (length > maxSize)
    {nBytes += length; maxSize=0;}
  else
    {
     short fltevt = get_fltevt();

     hdr->recid  = RID_SPARSE_SLB0;
     hdr->fltevt = fltevt;
     hdr->compid = COMP_ID_SVD;
     hdr->slbid  = 1;
     hdr->nodeid = 0;

     short sparselen;
     ESP_put_short( (short*)(&length), &sparselen, 1);
     memcpy(buf + nBytes, &sparselen, ssize);
     nBytes += ssize;

     int recid = RID_NORMAL_SPARSE;
     ESP_put_int(&recid, &recid, 1);
     memcpy(buf + nBytes, &recid, isize);
     nBytes += isize;

     ESP_put_short(&fltevt, &fltevt, 1);
     memcpy(buf + nBytes, &fltevt, ssize);
     nBytes += ssize;

     memcpy(buf+nBytes, sevt->data, datasize);
     nBytes += datasize;

     maxSize -= length;
    }

  if( sizeof(SLB_TAIL) <= maxSize )
    {
     hdr->length = nBytes + sizeof(SLB_TAIL);

     SLB_TAIL *tail = (SLB_TAIL*) (buf + nBytes);
     tail->recid = hdr->recid;
     tail->fltevt = hdr->fltevt;

     ESP_put_slb_hdr(hdr, hdr);
     ESP_put_slb_tail(tail, tail);
    }

  nBytes += sizeof(SLB_TAIL);

  return nBytes;
}
