// *****************************************************************************
//
// source:    daq_targ_if.C
//
// type:      source code
//
// created:   14 Mar 2001
//
// author:    Brian Petersen, Lars Soezueer
//            email: soezueer@mail.desy.de
//
// purpose:   functions to deliver FED ID and content to the class
//            DAQ_TARG_IF
//            called by DAQ_MC_IF::buildFED and DAQ_UNSPARSE_IF::setSLBs
//
// changes:   
//
// *****************************************************************************
#include "arte/ArteMessage.hh"
#include "daqif/daq_targ_if.hh"
#include "arte/arte.hh"

extern "C" {
#include "esp.h"
#include "tgts_raw.h"
	   }

unsigned int DAQ_TARG_IF::getFED_IDs(unsigned short *buf, unsigned int maxIDs)
{
  if(maxIDs < 1)
    return 0;

  buf[0] = 0;

  return 1;

}

int DAQ_TARG_IF::getFED(unsigned short fedid, FED_HDR *buf, unsigned int maxSize)
{

  if(fedid != 0)
    return 1;

  unsigned int length;
  if( (length = sizeof(FED_HDR) + sizeof(FED_TAIL) + sizeof(TGTS_RAW)) > maxSize)
    return 2;

  buf->length = length;
  buf->recid = 0x1000;
  buf->fltevt = 0;
  buf->bxid = 0;
  buf->compid = COMP_ID_TARG;
  buf->fedid = fedid;

  TGTS_RAW *traw = (TGTS_RAW *) (buf + 1);

  traw->time = faketime;
  traw->ia = 1000000.;
  traw->bg = 10000.;

  faketime += 1000; // why not?

  /*
    Assume the following order: above1, below1, inner1, outer1, above2, below2, inner2, outer2

       Above:  pos -> -pos
       Below:  pos ->  pos
     Used to be:  Inner:  pos ->  pos
     Used to be:  Outer:  pos -> -pos
     Now reverse...
   */

  for(int i = 0; i < 8; i++)
    traw->pos[i] = -15.;

struct mytarg {         /* This is hack to get rotations in - BP */
float x;
float y;
float z;
float dx;
float dy;
float dz;
float x0bm;
float y0bm;
} targmem,*targ=&targmem;

for (ArteTableIterator<GTAR> gt(1); gt.valid(); gt.next()) {

    // wnr doesn't correspond to the target group's wire number. So I'll distinguish
    // based on coordinates (not nice, I know). I'll also assume not rotations --
    // hope that's right!
    
    // wire 2 needs to be rotated, so we do the the general rotations first:
    
    targ->x=gt->rot[0]*(gt->x)+gt->rot[1]*(gt->y)+gt->rot[2]*(gt->z);  
    targ->y=gt->rot[3]*(gt->x)+gt->rot[4]*(gt->y)+gt->rot[5]*(gt->z);
    targ->z=gt->rot[6]*(gt->x)+gt->rot[7]*(gt->y)+gt->rot[8]*(gt->z);

    targ->dx=gt->rot[0]*(gt->dx)+gt->rot[1]*(gt->dy)+gt->rot[2]*(gt->dz);  
    targ->dy=gt->rot[3]*(gt->dx)+gt->rot[4]*(gt->dy)+gt->rot[5]*(gt->dz);
    targ->dz=gt->rot[6]*(gt->dx)+gt->rot[7]*(gt->dy)+gt->rot[8]*(gt->dz);

    // BP: These I had to hardcode :-(
    targ->x0bm=0.315;
    targ->y0bm=0.0;

    int station = (targ->z > -3.0)? 0 : 4;
    int vert = (targ->dx < 0.5)? 1 : 0; // vertical wire if skinny in x
    int wnr;

    if (gt->eff==0.) continue;      //Skip unused wires

    if(vert == 1)
      wnr = (targ->x - targ->x0bm > 0)? station + 2 : station + 3;
    else
      wnr = (targ->y - targ->y0bm > 0)? station : station + 1;

    switch (wnr) {
    case 0: traw->pos[wnr] = -targ->y; 
            break;
    case 1: traw->pos[wnr] =  targ->y;
            break;
    case 2: traw->pos[wnr] =  -targ->x;
            break;
    case 3: traw->pos[wnr] = targ->x;
            break;
    case 4: traw->pos[wnr] = -targ->y;
            break;
    case 5: traw->pos[wnr] =  targ->y;
            break;
    case 6: traw->pos[wnr] =  -targ->x;
            break;
    case 7: traw->pos[wnr] = targ->x;
            break;
    }

    traw->pos[wnr] *= 10.; // cm to mm

  }

  FED_TAIL *tail = (FED_TAIL *) (traw+1);
  tail->fltevt = buf->fltevt;

  ESP_put_fed_hdr(buf, buf);
  ESP_put_int( (int*) traw, (int*) traw, sizeof(TGTS_RAW)/sizeof(int) );
  ESP_put_fed_tail(tail, tail);

  return 0;
}

