*-- Author :    HBSTAFF   08/12/95
************************************************************************
*
* DGMUOO
* first version was written by A. Snijko at beginning of 1995
* corrected by B.Fominykh 04/12/95 - algorithm of digitization was changed
* corrected by B.Fominykh 07/02/96 - detailed structure of pad chambers
* corrected by B.Fominykh 12/02/96 - efficiency of chambers
* corrected by B.Fominykh 17/03/96 - digi for Ratnikov +
*                                    new pad parametrization
* corrected by B.Fominykh 05/06/96 - temporary ARTE tables
* corrected by B.Fominykh 20/11/96 - momentum cut for slow electrons +
*                                    cuts on collected charge
* modified by physjg      05/05/00 - for MC masking (call mumcmask)
* modified by physjg      05/29/00 - set 1/4 charge accepatnce for pads.
* modified by physjg      06/10/00 - check eff. channel by channel after checking mask
* modified by B.Fominykh  11/01/01 - bug correction
* modified by B.Fominykh  27/11/01 - pad chamber efficiency implemented
* modified by B.Fominykh  23/04/02 - efficiency from DB
* modified by O.Barsukova  6/05/02 - digi output instead of hitb
*
* This routine makes a digitization of muon tube and pad chambers
* INPUT: MC-impact points to detector collected in table MIMP
* OUTPUT: hits in tables DIGB and HITB
*
*       called from DGMAIN
*
* Arguments  kind         description
* IPL        input        MC-detector number
*
************************************************************************


      SUBROUTINE DGMUOO(IPL)
C     =======================
*
      IMPLICIT NONE
      INTEGER IPL
#include "arte/Arte.fi"
#include "auto/GEO.fi"
#include "auto/MC.fi"
#include "auto/DIGB.fi"
#include "auto/HITB.fi"
      REAL DISWIR,ACCUR,XFWIR,RNDM,EFF,ALSX,ALSY,YS1,YS2
      REAL XD1,XD2,YD1,YD2,ZD1,ZD2,COSR,SINR
      INTEGER NHITS,IH,IHH,IHRT,IFHIT
      REAL PMCIP,XMCIP,YMCIP,ZMCIP,XIPD,YIPD,X1,Y1,X2,Y2,TGX,TGY
      REAL YSLUN,YSLDN,YW1,YW2
      REAL YEFF,DYEFF
      REAL XL1,XL2,YL1,YL2,DY1,DY2
      REAL ZPAD,DYPD1,DYPD2,DYPD3,DY
      INTEGER NWMX,NPMX,NWM1
      INTEGER I,J,K,I1,I2,IST,IFN
      INTEGER NHITOD,NHITOT,NCROS
      INTEGER NMCS,NMS
      INTEGER IR,JJ,IPWW,NMCTRW,NIPWW
      REAL DXPAD,YREF,EST
      REAL XW,YW,ZW,XCR(2),YCR(2),ZCR(2)
      REAL ZWIR,ZC,Z1,Z2,XWWD
      REAL SS,SWTCUT,SWPCUT,SSPCUT,WNOR
      
      INTEGER IWIRH,NIPWIR,IPWIRH
      REAL    XWIRH,ZWIRH,SSWIRH
      INTEGER NIPPAD,IPPADH,NWMAX,NPMAX
      REAL    SSPAD,XPADH,YPADH,ZPADH,SSP,YWL,YWU
      COMMON/WRKSP1/IWIRH(16),NIPWIR(16),XWIRH(16),ZWIRH(16),
     +      SSWIRH(16),SSPAD(31,2),NIPPAD(31,2),XPADH(31,2),
     +      YPADH(31,2),ZPADH(31,2),SSP(31,2),YWL(31),YWU(31)


* parameters
      DATA SWTCUT  / 0.001 /
      DATA SWPCUT  / 0.001 /
      DATA SSPCUT  / 0.001 /

*
#include "arte/Arte.ff"
#include "auto/GEO.ff"
#include "auto/MC.ff"
#include "auto/DIGB.ff"
#include "auto/HITB.ff"

      IPWIRH(J,I) =IQ(JTMP1+IQ(JTMP1+1)*(J-1)+2+I)
      IPPADH(J,I,K) =IQ(JTMP2+IQ(JTMP2+1)*(2*J+K-3)+2+I)
C----+===1=========2=========3=========4=========5=========6=========7==*******
*
* check-point
      IFHIT  = IGEDE_MIM1(IPL)          ! pointer to MIMP table
      NHITS = IGEDE_MIML(IPL) -IFHIT +1 ! number of MIMPs

      IF(NHITS.LE.0.OR.IFHIT.EQ.0) GOTO 998 ! no impact points

* book temporary ARTE tables
      CALL MMBWRK(JTMP1,0,16,300)
      CALL MMBWRK(JTMP2,0,32,300)

* clear indicators of hitted wires and pads
      NWMAX=16
      CALL VZERO(IWIRH,NWMAX)
      CALL VZERO(NIPWIR,NWMAX)
      NPMAX=62
      CALL VZERO(NIPPAD,NPMAX)
* clear SSPAD(31,2)
        DO I=1,31
        DO IR=1,2
          SSPAD(I,IR)=0.
        ENDDO
        ENDDO

* MC-superlayer number
      NMCS=OGEDE_GESL(IPL)
* local superlayer of muon system
      NMS=I_V(JGESL_NSL,NMCS,JGESL)
* min and max Y of superlayer for absolute pad numeration
      IF(NMS.EQ.3.OR.NMS.EQ.4) THEN
        YSLUN= 10000.
        YSLDN= 10000.
        JJ=IGESL_NPL(NMCS)
        J =IGESL_FPL(NMCS)
        DO I=J,J+JJ-1
          YD1 =  FGEDE_YMIN(I)
          YD2 =  FGEDE_YMAX(I)
          IF(ABS(YD1/(YD2-YD1)).GT.0.7) THEN
           IF(YD1.LT.YSLDN) YSLDN=YD1
          ELSE
            IF(YD1.LE.YSLUN) YSLUN=YD1
          ENDIF
        ENDDO
      ENDIF
* get parameters of detector IPL in detector coordinate system
* definition: min X, min Y corner has coordinates x=0,y=0
      DISWIR =  FGEDE_PIT(IPL)   ! distance between wires
      ACCUR  =  FGEDE_DOD(IPL)   ! half of x-size of sensitive volume
      XFWIR  =  FGEDE_XFW(1,IPL) ! X-position of first wire
      XD1 =  FGEDE_XMIN(IPL)    ! x of left corner
      XD2 =  FGEDE_XMAX(IPL)    ! x of right corner
      YD1 =  FGEDE_YMIN(IPL)    ! y of low edge
      YD2 =  FGEDE_YMAX(IPL)    ! y of upper edge
      YS1 =  FGEDE_SYMN(IPL)    ! Ymin of sensitive volume
      YS2 =  FGEDE_SYMX(IPL)    ! Ymax of sensitive volume
* up or down 
      IF(ABS(YD1/(YD2-YD1)).GT.0.7) THEN
        YREF=YD2-YD1
      ELSE
        YREF=0.
      ENDIF
* number of wires in detector
      NWMX=IGEDE_NWI(1,IPL)
      NWM1=NWMX/2
* z-position of begining and end of MC-volume (detector)
      ZD1=FGEDE_ZMIN(IPL)
      ZD2=FGEDE_ZMAX(IPL)
      ZWIR=FGEDE_ZFW(1,IPL)   ! z-position of wire
* half of Z-size of sensetive volume for tube chambers
      IF(NMS.EQ.1.OR.NMS.EQ.2) 
     +       ZC=0.5*(ZD2-ZD1)-DISWIR+2.*ACCUR
* sin,cos of rotation angle
      COSR=FGEDE_ROT(1,IPL)
      SINR=FGEDE_ROT(2,IPL)
* pad parameters only for MUON-superlayers 3 and 4
        IF(NMS.EQ.3.OR.NMS.EQ.4) THEN
* pad  parametes
* number of pads in row
          NPMX =IGEDE_NWI(3,IPL)
* geometry of pads
          YL2  =0.01*REAL(IGEDE_NWI(4,IPL)) ! y-dist between pads
          DXPAD=FGEDE_XFW(2,IPL)            ! x-size of pad
          DYPD1=FGEDE_XFW(3,IPL)            ! y-size of first pad
          DYPD2=FGEDE_XFW(4,IPL)            ! y-size of standard pad
          ZPAD =FGEDE_ZFW(2,IPL)            ! z-position of pad
          DYEFF=FGEDE_ZFW(3,IPL)   ! y-dist from edge to point with eff 0
          YEFF =FGEDE_ZFW(4,IPL)   ! efficiency between two pads
          ALSX =FGEDE_SXMX(IPL)-FGEDE_SXMN(IPL)
          XL1  =(XD2-XD1-ALSX)/2.  ! x-dist from edge to pad
          XL2  =(ALSX-2.*DXPAD)    ! x-dist between pads
          ALSY =YS2-YS1
          YL1  =(YD2-YD1-ALSY)/2.  ! y-dist from edge to pad
* y-size of last pad
          DYPD3=ALSY-DYPD1-YL2-(DYPD2+YL2)*REAL(NPMX-2)
* half of Z-size of sensitive volume
          ZC=ABS(ZWIR-ZPAD)
* low and up y-coordinates of pads
          YWL(1)=YL1-0.5*YL2
          YWU(1)=YL1+DYPD1+0.5*YL2
          DO I=2,NPMX
            YWL(I)=YL1+DYPD1+0.5*YL2+(DYPD2+YL2)*REAL(I-2)
            YWU(I)=YL1+DYPD1+(DYPD2+YL2)*REAL(I-1)+0.5*YL2
          ENDDO
          YWU(NPMX)=YL1+ALSY+0.5*YL2
*
        ENDIF
*
* loops over impact points to  detector IPL
      DO 20 IH=1,NHITS
        IHH = IH+IFHIT-1    ! pointer to impact point in bank MIMP
* momentum
        PMCIP=F_V(JMIMP_P,IHH,JMIMP)
* coordinates of MC-impact point from bank MIMP
        XMCIP=F_V(JMIMP_X,IHH,JMIMP)
        YMCIP=F_V(JMIMP_Y,IHH,JMIMP)
        ZMCIP=F_V(JMIMP_Z,IHH,JMIMP)
* rotate MC-impact point to detector system
        XIPD=XMCIP*COSR-YMCIP*SINR-XD1
        YIPD=XMCIP*SINR+YMCIP*COSR-YD1
* TGX=DX/DZ AND TGY=DY/DZ of track inside detector in detector system
        TGX=FMIMP_TX(IHH)*COSR-FMIMP_TY(IHH)*SINR
        TGY=FMIMP_TX(IHH)*SINR+FMIMP_TY(IHH)*COSR
* start (1) and finish (2) points 
        Z1=ZWIR-ZC
        Z2=ZWIR+ZC
* forward direction
        IF(PMCIP.GE.0..AND.ZMCIP.GT.Z2) GOTO 20
        IF(PMCIP.GE.0..AND.ZMCIP.GT.Z1) Z1=ZMCIP
* backward direction
        IF(PMCIP.LT.0..AND.ZMCIP.LT.Z1) GOTO 20
        IF(PMCIP.LT.0..AND.ZMCIP.LT.Z2) Z2=ZMCIP
        X1=XIPD+TGX*(Z1-ZMCIP)
        Y1=YIPD+TGY*(Z1-ZMCIP)
        X2=XIPD+TGX*(Z2-ZMCIP)
        Y2=YIPD+TGY*(Z2-ZMCIP)
*** start the wire business ***
* nearest wire for 1st point
        I1=INT((X1-XFWIR)/DISWIR+0.5)+1
        IF(I1.LT.1.AND.X2.GT.X1) I1=1
        IF(I1.GT.NWMX.AND.X2.LT.X1) I1=NWMX
* nearest wire for 2nd point
        I2=INT((X2-XFWIR)/DISWIR+0.5)+1
        IF(I2.LT.1.AND.X1.GT.X2) I2=1
        IF(I2.GT.NWMX.AND.X1.LT.X2) I2=NWMX
* check for sure 
        IF(I1.LT.1.AND.I2.LT.1)       GOTO 20
        IF(I1.GT.NWMX.AND.I2.GT.NWMX) GOTO 20
*
        IF(I1.LT.1) I1=1
        IF(I1.GT.NWMX) I1=NWMX
        IF(I2.LT.1) I2=1
        IF(I2.GT.NWMX) I2=NWMX
*
* clear SSP
        DO I=1,31
        DO IR=1,2
          SSP(I,IR)=0.
        ENDDO
        ENDDO
* loops over wires
        IST=I1
        IF(I1.GT.I2) IST=I2
        IFN=I2
        IF(I1.GT.I2) IFN=I1
        DO 10 I=IST,IFN
* wire x-coordinate
          XWWD=XFWIR+REAL(I-1)*DISWIR
* calculate the length of particle trace in sensitive volume near the wire
          NCROS=0
* cross the 1st sensitive volume boundary
          ZW=ZWIR-ZC
          IF(ZW.LT.Z1-0.001) GOTO 71
          XW=XIPD+TGX*(ZW-ZMCIP)
          IF(XW.LT.XWWD-ACCUR) GOTO 71 
          IF(XW.GT.XWWD+ACCUR) GOTO 71 
          YW=YIPD+TGY*(ZW-ZMCIP)
          IF(YW.LT.0.) GOTO 71 
          IF(YW.GT.YD2-YD1) GOTO 71 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
  71      CONTINUE
* cross the 2nd sensitive volume boundary
          ZW=ZWIR+ZC
          IF(ZW.GT.Z2+0.001) GOTO 72
          XW=XIPD+TGX*(ZW-ZMCIP)
          IF(XW.LT.XWWD-ACCUR) GOTO 72 
          IF(XW.GT.XWWD+ACCUR) GOTO 72 
          YW=YIPD+TGY*(ZW-ZMCIP)
          IF(YW.LT.0.) GOTO 72 
          IF(YW.GT.YD2-YD1) GOTO 72 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
          IF(NCROS.EQ.2) GOTO 81
  72      CONTINUE
* cross the 3d sensitive volume boundary
          XW=XWWD-ACCUR
          IF(ABS(TGX).LT.1.E-6) GOTO 74
          IF(XW.GT.X1+1.E-5.AND.XW.GT.X2+1.E-5) GOTO 74
          IF(XWWD+ACCUR.LT.X1-1.E-5.AND.XWWD+ACCUR.LT.X2-1.E-5) GOTO 74
          IF(XW.LT.X1-1.E-5.AND.XW.LT.X2-1.E-5) GOTO 73
          ZW=ZMCIP+(XW-XIPD)/TGX
          IF(ZW.LT.ZWIR-ZC) GOTO 73 
          IF(ZW.GT.ZWIR+ZC) GOTO 73 
          YW=YIPD+TGY*(ZW-ZMCIP)
          IF(YW.LT.0.) GOTO 73 
          IF(YW.GT.YD2-YD1) GOTO 73 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
          IF(NCROS.EQ.2) GOTO 81
  73      CONTINUE
* cross the 4th sensitive volume boundary
          XW=XWWD+ACCUR
          IF(XW.GT.X1+1.E-5.AND.XW.GT.X2+1.E-5) GOTO 74
          ZW=ZMCIP+(XW-XIPD)/TGX
          IF(ZW.LT.ZWIR-ZC) GOTO 74 
          IF(ZW.GT.ZWIR+ZC) GOTO 74 
          YW=YIPD+TGY*(ZW-ZMCIP)
          IF(YW.LT.0.) GOTO 74
          IF(YW.GT.YD2-YD1) GOTO 74 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
          IF(NCROS.EQ.2) GOTO 81
  74      CONTINUE
* cross the 5th sensitive volume boundary
          YW=0.
          IF(ABS(TGY).LT.1.E-4) GOTO 76
          IF(YW.GT.Y1+1.E-5.AND.YW.GT.Y2+1.E-5) GOTO 76
          IF(YD2-YD1.LT.Y1-1.E-5.AND.YD2-YD1.LT.Y2-1.E-5) GOTO 76
          IF(YW.LT.Y1-1.E-5.AND.YW.LT.Y2-1.E-5) GOTO 75
          ZW=ZMCIP+(YW-YIPD)/TGY
          IF(ZW.LT.ZWIR-ZC) GOTO 75 
          IF(ZW.GT.ZWIR+ZC) GOTO 75 
          XW=XIPD+TGX*(ZW-ZMCIP)
          IF(XW.LT.XWWD-ACCUR) GOTO 75 
          IF(XW.GT.XWWD+ACCUR) GOTO 75 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
          IF(NCROS.EQ.2) GOTO 81
  75      CONTINUE
* cross the 6th sensitive volume boundary
          YW=YD2-YD1
          IF(YW.GT.Y1+1.E-5.AND.YW.GT.Y2+1.E-5) GOTO 76
          IF(ABS(TGY).LT.1.E-4) GOTO 76
          ZW=ZMCIP+(YW-YIPD)/TGY
          IF(ZW.LT.ZWIR-ZC) GOTO 76
          IF(ZW.GT.ZWIR+ZC) GOTO 76 
          XW=XIPD+TGX*(ZW-ZMCIP)
          IF(XW.LT.XWWD-ACCUR) GOTO 76 
          IF(XW.GT.XWWD+ACCUR) GOTO 76 
          NCROS=NCROS+1
          XCR(NCROS)=XW
          YCR(NCROS)=YW
          ZCR(NCROS)=ZW
  76      CONTINUE
  81      CONTINUE
* no crosses - nothing to do
          IF(NCROS.LT.1) GOTO 10
          IF(ABS(XCR(1)-XWWD).GT.ACCUR+0.01) GOTO 10
* the case when the particle was borned inside the sensitive volume
          IF(NCROS.EQ.1) THEN
            XCR(2)=XIPD
            YCR(2)=YIPD
            ZCR(2)=ZMCIP
          ENDIF
          IF(ABS(XCR(2)-XWWD).GT.ACCUR+0.01) GOTO 10
* the length of particle trace inside sensitive volume
       SS=SQRT((XCR(1)-XCR(2))**2+
     +         (YCR(1)-YCR(2))**2+(ZCR(1)-ZCR(2))**2)
          IF(SS.LT.0.01) GOTO 10
*
* cut on energy for slow electrons
          IF(ABS(PMCIP).LT.0.02) THEN
            CALL DGMUCUT(NMS,IPL,I,PMCIP,XIPD,YIPD,ZMCIP,TGX,TGY,EST)
            IF(ABS(PMCIP).LT.EST) GOTO 10
          ENDIF
* distribute SS between pads
          IF(NMS.EQ.3.OR.NMS.EQ.4) THEN
* row number
            IR=1
            IF(I.GT.NWM1) IR=2
            YW1=ABS(YCR(1)-YREF)
            YW2=ABS(YCR(2)-YREF)
            IF(YW1.GT.YW2) THEN
              YW1=YW2
              YW2=ABS(YCR(1)-YREF)
            ENDIF
            DY1=YW1-DYEFF
            DY2=YW2+DYEFF
            WNOR=DYEFF+YW2-YW1
* loopes over pads
      DO 91 J=1,NPMX
        IF(YWU(J).LT.DY1) GOTO 91
        IF(YWL(J).GT.DY2) GOTO 92
* all charge to one pad
      IF(DY1.GE.YWL(J).AND.DY2.LE.YWU(J)) THEN
        SSP(J,IR)=SSP(J,IR)+SS
        GOTO 92
      ENDIF
* only a part of total charge
      IF(YWL(J).LE.DY1) THEN
        IF(YWU(J).GT.YW1) THEN
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*0.5*DYEFF
          IF(YWU(J).GT.YW2) THEN
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YW2-YW1)
            IF(YWU(J).GT.DY2) THEN
              SSP(J,IR)=SSP(J,IR)+SS/WNOR*0.5*DYEFF
              GOTO 91
            ELSE
              SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +        (1.-0.5*(YWU(J)-YW2)/DYEFF)*(YWU(J)-YW2)
              GOTO 91
            ENDIF
          ELSE
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YWU(J)-YW1)
            GOTO 91
          ENDIF
        ELSE
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*0.5/DYEFF*
     +              (YWU(J)-DY1)**2
          GOTO 91
        ENDIF                
      ENDIF

      IF(YWL(J).GT.DY1.AND.YWL(J).LE.YW1) THEN
        IF(YWU(J).GT.YW1) THEN
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +      (1.-0.5*(YW1-YWL(J))/DYEFF)*(YW1-YWL(J))
          IF(YWU(J).GT.YW2) THEN
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YW2-YW1)
            IF(YWU(J).GT.DY2) THEN
              SSP(J,IR)=SSP(J,IR)+SS/WNOR*0.5*DYEFF
              GOTO 91
            ELSE
              SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +        (1.-0.5*(YWU(J)-YW2)/DYEFF)*(YWU(J)-YW2)
              GOTO 91
            ENDIF
          ELSE
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YWU(J)-YW1)
            GOTO 91
          ENDIF
        ELSE
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +              0.5*(YWL(J)+YWU(J)-2.*DY1)
          GOTO 91
        ENDIF                
      ENDIF

      IF(YWL(J).GT.YW1.AND.YWL(J).LE.YW2) THEN
        IF(YWU(J).GT.YW2) THEN
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YW2-YWL(J))
          IF(YWU(J).GT.DY2) THEN
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*0.5*DYEFF
            GOTO 91
          ELSE
            SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +      (1.-0.5*(YWU(J)-YW2)/DYEFF)*(YWU(J)-YW2)
            GOTO 91
          ENDIF
        ELSE
          SSP(J,IR)=SSP(J,IR)+SS/WNOR*(YWU(J)-YWL(J))
          GOTO 91
        ENDIF
      ENDIF

      IF(YWL(J).GT.YW2.AND.YWL(J).LE.DY2) THEN
        SSP(J,IR)=SSP(J,IR)+SS/WNOR*
     +    0.5/DYEFF*(DY2-YWL(J))**2
        GOTO 91
      ENDIF

  91        CONTINUE
  92        CONTINUE
*
          ENDIF
*
* fill the hited wires indicator and save info for bank HITB
          JJ=NIPWIR(I)+1
          IF(JJ.GT.IQ(JTMP1+2)) CALL MMBOOK(JTMP1,JJ-IQ(JTMP1+2))
          NIPWIR(I)=JJ
          IQ(JTMP1+IQ(JTMP1+1)*(JJ-1)+2+I)= IHH
* collect charge from different i.p.
          SSWIRH(I) = SSWIRH(I) + SS
          IF(IWIRH(I).EQ.1) GOTO 10
          XWIRH(I) = XWWD
          ZWIRH(I) = ZWIR
          IWIRH(I) = 1
  10    CONTINUE
*
*****  start the pad business *****

* only MUON-superlayers 3 and 4 are accepted
        IF(NMS.NE.3.AND.NMS.NE.4) GOTO 20
* loop over 2 rows of pads
        DO 15 IR=1,2
* loop over pads
          DO 13 I=1, NPMX
* was pad hitted?
            IF(SSP(I,IR).LE.0.) GOTO 13 
* first time?
            IF(SSPAD(I,IR).LE.0.) THEN
* x-position of pad center
              IF(IR.EQ.1)
     +        XPADH(I,IR)=XD1+XL1+DXPAD*.5
              IF(IR.EQ.2)
     +        XPADH(I,IR)=XD2-XL1-DXPAD*.5
* y-position of pad center (up and down separately)
              IF(YREF.EQ.0.) THEN
                YPADH(I,IR)=YD1+0.5*(YWL(I)+YWU(I))
              ELSE
                YPADH(I,IR)=YD2-0.5*(YWL(I)+YWU(I))
              ENDIF
              ZPADH(I,IR)=ZPAD
            ENDIF
* collect charges from different i.p.
*            SSPAD(I,IR)=SSPAD(I,IR)+SSP(I,IR)

* 2000,0529 physjg : pad should only get about 1/4 induced charges 
            SSPAD(I,IR)=SSPAD(I,IR)+SSP(I,IR)/4

* +1 to number of particles hitted this pad
            JJ=NIPPAD(I,IR)+1
            IF(2*JJ.GT.IQ(JTMP2+2))
     +        CALL MMBOOK(JTMP2,2*JJ-IQ(JTMP2+2))
            NIPPAD(I,IR)=JJ
* save pointer to impact point
            IQ(JTMP2+IQ(JTMP2+1)*(2*JJ+IR-3)+2+I)=IHH
  13      CONTINUE
  15    CONTINUE
  20  CONTINUE


*     write the hitted muon wires to table DMUT
*     loops over wires of detector

      DO 30 IH=1,NWMX
*     check if the wire was hitted
         IF(IWIRH(IH).NE.1) GOTO 30

*     cut on collected charge
         IF(NMS.GE.3.AND.SSWIRH(IH).LT.SWPCUT) GOTO 30
         IF(NMS.LE.2.AND.SSWIRH(IH).LT.SWTCUT) GOTO 30


*     EFF = FGEDE_EFF(IPL)       ! efficiency from geo-file
         call mueffw(ipl,IH,EFF) ! efficiency from DB
         CALL GRNDM(RNDM,1)
***         print*,'dgmuoo: tube eff ',EFF,' rnd ',RNDM,
***     *          '  gede,wire ',ipl,IH 
         IF(RNDM.GT.EFF) GOTO 30


*     MuMCTubHit2Digi
         IF (NMS .EQ. 1 .OR. NMS .EQ. 2) CALL mumctubhit2digi(ipl,IH)
*     MuMCPadHit2Digi
         IF (NMS .EQ. 3 .OR. NMS .EQ. 4) CALL mumcpadhit2digi(ipl,IH)
         
 30   CONTINUE
      

* write the hitted muon pads to table DMUP
* loops over 2 rows of pad chamber
      DO 40 IR=1,2
* loops over pads
         DO 39 IH=1,NPMX
* cut on collected charge
            IF(SSPAD(IH,IR).LE.SSPCUT) GOTO 39

*           EFF = FGEDE_EFF(IPL)         ! efficiency from geo-file
            call mueffp(ipl,IR,IH,EFF)   ! efficiency from DB
            CALL GRNDM(RNDM,1)
***            print*,'dgmuoo: pad eff ',EFF,' rnd ',RNDM,
***     *             '  gede,row,pad ',ipl,IR,IH 
            IF(RNDM.GT.EFF) GOTO 39


*     MuMCPad2Hit2Digi
            IF (NMS .EQ. 3 .OR. NMS .EQ. 4)
     @           CALL mumcpad2hit2digi(ipl,IR,IH)

 39      CONTINUE
 40   CONTINUE

 998  CONTINUE

      RETURN
      END
