//
//   MC Digitization Routine for RICH
//
//
//   Author: R.Eckmann
//   Modified: D.Dujmic (1999/Feb/2)
//
//       25 May, 2000 (DD): HITB->chid=number of pmt channels
//
///////////////////////////////////////////////////////////////////


#include "rich/PhotonDetection.h"
#include "rich/vessel.h"
#include "rich/PhiLambda.h"


int gasfix(Photon &ph, ArtePointer<MIMP> mimp);

extern "C" {

  extern void ricini_();


  void dgrich_(int *gede) {

    if (!RICH_Detector)  ricini_();

    int  daqid;
    DIGB digit;
    HITB hit;
    Photon p;

    ArtePointer<GEDE> gept(*gede);
    gept->dig1=ArteTable<DIGB>::end();
    gept->hit1=ArteTable<HITB>::end();

    // --- loop over RICH MIMP's
    int n0=gept->mim1.index();
    int n1=gept->miml.index()+1;
    for (int imimp=n0; imimp<n1; imimp++) {

      ArtePointer<MIMP> mimp(imimp);
      if (mimp->gede.index()!=unsigned(*gede)) {
	printf("dgrich: no photon %d %d\n",mimp->gede.index(),*gede);getchar();
	continue;
      }

      // --- conversion of MIMP to Photon
      p.x()=Vec3(mimp->x*10,mimp->y*10,mimp->z*10);
      p.n()=Vec3(mimp->tx,mimp->ty,mimp->p);
      p.e()=norm(p.n());
      p.n()/=p.e();
      if  (fabs(mimp->q)>1e-6 || fabs(mimp->time)<=1) {  // new mimp structure (mimp->q is used, mimp->time is cos)   
	p.polarisation()=FromPolar(1,mimp->time,mimp->q);
      }
      else {  // old mimp structure
	int ipol=int(mimp->time);     
	p.polarisation()=
	  FromPolar(1,double(ipol/10000)/1000.,double(ipol%10000)/1000.);
      }

      if (gasfix(p,mimp)) continue; 
	
      // --- raytracing & detection
      if (RICH_Vessel->raytrace(&p)) continue;
      if (RICH_Detector->dodetect(&p,&daqid)) continue;
      if (RICH_Detector->ChannelExist(daqid)) {
	
	// --- check if cell is hit twice
	ArteTable<HITB>::iterator ihitb=gept->hit1;
	ArteTable<DIGB>::iterator idigb=gept->dig1;
	for ( ;idigb!=ArteTable<DIGB>::end(); ++idigb,++ihitb)
	  if ((*idigb)->id==daqid) break;
	ArtePointer<DIGB> digb(idigb);
	ArtePointer<HITB> hitb(ihitb);

	// --- append if id is not present in the table
	if (idigb==ArteTable<DIGB>::end()) {
	  // --- hit id
	  digit.id  =daqid;
	  digb=ArteTable<DIGB>::push_back(digit);

	  // --- hit info
	  Vec3 pos=RICH_Detector->pos();
	  hit.x=pos.x()/10.;
	  hit.y=pos.y()/10.;
	  hit.z=pos.z()/10.;
	  int ci=RICH_Detector->ChannelIndex();
	  hit.q=RICH_LambdaPhi->phi(ci);
	  hit.r=RICH_LambdaPhi->lambda(ci);
	  hit.gede=mimp->gede;
	  hit.chid=RICH_Detector->pm().numchannel();
	  hitb=ArteTable<HITB>::push_back(hit);

	  // relation: 
	  // hit to digit: one to one relation (used: many digit to one hit)
	  createRelation(hitb,digb,&HITB::digb);
	}        

	// relation: 
	// mimp to digit: many to many relation
	createRelation(mimp,digb,&MIMP::digb);
	// relation: 
	// mimp to hit:   many to many relation
	createRelation(hitb,mimp,&HITB::mimp);
      }
    }
    gept->digl=ArteTable<DIGB>::end()-1;
    gept->hitl=ArteTable<HITB>::end()-1;
  }


	   } // end extern "C"
