// *******************************************************************
// source:    DQ_ITR_hit_evnt.C
// author:    Sonja Wunderlich
// called by: DQ_evnt()
//
// changes:
// 26 Jun 2000 (AG) creation
// 10/30 (SW) count hits per event for every half station
// 12/19 (SW) hits per event does not work
// 12/19 (SW) hits distribution for every half station
// 12/19 (SW) count dead helix chips and dead readout
// 12/19 (SW) publish 2d occupancyplots
// 01/17 (SW) fix bug in dead readout counting
// 01/22 (SW) Funktion to fill rhp-and hbook histograms    
// 01/22 (SW) Bug with dead helix counting when TR<2, now it is when TR<1
// 01/25 (SW) hits per event per ITR
// 01/27 (SW) 2d histogram not working/off chips and chamber
// 01/30 (SW) 2d histogram real occupancy per chamber
// 01/30 (SW) if (targetrate < 1.0) targetrate = 1.0; realy dirty!!!! 
// 2/5   (SW) fix another bug in the counting of deadhelixes...
// 2/5   (SW) split count of deadsigbord in deadsigboard and deadchamber
// 2/8   (SW) change numbering of SLindex now 0 = +1; 1 = -1 ...
// 2/9   (SW) take in account dead chambers at hit distribution
// 3/6   (SW) remove the calculation of dead behavior
// 4/4   (SW) include noise behavior check
// 4/9   (SW) include working chamber histogram (over the run)
// 4/12  (SW) reset hitcounter (all) by ResetByEvent (2000)
// 4/13  (SW) include BX specturm per chamber (only random triggered events
// 5/17  (SW) fill hitsSL in a 2 dim plot 
// 5/22  (SW) remove the 20 hitsperevent -plots per station 
// 7/3   (SW) hitcalculation only for random triggert events
// 9/4   (SW) take care to use no DQ when there are Monte Carlo's  
// 1/4/02 (JK) Adaption to new geometry (missing magnet stations), history histograms disabled
// 04/07/02 (JH) error printout commented out
// ******************************************************************
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#include "rhp.h"
#include "hbook.h"
#include "event.h"
#include "itrleda.h"

#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_ITR.h"

#include "ITR_marvin.h"

const extern int itr_evnt_cnt;
const float k_lessthanfactor = 0.5;
//static int chamber[184];
//static int chamberinit = 0;


int DQ_ITR_Noisebehavior();

int DQ_ITR_hit_evnt(void){
  static int event_count = 0;

#if defined (DQ_ITR_COMPILE)
  
  if(event_count == 0)
    cout<<"EEEEEEEEEEEEEEEEEE DQ_ITR_hit_event.C EEEEEEEEEEEEEEEEEEEEEEEE"<<endl;
#endif

  ARTE_mark( "DQ_ITR_hit_evnt 0", 0 );

  static int MeanAverageHitsEventSL[20] = {0};

  int SLindex, SLnr;
  int helix_counter;
  int strips = 0;
  int hi_id = 0;
  int chambernr = 0;   
  float norm;                        //normalize on target rate

  // check (again) if this is Monte Carlo
  if (ArteTable<MTRA>::size() > 0) {
    ARTE_mark( "DQ_ITR_hit_evnt 1", 0 );  
    return (GRR_OK);
  }



#if defined (DQ_ITR_CONTROL)
  if(faDDAQFCS_tcode() >= 8){                   // only randomtrigger 
    ArtePointer<DIHD> dihd(1);
    int BX = dihd->bx;
    DQ_ITR_my_hf1_fill(h_BXSpectrumEvent, LVL_HIT, (float)BX, 1.0);
  }
#endif

  event_count ++ ;
  float targetrate;                  // target rate 
  targetrate = faDTARia();            //get target rate
  if ( targetrate > 1.0)
    norm= 1./(float)targetrate ;  
  else 
    norm = 1;

#if defined (DQ_ITR_CONTROL)
  DQ_ITR_my_hf1_fill(h_ITR_targetrate, LVL_HIT, targetrate, 1.);   

#endif

  float ITR_hitsSL[20] = {0};
  float ITR_hitsallworkingCham = 0;
  int ITR_hitsall = 0;
  int ITR_hitsallundercut= 0;
   
  // calculate hits per event and halfstation
  for (ArteTableIterator<HITB> hitbit; hitbit.valid(); hitbit.next()) {
    ArtePointer<HITB> hitb (*hitbit);
    ArtePointer<GEDE> gede (hitb->gede);
    if(faDDAQFCS_tcode() >= 8){                   // only randomtrigger 
      if (gede->cmp == 3){                        //  ITR is component 3 in ARTE 

	string test_s, hwc_s;
	test_s = gede->hwc;
	hwc_s = test_s.substr(0,6);

	//SLindex = 2*(gede->gesl->nsl - 1);        // we have 10 stations 
	SLindex = 2*(DQ_ITR_SL_Chambers(hwc_s) - 1);        // we have 10 stations 
	if (gede->sec > 4)                         
	  SLindex++;
	float SignalOverNoise = hitb->q;
	if(SignalOverNoise >= SigalOverNoiseCut){
	  ITR_hitsall ++;
	  ITR_hitsSL[SLindex]++;
	}
	else
	  ITR_hitsallundercut++;
      }
    }
  }
  int hitsHel[k_numChamber][k_HelixPerChamber];
  for (int i=0; i < k_numChamber; i++)
    for (int j=0; j < k_HelixPerChamber; j++)
      hitsHel[i][j] = 0;
  
  int numChamSL[20]={8, 8, 4, 4, 4, 4, 4, 4, 16, 16, 8, 8, 8, 8, 16, 16, 12, 12, 12, 12};

  int numWorkingChamSL[20];
  for(SLindex=0; SLindex < k_numHalfStation; SLindex++) 
    numWorkingChamSL[SLindex] = numChamSL[SLindex];
  
  int numWorkingCham = 0;  
  static int hitcounter[k_numChamber] = {0};  
  int hitcounterBX[k_numChamber];    
  for (int i=0; i < k_numChamber; i++)
    hitcounterBX[i] = 0;
  static int hitcounterTyp1 = 0, hitcounterTyp2  = 0, hitcounterTyp3 = 0;
  int deadtyp1 = 0, deadtyp2 = 0, deadtyp3 = 0;
  float meanHitsTyp1, meanHitsTyp2,meanHitsTyp3;
  int numDeadChamTyp1, numDeadChamTyp2, numDeadChamTyp3;
  meanHitsTyp1 = meanHitsTyp2 = meanHitsTyp3 = 0.; 
  numDeadChamTyp1 = numDeadChamTyp2 = numDeadChamTyp3 = 0;
   
  for (ArteTableIterator<HITB> hitbit; hitbit.valid(); hitbit.next()) {
    ArtePointer<HITB> hitb(*hitbit);
    ArtePointer<GEDE> gede(hitb->gede);
    ArtePointer<DIHD> dihd(1);

    if (gede->cmp == 3){                        /*  ITR is component 3 in ARTE */
      

      string test_s, hwc_s;
      test_s = gede->hwc;
      hwc_s = test_s.substr(0,6);
      if (DQ_ITR_issKnown(hwc_s)!= 0)
	chambernr = (DQ_ITR_chambersIndex(hwc_s)); 

      float SignalOverNoise = hitb->q;

      DQ_ITR_my_hf1_fill(h_Signalovernoise,LVL_HIT, SignalOverNoise, 1.);

      if(SignalOverNoise >= SigalOverNoiseCut){

	// *************************************************************************
	//    for debugging: to get access to full occupancy histogram, replace 
	//    "float hitb_bin= hitb->y/4.;" with "float hitb_bin= hitb->y/1.;"

	float hitb_bin= hitb->y/4.;

	DQ_ITR_my_hf2_fill(h_Occupancy_raw, LVL_HIT, chambernr, 
			   hitb_bin,1.); 

	// *************************************************************************

	if (chambernr > k_numChamber) {
	  cout << " DQ_ITR_hit_event.C : GRR_INDEX_OVERFLOW_AND_BEING_LUCKY_NO_SIGSEGV   
                  chamber id = " << hwc_s << endl ;
	  continue;
	}
      
	hitcounter[chambernr]++;      
	if(faDDAQFCS_tcode() >= 8){      //BX spectrum only for random triggered events

	  string test_s, hwc_s;
	  test_s = gede->hwc;
	  hwc_s = test_s.substr(0,6);

	  SLnr = 2*(DQ_ITR_SL_Chambers(hwc_s) - 1);
	  if (gede->sec > 4)                         
	    SLnr++;

	  //hitcounterBX[chambernr] ++;
      


	  if(chambernr < k_Typ1)
	    hitcounterTyp1 ++;
	  else if (chambernr >= k_Typ1 && chambernr < k_Typ1Typ2)
	    hitcounterTyp2 ++;
	  else if (chambernr >= k_Typ1Typ2)
	    hitcounterTyp3 ++;
      
	  strips = (int) (hitb->y);
	  int chipIndex = (int) (strips / 128.);   
      
	  hitsHel [chambernr][chipIndex] ++;


	  int BXnumber= dihd->bx;
	  //DQ_ITR_my_hf2_fill(h_BXSpectrum, LVL_HIT, chambernr, 
	  //	     (float)BXnumber, hitcounterBX[chambernr]);
	  DQ_ITR_my_hf2_fill(h_BXSpectrum, LVL_HIT, chambernr, 
			     (float)BXnumber, 1.0);
	}
      }
    }
  }  
  static int reset_counter = 0;    
  
  if (event_count > ( reset_counter * ResetByEvent) + EventForCalculation ) {
    for(chambernr = 0; chambernr < k_Typ1Typ2Typ3; chambernr ++){
      if (chambernr < k_Typ1){
	meanHitsTyp1 =(float) hitcounterTyp1/
	  ((k_Typ1 - deadtyp1) 
	   * event_count - reset_counter * ResetByEvent);
      }     
      else if (chambernr>= k_Typ1 && chambernr<k_Typ1Typ2){
	meanHitsTyp2 = (float)hitcounterTyp2/ 
	  ((k_Typ1Typ2 - k_Typ1 - deadtyp2)
	   *event_count - reset_counter * ResetByEvent );
      }
      else if (chambernr >= k_Typ1Typ2 && chambernr < k_Typ1Typ2Typ3){
	meanHitsTyp3 = (float)hitcounterTyp3/ 
	  ((k_Typ1Typ2Typ3 - k_Typ1Typ2 - deadtyp3)
	   *event_count - reset_counter * ResetByEvent);
      	meanHitsTyp3 = 100.0;                //SWSW has to be removed in 2001
      }
    }
    
    for(chambernr = 0; chambernr < k_Typ1Typ2Typ3; chambernr ++){
      if (DQ_ITR_indexChambers(chambernr + 1) >= 0){
	SLindex = DQ_ITR_indexChambers(chambernr + 1);



// 	hi_id  = (DQ_ITR_indexChamber(chambernr + 1));  /// <---------------------------  needs to be checked
// 	SLindex = 2*(hi_id / 100 - 1);
// 	if(hi_id % 2 != 0)
// 	  SLindex ++;
// 	if (SLindex != DQ_ITR_indexChambers(chambernr+1))
// 	  cout << "Error!!! " <<"Old SL : " << SLindex << " New SL : " << DQ_ITR_indexChambers(chambernr+1) << " chambernr : " << chambernr + 1 << endl;
// 	else
// 	  cout << "new SLnumber ok" << endl;
// 	//cout << "Old SL : " << SLindex << " New SL : " << DQ_ITR_indexChambers(chambernr+1) << " chambernr : " << chambernr + 1 << endl;



      } 
      else { 
	cout << "DQ_ITR_hit_evnt.C : GRR_INDEX:  SLindex, chambernr, hi_id = " << SLindex << ", " << chambernr << ", " << hi_id << endl;
	continue;
      }
      
      float meanHits = ((float)hitcounter[chambernr] / 
			((float)event_count - reset_counter * ResetByEvent));
      float deadcut = 0.5;
      if (chambernr < k_Typ1){
	if (meanHits < (deadcut * meanHitsTyp1)){
	  numWorkingChamSL[SLindex]-=1;
	  numDeadChamTyp1++;
	}
	deadtyp1 = numDeadChamTyp1;
      }
      
      else if (chambernr >= k_Typ1 && chambernr < k_Typ1Typ2){
	if (meanHits < (deadcut * meanHitsTyp2)){
	  numWorkingChamSL[SLindex]-=1;
	  numDeadChamTyp2 ++;
	}
	deadtyp2 = numDeadChamTyp2;
      }
      
      else if (chambernr >= k_Typ1Typ2 && chambernr < k_Typ1Typ2Typ3){
	if (meanHits < (deadcut * meanHitsTyp3)){
	  numWorkingChamSL[SLindex]-= 1;
	  numDeadChamTyp3 ++;
	}
	deadtyp3 = numDeadChamTyp3;
      }
      else
	continue;
    }

    //    cout << deadtyp1<<"  "<<  deadtyp2<<"  "<<deadtyp3 <<endl;
  }  

  // ************* History Histograms disabled  
//   static int MeannumWorkingChamSL[20];
//   float persent[20] = {.0};
//   if (event_count > (reset_counter * ResetByEvent)+ EventForCalculation) {
//     for(SLindex = 0; SLindex < k_numHalfStation; SLindex++){
//       numWorkingCham += numWorkingChamSL[SLindex];
//       MeannumWorkingChamSL[SLindex] += numWorkingChamSL[SLindex];
    
//       if (event_count % TimeHistogramAbdate  ==  0){
// 	if(MeannumWorkingChamSL[SLindex] != 0)
// 	  persent[SLindex] = (float)MeannumWorkingChamSL[SLindex]/
// 	    ((float)numChamSL[SLindex]*(float)TimeHistogramAbdate);
	
// 	else
// 	  persent[SLindex] = .0;
	
// // 	DQ_ITR_my_hf2_fill(h_numWorkingCham, LVL_HIT, SLindex,
// // 			   event_count / TimeHistogramAbdate, 
// // 			   persent[SLindex]);
//       }
//     }
//     //   cout<<"working  " <<numWorkingCham<<endl;
//   }
  
//   if (event_count % TimeHistogramAbdate == 0 )
//     memset ((void*) MeannumWorkingChamSL,   0, sizeof(int) * SLindex);
  
  // normalize number of hits by number of working chambers in each
  // half superlayer.  Thus we can see more easily if one
  // half superlayer behaves strangely. 
  // NOTE: relation between SLindex and half superlayer: 0= +1, 1=-1, 2=+3, 3=-3...
  

  if (event_count > (reset_counter * ResetByEvent) + EventForCalculation) {
    
    for (SLindex=0; SLindex < k_numHalfStation; SLindex++) {
      if(numWorkingChamSL[SLindex] == 0)
	continue;
      if (numWorkingChamSL[SLindex] > 0) {
	ITR_hitsSL[SLindex] /= (float) numWorkingChamSL[SLindex];
      } 
      else
	ITR_hitsSL [SLindex] = 0;
      DQ_ITR_my_hf2_fill( h_HitsPerEventSL, LVL_HIT, 
			  SLindex,((float)ITR_hitsSL[SLindex] * (float)norm), 1.);
      // *****************************************
      //    time dependency of hits per EventSL
      
      MeanAverageHitsEventSL[SLindex] += (int)ITR_hitsSL[SLindex];

  // ************* History Histograms disabled  
//       if (event_count % TimeHistogramAbdate  == 0 ) {   
// 	DQ_ITR_my_hf2_fill(h_HitsPerEventSLMeanTenEvent, LVL_HIT, SLindex, 
// 			   (event_count/TimeHistogramAbdate), 
// 			   ((MeanAverageHitsEventSL[SLindex])/(float)TimeHistogramAbdate)); 
// 	MeanAverageHitsEventSL[SLindex] = 0;
//       }
    }
  }
  if (numWorkingCham > 0) {
    ITR_hitsallworkingCham = (float) ITR_hitsall / (float)numWorkingCham;
    
  }
  else
    ITR_hitsallworkingCham = 0;
  
  DQ_ITR_my_hf1_fill( h_HitsPerEvent, LVL_HIT,
		      (ITR_hitsallworkingCham *  norm), 1.); 
  //noise cut behavior 0: hits over noise, 1:hits under cut
 
  DQ_ITR_my_hf1_fill( h_hitsoverSNcut, LVL_HIT,0,ITR_hitsall);
  DQ_ITR_my_hf1_fill( h_hitsoverSNcut, LVL_HIT,1,ITR_hitsallundercut);
		       

  for(chambernr= 0; chambernr < k_numChamber; chambernr ++){  
    for(helix_counter = 0; helix_counter < k_HelixPerChamber; helix_counter++) {
      DQ_ITR_my_hf2_fill(h_Occupancy_bin, LVL_HIT, chambernr, 
			 helix_counter, hitsHel[chambernr][helix_counter]); 
    }    
  }
  /*************************************************************
   * reset hitcounter to find sudden problems with the helix  */

  if(event_count % ResetByEvent == 0){
    hitcounterTyp1 = 0; hitcounterTyp2  = 0; hitcounterTyp3 = 0;
    memset ((void*) hitcounter,   0, sizeof(int) * k_numChamber);
    reset_counter++;
  }

  DQ_ITR_my_hf1_fill(h_event, LVL_HIT, 0, 1.);
  ARTE_mark( "DQ_ITR_hit_evnt 1", 0 );  
  DQ_ITR_Noisebehavior();
  return GRR_OK; 
}
 
int DQ_ITR_Noisebehavior(){
  static int event_count = 0;
#if defined (DQ_ITR_COMPILE)
  if(event_count == 0)
    cout<<"EEEEEEEEEEEEEE DQ_ITR_hit_event.C/ DQ_ITR_Noisebehavior EEEEEEE"<<endl;
#endif

  float SignalOverNoise, ClusterCharge;
  float Noise;
  int Clusterwidth;
  float AverageNoise[k_numChamber];
  static float MeanAverageNoise[k_numChamber];
  float SumNoise[k_numChamber];
  int cluster_counter[k_numChamber];
  static int chambernr = 0;

  memset ((void*) SumNoise,   0, sizeof(int) * k_numChamber);
  memset ((void*) cluster_counter,   0, sizeof(int) * k_numChamber);
  memset ((void*) AverageNoise,   0, sizeof(int) * k_numChamber);

  event_count++; 

  if(event_count == 1)
    memset ((void*) MeanAverageNoise,   0, sizeof(int) * k_numChamber);

  DQ_ITR_buildIndex();
  ITR_Marvin_Cluster_t *cluster;

  ITR_Marvin_Event_t *marvinEvent = ITR_marvin_fetch_event();
  if ( marvinEvent == NULL ) { 
    cout <<"DQ_ITR_hit_evnt.C : NULL pointer to marvin information!"<<endl;
    return GRR_FISHY;
  }
  
  ITR_Marvin_Sevt_t *sevt = ITR_marvin_fetch_sevt();
  if ( sevt == NULL ) { 
    cout<<"DQ_ITR_hit_evnt.C : NULL pointer to sparsified information!"<<endl;
    return GRR_FISHY;
  }
  cluster = ITR_marvin_sevtCluster((char*)sevt->data, -1 );

  while ( cluster ) {

    LedaPointer<class ITR_Chamber> 
      ledaChamber = ITR_Chamber->get->key(cluster->chamber);
    LedaPointer<class ITR_nede>    
      ledaNede = ledaChamber->tonede;


    
    int superlayer = ledaNede->superl_db;
    int layer      = ledaNede->lay_db;
    int sector     = ledaNede->sec_db[0];
    int hi_id = superlayer * 100 + layer * 10 + sector;

//     string test_s, hwc_s;
//     test_s = gede->hwc;
//     hwc_s = test_s.substr(0,6);
//     if (DQ_ITR_issKnown(hwc_s)!= 0)
//       chambernr = (DQ_ITR_chambersIndex(hwc_s) - 1); 
//     else
//       cout<<"DQ_ITR_hit_evnt.C : chambernr not known  Chamber: "<< hwc_s <<endl; 

    
    if( DQ_ITR_isKnown(hi_id)!= 0)
      chambernr = (DQ_ITR_chamberIndex(hi_id) - 1); 
    else
    //  cout<<"DQ_ITR_hit_evnt.C : chambernr not known  Chamber: "<<hi_id<<endl; 

    

    SignalOverNoise = cluster->sn; 
    ClusterCharge = cluster->charge;
    Clusterwidth = cluster->width;
    if (SignalOverNoise != 0)
      Noise = ClusterCharge/(SignalOverNoise *sqrt(Clusterwidth));
    else
      Noise = 0;
    //cout << "b:" << Noise << ":" << ClusterCharge << ":" << SignalOverNoise << ":" << Clusterwidth << endl;
    SumNoise[chambernr] += Noise;
    cluster_counter[chambernr]++;
    // next cluster
    cluster = ITR_marvin_sevtCluster(NULL, -1);
  }
  
  for(chambernr = 0; chambernr < k_numChamber; chambernr++){
 
    if(cluster_counter[chambernr]!= 0){
      AverageNoise[chambernr] = SumNoise[chambernr]/cluster_counter[chambernr];
      MeanAverageNoise[chambernr] +=  AverageNoise[chambernr];
      //cout << "a:" << chambernr << ":" << SumNoise[chambernr] << ":" << cluster_counter[chambernr] << ":" << AverageNoise[chambernr] << ":" << MeanAverageNoise[chambernr] << endl;
    } 
    else
      AverageNoise[chambernr]=-1;

    DQ_ITR_my_hf2_fill(h_NoisePerChamber, LVL_HIT, chambernr, 
		       AverageNoise[chambernr], 1.); 


  // ************* History Histograms disabled  
//     if (event_count % TimeHistogramAbdate  == 0 ) {   
//       DQ_ITR_my_hf2_fill(h_NoiseMeanTenEvent, LVL_HIT, chambernr, 
//       			 (event_count/TimeHistogramAbdate), 
//       			 ((MeanAverageNoise[chambernr])/(float)TimeHistogramAbdate)); 
//     }
  }
  if (event_count % TimeHistogramAbdate  == 0 )
    memset ((void*) MeanAverageNoise,   0, sizeof(int) * k_numChamber);
  
  return GRR_OK;
}
  
