
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
// A. Gellrich
//
// Filling of histograms
//
// hints:
//        components: COMP_SVD
//                    COMP_ITR
//                    COMP_HIPT
//                    COMP_OTR
//                    COMP_RICH
//                    COMP_TRD
//                    COMP_ECAL
//                    COMP_MUON
//                    COMP_FLT
//                    COMP_SLT
//                    COMP_TLT
//                    COMP_FARM
//
//         levels:    LVL_RAW
//                    LVL_HIT
//                    LVL_REC
//
//         histogram id: 1 - 65535
//
// changes:
// 22 Oct 1999 (AG) code polishing
// 27 Oct 1999 (AG) use defines from DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 12 Dec 2000 (MB) changed something
// 14 Feb 2002 (UH) code polishing
// 03 Apr 2002 (UH) update for new gatherer/monitor code
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"
#include "event.h"
#include "tgts_raw.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "arte/RSEG_cmp.hh"
#include "dq/DQ.h"
#include "dq/DQ_MUON.h"

//
// function code
//
int
DQ_MUON_rec_evnt()
{
  static const float z_mu1 = 1620.;

  // show up
  ARTE_mark( "DQ_MUON_rec_evnt 1", 0 );

  // filling
  unsigned int n_muontracks = 0;
  
  for(ArteTable< RTRA >::iterator rtraIt = ArteTable< RTRA >::begin();
      rtraIt != ArteTable< RTRA >::end(); ++rtraIt) {
    ArtePointer< RTRA > rtra = *rtraIt;
    // a track is counted if a minimum muon ID criterion is fulfilled 
    if( rtra->lmmu > 0.0001 ) {
      // search for OTR segment
      for( ArteRelationIterator< RSEG > rsegIt = rtra->rseg.begin();
	   rsegIt !=  rtra->rseg.end(); ++rsegIt) {
	ArtePointer< RSEG> rseg = *rsegIt;
	if (rseg->cmp & Rsegc::trit ) {
	  float x = rseg->xe + rseg->txe * ( z_mu1 - rseg->ze );
	  float y = rseg->ye + rseg->tye * ( z_mu1 - rseg->ze );
	  DQ_MUON_hfill( TRACKPOS, LVL_REC, x, y, 1. );
	}
      }
      DQ_MUON_hf1( NHITS, LVL_REC, rtra->hmu, 1. );
      n_muontracks++; 
    }
  }
  DQ_MUON_hf1( NTRACKS, LVL_REC, n_muontracks, 1 );
  DQ_MUON_hf1( IARATE, LVL_REC, faDTARia(), 1 );

  ARTE_mark( "DQ_MUON_rec_evnt 1", 0 );
  return 0; // return error code (0 = okay)
}
