
// ******************************************************************
//
// D. Samtleben 10 Aug 1999
// A. Gellrich
//
// hints:
//        components: COMP_SVD
//                    COMP_ITR
//                    COMP_HIPT
//                    COMP_OTR
//                    COMP_RICH
//                    COMP_TRD
//                    COMP_ECAL
//                    COMP_MUON
//                    COMP_FLT
//                    COMP_SLT
//                    COMP_TLT
//                    COMP_FARM
//
//         levels:    LVL_RAW
//                    LVL_HIT
//                    LVL_REC
//
//         histogram id: 1 - 65535
//
// changes:
// 22 Oct 1999 (AG) code polsihing
// 27 Oct 1999 (AG) use defines in DQ.h
//  1 Nov 1999 (AG) get rid of dq_hid.h
// 12 Dec 2000 (MB) changed something
// 14 Feb 2002 (UH) code polishing
// 03 Apr 2002 (UH) new names for top level histograms
//
// ******************************************************************

// standard includes
#include <stdlib.h>
#include <iostream>

// ONLINE includes
#include "rhp.h"
#include "grr.h"

// Arte includes
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
#include "dq/DQ_MUON.h"

// ROOT includes
#ifdef DQ_MUON_TEST
#include <map>
#include "TH1.h"
#include "TH2.h"
#include "TFile.h"
extern TFile* rootfile;
extern map<int,TH1*> h1_map;
extern map<int,TH2*> h2_map;
#endif

const int debug_level = 2;

//
// function code
//
int
DQ_MUON_rec_init()
{
  // show up
  ARTE_mark( "DQ_MUON_rec_init 0", 0 );
  cout << "DQ_MUON_rec_init" << endl;

  // booking
  DQ_MUON_hbook1(NTRACKS,LVL_REC,
		 "MUON: number of tracks per event*",30,-0.5,29.5);
  DQ_MUON_hbook1(IARATE,LVL_REC,
		 "MUON: interaction rate",
		 500, 0, 50 );
  DQ_MUON_hbook1(NHITS,LVL_REC,
		 "MUON: number of hits per track",50,-0.5,49.5);
  DQ_MUON_hbook2(TRACKPOS,LVL_REC,
		 "MUON: positions of reconstructed tracks in MU1*",
		 80,-400,400,60,-300,300);
  
  ARTE_mark( "DQ_MUON_rec_init 1", 0 );
  return 0; // return error code (0 = okay)
}

//
//
// COLLECTION OF SOME UTILITY FUNCTIONS
//
//

// Wrapper functions to book and fill RHP and private histograms 
int DQ_MUON_hbook1(int ID, int level, char* title, 
			  int nbins, float xmin, float xmax) {
  int rc = 0;
  rc = rhp_hbook1(DQ_ID(COMP_MUON,level,ID),title,nbins,xmin,xmax);
  if(rc == GRR_DEFINED) {
    cout << " * DQ_MUON_hbook1: Error - histogram ID already in use" << endl;
    return -1;
  } else if(rc == GRR_FISHY) {
    cout << " * DQ_MUON_hbook1: Error - not enough memory" << endl;
    return -1;
  }

#ifdef DQ_MUON_TEST
  char histoid[MUON_MEDIUM_STRING];
  sprintf(histoid,"H10x%x",DQ_ID(COMP_MUON,level,ID));
  TH1* hnew = new TH1F(histoid,title,nbins,xmin,xmax);
  h1_map.insert(pair<int,TH1*>(DQ_ID(COMP_MUON,level,ID),hnew));
#endif
  return rc;
}

int DQ_MUON_hbook2(int ID, int level, char* title, 
			  int nbinsx, float xmin, float xmax,
			  int nbinsy, float ymin, float ymax) {
  int rc = 0;
  rc = rhp_hbook2(DQ_ID(COMP_MUON,level,ID),
		  title,nbinsx,xmin,xmax,nbinsy,ymin,ymax);
  if(rc == GRR_DEFINED) {
    cout << " * DQ_MUON_hbook2: Error - histogram ID already in use" << endl;
    return -1;
  } else if(rc == GRR_FISHY) {
    cout << " * DQ_MUON_hbook1: Error - not enough memory" << endl;
    return -1;
  }  
#ifdef DQ_MUON_TEST
  char histoid[MUON_MEDIUM_STRING];
  sprintf(histoid,"H20x%x",DQ_ID(COMP_MUON,level,ID));
  TH2* hnew = new TH2F(histoid,title,nbinsx,xmin,xmax,nbinsy,ymin,ymax);
  h2_map.insert(pair<int,TH2*>(DQ_ID(COMP_MUON,level,ID),hnew));
#endif
  return rc;
}

int DQ_MUON_hf1(int ID, int level, float x, float weight) {
  int rc = 0;
  int full_id = DQ_ID(COMP_MUON,level,ID);
  rc = rhp_hf1(full_id,x,weight);
#ifdef DQ_MUON_TEST
  if(h1_map.find(full_id) != h1_map.end()) {
    h1_map[full_id]->Fill(x,weight);
  } else {
    cout << " * DQ_MUON_hf1: warning - histogram ID 0x" << hex << full_id
	 << " not booked" << endl;
  }
#endif
  return rc;
}

int DQ_MUON_hfill(int ID, int level, float x, float y, float weight) {
  int rc = 0;
  int full_id = DQ_ID(COMP_MUON,level,ID);
  rc = rhp_hfill(full_id,x,y,weight);
#ifdef DQ_MUON_TEST
  if(h2_map.find(full_id) != h2_map.end()) {
    h2_map[full_id]->Fill(x,y,weight);
  } else {
    cout << " * DQ_MUON_hf2: warning - histogram ID 0x" << hex << full_id
	 << " not booked" << endl;
  } 
#endif
  return rc;
}

