

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "rhp.h"

// Arte include
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "dq/DQ.h"
//#include "dq/DQ_SVD.h"
//#include "cats/hbook.h"
#include "arte/RSEG_cmp.hh"

extern "C" {
       void rk5clip_(double& z_in ,double* p_in
                    ,double& z_out,double* p_out,double* rkd,int& ierror);
}


// ******************************************************************
//
// D.Kruecker
//
// Declaration of Histograms and Initialization for tracking
//
// creation: 4 Apr 2002
//
// changes:
//
// ******************************************************************
//
int DQ_RECO_TRACK_init(){

  // show up
  ARTE_mark( "DQ_RECO_TRACK_init 0", 0 ); 
  cout << "DQ_RECO_TRACK_init" << endl;

  //RSEG based i.e. SVD-MAIN match
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10001),"TRACK: Matches/SVD-segments",
	     50, 0, 2 );
  //alignment check
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10002),"TRACK: OTR+ITR-segments/SVD-segments",
	     50, 0, 10 );
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10003),"TRACK: SVD-MAIN-Matches SVD_x-MAIN_x at z=415",
	     50, -4, 4 );
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10004),"TRACK: SVD-MAIN-Matches SVD_y-MAIN_y at z=415",
	     50, -40, 40 );
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10005),"TRACK: SVD-MAIN-Matches SVD_tx-MAIN_tx at z=415",
	     50, -0.01, 0.01 );
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10006),"TRACK: SVD-MAIN-Matches SVD_ty-MAIN_ty at z=415",
	     50, -0.1, 0.1 );

  //RTRA based
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10010),"TRACK: hits on RTRA All-Si-In-Ou-Ri-Ca-Mu *",
	     8, 0, 8 );
  rhp_hbook1(DQ_ID(COMP_RECO, LVL_REC,10011),"TRACK: MAIN/SVD RTRA",
	     50, 0, 2 );

  //momentum check
  rhp_hbook2(DQ_ID(COMP_RECO, LVL_REC,10014),"TRACK: dtx*p vs. y",
              30, -150,150, 50,0,1 );


  // show up
  ARTE_mark( "DQ_RECO_TRACK_init 1", 0 ); 

  return 0; // return error code (0 = okay)
};


// ******************************************************************
// 
// D.Kruecker
//
// Filling of histograms for tracking
//
// creation:  4 Apr 2002
//
// changes:
//
// ******************************************************************
//
int DQ_RECO_TRACK_evnt()
{
  // show up
  ARTE_mark( "DQ_RECO_TRACK_evnt 0", 0 );

// part A: checking the matches

  // look for #SVD/OTR+ITR matches
  // and  for #OTR+ITR segments
  // then normalize by #SVD segments
  //
  int   SVD   = 0;  // # silicon segments (RSEG, fit==0)
  float MAIN  = 0;  // # OTR and ITR segments (RSEG, fit==0)
  float MATCH = 0;  // # SVD-ITR/OTR matches (RSEG, fit==7)

  for(ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
      rseg != ArteTable<RSEG>::end(); ++rseg){

      // alive?
      if((rseg->cmp & Rsegc::bitsign)!=0)  continue;

      if((*rseg)->fit == 0) {
        // count SVD
        if( (*rseg)->cmp & Rsegc::vxd ) SVD++;
        // count main tracker (ITR+OTR)
        if( (*rseg)->cmp & Rsegc::patt ) MAIN++;
      };

      // match: SVD-MAIN (might be in magnet)
      if((*rseg)->fit == 7) {
         if(! ( (*rseg)->cmp & Rsegc::vxd  ) ) continue; 
         if(! ( (*rseg)->cmp & Rsegc::patt ) ) continue; 
         MATCH++;         

        // check the alignment for these matches
        // 
        ArteRelationIterator<RSEG> rsegi = (*rseg)->rend.begin();
        if( rsegi == (*rseg)->rend.end() )  continue;
        // segment main tracker
        ArtePointer<RSEG> rmain  =  *rsegi;
        if( ++rsegi == (*rseg)->rend.end()) continue;
        // segment silicon
        ArtePointer<RSEG> rsvd =  *rsegi;
        //cout<<rsvd->ze<<" "<<rmain->ze<<endl;

        //extrapolate to z=415, momentum from RTRA
        
        //nothing is unique in this world
        ArteRelationIterator<RTRA> pick;
        for( ArteRelationIterator<RTRA> rtraiS = rmain->rtra.begin();
             rtraiS != rmain->rtra.end(); ++rtraiS){
             pick = rtraiS;
             for( ArteRelationIterator<RTRA> rtraiM = rmain->rtra.begin();
                  rtraiM != rmain->rtra.end(); ++rtraiM){
                  if((*rtraiM)==(*rtraiS)) break;
             }
        }
        // RTRA momentum
        double p=(*pick)->pf;

        //extrapolate SVD
        double Zto=415;
        double zin=rsvd->ze;
        double pin[5] = {rsvd->xe,rsvd->ye,rsvd->txe,rsvd->tye,p};
        double rkd[25];
        double pout1[5];
        int iErr;
        rk5clip_(zin, pin, Zto, pout1, rkd, iErr);
        if(iErr) continue;

        //extrapolate MAIN
        Zto=415;
        zin=rmain->ze;
        double pin2[5] = {rmain->xe,rmain->ye,rmain->txe,rmain->tye,p};
        double pout2[5];
        rk5clip_(zin, pin2, Zto, pout2, rkd, iErr);
        if(iErr) continue;

        rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10003),pout1[0]-pout2[0],1);
        rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10004),pout1[1]-pout2[1],1);
        rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10005),pout1[2]-pout2[2],1);
        rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10006),pout1[3]-pout2[3],1);

      };
  }
  if(SVD!=0){
    rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10001),MATCH/SVD,1);
    if(MAIN!=0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10002),MAIN/SVD,1);
  };

// part B: checking RTRA

  float MA2=0;
  float SVD2=0;
  for(ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
      rtra != ArteTable<RTRA>::end(); ++rtra){
      if( (*rtra)->fit!=0 ) continue;
      if( (*rtra)->hsi>0&&((*rtra)->hin>0||(*rtra)->hou>0) ) MA2++;
      if( (*rtra)->hsi>0 ) SVD2++;
      rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),0.5,1);
      if((*rtra)->hsi>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),1.5,1);
      if((*rtra)->hin>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),2.5,1);
      if((*rtra)->hou>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),3.5,1);
      if((*rtra)->hri>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),4.5,1);
      if((*rtra)->hca>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),5.5,1);
      if((*rtra)->hmu>0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10010),6.5,1);
      ArtePointer<RSEG>  svd=0;
      ArtePointer<RSEG> main=0;
      for(ArteRelationIterator<RSEG> ir=rtra->rseg.begin();
        ir!=rtra->rseg.end();++ir) {
        if( (*ir)->fit!=0 ) continue;
        if((*ir)->zf>5&&(*ir)->zf<206)    svd=*ir; 
        if((*ir)->zf>208&&(*ir)->zf<852) main=*ir; 
      }    
      if(svd!=0&&main!=0&&rtra->pf!=0) {
         float a=-(svd->txe-main->txe)/rtra->pf;
         rhp_hfill(DQ_ID(COMP_RECO,LVL_REC,10014),main->ye,a,1);
      }
  }

  if(SVD2!=0 && MA2!=0)
      rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10011),MA2/SVD2,1);

/*
// for debugging
  static float average(0);
  static float asvd(0);
  static int caver(0);
  static int bin(0);
  if(SVD2!=0){
    if(MA2!=0) rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10011),MA2/SVD2,1);
    average+=MA2/SVD2;
    asvd+=SVD2;
    caver++;
  };
  if(caver==400) {
    average/=caver;
    asvd/=caver;
    rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10012),bin+0.5,average);
    rhp_hf1(DQ_ID(COMP_RECO,LVL_REC,10013),bin+0.5,asvd);
    average=0;
    caver=0;
    bin++;
  }
*/


  // show up
  ARTE_mark( "DQ_RECO_TRACK_evnt 1", 0 );

  return 0; // return error code (0 = okay)
}
