// ********************************************************************
//
// Routine:      DQ_evnt.C
// Author:       D.Samtleben
//
// Description:  calls the different DQ-routines
//
// Changes:
// 09 Sep 1999 (AG) replace cerr by cout
// 18 Oct 1999 (AG) introduce DQ.hh
// 20 Oct 1999 (AG) use DQ.h
// 28 Oct 1999 (AG) remove not used functions
// 17 Nov 1999 (AG) invoke DQ_FARM_rec_evnt()
// 24 Nov 1999 (AG) add MUON rec
// 03 Apr 2000 (AG) introduce ECAL_raw
// 04 Apr 2000 (AG) introduce timing
// 05 Apr 2000 (AG) introduce DQ_MUON_raw
// 11 Apr 2000 (AG) remove printout
// 25 Apr 2000 (AG) use ARTE_mark()
// 26 Jun 2000 (AG) introduced DQ_ITR()
//  5 Jul 2000 (AG) introduce DQ_RECO_rec_evnt()
// 10 Jul 2000 (AG) check for incomming rpm messages
// 09 Aug 2000 (AG) arguments to faCheckRhp() changed.
// 29 Nov 2000 (AG) polished printout
//  6 Dec 2000 (AG) printout cputime
// 13 Dec 2000 (AG) added DQ_SLT
// 19 Dec 2000 (AG) added DQ_FLT
// 28 Aug 2001 (JH) removed rhp printout
// 23 Oct 2001 (JH) added DQ_FARM_raw_evnt
// 19 Nov 2001 (JH) Publish rhp histograms
// 17 Dec 2001 (JH) Do not call raw_evnt functions if no DEVT
// 19 Dec 2001 (JH) do rhp publication in acwrite
// 26 May 2002 (IV) introduced DQ_OTR_raw_evnt()
// 07 Jun 2002 (JH) Call DQ levels for components appropriately
// 18 Jul 2002 (JH) Call DQ raw level if event was not reconstructed
// 24 Jul 2002 (JH) do not call raw level for MC
//
// ********************************************************************

// standard includes
#include <iostream.h>
#include <stdlib.h>
#include <stdio.h>

// ONLINE includes
#include "event.h"
#include "rhp.h"

// ARTE includes
#include "arte/arte.hh"
#include "arte/ArteProto.hh"
#include "arte/Chrono.hh"
#include "arte/ChronoManager.hh"
#include "reco/rcflags.hh"
#include "farm/farm.hh"
#include "farm/fa.hh"
#include "dq/DQ.h"
#include "dq/DQ_SVD.h"
#include "dq/DQ_ITR.h"
#include "dq/DQ_HIPT.h"
#include "dq/DQ_OTR.h"
#include "dq/DQ_RICH.h"
#include "dq/DQ_TRD.h"
#include "dq/DQ_ECAL.h"
#include "dq/DQ_MUON.h"
#include "dq/DQ_FLT.h"
#include "dq/DQ_SLT.h"
#include "dq/DQ_TLT.h"
#include "dq/DQ_FARM.h"
#include "dq/DQ_CORR.h"
#include "dq/DQ_RECO.h"

// externals
extern int FATIME;
extern int FADEB;

// prototypes
extern "C" void dqkuip_();

//
// initialize the Xyz kuip commands
//
static class DQIniter
{
public:
  DQIniter() { atregkuip( dqkuip_, 1 ); } 
} _dqIniter;

//********************************************************************
// DQ_evnt
//********************************************************************
int
DQ_evnt()
{
  ARTE_mark( "DQ_evnt 0", 0 );

  // start timing
  //
  static Chrono *cron_DQ_evnt = 0;
  if( !cron_DQ_evnt ){
    ChronoManager CM;
    cron_DQ_evnt = CM.createChrono( "cron_DQ_evnt" );
  }
  cron_DQ_evnt->start();

  // print settings
  // DQ_print();

  // get raw event for real data
  EVENT_HDR *event = NULL; 
  if (ArteTable<DEVT>::size() != 0){
     ARTE_mark( "ARTE_get_event", 0 );
     event = ARTE_get_event();
     if (!event) ARTE_error(GRR_SERROR, "failed to get pointer to raw event for raw DQ",0);
  }
  //
  // event functions raw level
  //
  if (event) {
    // HIPT
    if( DQ_flag( COMP_HIPT, LVL_M_RAW ) ){
      ARTE_mark( "DQ_HIPT_raw_evnt", 0 );
      DQ_HIPT_raw_evnt( event );
    }
    // OTR
    if( DQ_flag( COMP_OTR,  LVL_M_RAW ) ){
      ARTE_mark( "DQ_OTR_raw_evnt", 0 );
      DQ_OTR_raw_evnt( event );
    }
    // RICH
    if( DQ_flag( COMP_RICH, LVL_M_RAW ) ){
      ARTE_mark( "DQ_RICH_raw_evnt", 0 );
      DQ_RICH_raw_evnt( event );
    }
    // ECAL
    if( DQ_flag( COMP_ECAL, LVL_M_RAW ) ){
      ARTE_mark( "DQ_ECAL_raw_evnt", 0 );
      DQ_ECAL_raw_evnt( event );
    }
    // MUON
    if( DQ_flag( COMP_MUON, LVL_M_RAW ) ){
      ARTE_mark( "DQ_MUON_raw_evnt", 0 );
      DQ_MUON_raw_evnt( event );
    }
    // FARM
    if( DQ_flag( COMP_FARM, LVL_M_RAW ) ){
      ARTE_mark( "DQ_FARM_raw_evnt", 0 );
      DQ_FARM_raw_evnt( event );
    }  
  }

  //
  // hit and rec level only if event was reconstructed
  //
  if (!faLastDownscaled()) {
    // SVD
    if( DQ_flag( COMP_SVD,  LVL_M_HIT ) ){
      ARTE_mark( "DQ_SVD_hit_evnt", 0 );
      if (recoVDS>0) DQ_SVD_hit_evnt();
    }
    if( DQ_flag( COMP_SVD,  LVL_M_REC ) ){
      ARTE_mark( "DQ_SVD_rec_evnt", 0 );
      if (recoVDS>1) DQ_SVD_rec_evnt();
    }
    // ITR
    if( DQ_flag( COMP_ITR,  LVL_M_HIT ) ){
      ARTE_mark( "DQ_ITR_hit_evnt", 0 );
      if (recoITR>0) DQ_ITR_hit_evnt();
    }
    if( DQ_flag( COMP_ITR,  LVL_M_REC ) ){
      ARTE_mark( "DQ_ITR_rec_evnt", 0 );
      if (recoITR>1) DQ_ITR_rec_evnt();
    }
    // OTR
    if( DQ_flag( COMP_OTR,  LVL_M_HIT ) ){
      ARTE_mark( "DQ_OTR_hit_evnt", 0 );
      if (recoOTR>0) DQ_OTR_hit_evnt();
    }
    if( DQ_flag( COMP_OTR,  LVL_M_REC ) ){
      ARTE_mark( "DQ_OTR_rec_evnt", 0 );
      if (recoOTR>1) DQ_OTR_rec_evnt();
    }
    // RICH
    if( DQ_flag( COMP_RICH, LVL_M_HIT ) ){
      ARTE_mark( "DQ_RICH_hit_evnt", 0 );
      if (recoRICH>0) DQ_RICH_hit_evnt();
    }
    if( DQ_flag( COMP_RICH, LVL_M_REC ) ){
      ARTE_mark( "DQ_RICH_rec_evnt", 0 );
      if (recoRICH>1) DQ_RICH_rec_evnt();
    }
    // ECAL
    if( DQ_flag( COMP_ECAL, LVL_M_HIT ) ){
      ARTE_mark( "DQ_ECAL_hit_evnt", 0 );
      if (recoECAL>0) DQ_ECAL_hit_evnt();
    }
    if( DQ_flag( COMP_ECAL, LVL_M_REC ) ){
      ARTE_mark( "DQ_ECAL_rec_evnt", 0 );
      if (recoECAL>1) DQ_ECAL_rec_evnt();
    }
    // MUON
    if( DQ_flag( COMP_MUON, LVL_M_HIT ) ){
      ARTE_mark( "DQ_MUON_hit_evnt", 0 );
      if (recoMUON>0) DQ_MUON_hit_evnt();
    }
    if( DQ_flag( COMP_MUON, LVL_M_REC ) ){
      ARTE_mark( "DQ_MUON_rec_evnt", 0 );
      if (recoMUON>1) DQ_MUON_rec_evnt();
    }
    // FLT
    if( DQ_flag( COMP_FLT,  LVL_M_REC ) ){
      ARTE_mark( "DQ_FLT_rec_evnt", 0 );
      DQ_FLT_rec_evnt();
    }
    // SLT
    if( DQ_flag( COMP_SLT,  LVL_M_REC ) ){
      ARTE_mark( "DQ_SLT_rec_evnt", 0 );
      DQ_SLT_rec_evnt();
    }
    // TLT
    if( DQ_flag( COMP_TLT,  LVL_M_REC ) ){
      ARTE_mark( "DQ_TLT_rec_evnt", 0 );
      DQ_TLT_rec_evnt();
    }
    // FARM
    if( DQ_flag( COMP_FARM, LVL_M_HIT ) ){
      ARTE_mark( "DQ_FARM_hit_evnt", 0 );
      if (recoStAl) DQ_FARM_hit_evnt();
    }
    if( DQ_flag( COMP_FARM, LVL_M_REC ) ){
      ARTE_mark( "DQ_FARM_rec_evnt", 0 );
      if (recoStAl) DQ_FARM_rec_evnt();
    }
    // CORR
    if( DQ_flag( COMP_CORR, LVL_M_HIT ) ){
      ARTE_mark( "DQ_CORR_hit_evnt", 0 );
      if (recoStAl) DQ_CORR_hit_evnt();
    }
    if( DQ_flag( COMP_CORR, LVL_M_REC ) ){
      ARTE_mark( "DQ_CORR_rec_evnt", 0 );
      if (recoStAl) DQ_CORR_rec_evnt();
    }
    // RECO
    if( DQ_flag( COMP_RECO, LVL_M_REC ) ){
      ARTE_mark( "DQ_RECO_rec_event", 0 );
      if (recoStAl) DQ_RECO_rec_evnt();
    } 
  }

  // stop timing
  //
  cron_DQ_evnt->stop();
  float cputime = cron_DQ_evnt->lastTime();
  if( FATIME ) printf( "\033[1m TIME: DQ=%4.2f \033[0m \n", cputime );
  if( DQ_flag( COMP_FARM, LVL_M_HIT ) )
    rhp_hf1( DQ_ID( COMP_FARM, LVL_HIT, 15 ), cputime, 1 );

  ARTE_mark( "DQ_evnt 1", 0 );
  return 0;
}
