
// *****************************************************************************
//
// source:    DQrhp2hbook.C
//
// type:      source code
//
// created:   13 Sep 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    DQrhp2hbook
//
// called by: usstop
//
// purpose:   creates HBOOK file name from EMG partion name if set
//
// hints:     leaves HBOOK file name unchanged if EMGpart is NOT set
//            EMG partition name is appended to original HBOOK name
//
// changes:
// 13 Sep 2000 (AG) creation
// 14 Sep 2000 (AG) addeded machine name to histogram file
// 15 Sep 2000 (AG) added writing to erwin
// 01 Nov 2001 (JH) Fixed bug when writting to log file
//
// *****************************************************************************

// standard includes
#include <iostream.h>
#include <string.h>
#include <unistd.h>

// ONLINE includes
#include "rhp.h"
#include "rhp2hbook.h"

// ARTE includes
#include "farm/farm.hh"

// externals
extern int  FADEB;     // debug flag
extern char *PARNAME;  // emg partition name
extern char *DQ_HBOOK; // HBOOK file name
extern char *LOGFILE;  // farm log file

// prototypes
#ifdef __linux
extern "C" int  gethostname( char *hostname, unsigned int length );
#endif

//
// function code
//
void
DQrhp2hbook()
{
  ARTE_mark( "DQrhp2hbook 0", 0 );
  cout << "DQrhp2hbook" << endl;

  // host name
  static char *hostName = 0;
  if( !hostName ) hostName = new char[10];
  memset( hostName, '\0', 10 );
  gethostname( hostName, 9 );

  // inits
  char *hbookFile = new char[80];
  memset( hbookFile, '\0', 80 );
  strcpy( hbookFile, DQ_HBOOK );

  // build HBOOK file name from EMG partition name
  if( PARNAME ){
    if( FADEB ) cout << "HBOOK: hostname:     " << hostName << endl;
    if( FADEB ) cout << "HBOOK: EMG partition:" << PARNAME  << endl;
    strcat( hbookFile, "_" );
    strcat( hbookFile, hostName );
    strcat( hbookFile, "_" );
    strcat( hbookFile, PARNAME );
  }
  
  if( DQ_HBOOK ){
    cout << "HBOOK: file name: " << hbookFile << endl;
    rhp_2hbook( 0, 0, "anything" ); // convert rhp to hbook
    char *str = new char[80];
    memset( str, '\0', 80 );
    strcpy( str, "HBOOK: rhp_2file file: " );
    strcat( str, hbookFile );
    ARTE_error( GRR_SINFO, str, 0 );
    ARTE_log( LOGFILE, str);
    rhp_2file( hbookFile ); // write to hbook file
  }
  
  ARTE_mark( "DQrhp2hbook 1", 0 );
}
