
// *****************************************************************************
//
// source:    faCatchSIG.C
//
// type:      source code
//
// created:   23 Sep 1997
//
// author:    Andreas Gellrich
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: Andreas.Gellrich@desy.de
//
// purpose:   Catch signals and handle them.
//
// symbols:   faCatchSig
//
// called by: faInit
//
// changes:         
// 23 Feb 1999 (AG) include prototypes here
// 11 Mar 1999 (AG) polishing
// 25 Mar 1999 (AG) use atstopc_()
// 01 Jun 1999 (AG) put includes
// 18 Aug 1999 (AG) provide only one signal catching routine
// 02 Sep 1999 (AG) fixed segmentation violation problem
// 07 Sep 1999 (AG) use atstopcc_( char * )
// 13 Sep 1999 (AG) signal catcher output
// 13 Oct 1999 (AG) write to log-file
// 14 Oct 1999 (AG) added unistd.h
// 18 Oct 1999 (AG) polishing
// 03 Nov 1999 (AG) add process id to logfile
// 03 Nov 1999 (AG) log sig to file
// 11 Nov 1999 (AG) improve log
// 24 Nov 1999 (AG) use ARTE_error()
// 26 Nov 1999 (AG) correct logfile/logsig handling
// 29 Nov 1999 (AG) polishing
// 29 Feb 2000 (AG) linux -> __linux
// 22 Mar 2000 (AG) improve print-out
// 31 Mar 2000 (AG) change error codes
// 12 Apr 2000 (AG) use ARTE_mark()
// 02 May 2000 (AG) use ARTE_log()
// 12 Oct 2000 (AG) prepare for setting error flag in case of problems
// 17 Jan 2001 (AG) added comments
// 20 Dec 2001 (JH) Send FATAL msg to error logger for SIGV
// 23 Jan 2002 (JH) Halt smc and rpm
//
// *****************************************************************************

// standard includes
#include <iostream.h>
//#include <signal.h>
#include <sys/signal.h>
#include <string.h>

// Arte includes
#include "arte/arte.hh"
#include "farm/fa.hh"
#include "rpm.h"
#include "smc.h"
#include "farm/farm.hh"

// externals
extern char *LOGFILE; // log-file
extern char *LOGSIG;  // log-file
extern int FAON;

extern char *ARTE_MARK_TEXT;
extern int   ARTE_MARK_CODE;

// C prototypes
extern "C" void atstopcc_( char *txt );
void faSetErrmask( unsigned int errmask );

//
// program code
//
void
faCatchSIG( int sig )
{
  // inits
  char txt[200];
  memset( txt, '\0', 200 );

  // error logger
  switch( sig ){
  case SIGINT:
    sprintf( txt, "faCatchSIG(): Caught SIGINT mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SINFO, txt, 0 );
    break;
  case SIGQUIT:
    sprintf( txt, "faCatchSIG(): Caught SIGQUIT mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SINFO, txt, 0 );
    break;
  case SIGILL:
    sprintf( txt, "faCatchSIG(): Caught SIGILL mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SFATAL, txt, 0 );
    break;
  case SIGBUS:
    sprintf( txt, "faCatchSIG(): Caught SIGBUS mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SFATAL, txt, 0 );
    faSetErrmask( 0xffff );
    break;
  case SIGSEGV:
    sprintf( txt, "faCatchSIG(): Caught SIGSEGV mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SFATAL, txt, 0 );
    faSetErrmask( 0xffff );
    break;
  case SIGTERM:
    sprintf( txt, "faCatchSIG(): Caught SIGTERM mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SINFO, txt, 0 );
    break;
  default:
    sprintf( txt, "faCatchSIG(): Caught signal mod:%s code=%d",
	     ARTE_MARK_TEXT, ARTE_MARK_CODE );
    cout << txt << endl;
    ARTE_error( GRR_SINFO, txt, 0 );
  }

  //
  // write to logfile
  //
  memset( txt, '\0', 200 );
  sprintf( txt, "Caught sig=%d mod:%s", sig, ARTE_MARK_TEXT );
  ARTE_log( LOGFILE, txt );

  //
  // write to logsig
  //
  if( sig ){
    memset( txt, '\0', 200 );
    sprintf( txt, "signal=%d", sig );
    ARTE_log( LOGSIG, txt );
  }

  // release emg and rpm in online running
  if(FAON) {
    printf("halting rpm\n");
    rpm_halt();
    printf("halting smc\n");
    smc_halt();    
  }

  // call atstop
  atstopcc_( txt );
}

//
// function
//
void
faSetErrmask( unsigned int errmask )
{
  //int size = ArteTable<EVRC>::size();
  //ArtePointer<EVRC> evrc( size );
  //evrc->errmask = errmask;
}
