/************************************************************************
*
* CRKIBK
* written by Siegmund Nowak, 13/02/96 19.56.54
*
* This routine transforms the temporary GEANT vertex and kinematic banks
* into  ARTE banks MVER and MTRA
*
*       called from GUOUT
*
***********************************************************************/
#include <vector.h>
#include <stdio.h>
#include "arte/arte.hh"
#include "arte/ArteFlag.h"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/ZGMOTH.h"
#include "gean/zgkine.hh"


extern "C" {
  void crkibk_();
  int  tpgear_(int*);
  void mmfree_(int&);
}

void crkibk_() {
   extern vector<hgmtra> *hg_mtra;
   extern vector<int>    *hg_vert;

   vector<hgmtra>::iterator mt, mo;   
   vector<int>::iterator mv;     
   zgkine ZGKINE;

   if(!arteflag_.mimpsw || hg_mtra->size() == 0) return;

// mark tracks with mimps or where daughters have mimps
   for(int itr=hg_mtra->size(); itr >0; itr--){
      mt = hg_mtra->begin() + itr -1;
      if(!mt->nhit) continue;                  //    no hits
      if(mt->mtra)  continue;                  //    track already booked
      int mo_track = ZGKINE.MotherTrack(itr);
      if(!mo_track) continue;
      mo = hg_mtra->begin() + mo_track -1;        
      if(!mo->nhit) mo->nhit = -1;
   }

// install mtra and mver and relations

   for(unsigned int itr=1; itr <= hg_mtra->size(); itr++){
      mt = hg_mtra->begin() + itr -1;
      int printflag=0;
      if(itr >= 261 && itr <=264) printflag = 1;
      if(itr == 367 || itr ==368) printflag = 1;
      if(mt->mtra) {                              // track already booked
         ArtePointer<MTRA> map(mt->mtra);   
         map->ml   = mt->len;
         map->leng = mt->len;
         map->flag = mt->flag;
         continue;
      }

      if(!mt->nhit) continue;                    // no hits

//    define geant particle code, geant vertex origin and geant mother track

      float *ptx = ZGKINE.TrackBank(itr);  
      int ipart = int(*(ptx+5));
      int itopp = tpgear_(&ipart); //transform particle id from geant to arte
      if(!itopp) {           
         arteMsg << "crkibk: particle code = 0";        
         static int idcount = 8762135;  arteMsg.warn(idcount);
      }
      unsigned tr_vertex = int(*(ptx+6));
      if(!tr_vertex) continue;
      float *pvx = ZGKINE.VertexBank(tr_vertex);
      int mo_track = int(*(pvx+5));; 

//    book and fill track bank
      MTRA ma;
      memcpy(&ma.px,ptx+1,12);
      ma.ml   = mt->len;
      ma.leng = mt->len;
      ma.flag = mt->flag;
      ArtePointer<TOPP> tp(itopp);
      if(tp->wid > 1.e-9) {                 // it is a resonance
         ma.flag = 2; 
         ma.mass = tp->mass;
      }
      ArteTable<MTRA>::push_back(ma);

      // update temporary track bank and fill MTRA bank and rel topp - track
      int nmtra = ArteTable<MTRA>::size();              
      mt->mtra = nmtra;
      ArtePointer<MTRA> map(nmtra);
      createRelation( tp, map, &TOPP::mtra);

//    book and fill vertex bank
//    first look for the vertex which track origin is not a resonance
      int iorig = 0;
      if(mo_track > 0) {
         mo = hg_mtra->begin() + mo_track -1;
         createRelation(ArtePointer<MTRA>(mo->mtra), map,&MTRA::etra);

         //define particle id of mother 
         float *pmx = ZGKINE.TrackBank(mo_track);
         int mpart =int(*(pmx+5));
         int mtopp = tpgear_(&mpart);
         ArtePointer<TOPP> mtp(mtopp);                
         if(mtp->wid > 1.e-9) {            // mother is a resonance 
            iorig = -mo->mtra;
            // as long as mother = resonance go to the previous mother track
            while (mo_track &&  mtp->wid > 1.e-9) {  
               tr_vertex = ZGKINE.VertexOrigin(mo_track); 
               pvx = ZGKINE.VertexBank(tr_vertex);
               mo_track=int(*(pvx+5)); 
               if(mo_track) {
         	  ptx = ZGKINE.TrackBank(mo_track); 
                  mpart=int(*(ptx+5));
                  mtopp = tpgear_(&mpart);
                  mtp = ArtePointer<TOPP>(mtopp);
               }
            } 
         }
      }

//    no more mother or mother is not a resonance - look for the vertex origin
      int imv =0; 
      if(tr_vertex <= hg_vert->size()) imv = *(hg_vert->begin()+tr_vertex-1); 
      if(!imv)  {                            // vertex not yet  booked
         MVER mver;                         // book and fill hg_vert and mver
         pvx = ZGKINE.VertexBank(tr_vertex);
         memcpy(&mver.x,pvx+1,12);
         mver.z = mver.z - zgmoth_.ZZERO ; 
         ArteTable<MVER>::push_back(mver);
         int nmver = ArteTable<MVER>::size();                 
         ZGKINE.mvpush(tr_vertex,nmver);
         ArtePointer<MVER> mmv(nmver);
         if(mo_track) {                       
            mmv->flag = 4;
            mo = hg_mtra->begin() + mo_track -1;         
            int nmoth = mo->mtra;
            ArtePointer<MTRA> moth(nmoth);
            if(iorig < 0 || moth->flag == 3) mmv->flag = 2;
            createRelation( moth, mmv, &MTRA::ever);  
	 }
      }

//    fill mtra_orig and relation mtmv
      mv = hg_vert->begin() + tr_vertex -1;
      int nmver = *mv;
      createRelation(ArtePointer<MVER>(nmver),map,&MVER::etra);

      if(iorig < 0) map->orig = iorig; 
      else map->orig = nmver;
   }                                                     // end track loop

   mmfree_(ArtePointer<MTRA>::getLink().getRef());
   mmfree_(ArtePointer<MVER>::getLink().getRef());  
}


