
      SUBROUTINE GHELX2 (CHARGE,STEP,VECT,VOUT)

C----------------------------------------------
C.
C.    ******************************************************************
C.    *                                                                *
C.    *  Performs the tracking of one step in a magnetic field         *
C.    *  The trajectory is assumed to be a helix in a constant field   *
C.    *  taken at the mid point of the step.                           *
C.    *  Parameters:                                                   *
C.    *   input                                                        *
C.    *     STEP =arc length of the step asked                         *
C.    *     VECT =input vector (position,direction cos and momentum)   *
C.    *     CHARGE=  electric charge of the particle                   *
C.    *   output                                                       *
C.    *     VOUT = same as VECT after completion of the step           *
C.    *                                                                *
C.    *    ==>Called by : <USER>, GUSWIM                               *
C.    *       Author    M.Hansroul  *********                          *
C.    *    original routine  GHELX3 for field along z axis             *
C     *    changes by Siegmund Nowak 03/93 to accept field along y     *
C.    ******************************************************************

      IMPLICIT NONE
      INTEGER LIMFLG,I
      REAL CHARGE,STEP,VECT(7),VOUT(7),EC,STEP0,STEP1,STEP2
      REAL RAD,TET,CTET,STET,CPW,CPT
      REAL XYZ(3),FIELD(3),VLOW(3),VHIGH(3),FELDA(3),FELDB(3)
      DATA EC/2.9979251E-4/
C.
C.    ------------------------------------------------------------------
C.
C       UNITS ARE KGAUSS,CENTIMETERS,GEV/C
C
      LIMFLG = 0
      STEP0 = STEP
      DO 11 I = 1,3
      VLOW(I) = VECT(I)
 11   VHIGH(I) = VECT(I) + STEP * VECT(I+3)

      IF (CHARGE.EQ.0.) GO TO 80

      CALL GUFLD(VLOW,FELDA)
      CALL GUFLD(VHIGH,FELDB)
      IF(FELDA(2).LT.1.E-3 .AND. FELDB(2).GT.1.E-3) LIMFLG = 1
      IF(FELDA(2).GT.1.E-3 .AND. FELDB(2).LT.1.E-3) LIMFLG = 2
      IF(LIMFLG .NE.0) THEN
         STEP1 = STEP
 12      STEP1 = 0.5 * STEP1
         XYZ(1) = VLOW(1) + STEP1 * VECT(4)
         XYZ(2) = VLOW(2) + STEP1 * VECT(5)
         XYZ(3) = VLOW(3) + STEP1 * VECT(6)
         CALL GUFLD(XYZ,FIELD)
         IF(FIELD(2).EQ.FELDA(2)) THEN
            VLOW(1) = XYZ(1)
            VLOW(2) = XYZ(2)
            VLOW(3) = XYZ(3)
         ELSE IF(FELDB(2).NE. FIELD(2)) THEN
            CALL MEERRO ('GHELX2 -STOP')
         ENDIF
         IF(STEP1.GT..001) GO TO 12
         STEP2 = (XYZ(3)-VECT(3))/VECT(6)
         IF(FELDA(2).LT.1.E-3 .AND. FELDB(2).GT.1.E-3) THEN
            LIMFLG = 1
            STEP0 = STEP - STEP2
         ELSE
            LIMFLG = 2
            DO 14 I = 1,3
            VHIGH(I) = VLOW(I)
 14         VLOW(I) = VECT(I)
            STEP0 = STEP2
            STEP2 = STEP - STEP0
         ENDIF
      ENDIF
      XYZ(1) = VLOW(1) + 0.5 * STEP0 * VECT(4)
      XYZ(2) = VLOW(2) + 0.5 * STEP0 * VECT(5)
      XYZ(3) = VLOW(3) + 0.5 * STEP0 * VECT(6)
      CALL GUFLD(XYZ,FIELD)
      IF (FIELD(2).LT.1.E - 3) GO TO 80


      RAD = VECT(7)/(EC*FIELD(2)*CHARGE)
      TET    = STEP0 /RAD
      STET   = SIN(TET)
      CTET   = COS(TET)
C
      CPW     = VECT(5)
      CPT     = SQRT(ABS((1.+CPW)*(1.-CPW)))
      IF(CPT.LT.1.E-6) GOTO 80
C
      VOUT(1) = VLOW(1) + RAD*((CTET-1.)*VECT(6) + STET*VECT(4))
      VOUT(2) = VLOW(2) + STEP0 * CPW
      VOUT(3) = VLOW(3) + RAD*((1.-CTET)*VECT(4) + STET*VECT(6))
      VOUT(4) = -STET*VECT(6) + CTET*VECT(4)
      VOUT(5) = CPW
      VOUT(6) = STET*VECT(4) + CTET*VECT(6)
      VOUT(7) = VECT(7)
      IF(LIMFLG.EQ.2) THEN
         DO 15 I = 1,3
 15      VOUT(I) = VOUT(I) + STEP2 * VOUT(I+3)
      ENDIF
      GO TO 99
C
  80  CONTINUE
      DO 90 I=1,3
         VOUT(I) = VECT(I) + STEP * VECT(I+3)
         VOUT(I+3) = VECT(I+3)
  90  CONTINUE
      VOUT(7) = VECT(7)
C
  99  RETURN
      END
