/*   Controls tracking of all particles belonging to the current *
*    event.                                                      *
*                                                                *
*   Called by : GUTREV, called by GTRIG                          *
*   Authors   : R.Brun, F.Bruyant                                *
*                                                                *
*****************************************************************/
#include <stdio.h>
#include <vector.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/gclink.h"
#include "gean/gcflag.h"
#include "gean/gckine.h"
#include "gean/gcnum.h"
#include "gean/gcstak.h"
#include "gean/gctmed.h"
#include "gean/gctrak.h"
#include "gean/zgtrig.hh"
#include "gean/zgkine.hh"

extern "C" {
  void gutrev_();
  void gsstak_(int *);
  void gltrac_();
  void gutrak_();
  void crkibk_();
}
void ustrig( vector<int> *, vector<int> * );
extern vector<hptrig> *P_trig;
extern vector<hgmtra> *hg_mtra;
// ------------------------------------------------------------------

void gutrev_() {

  int i2=2;
  gcnumx_.ntmsto = 0, gcnum_.nstmax = 0, gcnumx_.nalive = 0;

  vector<int> *ptrig = new vector<int>;
  zgkine ZGKINE;
  int nv = gcnum_.nvertx;
  for( int iv = 1; iv<=nv; iv++) {      // for each vertex in turn ..
     float *pjx = ZGKINE.VertexBank(iv);
     int nt = int(*(pjx+7));
     if (nt <= 0) continue;
     gctrak_.tofg   = *(pjx+4);
     gctrak_.safety = 0.;

     for(int it= 0; it<nt; it++) { //loop over tracks attached to curr vertex
        gckine_.itra = int(*(pjx+8+it));
        float *tjx = ZGKINE.TrackBank(gckine_.itra);
        if(*((int *)tjx)&1) continue;
	gckine_.ivert = int(*(tjx+6));
        if (gckine_.ivert != iv) {
           arteMsg << "gtreve :  abnormal track/vertex connection " 
                   << iv+1  << " " << int(*(tjx+6)) ;
           arteMsg.warn();
        }
        int trig = 1; 
        gckine_.ipart = int(*(tjx+5));
        if(P_trig != NULL &&  P_trig->size() > 0) {
          float pt = *(tjx+1) * *(tjx+1) + *(tjx+2) * *(tjx+2);
          float p  = sqrt( pt + *(tjx+3) * *(tjx+3));
          pt = sqrt(pt);
          trig = 0;
          for(unsigned n=0; n<P_trig->size(); n++) { 
             vector<hptrig>::iterator nt = P_trig->begin() + n;
             if(gckine_.ipart == nt->ipart && p > nt->p && pt > nt->pt ) {
                trig = 1;
	   	ptrig->push_back(gckine_.itra);
                break;
	     }
          }
        }
        if(!trig) continue; 
	memcpy(gckine_.pvert,tjx+1,12); 
        memcpy(gckine_.vert,pjx+1,12);
        gsstak_(&i2);
     }
//   start tracking phase
     while (gcnumx_.nalive) {     // pick-up next track in stack
        gcnumx_.nalive --;
        gltrac_();    // initialize tracking parameters
        if (!gctmed_.numed) continue;
        gutrak_();
        if (gcflag_.ieotri) {
	  if(ptrig != NULL) delete ptrig; 
          return;
        }
     }
   
   }

   crkibk_();

   if(P_trig == NULL || P_trig->size() == 0 ) {
      if(ptrig != NULL) delete ptrig;
      return;
   }


   vector<int> *mtrig = new vector<int>;
   vector<int> *ntrig = new vector<int>;
   for (unsigned i=0; i<ptrig->size(); i++) {
    vector<int>::iterator nt = ptrig->begin() + i;
    vector<hgmtra>::iterator kt =  hg_mtra->begin() + *nt -1;
    mtrig->push_back( kt->mtra);
    ntrig->push_back( kt->nhit); 
   }

   ustrig(mtrig,ntrig);
   
   if(mtrig != NULL) delete mtrig;
   if(ntrig != NULL) delete ntrig;

   if (gcflag_.ieotri) { 
     if(ptrig != NULL) delete ptrig;
     return; 
   }  


   for( int iv = 1; iv<=nv; iv++) {      // for each vertex in turn ..
     float *pjx = ZGKINE.VertexBank(iv);
     int nt = int(*(pjx+7));
     if (nt <= 0) continue;
     gctrak_.tofg   = *(pjx+4);
     gctrak_.safety = 0.;
 
     for(int it= 0; it<nt; it++) { //loop over tracks attached to curr vertex
        gckine_.itra = int(*(pjx+8+it));
        int itrig = 1;
        for(unsigned ij =0;ij < ptrig->size(); ij++) {
          vector<int>::iterator nt = ptrig->begin() + ij;
	  if(*nt == gckine_.itra) {
	    itrig=0;
            break;
          }
        }
        if(!itrig) continue; 
        float *tjx = ZGKINE.TrackBank(gckine_.itra);
        if(*((int *)tjx)&1) continue;
	gckine_.ivert = int(*(tjx+6));
        if (gckine_.ivert != iv) {
           arteMsg << "gtreve :  abnormal track/vertex connection " 
                   << iv+1  << " " << int(*(tjx+6)) ;
           arteMsg.warn();
        }
        gckine_.ipart = int(*(tjx+5));
	memcpy(gckine_.pvert,tjx+1,12); 
        memcpy(gckine_.vert,pjx+1,12);
        gsstak_(&i2);
     }     
//   start tracking phase
     while (gcnumx_.nalive) {     // pick-up next track in stack
        gcnumx_.nalive --;
        gltrac_();    // initialize tracking parameters
        if (!gctmed_.numed) continue;
        gutrak_();
        if (gcflag_.ieotri) { 
  	   if(ptrig != NULL) delete ptrig;
           return;
        }
     }
   }
   crkibk_();
   if(ptrig != NULL) delete ptrig;
 
}
