
/***********************************************************************
*
* KACORR
* written by Siegmund Nowak     04/02
*
* Sample for different ARTE correction by kumac commands:
*
* kacorr020103() action routine for the kumac command CORR020103 which 
* introduces additional air walls, replaces vessel end cup sphere by a box
* (in the arte table NWAL). Foreseen for use with already existing geometries,
* especially geovers 2.0103)
*
************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <math.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "arte/ArteUtil.h"
#include "arte/ArteFlag.h"
#include "gean/ZGMOTH.h"


extern "C" {
  int  matnum_(char*,int);



void kacorr020103() {

  float wzmin[5],wzmax[5],wxmin[5],wxmax[5],wymin[5],wymax[5];
  float xcenter=0., ycenter=0.;
  int fullflag = 0;

  wzmax[0] = 270.;
  wzmin[1] = 270.1;
  int imat = matnum_("AIR",3);
  ArtePointer<GMAT> gm(imat);

  if(!ArteTable<NESL>::size()) {
    arteMsg << "to apply  corr020103 GESL and GWAL banks must exist";
      arteMsg.warn();
      return; 
  }

  for( ArteTableIterator<GESL> gs; gs.valid(); gs.next()){
     if(!strncmp(gs->name,"RPOT",4)) { 
        for(int j=(*gs)->fwl; j<(*gs)->fwl+(*gs)->nwl; j++){
	  ArtePointer<GWAL> nw(j);
          if(!strncmp(nw->gvol,"VPSM",4)) {
            if(nw->shp == 1) {
	      arteMsg << "correction 020103 already done";
              arteMsg.warn();
              fullflag = 1;
              break; 
            } 
            float zmin = nw->zmin - nw->xmax;
            float zmax = nw->zmin - nw->xmin;
            float ro =  nw->xmin * sin(nw->zmax* AR_DEGRAD);
            nw->zmin = zmin; 
            nw->zmax = zmax;
            nw->xmin = -ro;
            nw->xmax = ro;
            nw->ymin = -ro;
            nw->ymax = ro;      
            nw->shp = 1;
            nw->xmad =0;   nw->ymad = 0.;    nw->zmad = 0.;
            break;
          }
        }
	if(fullflag) break;
     }
     else if(!strncmp(gs->name,"MAGN",4)) { 
        xcenter = 0.5*(gs->x1 + gs->x2);
        ycenter = 0.5*(gs->y1 + gs->y2);
     } 
     else if(!strncmp(gs->name,"MC01",4)) wzmin[0] = gs->z2 + .1;

     else if(!strncmp(gs->name,"PC01",4)) {
       if(gs->nsl != 2) {
         fullflag = 1;
         break; 
       }
     }
     else if(!strncmp(gs->name,"MS10",4)) {
       if(gs->nsl != 2) {
         fullflag = 1;
         break; 
       }
       wzmax[4] = gs->z1 - .1;          
     }
     else if(!strncmp(gs->name,"PTL1",4)) {
       wzmax[1] = gs->z1 - .1; 
       wzmin[2] = gs->z2 + .1; 
     }
     else if(!strncmp(gs->name,"PTL2",4)) {
       wzmax[2] = gs->z1 - .1;
       wzmin[3] = gs->z2 + .1; 
     }
     else if(!strncmp(gs->name,"PTL3",4)) {
       wzmax[3] = gs->z1 - .1;
       wzmin[4] = gs->z2 + .1;
     }
  }
  if(fullflag) return;

  float tgx = tan(0.250);
  float tgy = tan(0.160); 
  for(int i=0; i<5;i++) {
     wxmin[i] = xcenter - tgx*wzmax[i]; 
     wxmax[i] = xcenter + tgx*wzmax[i];
     wymin[i] = ycenter - tgy*wzmax[i];
     wymax[i] = ycenter + tgy*wzmax[i];
     struct GWAL nw;
     memset(&nw,0,sizeof(nw));
     nw.shp = 1;
     nw.xmin = xcenter - tgx*wzmax[i];
     nw.xmax = xcenter + tgx*wzmax[i];
     nw.ymin = ycenter - tgy*wzmax[i];
     nw.ymax = ycenter + tgy*wzmax[i];
     nw.zmin = wzmin[i];
     nw.zmax = wzmax[i];
     nw.rot[0] = 1.; nw.rot[4] = 1.; nw.rot[8] = 1.;
     nw.nsl = 1;
     float xms = (nw.zmax - nw.zmin) /gm->radl;  
     nw.ms = xms*pow(0.0136*(1.+ 0.038*log(double(xms))),2.);
     strncpy(nw.gvol,"    ",4);  
     ArteTable<GWAL>::push_back(nw);
     int ngwal = ArteTable<GWAL>::size(); 
     ArtePointer<GWAL> NW(ngwal);
     createRelation(gm, NW, &GMAT::gwal);
  } 
}

}



