// 
// Boris Fominykh  18.01.1998
//
// updated 24.01.2001 by B.Fominykh 
//   - implemented volumes MU01, MU02, MU03, MU04 instead of MUON

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <iostream.h>
#include <math.h>
#include "gean/hgdat.h"
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include "gean/ZGESL.h"

#include "gean/ZGMOTH.h"

#include "gean/zgvers.hh"
#include "gean/ZGDETV.h"

#include "gean/MuGeo.h"
#include "gean/MuGeoMother.icc"
#include "gean/MuGeoSimple.icc"
#include "gean/MuGeoChamTube.icc"
#include "gean/MuGeoChamPad.icc"
#include "gean/MuGeoChamPix.icc"

FILE *fmu0, *utfilp(const char*,const char*); 

extern "C" void muarrange_();

extern "C" {
void mugeo_(char* version) {

// control parameter of the card print
  int PrintCont = 0;

// create mother volumes
  struct MuMothVol Vol;
  int icheck=Vol.Mother();
  if(icheck != 0) {
    arteMsg << "MUGEO: problems with overall volumes";
    arteMsg.error();
  }


// make the objects for dead material
  char* BoxCh = "BOX";
  char* BoxSh = "BOX ";
  MuGeoSimple box(&Vol,BoxSh);
  char* TubeCh = "TUBE";
  MuGeoSimple tube(&Vol,TubeCh);
  char* TubsCh = "TUBS";
  MuGeoSimple tubs(&Vol,TubsCh);
  char* ConeCh = "CONE";
  MuGeoSimple cone(&Vol,ConeCh);

// make the objects for muon chambers
  char* TubChambCh = "TUBE";
  char* PadChambCh = "PAD";
  char* PixChambCh = "PIX";

  char  Version[6];
  if(strcmp(version,"SHORT") && strcmp(version,"LONG")) {
    arteMsg << "MUGEO - unknown version: " << version;
    arteMsg.error();
  }
  strcpy(Version,version);
  MuGeoChamTube chamtub(&Vol,Version);
  MuGeoChamPad  champad(&Vol,Version);
  MuGeoChamPix  champix(&Vol,Version);

// name of actual mother volume
  char MoVol[5];

// offset and rotation parameters
  float Xofs=0.;
  float Yofs=0.;
  float Zofs=0.;
  int  Irotm=0;
  float Angle=0.;
  int   NrotmD=0;
  int   IrotmD[20];
  float AnglD[20];

// layer of chambers
  char ChamMat[15];
  char ChamGas[15];
  char ChamWall[15];
  strcpy(ChamMat,"NOTHING");
  strcpy(ChamGas,"NOTHING");
  strcpy(ChamWall,"NOTHING");

  const int Nchpar=36;
  float ChamPar[Nchpar];
  for(int i=0; i<Nchpar; i++) ChamPar[i]=100000.;

  char Type[5];
  int NumCham=0;
  int IdNum[100];
  float PlateCode[100];
  float UpDown[100];
  float LenCham[100];
  float DLenCham[100];
  float XposCham[100];
  float YposCham[100];
  float XbaseCham=0.;
  float YbaseCham=0.;
  float ZbaseCham=0.;
  int Musl=-1;

// open the input file
  char buffer[256]; char filnam[80];
  zgvers ZGVERS;
  strcpy(filnam,ZGVERS.Get_vers_datafile(LGESL_MUON));
  fmu0= utfilp(filnam,"");

// read the file contents line by line
  int icd=0;
  int npar=0;
  float par[10];
  char type[5];

// loops over lines: max 5000 lines
  for(int icc=0; icc<5000; icc++) {
    if(fgets(buffer,256,fmu0) == NULL) {
      fclose(fmu0);
      if(feof(fmu0))         return;
      else { 
        arteMsg << "MUGEO: error in reading " << filnam;
        arteMsg.error();
      }
    }

// card number
    ++icd;
    char *card = &buffer[0];
    int sens=0;
// read the first key word 
    char idr[13];
    int nitem=sscanf(card,"%s",idr);
// pass  blank lines and commented cards
    if(nitem == -1) continue;
    if(idr[0]=='#' || idr[0]=='*' ||idr[0]=='/') continue;
    card+=itemlength(card,nitem);

// print first 80 characters of card contents
    if(PrintCont == 1) {
      char prbuf[82];
      strncpy(prbuf,buffer,80);
      prbuf[80] = '\0'; 
      arteMsg << icd << ": " << prbuf;
      arteMsg.allp();
    }

// end of file
    if(!strcmp(idr,"end")) {
      fclose(fmu0);
//      if(zgdetv_.DETV[LGESL_MUON-1] == 'L') muarrange_();
      return ;
    }

// control parameters
    else if(!strcmp(idr,"print")) {
      PrintCont = 1;
      arteMsg << "MUGEO: print the cards";
      arteMsg.warn();
    }

    else if(!strcmp(idr,"no_print")) {
      PrintCont = 0;
      arteMsg << "MUGEO: no card print";
      arteMsg.warn();
    }

// set mother volume
    else if(!strcmp(idr,"volume")) {
      nitem=sscanf(card,"%s",MoVol);
      if(nitem != 1) break;
      int nvol=-1;
      if(!strcmp(MoVol,"MUON")) nvol=0;
      if(!strcmp(MoVol,"MU01")) nvol=1;
      if(!strcmp(MoVol,"MU02")) nvol=2;
      if(!strcmp(MoVol,"MU03")) nvol=3;
      if(!strcmp(MoVol,"MU04")) nvol=4;
      if(nvol == -1) {
       arteMsg << "MUGEO: unknown volume - " << MoVol;
       arteMsg.warn();
       break;
      }
      else {
	int icheck=Vol.MoVol(nvol);
	if(icheck != 0) {
	  arteMsg << "MUGEO: problems with volume " << MoVol;
	  arteMsg.error();
	}
      }
    }

// frame
    else if(!strcmp(idr,"frame")) {
      nitem=sscanf(card,"%f %f %f",&Xofs,&Yofs,&Zofs);
      if(nitem != 3) break;
    }

//  frame_end
    else if(!strcmp(idr,"frame_end")) {
      Xofs=0.; Yofs=0.; Zofs=0.;
    }

//  rotation angle in degrees
    else if(!strcmp(idr,"rotation")) {
       nitem=sscanf(card,"%f",&Angle);
       if(nitem != 1) break;
//     is it zero?
       if(Angle == 0.) Irotm=0;
       else {        
//        check if the rotation matrix exists?
          if(NrotmD > 0) {
             for(int i=0; i<NrotmD; i++) {
                if(AnglD[i] == Angle) {
                  Irotm=IrotmD[i];
                  goto m1;
                }
             }
	  }
//        book a new rotation matrix
          float fzero=0.;
          float f90=90.;
          float f90p=90.-Angle;
          float anglm=-Angle;
          crrotm_(&Irotm,&f90,&anglm,&f90,&f90p,&fzero,&fzero);
//        add it to the list of used rotation matrixes
          AnglD[NrotmD]=Angle;
          IrotmD[NrotmD]=Irotm;
          NrotmD+=1;
       }
    }

//  rotation_end
    else if(!strcmp(idr,"rotation_end")) {
      Angle=0.;
      Irotm=0;
      continue;
    }

// chamber structure
    else if(!strcmp(idr,"CHAMB_STRUCT")) {
      nitem=sscanf(card,"%s",type);
      if(nitem != 1) break;
      if(strcmp(type,TubChambCh) && strcmp(type,PadChambCh) && strcmp(type,PixChambCh)) {
       arteMsg << "MUGEO: unknown chamber type - " << type << " - in CHAMB_STRUCT";
       arteMsg.warn();
       break;
      }
      card+=itemlength(card,nitem);
// loops over parameters
      char key[10];
      Musl=-1;
  mk1:;
      nitem=sscanf(card,"%s",key);
      if(!strcmp(key,"End")) {
        if(!strcmp(type,TubChambCh)) chamtub.Param(ChamMat,ChamGas,ChamWall,ChamPar);
        else if(!strcmp(type,PadChambCh)) champad.Param(ChamMat,ChamGas,ChamWall,Musl,ChamPar);
        else if(!strcmp(type,PixChambCh)) champix.Param(ChamMat,Musl,ChamPar);

        strcpy(ChamMat,"NOTHING");
        strcpy(ChamGas,"NOTHING");
        strcpy(ChamWall,"NOTHING");
        Musl=-1;
        for(int i=0; i<Nchpar; i++) ChamPar[i]=100000.;
        continue;
      }

      card+=itemlength(card,nitem);
      if(!strcmp(key,"Mater")) {
        nitem=sscanf(card,"%s",ChamMat);
        if(nitem != 1) break;
        card+=itemlength(card,nitem);
      }
      else if(!strcmp(key,"MU1"))  Musl=0;
      else if(!strcmp(key,"MU2"))  Musl=1;
      else if(!strcmp(key,"MU3"))  Musl=2;
      else if(!strcmp(key,"MU4"))  Musl=3;
      else if(!strcmp(key,"MatWall")) {
        nitem=sscanf(card,"%s",ChamWall);
        if(nitem != 1) break;
        card+=itemlength(card,nitem);
      }
      else if(!strcmp(key,"MatGas")) {
        nitem=sscanf(card,"%s",ChamGas);
        if(nitem != 1) break;
        card+=itemlength(card,nitem);
      }

      else {
        float param;
        nitem=sscanf(card,"%f",&param);
        if(nitem != 1) continue;
        card+=itemlength(card,nitem);
        if(!strcmp(key,"Eff"))               ChamPar[0]=param;
        else if(!strcmp(key,"Xwidth"))       ChamPar[1]=param;
        else if(!strcmp(key,"Zwidth"))       ChamPar[2]=param;
        else if(!strcmp(key,"Overlap"))      ChamPar[3]=param;
        else if(!strcmp(key,"Pitch"))        ChamPar[4]=param;
        else if(!strcmp(key,"DriftDist"))    ChamPar[5]=param;
        else if(!strcmp(key,"Wall"))         ChamPar[6]=param;
        else if(!strcmp(key,"Zpad_shift"))   ChamPar[7] =param;
        else if(!strcmp(key,"Zwire_pad"))    ChamPar[8] =param;
        else if(!strcmp(key,"Xsize"))        ChamPar[9] =param;
        else if(!strcmp(key,"Xedge"))        ChamPar[10]=param;
        else if(!strcmp(key,"Xgap"))         ChamPar[11]=param;
        else if(!strcmp(key,"Y1size"))       ChamPar[12]=param;
        else if(!strcmp(key,"Y2size"))       ChamPar[13]=param;
        else if(!strcmp(key,"Yedge"))        ChamPar[14]=param;
        else if(!strcmp(key,"Ygap"))         ChamPar[15]=param;
        else if(!strcmp(key,"Yeff0"))        ChamPar[16]=param;
        else if(!strcmp(key,"Eff_middle"))   ChamPar[17]=param;
        else if(!strcmp(key,"Ywidth"))       ChamPar[18]=param;
        else if(!strcmp(key,"NXpix"))        ChamPar[19]=param;
        else if(!strcmp(key,"NYpix"))        ChamPar[20]=param;
        else if(!strcmp(key,"NXpixCon"))     ChamPar[21]=param;
        else if(!strcmp(key,"NYpixCon"))     ChamPar[22]=param;
        else if(!strcmp(key,"Ysize"))        ChamPar[23] =param;
        else if(!strcmp(key,"RholeSens"))    ChamPar[24] =param;
        else if(!strcmp(key,"Npad"))         ChamPar[25] =param;
        else if(!strcmp(key,"Nwire"))        ChamPar[26] =param;
        else if(!strcmp(key,"PlLen1"))       ChamPar[27] =param;
        else if(!strcmp(key,"PlBas1"))       ChamPar[28] =param;
        else if(!strcmp(key,"PlLen2"))       ChamPar[29] =param;
        else if(!strcmp(key,"PlBas2"))       ChamPar[30] =param;
        else if(!strcmp(key,"PlLen3"))       ChamPar[31] =param;
        else if(!strcmp(key,"PlBas3"))       ChamPar[32] =param;
        else if(!strcmp(key,"PlWid"))        ChamPar[33] =param;
        else if(!strcmp(key,"PlThick"))      ChamPar[34] =param;
        else if(!strcmp(key,"PlShift"))      ChamPar[35] =param;
        else {
           arteMsg << "MUGEO - unknown key word: " << key;
           arteMsg.warn();
           break;
        }
      }
      goto mk1;
    }

// chamber layer
    else if(!strcmp(idr,"CHAMB_LAYER")) {
      int nchlay;
      float xchlay,ychlay,zchlay;
      char msl[4];
      nitem=sscanf(card,"%s %s %d %f %f %f",type,msl,&nchlay,&xchlay,&ychlay,&zchlay);
      if(nitem != 6) break;
      card+=itemlength(card,nitem);
      if(strcmp(type,TubChambCh) && strcmp(type,PadChambCh) && strcmp(type,PixChambCh)) {
       arteMsg << "MUGEO - unknown chamber type: " << type;
       arteMsg.warn();
       break;
      }
      card+=itemlength(card,nitem);
      NumCham=nchlay;
      if(!strcmp(msl,"MU1"))      Musl=0;
      else if(!strcmp(msl,"MU2")) Musl=1;
      else if(!strcmp(msl,"MU3")) Musl=2;
      else if(!strcmp(msl,"MU4")) Musl=3;
      for(int ii=0; ii<NumCham; ii++) {
        UpDown[ii]=0.;
        IdNum[ii]=0;
        PlateCode[ii]=100000.;
        LenCham[ii]=100000.;
        DLenCham[ii]=100000.;
        XposCham[ii]=100000.;
        YposCham[ii]=100000.;
      }
      XbaseCham=xchlay;
      YbaseCham=ychlay;
      ZbaseCham=zchlay;
      strcpy(Type,type);
      continue;
    }
// chamber parameters
    else if(!strcmp(idr,"CHAMB")) {
      char lexy[10];
      int nchmin, nchmax;
      nitem=sscanf(card,"%d %d %s",&nchmin,&nchmax,lexy);
      if(nitem != 3) break;
      card+=itemlength(card,nitem);
      if(nchmin > nchmax) break;
      if(nchmin < 1 || nchmin > NumCham) break;
      if(nchmax < 1 || nchmax > NumCham) break;
      for(int ichb=0; ichb<nchmax-nchmin+1; ichb++) {
        if(!strcmp(lexy,"ID_NUM"))
          nitem=sscanf(card,"%d",&IdNum[nchmin+ichb-1]);
        else if(!strcmp(lexy,"PLATE")) 
          nitem=sscanf(card,"%f",&PlateCode[nchmin+ichb-1]);
        else if(!strcmp(lexy,"LENGTH")) 
          nitem=sscanf(card,"%f",&LenCham[nchmin+ichb-1]);
        else if(!strcmp(lexy,"DLENDN")) {
          UpDown[nchmin+ichb-1]=-1.;
          nitem=sscanf(card,"%f",&DLenCham[nchmin+ichb-1]);
        }
        else if(!strcmp(lexy,"DLENUP")) {
          UpDown[nchmin+ichb-1]=1.;
          nitem=sscanf(card,"%f",&DLenCham[nchmin+ichb-1]);
        }
        else if(!strcmp(lexy,"X-POS")) 
          nitem=sscanf(card,"%f",&XposCham[nchmin+ichb-1]);
        else if(!strcmp(lexy,"Y-POS")) 
          nitem=sscanf(card,"%f",&YposCham[nchmin+ichb-1]);
        else {
           arteMsg << "MUGEO: parameter " << lexy << " is unknown";
           arteMsg.warn();
           goto m2;
        }
       if(nitem != 1) break;
       card+=itemlength(card,nitem);
      }
      continue;
    }
// end of chamber layer
    else if(!strcmp(idr,"CHAMB_END")) {
// check that all parameters are filled
      int ierr=0;
      for(int ii=0; ii<NumCham; ii++) {
        if(IdNum[ii] == 0.) {
          ierr+=1;
          arteMsg << "MUGEO: ID_NUMBER of chamber ";
          arteMsg << dec << ii+1 << " is unknown";
          arteMsg.warn();
        }
        if(XposCham[ii] == 100000.) {
          ierr+=1;
          arteMsg << "MUGEO: X-pos of chamber ";
          arteMsg << dec << ii+1 << " is unknown";
          arteMsg.warn();
        }
        if(YposCham[ii] == 100000.) {
          ierr+=1;
          arteMsg << "MUGEO: Y-pos of chamber ";
          arteMsg << dec << ii+1 << " is unknown";
          arteMsg.warn();
        }
        if(!strcmp(Type,TubChambCh) || !strcmp(Type,PadChambCh)) {
          if(LenCham[ii] == 100000.) {
            ierr+=1;
            arteMsg << "MUGEO: LENGTH of chamber ";
            arteMsg << dec << ii+1 << " is unknown";
            arteMsg.warn();
          }
        }
        if(!strcmp(Type,TubChambCh)) {
          if(PlateCode[ii] == 100000.) {
            ierr+=1;
            arteMsg << "MUGEO: PLATE code of tube chamber ";
            arteMsg << dec << ii+1 << " is unknown";
            arteMsg.warn();
          }
          if(UpDown[ii] == 0.) {
            ierr+=1;
            arteMsg << "MUGEO: UpDown of tube chamber ";
            arteMsg << dec << ii+1 << " is unknown";
            arteMsg.warn();
          }
          if(DLenCham[ii] == 100000.) {
            ierr+=1;
            arteMsg << "MUGEO: DLENgth of tube chamber ";
            arteMsg << dec << ii+1 << " is unknown";
            arteMsg.warn();
          }
        }
      }
      if(ierr > 0) {
        arteMsg << "MUGEO: " << dec << ierr;
        arteMsg << " errors - don't create the chamber layer";
        arteMsg.warn();
        continue;
      }
// book the chambers
      for(int ichm=0; ichm<NumCham; ichm++) {
        int Chamid=IdNum[ichm];
        par[0]=XposCham[ichm]+XbaseCham+Xofs;
        par[1]=YposCham[ichm]+YbaseCham+Yofs;
        par[2]=ZbaseCham+Zofs;
        par[3]=LenCham[ichm];
        if(!strcmp(Type,TubChambCh)) {
          par[4]=DLenCham[ichm];
          par[5]=UpDown[ichm];
          par[6]=PlateCode[ichm];
          icheck = chamtub.Position(&Vol,Chamid,Musl,Irotm,Angle,par);
	  if(icheck != 0) goto m2;
        }
        else if(!strcmp(Type,PadChambCh)) {
          par[4]=0.; par[5]=1.;
          par[6]=PlateCode[ichm];
          icheck = champad.Position(&Vol,Chamid,Musl,par);
	  if(icheck != 0) goto m2 ;
        }
        else if(!strcmp(Type,PixChambCh)) {
          par[3]=0.; par[4]=0.; par[5]=1.;
          icheck = champix.Position(&Vol,Chamid,Musl,par);
	  if(icheck != 0) goto m2;
        }
      }
      continue;
    }

// analysis of the rest of the card in the case of dead material
// material & manon
    else if(!strcmp(idr,BoxCh) || !strcmp(idr,TubsCh) || 
            !strcmp(idr,TubeCh) || !strcmp(idr,ConeCh) ) {
       char matname[20], manon[5], vacuum[10], insname[10];
       int ivacuum=-1;
       nitem=sscanf(card,"%s %s",matname,manon) ;
       if(nitem != 2) break;
       card+=itemlength(card,nitem);
//     in case of tube or cone define inner filling
       if(!strcmp(idr,TubeCh) || !strcmp(idr,ConeCh)) {
          nitem=sscanf(card,"%s",vacuum) ;
          if(nitem != 1) break;
          card+=itemlength(card,nitem);
          if     (!strcmp(vacuum,"VACONLY"))  ivacuum=1;
          else if(!strcmp(vacuum,"VACMANY"))  ivacuum=2;
          else if(!strcmp(vacuum,"AIRONLY"))  ivacuum=3;
          else if(!strcmp(vacuum,"AIRMANY"))  ivacuum=4;
          else if(!strcmp(vacuum,"NOTHING"))  ivacuum=0;
          else break;
       } 
//     parameter definition 
       if(!strcmp(idr,BoxCh) || !strcmp(idr,TubeCh)) npar=6;
       else  npar = 8;
       for(int i=0; i<npar; i++) {
          nitem=sscanf(card,"%f",&par[i]);
          if(nitem != 1) goto m2;
          card+=itemlength(card,nitem);
       }
       par[0]=par[0]+Xofs;
       par[1]=par[1]+Yofs;
       par[2]=par[2]+Zofs;

//     posit volumina
       if(!strcmp(idr,BoxCh)) 
          icheck = box.Position(matname,manon,sens,Irotm,Angle,par);
       else if(!strcmp(idr,TubeCh)) 
          icheck = tube.Position(matname,manon,sens,0,0.,par);
       else if(!strcmp(idr,TubsCh)) 
          icheck = tubs.Position(matname,manon,sens,0,0.,par);
       else if(!strcmp(idr,ConeCh)) {
          icheck = cone.Position(matname,manon,sens,0,0.,par);
      }

//    posit vacuum or air inside tube or cone 
      if(ivacuum > 0) {
        par[5]=par[4];
        par[4]=0.;
//      vacuum or air inside
        if(ivacuum < 3)  strcpy(insname,"VACUUM_MU");
        else             strcpy(insname,"AIR_MU");
        if(ivacuum == 1 || ivacuum == 3) strcpy(manon,"ONLY");
        else                             strcpy(manon,"MANY");
        if(!strcmp(idr,TubeCh)) 
          icheck = tube.Position(insname,manon,0,0,0.,par);
        else if(!strcmp(idr,ConeCh)) {
          par[7]=par[6];
          par[6]=0.;
          icheck = cone.Position(insname,manon,0,0,0.,par);
        }
      }
  }
  else { 
    arteMsg << "MUGEO: shape " << dec << idr << " is unknown - pass the line";
    arteMsg.warn();
  }
// end of the line analysis
 m1:;
  }
// output in the case of error
 m2:
    arteMsg << "MUGEO: input data error in card " << dec << icd;
    arteMsg.warn();
    fclose(fmu0);
    arteMsg << "MUGEO: close file " << filnam;
    arteMsg.error();
  return ;
}
}
