// ********************************************************************
//
// source:    otcaps.C
//
// type:      initialization of OTR caps geometry
//
// created:   Sep-2001
//
// author:    Oxana Barsoukova
//
// changes:
//
// ********************************************************************

#include <stdio.h>
#include <math.h>
#include <arte/arte.hh>
#include "gean/ZGESL.h"

#define m_pi 3.14159265358979323846

typedef struct
{
  float ang;
} strA2;

extern "C"
{
  extern strA2 a2_;
  void gsposp_(char*,int*,char*,float*,float*,float*,int*,char*,float*,int*); 
  void crgwal_(char*,int*);
  void crrotm_(int*,float*,float*,float*,float*,float*,float*);
}

extern "C" {

void capsmc_(char* csl,float* xp_sl,float* yp_sl,float* zp_sl,int* istereo)
{
// This function assignes MC caps, besides MC05 superlayer

  strA2 & ref = a2_;

  int nnwal;
  float xp_sl_in=*xp_sl, yp_sl_in=*yp_sl, zp_sl_in=*zp_sl;
  int istereo_in=*istereo;
  int npar;
  char smod[5];

  // _pls_ means +x half, _mns_ means -x half
  float xmin_pls_rot[3], xmax_mns_rot[3];
  float x_pls_lab = 0;
  float x_mns_lab = 0;

  float z_max, z_min, y_middle_rot_pls, y_width_rot_pls=0, y_middle_rot_mns, y_width_rot_mns=0;
  float ymin_pls_rot[3], ymax_pls_rot[3],ymin_mns_rot[3], ymax_mns_rot[3];
  float y_lab_pls = 0, y_lab_mns = 0;
  float cosang = 0, sinang = 0;
  float x_min1,x_min2;

  float xp, yp, zp;
  float par[6];
  int rotm;
  float shift_x, shift_z;
  float y_add_c = 0, y_add_p = 0; // +/-7cm. for caps on the top and bottom edges

  float theta1,phi1,theta2,phi2,theta3,phi3;
  
  strncpy(smod,csl,4);
  smod[4] = '\0';
  
  char vol[5];
  strncpy(vol,"WTCO",4);
  vol[4] = '\0';
  
  char manon[5];
  strncpy(manon,"ONLY",4);
  manon[4] = '\0';
  
  npar = 6;
  shift_z = 0.3;
  
  theta3 = 0;
  phi3 = 0;

  // loop on stereo layers
  for (int j = 1; j <= istereo_in; j++)
    {
      for (int i=0; i<3; i++)
	{
	  xmin_pls_rot[i]=9000;
	  xmax_mns_rot[i]=-9000;
	  
	  ymin_pls_rot[i]=9000;
	  ymax_pls_rot[i]=-9000;

	  ymin_mns_rot[i]=9000;
	  ymax_mns_rot[i]=-9000;
	}
      z_max = 0;
      z_min = 100000;
      x_min1=9000, x_min2=9000;

      // rotation matrixes for MCs
      if (j == 1)
        {
          theta1 = 90;
          phi1 = ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 + ref.ang*180/m_pi;
        }
      else if (j == 2)
        {
          theta1 = 90;
          phi1 = 0;
          theta2 = 90;
          phi2 = 90;
        }
      else
        {
          theta1 = 90;
          phi1 = -ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 - ref.ang*180/m_pi;
        }
      
      crrotm_(&rotm,&theta1,&phi1,&theta2,&phi2,&theta3,&phi3);

      // search for the nearest modules to caps so as to calculate positions of 
      // minimal x in +half, maximal x in -half and maximal z in each stereo layer

      for (ArteTableIterator<NEDE> NedeIt; NedeIt.valid(); NedeIt.next())
	{
	  ArtePointer<NEDE> nede(NedeIt);
	  if (!strncmp(smod, nede->nesl->name, 4))
	    if (nede->lay == j)
	      {
		cosang = nede->rot[0];
		sinang = nede->rot[1];
		
		// array elements contain: [0] - for sector 1 (-half),
		// [1] - for sector 2 (-half), [2] - for sectors 1, 2 (+half)

		if (nede->zmax > z_max) z_max = nede->zmax;
		if (nede->zmin < z_min) z_min = nede->zmin;
		
		if (!strncmp(nede->hwc, "-", 1))
		  {
		    if (nede->sec == 1 && nede->xmax > xmax_mns_rot[0]) 
		      {
			xmax_mns_rot[0]=nede->xmax;
			ymin_mns_rot[0]=nede->ymin;
			ymax_mns_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 2 && nede->xmax > xmax_mns_rot[1]) 
		      {
			xmax_mns_rot[1]=nede->xmax;
			ymin_mns_rot[1]=nede->ymin;
			ymax_mns_rot[1]=nede->ymax;
		      }
		  }
		if (!strncmp(nede->hwc, "+", 1)){
		  
                    if (nede->sec == 1 && nede->xmin < x_min1) 
                      {
                        x_min1 = nede->xmin;
                        ymax_pls_rot[2]=nede->ymax;
                      }
                    if (nede->sec == 2 && nede->xmin < x_min2)
                      {
                        x_min2 = nede->xmin;
                        ymin_pls_rot[2]=nede->ymin;
                      }
		}
	      }
	}	
      
      if (x_min1 < x_min2) 
        xmin_pls_rot[2] = x_min1;
      else
        xmin_pls_rot[2] = x_min2;
      
      // set caps according to specified sector's number in array
      for (int i=0; i<3; i++)
	{ 
	  // addition 7cm. to length of the cap
	  if (i == 0 || i == 2)
	    {
	      if (i == 2 &&
		  (!strncmp(smod, "MC03", 4) || !strncmp(smod, "MC04", 4) ||
		   !strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4)))
		{
		  y_add_c = 0;
		  y_add_p = 7;
		  
		} else {
		  
		  y_add_c = 3.5;
		  y_add_p = 3.5;
		  
		  if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
		    {
		      y_add_c = 0;
		      y_add_p = 0;
		    }
		}
	    }
	  else
	    {
	      y_add_c = -3.5;
	      y_add_p = 3.5;

	      if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
		{
		  y_add_c = 0;
		  y_add_p = 0;
		}
	    }

	  // shift from the end point of module by x
	  shift_x = (z_max - z_min)/2*tan(1.05) - 0.6;
	  if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
	    shift_x = (z_max - z_min)/2*tan(1.05) - 0.3;

	  zp = 0.5*(z_max + z_min) - zp_sl_in;
	  par[0] = 0.05;
	  par[2] = 2.4;
	  par[3] = 0;
	  par[4] = 300;
	  par[5] = 0;
	  
	  if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
	    par[2] = 1.5;

	  if (i < 2) 
	    {
	      // parallelepiped for -x half
	      y_middle_rot_mns = 0.5*(ymin_mns_rot[i] + ymax_mns_rot[i]);
	      y_width_rot_mns = 0.5*(ymax_mns_rot[i] - ymin_mns_rot[i]);
	      x_mns_lab = xmax_mns_rot[i]*cosang + y_middle_rot_mns*sinang;
	      y_lab_mns = y_middle_rot_mns*cosang - xmax_mns_rot[i]*sinang;

	      xp = x_mns_lab - shift_x - xp_sl_in;
	      yp = y_lab_mns + y_add_c - yp_sl_in;
	      par[1] = y_width_rot_mns*cosang + y_add_p;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	    } 
	  else 
	    {
	      // parallelepiped for +x half
	      y_middle_rot_pls = 0.5*(ymin_pls_rot[i] + ymax_pls_rot[i]);
	      y_width_rot_pls = 0.5*(ymax_pls_rot[i] - ymin_pls_rot[i]);
	      x_pls_lab = xmin_pls_rot[i]*cosang + y_middle_rot_pls*sinang;
	      y_lab_pls = y_middle_rot_pls*cosang - xmin_pls_rot[i]*sinang;

	      xp = x_pls_lab + shift_x - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	    }

	  if (j < 3 && i == 2)
	    {
	      // boxes, +x 
	      xp = x_pls_lab + shift_x - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max + shift_z - zp_sl_in;
	      par[0] = 3.7;
	      if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
		par[0] = 2.4;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	    }
	  if (j > 1 && i < 2)
	    {
	      // boxes, -x
	      xp = x_mns_lab - shift_x - xp_sl_in;
	      yp = y_lab_mns + y_add_c - yp_sl_in;
	      zp = z_min - shift_z - zp_sl_in;
	      par[0] = 3.7;
	      if (!strncmp(smod, "MC01", 4) || !strncmp(smod, "MC02", 4))
		par[0] = 2.4;
	      par[1] = y_width_rot_mns*cosang + y_add_p;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	    }
	}
    }   
  
}


 



void capspc14_(char* csl,float* xp_sl,float* yp_sl,float* zp_sl,int* istereo)
{
// Function specifies caps for PC01 and PC04 superlayers

  strA2 & ref = a2_;

  int nnwal;
  float xp_sl_in=*xp_sl, yp_sl_in=*yp_sl, zp_sl_in=*zp_sl;
  int istereo_in=*istereo;
  int npar;
  char smod[5];

  // _pls_ means +x half, _mns_ means -x half
  float xmin_pls_rot[3], xmax_mns_rot[3];
  float x_pls_lab;
  float x_mns_lab;

  // _rot_ means rotated coordinates system, _lab_ - general
  float z_max, z_min, y_middle_rot_pls, y_width_rot_pls, y_middle_rot_mns, y_width_rot_mns;
  float ymin_pls_rot[3], ymax_pls_rot[3],ymin_mns_rot[3], ymax_mns_rot[3];
  float y_lab_pls, y_lab_mns;
  float cosang = 0, sinang = 0;
  float x_min3,x_min4,x_max5,x_max6;

  float xp, yp, zp;
  float par[6], y1_prot, y2_prot, x56_prot = 0, x34_prot = 0;
  float y1_prot_lab, y2_prot_lab, x1_prot_lab, x2_prot_lab;
  int rotm;
  float shift_x, shift_z;
  float y_add_c = 0, y_add_p = 0; // +/-7cm. for caps on the top and bottom edges
  
  float theta1,phi1,theta2,phi2,theta3,phi3;

  strncpy(smod,csl,4);
  smod[4] = '\0';
  
  char vol[5];
  strncpy(vol,"WTCO",4);
  vol[4] = '\0';
  
  char manon[5];
  strncpy(manon,"ONLY",4);
  manon[4] = '\0';
  
  npar = 6;
  shift_z = 0.3;
  
  theta3 = 0;
  phi3 = 0;
  
  // loop by stereo layers
  for (int j = 1; j <= istereo_in; j++)
    {
      for (int i=0; i<3; i++)
	{
	  xmin_pls_rot[i]=9000;
	  xmax_mns_rot[i]=-9000;
	  
	  ymin_pls_rot[i]=9000;
	  ymax_pls_rot[i]=-9000;

	  ymin_mns_rot[i]=9000;
	  ymax_mns_rot[i]=-9000;
	}
      z_max = 0;
      z_min = 100000;
      x_min3=9000, x_min4=9000, x_max5=-9000, x_max6=-9000;
      y1_prot = 9000;
      y2_prot = -9000;

      // rotation matrixes for PC01, PC04
      if (j%2)
	{
          theta1 = 90;
          phi1 = 0;
          theta2 = 90;
          phi2 = 90;
	}
      else if (j == 2 || j == 6)
	{
          theta1 = 90;
          phi1 = -ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 - ref.ang*180/m_pi;
	}
      else
	{
          theta1 = 90;
          phi1 = ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 + ref.ang*180/m_pi;
	}
      
      crrotm_(&rotm,&theta1,&phi1,&theta2,&phi2,&theta3,&phi3);

      // search for the nearest modules to caps so as to calculate positions of 
      // minimal x in +half, maximal x in -half and max/min z in each stereo layer
      // in rotated coordinates system

      for (ArteTableIterator<NEDE> NedeIt; NedeIt.valid(); NedeIt.next())
	{
	  ArtePointer<NEDE> nede(NedeIt);
	  if (!strncmp(smod, nede->nesl->name, 4))
	    if (nede->lay == j)
	      {
		cosang = nede->rot[0];
		sinang = nede->rot[1];
		
                // array elements contain: [0] - for sector 1 (+/-half),
                // [1] - for sectors 5,6/3,4 (+/-half), [2] - for sector 2 (+/-half)

		if (nede->zmax > z_max) z_max = nede->zmax;
		if (nede->zmin < z_min) z_min = nede->zmin;
		
		if (!strncmp(nede->hwc, "-", 1))
		  {
		    if (nede->sec == 1 && nede->xmax > xmax_mns_rot[0]) 
		      {
			xmax_mns_rot[0]=nede->xmax;
			ymin_mns_rot[0]=nede->ymin;
			ymax_mns_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 5 && nede->xmax > x_max5) 
		      {
			x_max5 = nede->xmax;
			ymax_mns_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 6 && nede->xmax > x_max6)
		      {
			x_max6 = nede->xmax;
			ymin_mns_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmax > xmax_mns_rot[2]) 
		      {
			xmax_mns_rot[2]=nede->xmax;
			ymin_mns_rot[2]=nede->ymin;
			ymax_mns_rot[2]=nede->ymax;
		      }

                    if (nede->sec == 1 && nede->xmax >= -20 
                        && nede->xmax <= 20 && nede->ymin < y1_prot)
                      y1_prot = nede->ymin;
                    if (nede->sec == 2 && nede->xmax >= -20 
                        && nede->xmax <= 20 && nede->ymax > y2_prot)
                      y2_prot = nede->ymax;
		  }
		if (!strncmp(nede->hwc, "+", 1))
		  {
		    if (nede->sec == 1 && nede->xmin < xmin_pls_rot[0]) 
		      {
			xmin_pls_rot[0]=nede->xmin;
			ymin_pls_rot[0]=nede->ymin;
			ymax_pls_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 3 && nede->xmin < x_min3) 
		      {
			x_min3 = nede->xmin;
			ymax_pls_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 4 && nede->xmin < x_min4)
		      {
			x_min4 = nede->xmin;
			ymin_pls_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmin < xmin_pls_rot[2]) 
		      {
			xmin_pls_rot[2]=nede->xmin;
			ymin_pls_rot[2]=nede->ymin;
			ymax_pls_rot[2]=nede->ymax;
		      }
		  }    
	      }
	}	
      
      if (x_max5 > x_max6) 
	xmax_mns_rot[1] = x_max5;
      else
	xmax_mns_rot[1] = x_max6;
      
      if (x_min3 < x_min4) 
	xmin_pls_rot[1] = x_min3;
      else
	xmin_pls_rot[1] = x_min4;

      // set caps according to specified sector's number in array
      for (int i=0; i<3; i++)
	{
	  y_middle_rot_pls = 0.5*(ymin_pls_rot[i] + ymax_pls_rot[i]);
	  y_width_rot_pls = 0.5*(ymax_pls_rot[i] - ymin_pls_rot[i]);

	  y_middle_rot_mns = 0.5*(ymin_mns_rot[i] + ymax_mns_rot[i]);
	  y_width_rot_mns = 0.5*(ymax_mns_rot[i] - ymin_mns_rot[i]);
	  
	  // rot -> lab
	  x_pls_lab = xmin_pls_rot[i]*cosang + y_middle_rot_pls*sinang;
	  y_lab_pls = y_middle_rot_pls*cosang - xmin_pls_rot[i]*sinang;
	  x_mns_lab = xmax_mns_rot[i]*cosang + y_middle_rot_mns*sinang;
	  y_lab_mns = y_middle_rot_mns*cosang - xmax_mns_rot[i]*sinang;
	  
          // addition 7cm. to length of the cap
	  if (i == 0)
	    {
	      y_add_c = 3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 2)
	    {
	      y_add_c = -3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 1)
	    {
	      y_add_c = 0;
	      y_add_p = 0;
	    }

	  // shift from the end point of module by x
	  shift_x = (z_max - z_min)/2*tan(1.05) - 0.8;

	  if (j > 3)
	    {
	      xp = x_pls_lab + shift_x - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = 0.5*(z_max + z_min) - zp_sl_in;
	      
	      par[0] = 0.05;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 2.4;
	      par[3] = 0;
	      par[4] = 300;
	      par[5] = 0;
	      
	      // parallelepiped for +x half
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	      
	      // parallelepiped for -x half
	      xp = x_mns_lab - shift_x - xp_sl_in;
	      yp = y_lab_mns + y_add_c - yp_sl_in;
	      par[1] = y_width_rot_mns*cosang + y_add_p;
	      if (i == 1) // caps near sectors 5-6 are none ordinary
		{
		  par[4]=60;
		  xp = x_mns_lab - 0.5 - xp_sl_in;
		}
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	    }
	  if (j <= 3)
	    {
	      xp = x_pls_lab + 0.2 - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max - 0.4 - zp_sl_in;
	      
	      par[0] = 0.05;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.7;
	      par[3] = 0;
	      par[4] = 300;
	      par[5] = 0;
	      
	      // parallelepiped for +x half
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      // parallelepiped for +x half
	      xp = x_pls_lab + 5.2 - xp_sl_in;
	      zp = z_max - 1.9 - zp_sl_in;
	      par[2] = 0.9;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      // box for +x half
	      xp = x_pls_lab + 2.6 - xp_sl_in;
	      zp = z_max - 1 - zp_sl_in;
	      par[0] = 1.2;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      
	      // parallelepiped for -x half
	      xp = x_mns_lab - 0.1 - xp_sl_in;
	      yp = y_lab_mns + y_add_c - yp_sl_in;
              zp = z_min + 0.4 - zp_sl_in;
              par[0] = 0.05;
	      par[1] = y_width_rot_mns*cosang + y_add_p;
	      par[2] = 0.7;
	      par[4] = 300;
	      if (i == 1) // caps near sectors 5-6 are not ordinary
		{
		  zp = 0.5*(z_max + z_min) - zp_sl_in;
		  par[0] = 0.05;
		  par[2] = 1.6;
		  par[4] = 60;
		  xp = x_mns_lab + 0.3 - xp_sl_in;
		}
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      if (i != 1)
		{
		  // parallelepiped for -x half
		  xp = x_mns_lab - 5.2 - xp_sl_in;
		  zp = z_min + 1.9 - zp_sl_in;
		  par[2] = 0.9;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);

		  // box for -x half
		  xp = x_mns_lab - 2.4 - xp_sl_in;
		  zp = z_min + 1.1 - zp_sl_in;
		  par[0] = 1.2;
		  par[2] = 0.025;
		  par[4] = 0;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);
		}
	    }
	  	  
	  // boxes, +x 
	  if (j <= 5)
	    {
	      xp = x_pls_lab + shift_x - xp_sl_in;
	      if (j <= 3) xp = x_pls_lab + shift_x + 1.9 - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max + shift_z - zp_sl_in;
	      par[0] = 4.;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	      
	      if (i == 1) // -x, boxes for 5-6 sectors
		{
		  xp = x_mns_lab - 0.5 - xp_sl_in;
		  if (j <= 3) xp = x_mns_lab - 1 - xp_sl_in;
		  yp = y_lab_mns + y_add_c - yp_sl_in;
		  par[1] = y_width_rot_mns*cosang + y_add_p;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);
		}
	    }
	  
	  // boxes, -x
	  if (j > 1)
	    if (i == 0 || i == 2)
	      {
		xp = x_mns_lab - shift_x - 0.2 - xp_sl_in;
		if (j <= 3) xp = xp - 1.5;
		yp = y_lab_mns + y_add_c - yp_sl_in;
		zp = z_min - shift_z - zp_sl_in;
		par[0] = 4.;
		par[1] = y_width_rot_mns*cosang + y_add_p;
		par[2] = 0.025;
		par[4] = 0;
		nnwal = ArteTable<NWAL>::size();
		gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		crgwal_(vol,&nnwal);
	      }
          // x coordinates for the caps from the outside proton beam pipe 
          //(top and bottom) in rotated system
	  
          if (i == 1) 
            {
              x56_prot = xmax_mns_rot[i];
              x34_prot = xmin_pls_rot[i];
            }
	}

      //2 caps over proton beam pipe for each stereo layer separatly
      x1_prot_lab = x56_prot*cosang + y1_prot*sinang;
      y1_prot_lab = y1_prot*cosang - x56_prot*sinang;

      x2_prot_lab = x34_prot*cosang + y1_prot*sinang;
      y2_prot_lab = y1_prot*cosang - x34_prot*sinang;

      xp = 0.5*(x2_prot_lab + x1_prot_lab + 1) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) - 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
      
      x1_prot_lab = x56_prot*cosang + y2_prot*sinang;
      y1_prot_lab = y2_prot*cosang - x56_prot*sinang;

      x2_prot_lab = x34_prot*cosang + y2_prot*sinang;
      y2_prot_lab = y2_prot*cosang - x34_prot*sinang;
      
      xp = 0.5*(x2_prot_lab + x1_prot_lab + 1) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) + 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
    }   
  
}


void capspc23_(char* csl,float* xp_sl,float* yp_sl,float* zp_sl,int* istereo)
{
//  This function assignes caps for PC02 and PC03
  strA2 & ref = a2_;

  int nnwal;
  float xp_sl_in=*xp_sl, yp_sl_in=*yp_sl, zp_sl_in=*zp_sl;
  int istereo_in=*istereo;
  int npar;
  char smod[5];

  // _pls_ means +x half, _mns_ means -x half
  float xmin_pls_rot[3], xmax_mns_rot[3];
  float x_pls_lab;
  float x_mns_lab;

  float z_max, z_min, y_middle_rot_pls, y_width_rot_pls, y_middle_rot_mns, y_width_rot_mns;
  float ymin_pls_rot[3], ymax_pls_rot[3],ymin_mns_rot[3], ymax_mns_rot[3];
  float y_lab_pls, y_lab_mns;
  float cosang = 0, sinang = 0;
  float x_min3,x_min4,x_max5,x_max6;

  float xp, yp, zp;
  float par[6], y1_prot, y2_prot, x56_prot = 0, x34_prot = 0;
  float y1_prot_lab, y2_prot_lab, x1_prot_lab, x2_prot_lab;
  int rotm;
  float shift_x, shift_z;
  float y_add_c = 0, y_add_p = 0; // +/-7cm. for caps on the top and bottom edges
  
  float theta1,phi1,theta2,phi2,theta3,phi3;

  strncpy(smod,csl,4);
  smod[4] = '\0';
  
  char vol[5];
  strncpy(vol,"WTCO",4);
  vol[4] = '\0';
  
  char manon[5];
  strncpy(manon,"ONLY",4);
  manon[4] = '\0';
  
  npar = 6;
  shift_z = 0.3;

  theta3 = 0;
  phi3 = 0;
  
  // loop on stereo layers
  for (int j = 1; j <= istereo_in; j++)
    {
      for (int i=0; i<3; i++)
	{
	  xmin_pls_rot[i]=9000;
	  xmax_mns_rot[i]=-9000;
	  
	  ymin_pls_rot[i]=9000;
	  ymax_pls_rot[i]=-9000;

	  ymin_mns_rot[i]=9000;
	  ymax_mns_rot[i]=-9000;
	}
      z_max = 0;
      z_min = 100000;
      x_min3=9000, x_min4=9000, x_max5=-9000, x_max6=-9000;
      y1_prot = 9000;
      y2_prot = -9000;

      // rotation matrixes for PC02 and PC03
      if (j%2)
	{
	  theta1 = 90;
	  phi1 = 0;
	  theta2 = 90;
	  phi2 = 90;
	}
      else if (j == 2 || j == 6)
	{
	  theta1 = 90;
	  phi1 = ref.ang*180/m_pi;
	  theta2 = 90;
	  phi2 = 90 + ref.ang*180/m_pi;
	}
      else
	{
	  theta1 = 90;
	  phi1 = -ref.ang*180/m_pi;
	  theta2 = 90;
	  phi2 = 90 - ref.ang*180/m_pi;
	}

      crrotm_(&rotm,&theta1,&phi1,&theta2,&phi2,&theta3,&phi3);

      // search for the nearest modules to caps so as to calculate positions of 
      // minimal x in +half, maximal x in -half and maximal z in each stereo layer

      for (ArteTableIterator<NEDE> NedeIt; NedeIt.valid(); NedeIt.next())
	{
	  ArtePointer<NEDE> nede(NedeIt);
	  if (!strncmp(smod, nede->nesl->name, 4))
	    if (nede->lay == j)
	      {
		cosang = nede->rot[0];
		sinang = nede->rot[1];
		
                // array elements contain: [0] - for sector 1 (+/-half),
                // [1] - for sectors 5,6/3,4 (+/-half), [2] - for sector 2 (+/-half)

		if (nede->zmax > z_max) z_max = nede->zmax;
		if (nede->zmin < z_min) z_min = nede->zmin;
		
		if (!strncmp(nede->hwc, "-", 1))
		  {
		    if (nede->sec == 1 && nede->xmax > xmax_mns_rot[0]) 
		      {
			xmax_mns_rot[0]=nede->xmax;
			ymin_mns_rot[0]=nede->ymin;
			ymax_mns_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 5 && nede->xmax > x_max5) 
		      {
			x_max5 = nede->xmax;
			ymax_mns_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 6 && nede->xmax > x_max6)
		      {
			x_max6 = nede->xmax;
			ymin_mns_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmax > xmax_mns_rot[2]) 
		      {
			xmax_mns_rot[2]=nede->xmax;
			ymin_mns_rot[2]=nede->ymin;
			ymax_mns_rot[2]=nede->ymax;
		      }

		    if (nede->sec == 1 && nede->xmax >= -20 
			&& nede->xmax <= 20 && nede->ymin < y1_prot)
		      y1_prot = nede->ymin;
		    if (nede->sec == 2 && nede->xmax >= -20 
			&& nede->xmax <= 20 && nede->ymax > y2_prot)
		      y2_prot = nede->ymax;
		  }
		if (!strncmp(nede->hwc, "+", 1))
		  {
		    if (nede->sec == 1 && nede->xmin < xmin_pls_rot[0]) 
		      {
			xmin_pls_rot[0]=nede->xmin;
			ymin_pls_rot[0]=nede->ymin;
			ymax_pls_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 3 && nede->xmin < x_min3) 
		      {
			x_min3 = nede->xmin;
			ymax_pls_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 4 && nede->xmin < x_min4)
		      {
			x_min4 = nede->xmin;
			ymin_pls_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmin < xmin_pls_rot[2]) 
		      {
			xmin_pls_rot[2]=nede->xmin;
			ymin_pls_rot[2]=nede->ymin;
			ymax_pls_rot[2]=nede->ymax;
		      }
		  }    
	      }
	}	
      
      if (x_max5 > x_max6) 
	xmax_mns_rot[1] = x_max5;
      else
	xmax_mns_rot[1] = x_max6;
      
      if (x_min3 < x_min4) 
	xmin_pls_rot[1] = x_min3;
      else
	xmin_pls_rot[1] = x_min4;

      // set caps according to specified sector's number in array
      for (int i=0; i<3; i++)
	{
	  y_middle_rot_pls = 0.5*(ymin_pls_rot[i] + ymax_pls_rot[i]);
	  y_width_rot_pls = 0.5*(ymax_pls_rot[i] - ymin_pls_rot[i]);

	  y_middle_rot_mns = 0.5*(ymin_mns_rot[i] + ymax_mns_rot[i]);
	  y_width_rot_mns = 0.5*(ymax_mns_rot[i] - ymin_mns_rot[i]);
	  
	  // rot -> lab
	  x_pls_lab = xmin_pls_rot[i]*cosang + y_middle_rot_pls*sinang;
	  y_lab_pls = y_middle_rot_pls*cosang - xmin_pls_rot[i]*sinang;
	  x_mns_lab = xmax_mns_rot[i]*cosang + y_middle_rot_mns*sinang;
	  y_lab_mns = y_middle_rot_mns*cosang - xmax_mns_rot[i]*sinang;
	  
          // addition 7cm. to length of the cap
	  if (i == 0)
	    {
	      y_add_c = 3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 2)
	    {
	      y_add_c = -3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 1)
	    {
	      y_add_c = 0;
	      y_add_p = 0;
	    }

	  shift_x = (z_max - z_min)/2*tan(1.05) - 0.8;

	      xp = x_pls_lab + 0.2 - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max - 0.4 - zp_sl_in;
	      
	      par[0] = 0.05;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.7;
	      par[3] = 0;
	      par[4] = 300;
	      par[5] = 0;
	      
	      // parallelepiped for +x half
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      // parallelepiped for +x half
	      xp = x_pls_lab + 5.2 - xp_sl_in;
	      zp = z_max - 1.9 - zp_sl_in;
	      par[2] = 0.9;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      // box for +x half
	      xp = x_pls_lab + 2.6 - xp_sl_in;
	      zp = z_max - 1 - zp_sl_in;
	      par[0] = 1.2;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      
	      // parallelepiped for -x half
	      xp = x_mns_lab - 0.1 - xp_sl_in;
	      yp = y_lab_mns + y_add_c - yp_sl_in;
              zp = z_min + 0.4 - zp_sl_in;
              par[0] = 0.05;
	      par[1] = y_width_rot_mns*cosang + y_add_p;
	      par[2] = 0.7;
	      par[4] = 300;
	      if (i == 1) // caps near sectors 5-6 have different slope
		{
		  zp = 0.5*(z_max + z_min) - zp_sl_in;
		  par[0] = 0.05;
		  par[2] = 1.6;
		  par[4] = 60;
		  xp = x_mns_lab + 0.3 - xp_sl_in;
		}
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);

	      if (i != 1)
		{
		  // parallelepiped for -x half
		  xp = x_mns_lab - 5.2 - xp_sl_in;
		  zp = z_min + 1.9 - zp_sl_in;
		  par[2] = 0.9;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);

		  // box for -x half
		  xp = x_mns_lab - 2.4 - xp_sl_in;
		  zp = z_min + 1.1 - zp_sl_in;
		  par[0] = 1.2;
		  par[2] = 0.025;
		  par[4] = 0;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);
		}
	  	  
	  // boxes, +x 
	  if (j <= 5)
	    {
	      //	      xp = x_pls_lab + shift_x - xp_sl_in;
	      xp = x_pls_lab + shift_x + 1.9 - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max + shift_z - zp_sl_in;
	      par[0] = 4.;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	      
	      if (i == 1) // -x, boxes for 5-6 sectors
		{
		  //		  xp = x_mns_lab - 0.5 - xp_sl_in;
		  xp = x_mns_lab - 1.2 - xp_sl_in;
		  yp = y_lab_mns + y_add_c - yp_sl_in;
		  par[1] = y_width_rot_mns*cosang + y_add_p;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);
		}
	    }
	  
	  // boxes, -x
	  if (j > 1)
	    if (i == 0 || i == 2)
	      {
		//		xp = x_mns_lab - shift_x - 0.2 - xp_sl_in;
	        xp = x_mns_lab - shift_x - 1.7 - xp_sl_in;
		yp = y_lab_mns + y_add_c - yp_sl_in;
		zp = z_min - shift_z - zp_sl_in;
		par[0] = 4.;
		par[1] = y_width_rot_mns*cosang + y_add_p;
		par[2] = 0.025;
		par[4] = 0;
		nnwal = ArteTable<NWAL>::size();
		gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		crgwal_(vol,&nnwal);
	      }
	  
	  // x coordinates for the caps from the outside proton beam pipe 
	  //(top and bottom) in rotated system

	  if (i == 1) 
	    {
	      x56_prot = xmax_mns_rot[i];
	      x34_prot = xmin_pls_rot[i];
	    }
	}

      //2 caps over proton beam pipe for each stereo separatly
      x1_prot_lab = x56_prot*cosang + y1_prot*sinang;
      y1_prot_lab = y1_prot*cosang - x56_prot*sinang;

      x2_prot_lab = x34_prot*cosang + y1_prot*sinang;
      y2_prot_lab = y1_prot*cosang - x34_prot*sinang;

      xp = 0.5*(x2_prot_lab + x1_prot_lab + 1) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) - 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
      
      x1_prot_lab = x56_prot*cosang + y2_prot*sinang;
      y1_prot_lab = y2_prot*cosang - x56_prot*sinang;

      x2_prot_lab = x34_prot*cosang + y2_prot*sinang;
      y2_prot_lab = y2_prot*cosang - x34_prot*sinang;
      
      xp = 0.5*(x2_prot_lab + x1_prot_lab + 1) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) + 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
    }   
  
}


void capstc_(char* csl,float* xp_sl,float* yp_sl,float* zp_sl,int* istereo)
{
// Function specifies caps for TCs

  strA2 & ref = a2_;

  int nnwal;
  float xp_sl_in=*xp_sl, yp_sl_in=*yp_sl, zp_sl_in=*zp_sl;
  int istereo_in=*istereo;
  int npar;
  char smod[5];

  // _pls_ means +x half, _mns_ means -x half
  float xmin_pls_rot[3], xmax_mns_rot[3];
  float x_pls_lab;
  float x_mns_lab;

  // _rot_ means rotated coordinates system, _lab_ - general
  float z_max, z_min, y_middle_rot_pls, y_width_rot_pls, y_middle_rot_mns, y_width_rot_mns;
  float ymin_pls_rot[3], ymax_pls_rot[3],ymin_mns_rot[3], ymax_mns_rot[3];
  float y_lab_pls, y_lab_mns;
  float cosang = 0, sinang = 0;
  float x_min3,x_min4,x_max5,x_max6;

  float xp, yp, zp;
  float par[6], y1_prot, y2_prot, x56_prot = 0, x34_prot = 0;
  float y1_prot_lab, y2_prot_lab, x1_prot_lab, x2_prot_lab;
  int rotm;
  float shift_x, shift_z;
  float y_add_c = 0, y_add_p = 0; // +/-7cm. for caps on the top and bottom

  float theta1,phi1,theta2,phi2,theta3,phi3;
  
  strncpy(smod,csl,4);
  smod[4] = '\0';
  
  char vol[5];
  strncpy(vol,"WTCO",4);
  vol[4] = '\0';
  
  char manon[5];
  strncpy(manon,"ONLY",4);
  manon[4] = '\0';
  
  npar = 6;
  shift_z = 0.3;

  theta3 = 0;
  phi3 = 0;
  
  // loop by stereo layers
  for (int j = 1; j <= istereo_in; j++)
    {
      for (int i=0; i<3; i++)
	{
	  xmin_pls_rot[i]=9000;
	  xmax_mns_rot[i]=-9000;
	  
	  ymin_pls_rot[i]=9000;
	  ymax_pls_rot[i]=-9000;

	  ymin_mns_rot[i]=9000;
	  ymax_mns_rot[i]=-9000;
	}
      z_max = 0;
      z_min = 100000;
      x_min3=9000, x_min4=9000, x_max5=-9000, x_max6=-9000;
      y1_prot = 9000;
      y2_prot = -9000;

      // rotation matrixes for TCs
      if (j == 1)
	{
          theta1 = 90;
          phi1 = ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 + ref.ang*180/m_pi;
	}
      else if (j == 2)
	{
          theta1 = 90;
          phi1 = 0;
          theta2 = 90;
          phi2 = 90;
	}
      else
	{
          theta1 = 90;
          phi1 = -ref.ang*180/m_pi;
          theta2 = 90;
          phi2 = 90 - ref.ang*180/m_pi;
	}
      
      crrotm_(&rotm,&theta1,&phi1,&theta2,&phi2,&theta3,&phi3);

      // search for the nearest modules to caps so as to calculate positions of 
      // minimal x in +half, maximal x in -half and max/min z in each stereo layer
      // in rotated coordinates system

      for (ArteTableIterator<NEDE> NedeIt; NedeIt.valid(); NedeIt.next())
	{
	  ArtePointer<NEDE> nede(NedeIt);
	  if (!strncmp(smod, nede->nesl->name, 4))
	    if (nede->lay == j)
	      {
		cosang = nede->rot[0];
		sinang = nede->rot[1];
		
                // array elements contain: [0] - for sector 1 (+/-half),
                // [1] - for sectors 5,6/3,4 (+/-half), [2] - for sector 2 (+/-half)

		if (nede->zmax > z_max) z_max = nede->zmax;
		if (nede->zmin < z_min) z_min = nede->zmin;
		
		if (!strncmp(nede->hwc, "-", 1))
		  {
		    if (nede->sec == 1 && nede->xmax > xmax_mns_rot[0]) 
		      {
			xmax_mns_rot[0]=nede->xmax;
			ymin_mns_rot[0]=nede->ymin;
			ymax_mns_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 5 && nede->xmax > x_max5) 
		      {
			x_max5 = nede->xmax;
			ymax_mns_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 6 && nede->xmax > x_max6)
		      {
			x_max6 = nede->xmax;
			ymin_mns_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmax > xmax_mns_rot[2]) 
		      {
			xmax_mns_rot[2]=nede->xmax;
			ymin_mns_rot[2]=nede->ymin;
			ymax_mns_rot[2]=nede->ymax;
		      }

                    if (nede->sec == 1 && nede->xmax >= -20 
                        && nede->xmax <= 20 && nede->ymin < y1_prot)
                      y1_prot = nede->ymin;
                    if (nede->sec == 2 && nede->xmax >= -20 
                        && nede->xmax <= 20 && nede->ymax > y2_prot)
                      y2_prot = nede->ymax;
		  }
		if (!strncmp(nede->hwc, "+", 1))
		  {
		    if (nede->sec == 1 && nede->xmin < xmin_pls_rot[0]) 
		      {
			xmin_pls_rot[0]=nede->xmin;
			ymin_pls_rot[0]=nede->ymin;
			ymax_pls_rot[0]=nede->ymax;
		      }
		    if (nede->sec == 3 && nede->xmin < x_min3) 
		      {
			x_min3 = nede->xmin;
			ymax_pls_rot[1]=nede->ymax;
		      }
		    if (nede->sec == 4 && nede->xmin < x_min4)
		      {
			x_min4 = nede->xmin;
			ymin_pls_rot[1]=nede->ymin;
		      }
		    if (nede->sec == 2 && nede->xmin < xmin_pls_rot[2]) 
		      {
			xmin_pls_rot[2]=nede->xmin;
			ymin_pls_rot[2]=nede->ymin;
			ymax_pls_rot[2]=nede->ymax;
		      }
		  }    
	      }
	}	
      
      if (x_max5 > x_max6) 
	xmax_mns_rot[1] = x_max5;
      else
	xmax_mns_rot[1] = x_max6;
      
      if (x_min3 < x_min4) 
	xmin_pls_rot[1] = x_min3;
      else
	xmin_pls_rot[1] = x_min4;

      // set caps according to specified sector's number in array
      for (int i=0; i<3; i++)
	{
	  y_middle_rot_pls = 0.5*(ymin_pls_rot[i] + ymax_pls_rot[i]);
	  y_width_rot_pls = 0.5*(ymax_pls_rot[i] - ymin_pls_rot[i]);

	  y_middle_rot_mns = 0.5*(ymin_mns_rot[i] + ymax_mns_rot[i]);
	  y_width_rot_mns = 0.5*(ymax_mns_rot[i] - ymin_mns_rot[i]);
	  
	  // rot -> lab
	  x_pls_lab = xmin_pls_rot[i]*cosang + y_middle_rot_pls*sinang;
	  y_lab_pls = y_middle_rot_pls*cosang - xmin_pls_rot[i]*sinang;
	  x_mns_lab = xmax_mns_rot[i]*cosang + y_middle_rot_mns*sinang;
	  y_lab_mns = y_middle_rot_mns*cosang - xmax_mns_rot[i]*sinang;
	  
          // addition 7cm. to length of the cap
	  if (i == 0)
	    {
	      y_add_c = 3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 2)
	    {
	      y_add_c = -3.5;
	      y_add_p = 3.5;
	    }
	  else if (i == 1)
	    {
	      y_add_c = 0;
	      y_add_p = 0;
	    }

	  // shift from the end point of module by x
	  shift_x = (z_max - z_min)/2*tan(1.05) - 0.8;
	  xp = x_pls_lab + shift_x - xp_sl_in;
	  yp = y_lab_pls + y_add_c - yp_sl_in;
	  zp = 0.5*(z_max + z_min) - zp_sl_in;
	  
	  par[0] = 0.05;
	  par[1] = y_width_rot_pls*cosang + y_add_p;
	  par[2] = 2.4;
	  par[3] = 0;
	  par[4] = 300;
	  par[5] = 0;
	  
	  // parallelepiped for +x half
	  nnwal = ArteTable<NWAL>::size();
	  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	  crgwal_(vol,&nnwal);
	  
	  // parallelepiped for -x half
	  xp = x_mns_lab - shift_x - xp_sl_in;
	  yp = y_lab_mns + y_add_c - yp_sl_in;
	  par[1] = y_width_rot_mns*cosang + y_add_p;
	  if (i == 1) // caps near sectors 5-6 have another slope
	    {
	      par[4]=60;
	      xp = x_mns_lab - 0.5 - xp_sl_in;
	    }
	  nnwal = ArteTable<NWAL>::size();
	  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	  crgwal_(vol,&nnwal);

	  	  
	  // boxes, +x 
	  if (j == 1 || j == 2)
	    {
	      xp = x_pls_lab + shift_x - xp_sl_in;
	      yp = y_lab_pls + y_add_c - yp_sl_in;
	      zp = z_max + shift_z - zp_sl_in;
	      par[0] = 4.;
	      par[1] = y_width_rot_pls*cosang + y_add_p;
	      par[2] = 0.025;
	      par[4] = 0;
	      nnwal = ArteTable<NWAL>::size();
	      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
	      crgwal_(vol,&nnwal);
	      
	      if (i == 1) // -x, boxes for 5-6 sectors
		{
		  xp = x_mns_lab - 0.5 - xp_sl_in;
		  yp = y_lab_mns + y_add_c - yp_sl_in;
		  par[1] = y_width_rot_mns*cosang + y_add_p;
		  nnwal = ArteTable<NWAL>::size();
		  gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		  crgwal_(vol,&nnwal);
		}
	    }
	  
	  // boxes, -x
	  if (j == 2 || j == 3)
	    if (i == 0 || i == 2)
	      {
		xp = x_mns_lab - shift_x - xp_sl_in;
		yp = y_lab_mns + y_add_c - yp_sl_in;
		zp = z_min - shift_z - zp_sl_in;
		par[0] = 4.;
		par[1] = y_width_rot_mns*cosang + y_add_p;
		par[2] = 0.025;
		par[4] = 0;
		nnwal = ArteTable<NWAL>::size();
		gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
		crgwal_(vol,&nnwal);
	      }
	  
          // x coordinates for the caps from the outside proton beam pipe 
          //(top and bottom) in rotated system

          if (i == 1) 
            {
              x56_prot = xmax_mns_rot[i];
              x34_prot = xmin_pls_rot[i];
            }
	}

      //2 caps over proton beam pipe for each stereo layer
      x1_prot_lab = x56_prot*cosang + y1_prot*sinang;
      y1_prot_lab = y1_prot*cosang - x56_prot*sinang;
      
      x2_prot_lab = x34_prot*cosang + y1_prot*sinang;
      y2_prot_lab = y1_prot*cosang - x34_prot*sinang;
      
      xp = 0.5*(x2_prot_lab + x1_prot_lab + 1) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) - 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
      
      x1_prot_lab = x56_prot*cosang + y2_prot*sinang;
      y1_prot_lab = y2_prot*cosang - x56_prot*sinang;

      x2_prot_lab = x34_prot*cosang + y2_prot*sinang;
      y2_prot_lab = y2_prot*cosang - x34_prot*sinang;
      
      xp = 0.5*(x2_prot_lab + x1_prot_lab + 3) - xp_sl_in;
      yp = 0.5*(y2_prot_lab + y1_prot_lab) + 0.1 - yp_sl_in;
      zp = 0.5*(z_max + z_min) - zp_sl_in;
      par[0] = 0.5*(x2_prot_lab - x1_prot_lab) - 0.5;
      par[1] = 0.025;
      par[2] = 0.5*(z_max - z_min);
      par[4] = 0;
      nnwal = ArteTable<NWAL>::size();
      gsposp_(vol,&nnwal,smod,&xp,&yp,&zp,&rotm,manon,par,&npar);
      crgwal_(vol,&nnwal);
    }   
  
}







}
