      subroutine otropen(lun,cname,subdir)
************************************************************************
* Written by A.Lanyov (from S.Nowak geopen.F (from H.Albrechts ddopen.F))
* Updated by A.Lanyov Sep 2001 (corrected work if file is not found)
*
* Description: open file for OTR geometry input
*              Several directories are tried to open the file:
*
*              1a. .                  (current directory)
*              1b. ./general
*              2a. $otrgeom       (environment variable "otrgeom" if set)
*              2b. $otrgeom/general
*              3a. $HBROOT/GEO/subdir   (subdir is argument supplied)
*              3b. $HBROOT/GEO/subdir/general
*
* Here general can be a symbolic link
* to a directory with final/standard geometry.
* In this case only differences should be stored in other directories,
* so that size of the repository is reduced.
*
* Called from: OTR routines
*
* Argument in/out description
*   LUN    in     lun
*   CNAME  in/out file name with extension (on error CNAME is blanked)
*   SUBDIR in     subdirectory in system directory for looking the files
*
************************************************************************
      implicit none

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "auto/MEIO.fi"
#include "gean/otrgeo.fi"

        integer lun,ios,lenocc,l,lcur
        character*100 fulnam,dir,dir1
        character*(*) cname,subdir
        logical exist

#include "arte/Arte.ff"
#include "auto/MEIO.ff"

      LCUR = IMEIO_CURRENT(LUN)

C Already open ? if so - close the old one 
      IF (IMEIO_FLAG(LCUR) .EQ. 3 .OR.
     +    IMEIO_FLAG(LCUR) .EQ. 2)  CALL IOCLOS (LUN)

c 1a. Check current directory
        fulnam = cname
        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c 1b. Check current directory + subdirectory "general"
        fulnam = 'general/' // cname
        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c 2a. Check environment variable $otrgeom
        call getenvf('otrgeom',dir)
        l = lenocc(dir)
        dir(l+1:l+1) = '/'
        fulnam = dir(:lenocc(dir)) // cname

        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c 2b. Check environment variable $otrgeom + subdirectory "general"
        fulnam = dir(:lenocc(dir)) // 'general/' // cname

        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c 3a. Check corresponding system directory $HBROOT/GEO/subdir
c     where subdir is a parameter
        call getenvf('HBROOT',dir)
        l = lenocc(dir)
        dir(l+1:) = '/GEO/' // subdir(:lenocc(subdir)) // '/'
        fulnam = dir(:lenocc(dir)) // cname

        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c 3b. Check corresponding system directory $HBROOT/GEO/subdir
c                                          + subdirectory "general"
        fulnam = dir(:lenocc(dir)) // 'general/' // cname

        inquire(file=fulnam(:lenocc(fulnam)),exist=exist,iostat=ios)
        if (exist) goto 1

c Not found
        buf = 'File ' // cname // ' not found'
        call mewarn(buf)
        cname(:1) = ' '
        return

1       call ioconc (lun, fulnam(:lenocc(fulnam)), ' ', 0)
        call ioopen (lun, 'fi', 0, ios)

c        write (*,*) 'OTROPEN: File opened at ',fulnam(:lenocc(fulnam))

        return
        end
