#ifndef _GROVER_HH_
#define _GROVER_HH_    

/////////////////////////////////////////////////////////////////////////////
//
//
//         GROVER  - Generic Reconstruction Of VERtices
//	   ============================================
//
//
//                          -VERSION-                                   
static const char
GroverVersion[] =            "2.02";
//               
//
//	   Contributing authors: D.Emeliyanov
//                               V.Kekelidze
//                               I.Kisel
//                               M.Sang
//                               S.Schaller
//
//	   Adapted from ROVER by T.Perschke/J.van Tilburg
//
//         A flexible set of classes and utility functions for 
//         vertexing at HERA-B
//
//	   See documentation at MPI Munich for more information:
//
//   	   http://wwwhera-b.mppmu.mpg.de/analysis/grover.html
//
//         Current responsible: ==> Michael Dressel (dressel@mppmu.mpg.de)
//                                  --------------------------------------
//
//         Other people who may be able to help with queries:
//         Max Sang (max.sang@cern.ch)
//	   Ivan Kisel (ikisel@mppmu.mpg.de), 
//         Dmitry Emeliyanov (emelian@mail.desy.de)
//
// Modification Log:
//
//(YY-MM-DD) Person Modification(s) 
//
// 00-05-14  Max    First Version 0.01 - Attempt to get a common interface
//                  to VT and Rover. Some minor hacking of VT and major
//	            hacking of rover_primaries. No second_rove or 
// 		    graphics yet. Classes are reasonably complete but still
//      	    lots of work left to be done.
// 00-05-16  Max    Graphics added, removed RPGeoFit from GroverVertex and 
//                  put it back into GroverPrimaries. General feeling that I
//                  don`t want our classes to get fat. Could even strip
//		    out the flags and chi-squareds. These too are in
//		    danger of multiplying as more routines are written.
//		    If people need to store stuff they should use their own
//		    structures, not the Grover classes. second_rove added.
// 00-05-24  Max    Lots of changes and bug fixes. Graphics separated into
//                  its own module. Graphics added to GroverVTPrimaries().
//		    The 'particles' list of each returned vertex contains 
//		    particles with updated parameters - follow pointers for 
//		    original track parameters.
//		    Internal alignment in GroverPrimaries() works now :-)
// 00-05-26  Max    Added GroverPrimEval() - basically does the same sort of 
//                  thing as Verona by J.van Tilburg - calculates efficiencies
//		    etc for a pair of vertex lists (one real, one MC)
// 00-07-31  Max    v0.10 - Special version with all VT functionality 
//                  removed
// 00-09-07  Max    Bug removed (thanks Dmitry) in GroverVertex::SetSlope*()
// 00-09-15  Max    First version in Arte (1.00). VT functionality still 
//                  disabled!
// 00-09-26  Max    Some internal details in GroverVertex changed
// 00-09-27  Max    Added GroverParticle::GroverParticle(GroverVertex*)
// 00-10-01  Max    1) Added GroverAddBest(...) which adds best track in
//                     a list to the user's vertex.
//                  2) Changed default cuts to stop default calling of 
//                     secondary finders. GroverEvent(0) should now 
//                     have same effect as Rover primary finder.
// 00-10-11  Max    Added some stuff for future 3D display (Root/X3D)
// 00-10-12  MS/DE  Added multi-constraint fit GroverVertex::Fit(void)
// 00-10-12  Max    3D event display working :-) But only draws wires and
//                  vertices, not tracks :-( because I am away next week.
// 00-10-12  Max    Version 1.01 into Arte  
// 00-10-26  Max    Added global objects ggXX.. for ARTE implementation
// 00-10-26  Max    Added KUIP interface for Arte implementation
// 00-10-26  Max    Fixed a  bug in GroverParticle constructors which was
//                  screwing up the terms in p (momentum) :-(
// 00-10-27  Max    3D graphics working properly now; new safety checks for
//                  2D (HIGZ) display
// 00-10-29  Max    GroverEvent() completely rewritten
// 00-10-30  Max    Version 1.20 in Arte 03-08-r2
// 00-11-06  Max    Some improvements to GroverVertex.C
// 00-11-08  Max    Some minor changes to cuts, a couple of bug fixes, minor
//                  changes to 3D graphics and kuip interface.
// 00-11-13  Max    Version 1.21 in Arte 03-08-r3
// 00-11-23  Max    Added 'mobility' flag to GroverWire
// 00-11-24  Max    Added Michael Dressel's track selection function (TSEL 4)
// 00-11-24  Max    GroverRefitRSEG improved
// 00-11-26  Max    GroverRefitRSEG now allows refitting of single RSEGs
// 00-11-27  Max    Version 1.22 in Arte 03-08-r4
// 00-12-01  Max    Chi2 probability is now flat for constrained fits
// 00-12-21  Max    Added some operators (like == and <<) to classes
//                  Also made classes const-safe.
// 00-12-21  Max    Version 1.24 in Arte 03-08-r5 (used for rp002)
// 01-01-18  Max    GeoFit(vtx*) re-absorbed as member of GroverVertex
// 01-01-18  Max    GroverParticle and GroverVertex are now friends.
// 01-01-18  Max    Some dangerous public functions have been made protected!
//                  Others have been removed completely!
//                  Users should not notice the difference - please mail me
//                  if you use a member function which is no longer public.
// 01-01-19  Max    Added track selection 6 (Conde/Schmelling) - made default
// 01-01-19  Max    All inline functions moved to grover/grover_inlines.hh
// 01-01-30  Max    vtx.Chi2DistTo(X) definition changed where X is a vertex,
//                  wire or space point. Used to be
//                  => (spatial distance/error)^2, 
//                  Now the distance in error space, e.g. 
//                  => sqrt((Dx/dx)^2 + (Dy/dy)^2 + (Dz/dz)^2).
// 01-02-01  Max    Chi2 -> Unnormalised from now on! [#dof = vtx.Ndof()]
// 01-02-21  Max    ResetParticles() is now ONLY called during Fit() IF the
//                  user has not manually altered the track parameters or
//                  covariance matrix.
// 01-02-21  Max    Improved const safety
// 01-02-29  Max    Version 2.01 in Arte-dev
// 01-03-13  Max    Added some preprocessor flags GROVER_WITHOUT_XXXX to 
//                  remove dependence on Arte, HIGZ, Root, CATS etc.
// 01-03-16  Max    Added constants SUCCESS and FAILURE for function 
//                  return values
// 01-03-21  Max    Added include files to make 3D graphics easier to
//                  use in user code (3d.hh and 3d_cdf.hh)
// 01-03-29  Max    Michael Dressel is the new Grover guy
// 01-04-09  MD     In GeoFit set gparticle values direct instead with the
//                  GroverParticle maember functions. To quere tampered bug. 
// 01-04-17  MD     bug fix in GroverVertex::operator==
// 01-04-24  MD     chanfing filling of ggWires and writing of RTAR
//                  added sx, sy, sz to GroverWire
// 01-04-26  MD     Bug fix for extrapolating RSEG in GroverCatsExtrap
// 01-04-27  MD     Bug fix if RTAR not booked
//                  Make GroverCatsExtrapolate usable for RTRAs too.
// 01-05-07  MD     wire following made posible on MC events, but are you 
//                  sure you want that?
// 01-05-09  MD     reject non active wires see GroverFillFromGTAR.C
// 01-05-10  MD     correct ms coeff in GroverCatsExtrapolate
//                  bug fix in FakeWires.C
// 01-05-28  MD     bug fix in GroverVertex
// 01-06-01  MD     Bug fix in Chi2DistTo(const GroverVertex* vert) const
//                    and GroverParticle::Chi2DistTo(const float xx, 
//                                           const float yy, const float zz)
// 01-06-15  MD     cloneremoved track selection using Rsegc::Bit20
//                  available with /GROVER/STEERING/TSELECTION   7
//                  see file GroverFillBit20.C
// 01-07-02  MD     cloneremoved track rejecting VDS-only tracks
//                  available with /GROVER/STEERING/TSELECTION   8
//                  see file GroverFillMomentum.C
// 01-07-04  MD     GroverFillSmartNoVDSonly added, like 
//                  GroverFillSmarter but reject VDS-only tracks.
//                  available with /GROVER/STEERING/TSELECTION   9
// 01-07-05  MD     Change the WARNING in case of  WIREFOLLOEING. 
//                  It may improve the target positions for the first 
//                  events in a run but shoul NOT BE USED ON SELECTED EVENTS
// 01-07-05  MD     added Jing-GE Shiu's pointing constrained vertex fit
//                  Fitp to GroverVertex for testing. 
//                  See: ~physjg/public/pgrover/pgrover.ps.gz
// 01-07-06  MD     ROVERMAXTRACKS in GroverVertex::GeoFit 
//                  is 200 as in GroverPrimaries
// 01-07-16  MD     Bug fix in GroverEvent, now the rrel->flags are set to zero
//                  if RVER is replaced.
// 01-07-31  MD     bit20 name changes:
//                  GroverFillBit20 -> GroverFillClonerm
//                  GroverFillMomentum -> GroverFillClonermNoVDSonly
//                  GroverEvent is also changed due to this
// 01-08-28  MD     imported utility functions wrtten by Sven Schaller:
//                  GroverPx, GroverPy, GroverPz,  GroverListToLocal,
//                  GroverListToGlobal. See GroverUtil.C.
// 01-09-08  DE     Add GROVER_WITHOUT_ARTE envelope for Fitp.
//
// 02-04-08  DE     The new primary vertex finder algorithm
//
// 02-05-21  DE     Improving timing of the new PVF. New routine NewGeoFit()
//
/////////////////////////////////////////////////////////////////////////////




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                      Essential #includes                           //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#ifndef GROVER_WITHOUT_ARTE
#ifndef ARTE_HH
#include "arte/arte.hh"
#define ARTE_HH
#endif
#endif

#ifndef _VECTOR_H_
#include <vector>
#define _VECTOR_H_
#endif




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                          Grover classes                            //
//                                                                    //
////////////////////////////////////////////////////////////////////////

class GroverParticle;
class GroverVertex;
class GroverWire;
class GroverCuts;



////////////////////////////////////////////////////////////////////////
//                                                                    //
//                         Grover containers                          //
//                                                                    //
////////////////////////////////////////////////////////////////////////

typedef vector<GroverParticle> GroverParticleList;
typedef vector<GroverVertex>   GroverVertexList;
typedef vector<GroverWire>     GroverWireList;



////////////////////////////////////////////////////////////////////////
//                                                                    //
//                    Constants                                       //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// General number used to signify bad return values or other errors
static const int SUCCESS = 0, FAILURE = -999;

// Default mass for GroverParticles (pi+)
static const float DEFAULTMASS = 0.1395702;



////////////////////////////////////////////////////////////////////////
//                                                                    //
//                   GroverParticle class definition                  //
//                                                                    //
////////////////////////////////////////////////////////////////////////

class GroverParticle 
{
  friend class GroverVertex;
public:
  //
  //           CONSTRUCTORS
  //
  GroverParticle (void);                   
#ifndef GROVER_WITHOUT_ARTE
  GroverParticle (const ArtePointer<RSEG>);
  GroverParticle (const ArtePointer<RTRA>);
  GroverParticle (const ArtePointer<MTRA>);
#endif
  GroverParticle (GroverVertex*); // non-const - calls vtx.Fit() and/or vtx.Momentum()
 ~GroverParticle (void);   
  // 
  //           MEMBER FUNCTIONS
  //            
  // Get/transform coordinate system
  bool         InLocal(void) const;
  bool         InGlobal(void) const;
  int          LocalToGlobal(void);
  int          GlobalToLocal(void);
  // Spatial and chi2 distances to a wire, vertex, particle, or space point
  float        DistanceTo(const GroverParticle*) const;
  float        Chi2DistTo(const GroverParticle*) const;
  float        DistanceTo(const GroverWire*) const;
  float        Chi2DistTo(const GroverWire*) const;
  float        DistanceTo(const GroverVertex*) const;
  float        Chi2DistTo(const GroverVertex*) const;
  float        DistanceTo(const float x, const float y, const float z) const;
  float        Chi2DistTo(const float x, const float y, const float z) const;
  // Propagate one end of the particle segment to a plane in z.
  // (propagation through the magnet is NOT implemented).
  int          PropagateBeginTo(const float z = 0.0);    
  int          PropagateEndTo(const float z = 2000.);
  // Scalar momentum and inverse momentum at start of track
  float        Momentum(void) const;
  float        InvMomentum(void) const;
  int          SetMomentum(const float p_new);
  int          SetInvMomentum(const float invp_new);
  int          SetOMomentum(const float p_new);
  int          SetOInvMomentum(const float invp_new);
  // Track parameters and cov matrix at start of track
  float        BeginX(void) const;
  float        BeginY(void) const;
  float        BeginZ(void) const;
  float        BeginSlopeX(void) const;
  float        BeginSlopeY(void) const;
  float        BeginCov(const int i) const;             // triangular
  float        BeginCov(const int i, const int j) const;// square
  // -"- at end of 'track'
  float        EndX(void) const;
  float        EndY(void) const;
  float        EndZ(void) const;
  float        EndSlopeX(void) const;
  float        EndSlopeY(void) const;
  float        EndCov(const int i) const;             // triangular
  float        EndCov(const int i, const int j) const;// square
  // Reset the track parameters to their original values (INCLUDING momentum)
  int          ResetParameters(void);
  // Get the original z positions of the start and end of track.
  float        OriginalBeginZ(void) const;
  float        OriginalEndZ(void) const;
  // Number of hits in VDS
  int          Nhit(void) const;
  int          SetNhit(const int newnhit);
  // Electric charge (-999 if unknown)
  int          Charge(void) const;
  int          SetCharge(const int newq);
  // Chi2 contribution to a fit
  float        Chi2(void) const;
  int          SetChi2(const float c);
  // Mass as a number or a TOPP pointer
  float        Mass(void) const;
  int          SetMass(const float mass_new);
#ifndef GROVER_WITHOUT_ARTE
  int          SetMass(const ArtePointer<TOPP>);
  int          SetTopp(const ArtePointer<TOPP>);
  ArtePointer<TOPP> Topp(void) const;
#endif
  // 'Type of particle' flag tells you what was used to create
  //  this GroverParticle.
  //  1 = RTRA 
  //  2 = RSEG 
  //  3 = RCCL 
  //  4 = MTRA 
  // (5 - 49 reserved for future use)
  // 50 = GroverParticle
  // 51 = GroverParticleList
  // 52 = GroverVertex
  int          Type(void) const;
  // Check Type() and then read the ParentXXXX() you need. All others = 0
#ifndef GROVER_WITHOUT_ARTE
  ArtePointer<RTRA>   ParentRTRA(void) const;
  ArtePointer<RSEG>   ParentRSEG(void) const;
  ArtePointer<RCCL>   ParentRCCL(void) const;
  ArtePointer<MTRA>   ParentMTRA(void) const;
#endif
  GroverParticle*     ParentGP(void)   const;
  GroverParticleList* ParentGPL(void)  const;
  GroverVertex*       ParentGV(void)   const;
  // Index - for user convenience
  int          Index(void) const;
  int          SetIndex(const int i);
  // Dist - for user convenience
  float        Dist(void) const;
  int          SetDist(const float d);
  // Associations - for user convenience
  int          UsedInPrimary(void) const;
  int          SetUsedInPrimary(const int m);
  int          UsedInSecondary(void) const;
  int          SetUsedInSecondary(const int m);
  // Weights - for user convenience
  float        PrimaryWeight(void) const;
  int          SetPrimaryWeight(const float wprim_new);
  float        SecondaryWeight(void) const;
  int          SetSecondaryWeight(const float wsec_new);
  // Dump private members to stdout
  void         Dump(void) const;
protected:
  int          SetGlobal(const bool newg);
  int          SetLocal(const bool newl);
  int          SetBeginX(const float xb_new);
  int          SetBeginY(const float yb_new);
  int          SetBeginZ(const float zb_new);
  int          SetEndX(const float xe_new);
  int          SetEndY(const float ye_new);
  int          SetEndZ(const float ze_new);
  int          SetBeginSlopeX(const float axb_new);
  int          SetBeginSlopeY(const float ayb_new);
  int          SetEndSlopeX(const float axe_new);
  int          SetEndSlopeY(const float aye_new);
  int          SetBeginCov(const float new_covb[]);
  int          SetBeginCov(const int index, const float covb_new);
  int          SetEndCov(const float new_cove[]);
  int          SetEndCov(const int index, const float cove_new);
  bool         Tampered(void) const;
private:
  static const int COVDIM = 21; // x,y,ax,ay,p
  float xb, yb, zb, axb, ayb, covb[COVDIM];
  float xe, ye, ze, axe, aye, cove[COVDIM];
  float oxb, oyb, ozb, oaxb, oayb, ocovb[COVDIM];
  float oxe, oye, oze, oaxe, oaye, ocove[COVDIM];
  float p, op, mass;
  int   nhit, q, type, usedinprimary, usedinsecondary, index;
#ifndef GROVER_WITHOUT_ARTE
  ArtePointer<RTRA>   parRTRA;
  ArtePointer<RSEG>   parRSEG;
  ArtePointer<RCCL>   parRCCL;
  ArtePointer<MTRA>   parMTRA;
#endif
  GroverParticle*     parGP;
  GroverParticleList* parGPL;
  GroverVertex*       parGV;
  bool  inlocal, inglobal;
  float wprim, wsec, dist, chi2, chi2_m;
#ifndef GROVER_WITHOUT_ARTE
  ArtePointer<TOPP> topp;
#endif
  bool tampered;
};




////////////////////////////////////////////////////////////////////////
//                                                                    //
//               GroverVertex class definition                        //
//                                                                    //
////////////////////////////////////////////////////////////////////////

class GroverVertex 
{
  friend class GroverParticle;
public:
  //
  //           CONSTRUCTORS
  //
  GroverVertex();
  GroverVertex(const GroverParticle*);                       // vertex seed
  GroverVertex(const GroverParticle*,const GroverParticle*); // 2 track vtx
  GroverVertex(const GroverParticle*,const GroverParticle*,  // 3 track vtx
	       const GroverParticle*);                          
  GroverVertex(const GroverParticle*,const GroverParticle*,  // 4 track vtx
	       const GroverParticle*,const GroverParticle*);
  GroverVertex(const GroverParticle*,const GroverParticle*,  // 5 track vtx
	       const GroverParticle*,const GroverParticle*,
	       const GroverParticle*);
  GroverVertex(const GroverParticle*,const GroverParticle*,  // 6 track vtx
	       const GroverParticle*,const GroverParticle*,
	       const GroverParticle*,const GroverParticle*);
  GroverVertex(const GroverParticleList*);                   // N track vtx
#ifndef GROVER_WITHOUT_ARTE
  GroverVertex(const ArtePointer<RVER>);
  GroverVertex(const ArtePointer<MVER>);
#endif
  // Construct from a point in LOCAL coordinates
  GroverVertex(const float x, const float y, const float z);
  // Construct from a VT vertex index (with or without corresponding GPL)
  GroverVertex(const int index, const GroverParticleList*);
  GroverVertex(const int index);
 ~GroverVertex();
  //
  //           PUBLIC TRACK LISTS 
  //
  GroverParticleList particles;
  GroverParticleList trashcan;
  // 
  //           MEMBER FUNCTIONS
  //
  // Get and transform coordinate system
  int          LocalToGlobal(void);
  int          GlobalToLocal(void);
  bool         InLocal(void) const;
  bool         InGlobal(void) const;
  // Number of particles in internal particle list
  int          Size(void) const;
  // Add one or N particles to internal particle list
  int          Add(const GroverParticle*);     
  int          Add(const GroverParticleList*);
  // Remove particle number i (starts at 0) or remove by pointer to track in
  // internal particle list. This clears *all* constraint information.
  int          Remove(const int i);
  int          Remove(GroverParticle* ptr);
  // Perform a secondary vertex fit to mass m (m <= 0.0: unconstrained fit)
  int          Fit(const float m);
  // Vertex fit with mass constrained to a particle in the TOPP table
#ifndef GROVER_WITHOUT_ARTE
  int          Fit(ArtePointer<TOPP> hypothesis);
#endif
  // Fit which is constrained or unconstrained according to current
  // constraint flags (this is most general)
  int          Fit(void);
  // Quick fit for 2-prong vertices
  int          FastFit(void);
  // Robust fit for the new primary vertex finder
  int          RobustFit(void);
  // New primary vertex fit
  int          NewGeoFit(void);
  // Primary vertex fit:
  int          GeoFit(void);
  // pointing constrained vertex fit
#ifndef GROVER_WITHOUT_ARTE
  int GroverVertex::Fitp(float x0, float y0, float z0, float cv0[], RVER &newver);
  int GroverVertex::Fitp(ArtePointer<RVER> rver, RVER &newver);
#endif
  // Chi-squared of the last fit (0. if undefined)
  float        Chi2(void) const; 
  // Number of degrees of freedom in the last fit (-1 if never fit)
  int          Ndof(void) const;
  // Probability of chi2 (returns -0.5 if bad)
  float        Prob(void) const;
  // Parameters of vertex after fitting, and their cov matrix.
  float        X(void) const;
  float        Y(void) const;
  float        Z(void) const;
  float        Momentum(void); // non-const - calls EstimateMomentum()
  float        SlopeX(void);   // non-const - calls Momentum()
  float        SlopeY(void);   // non-const - calls Momentum()
  float        Cov(const int i) const;
  float        Mass(void) const;
  float        MassError(void) const;
  // Add a mass constraint, with a list of two or more particles (max 10)
  // to which this new constraint applies.
  int          AddConstraint(const float mass, 
			     const int p1,    const int p2, 
			     const int p3=-1, const int p4=-1, 
			     const int p5=-1, const int p6=-1, 
			     const int p7=-1, const int p8=-1, 
			     const int p9=-1, const int p10=-1);
  int          AddConstraint(const float mass, 
			     const GroverParticle* p1,   
			     const GroverParticle* p2, 
			     const GroverParticle* p3=0, const GroverParticle* p4=0, 
			     const GroverParticle* p5=0, const GroverParticle* p6=0, 
			     const GroverParticle* p7=0, const GroverParticle* p8=0, 
			     const GroverParticle* p9=0, const GroverParticle* p10=0);
#ifndef GROVER_WITHOUT_ARTE
  int          AddConstraint(const ArtePointer<TOPP>, 
			     const int p1,    const int p2, 
			     const int p3=-1, const int p4=-1, 
			     const int p5=-1, const int p6=-1, 
			     const int p7=-1, const int p8=-1, 
			     const int p9=-1, const int p10=-1);
  int          AddConstraint(const ArtePointer<TOPP>, 
			     const GroverParticle* p1,   const GroverParticle* p2, 
			     const GroverParticle* p3=0, const GroverParticle* p4=0, 
			     const GroverParticle* p5=0, const GroverParticle* p6=0, 
			     const GroverParticle* p7=0, const GroverParticle* p8=0, 
			     const GroverParticle* p9=0, const GroverParticle* p10=0);
#endif
  // Get number of constraints
  int          Nconstraints(void) const;
  // Get mass or TOPP code of constraint i
  float        ConstraintMass(const int i) const;
#ifndef GROVER_WITHOUT_ARTE
  ArtePointer<TOPP> ConstraintTOPP(const int i) const;
  // Get TOPP code if only one constraint used (returns 0 otherwise)
  ArtePointer<TOPP> ConstraintTOPP(void) const;
#endif
  // Remove all mass constraints
  int          ClearConstraints(void);
  // Spatial and chi2 distances to a wire, vertex, particle, or space point
  float        DistanceTo(const GroverParticle*) const;
  float        Chi2DistTo(const GroverParticle*) const;
  float        DistanceTo(const GroverVertex*) const;
  float        Chi2DistTo(const GroverVertex*) const;
  float        DistanceTo(const GroverWire*) const;
  float        Chi2DistTo(const GroverWire*) const;
  float        DistanceTo(const float x, const float y, const float z) const;
  float        Chi2DistTo(const float x, const float y, const float z) const;
  // Index - for user convenience
  int          Index(void) const;
  int          SetIndex(const int i);
  // Get/set wire number with which the vertex is associated
  int          Wire(void) const;
  int          SetWire(const int wire);
  // Call trk.ResetParameters() for every track in the particles list
  int          ResetParticles(void);
  // Dump private members to stdout
  void         Dump(void) const;
  //
  //           EXPERT / DEPRECATED FUNCTIONS
  //
  int          EstimateCov(const float scale=1.0, const float zcov=0.01); 
  void         dump_cons(void) const;
  int          UpdateDistances(void);
protected:
  int          SetChi2(const float newc);
  int          SetX(const float x);
  int          SetY(const float y);
  int          SetZ(const float z);
  int          SetMomentum(const float newp);
  int          SetSlopeX(const float newax);
  int          SetSlopeY(const float neway);
  int          SetCov(const float value[]);
  int          SetCov(const int i, const float value);
  int          SetBx(const float bx);
  int          SetBy(const float by);
  int          SetDecayLength(const float len);
  int          SetMass(const float newm); // initial estimate for fits
  int          SetMassError(const float newdm);
  float        EstimateMomentum(void);
  float        Bx(void) const;
  float        By(void) const;
  float        DecayLength(void) const;
  float        GCov(const unsigned int n) const;
private:
  static const int COVDIM  = 36; // x,y,z,ax,ay,p,bx.by
  static const int CONSTDIM= 10; // maximum number of constraints
  static const int MAXTRKS = 15; // max. # tracks for *constrained* fits
  int   index;
  float x, y, z, ax, ay, p, m, dm;
  bool  inlocal, inglobal;
  float cov[COVDIM], wprim, wsec;
  int   wnr, refntrincluster;
  float chi2;
  bool  fitted, mom_estimated;
  int   n_constraints;
  float constr_mass[CONSTDIM];
  int   constr_map[CONSTDIM][MAXTRKS];
#ifndef GROVER_WITHOUT_ARTE
  ArtePointer<TOPP> constr_topps[CONSTDIM];
#endif
  float bx, by, decay;
  vector<float> gcov;
};




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                    GroverWire class definition                     //
//                                                                    //
////////////////////////////////////////////////////////////////////////

class GroverWire 
{
public:
  //
  //           CONSTRUCTORS
  //
  GroverWire();                 
#ifndef GROVER_WITHOUT_ARTE
  GroverWire(const ArtePointer<GTAR>);
  GroverWire(const ArtePointer<RTAR>);
#endif
 ~GroverWire();            
  //
  //           MEMBER FUNCTIONS
  //
  // Wire number
  int          Number(void) const;
  // Get / transform coordinate system
  bool         InLocal(void) const;
  bool         InGlobal(void) const; 
  int          GlobalToLocal(void);
  int          LocalToGlobal(void);
  // Position of wire centre
  float        X(void) const;
  float        Y(void) const;
  float        Z(void) const;
  // Dimensions of wire
  float        SizeX(void) const;
  float        SizeY(void) const;
  float        SizeZ(void) const;
  // Error on Mean position
  float        SX(void) const;
  float        SY(void) const;
  float        SZ(void) const;
  // Wire shape (cylinder, band etc)
  int          Shape(void) const;
  // True if horizontal/vertical wire, false otherwise
  bool         Horizontal(void) const;
  bool         Vertical(void) const;
  // Spatial and chi2 distances to a wire, vertex, particle, or space point
  float        DistanceTo(const GroverParticle*) const;
  float        Chi2DistTo(const GroverParticle*) const;
  float        DistanceTo(const GroverVertex*) const;
  float        Chi2DistTo(const GroverVertex*) const;
  float        DistanceTo(const GroverWire*) const;
  float        DistanceTo(const float x, const float y, const float z) const;
  // Dump private members to stdout
  void         Dump(void) const;
  //
  //           EXPERT/DEPRECATED FUNCTIONS
  //
  int          SetX(const float x);
  int          SetY(const float y);
  int          SetZ(const float z);
  int          SetSizeX(const float dx);
  int          SetSizeY(const float dy);
  int          SetSizeZ(const float dz);
  int          SetSX(const float dx);
  int          SetSY(const float dy);
  int          SetSZ(const float dz);
  int          Mobile(void) const;
  int          Stick(void);
  int          Unstick(void);
  int          SetMobility(const int mob);
  float        Rot(const int i) const;
  int          NParticles(void) const;
  int          IncrementNParticles(void);
  int          DecrementNParticles(void);
  int          SetNParticles(const int n);
  int          NPrimaries(void) const;
  int          IncrementNPrimaries(void);
  int          DecrementNPrimaries(void);
  int          SetNPrimaries(const int n);
  int          NSecondaries(void) const;
  int          IncrementNSecondaries(void);
  int          DecrementNSecondaries(void);
  int          SetNSecondaries(const int n);
private:
  int   wnr;
  float x, y, z, dx, dy, dz, sx, sy, sz;
  int   shape;
  bool  horizontal, vertical;
  float rot[9];
  bool  intarget, inlocal, inglobal;
  int   ntr, nprim, nsec;
  int   mobile;
};




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                    GroverCuts class definition                     //
//                                                                    //
////////////////////////////////////////////////////////////////////////

class GroverCuts 
{
public:
  //
  //           CONSTRUCTORS
  //
  GroverCuts();    // default constructor - sets defaults!
 ~GroverCuts();    // default destructor
  //
  //           MEMBER FUNCTIONS
  //
  // formatted dump of current values to stdout
  void         Dump(void) const;
  //
  //           CUT VALUES
  //
  //
  //           1) General cuts
  //
  // Vertexing package for primary finding - 1=Grover, 2=VT (default 1)
  int   VertexingPackage;
  // Event display with HIGZ (default 0 = off)
  int   GroverDraw;
  // Which type of secondaries to search for? (defaults 1,1,1)
  int   FindJpsiPlusX;
  int   FindUnbiased;
  int   FindCharm;
  // Print out messages all over the place
  int   Verbose;
  // Use the evaluation package (Efficiency calculations etc)
  int   GPEActivate;
  // use HBOOK/PAW and/or ROOT in GPE (default Paw=0, Root=1)
  int   Paw;
  int   Root;
  //
  //           2) Cuts for MC studies
  //
  // Mass constraint to use in fit
  float MassConstraint;
  // insert real m and/or p into tracks for vertex resolution 
  // studies (defaults 0,0)
  int   CheatMass;
  int   CheatMomentum;
  // smear momenta by these factors (set to zero for no smearing)
  // const = constant term (all momenta), var = p dependent term
  // (defaults 0.0,0.0; TDR values = 0.0045,0.00005)
  int   CheatSmearing;
  float CheatSmearConst;
  float CheatSmearVar;
  //
  //           3) Cuts for VT - described in T.Lohse's manual
  //              Used to be parameters, now they are cuts 
  //              - VT modified by M.Sang 11/05/00
  // 
  // PARTICLE MASSES - please substitute the TOPP values if possible!
  double VTmPsi;
  double VTmKs;
  double VTmB;
  double VTmpi;
  double VTmmu;  
  double VTme;
  // max chi2 distance for trk/wire assignment. (default 10.)
  float VTch2wtr;
  // max chi2 distance for B0/wire assignment. (default 20.)
  float VTch2wB;
  // parameters for clustering tracks which have been assigned to a common
  // wire. VTxlmin is the minimum hit separation along a wire and VTxlwid
  // is the maximum width of a cluster. The clusters are used as seeds for 
  // primary vertices (default VTxlmin=.015, VTxlwid=.050)
  float VTxlmin;
  float VTxlwid;
  // min trks on a primary - ignored if it's a B0 primary, which can even
  // have 0 tracks left over (i.e. just B0 intersection with a wire)
  // (default 3)
  int   VTntmin;
  // max chi2 distance for a track to a primary vtx seed for trk/vtx merging 
  // (default 14.)
  float VTch2vtr;
  // max chi2 for merging 2 trks into a new secondary (default 6.)
  float VTch2stt;
  // max chi2 for adding a track to a secondary vertex (default 8.)
  float VTch2stv;
  // min chi2 distance of a secondary vertex from a wire (default 1.)
  float VTch2svw;
  // max impact parameter in (cm) of track to the B0 secondary if the 
  // search is restricted to the vicinity of the B0 (eg for charm decays)
  // (default 0.1)
  float VTBvimpc;
  // max # of primary vtx tracks assignable to a secondary (default 1)
  int   VTnmix;
  // max chi2 distance of 2 vertices for vertex merging (default 6.)
  float VTch2vv;
  // Set to 999 before first call to VT to override default cuts (default 999)
  int   VTtweak;
  // Event display with HIGZ (default 0 = off)
  int   VTDraw;
  //
  //           4) Cuts for the Rover Primary vertex finder
  //
  // Which tracks to use? 1=Rover (default) 2=all VDS RSEGs 3=all RTRAs
  int   RPTrackChoice;
  // How many VDS hits (minimum) in the tracks? (default 5)
  int   RPTrackQualNH;
  // minimum number of tracks in a primary (default 3)
  int   RPMinTrksInVtx;
  // maximum number of sigma for associating tracks to wires 
  // or each other (default 3.5)
  float RPTrkWireSigmaMax;
  //
  //           5) Cuts for the GroverPrimEval vertex finder evaluation
  //              routine (based on VERONA by J.van Tilburg)
  //
  // Only print the stuff every Nth event (default = 100)
  int   GPEUpdateFreq;
  // Minimum # hits for a track to be counted
  int   GPEMinHitsInTrack;
  // Minimum # tracks for a vertex to be counted
  int   GPEMinTracksInVertex;
  // Maximum chi2 for a vtx to be accepted
  float GPEMaxChi2;
  // Max chi2 per track for a vtx to be accepted
  float GPEMaxChi2PerTrack;
  // Size of windows (in cm) for matching real vertices with 
  // MC vertices (defaults 0.0200, 0.4000 respectively)
  float GPETransverseWindow;
  float GPELongitudinalWindow;
  // Chi (sqrt(chi2)) cut for same (default 5.0)
  float GPEChiWindow;
  //
  //           6) Cuts for the Grover secondary vertex finders
  //
  // How many VDS hits (minimum) in the tracks? (default 7)
  int   RSTrackQualNH;
  // J/psi seed trks: trk-trk chi2 dist (def. 20) and spatial dist (def. 0.1)
  float RSJpsiSeedMaxChi2;
  float RSJpsiSeedMaxDist;
  // Minimum impact parameter significance for a seed track (default 10.0)
  float RSMinSeedIPS;
  // Max chi2 contribution of a track to a 
  // secondary (otherwise rejected) (def. 5.0)
  float RSMaxChi2Contrib;
  // Maximum chi2 distance of trk to vtx to consider refitting (def. 10)
  float RSMaxTrkVtxChi2;
  // Maximum chi2 dist of two tracks to consider fitting as a seed (def. 10)
  float RSMaxTrkTrkChi2;
  // Maximum chi2 of a two-track seed after initial fit (default 10)
  float RSMax2TVtxChi2;
  // Maximum chi2/dof to accept a secondary (def. 5.0)
  float RSMaxChi2PDOF;
  //
  //           7) Stuff for the 3D display using ROOT
  //
  // To draw or not?
  int   GroverDraw3D;
  // Use X3D display?
  int   TDX3D;
  int   TDTargetRegion;
  int   TDAxes;
  int   TDWires;
  int   TDPrimaries;
  int   TDSecondaries;
  int   TDMCPrimaries;
  int   TDMCSecondaries;
  int   TDPrimaryTracks;
  int   TDSecondaryTracks;
  int   TDUnusedTracks;
  int   TDSpecialTracks;
  // colours
  int   TDWireColour;
  int   TDPrimaryColour;
  int   TDSecondaryColour;
  int   TDMCPrimaryColour;
  int   TDMCSecondaryColour;
  int   TDPrimaryTrackColour;
  int   TDSecondaryTrackColour;
  int   TDUnusedTrackColour;
  int   TDSpecialTrackColour;
  // number of segments in track "cylinder"
  int   TDTrackSmoothness;
  // track lengths in centimetres
  float TDPrimaryTrackLength;
  float TDSecondaryTrackLength;
  float TDUnusedTrackLength;
  float TDSpecialTrackLength;
  // axis lengths in centimetres
  float TDXAxisLength;
  float TDYAxisLength;
  float TDZAxisLength;
  float TDAxisRadius;
  // position of axis marker
  float TDAxisX;
  float TDAxisY;
  float TDAxisZ;
  // track quality cuts for drawing
  float TDTrackMinP;
  float TDTrackMinPt;
  float TDTrackMaxChi2;
  int   TDTrackMinNh;
  //
  //           8) STEERING OF GROVEREVENT()
  //
  // Steering GroverEvent()
  int GETrackChoice;
  int GERefillTracks;
  int GEFindPrimaries;
  int GERedoPrimaries;
  int GEFindSecondaries;
  int GERedoSecondaries;
  int GERefitRSEGTable;
  int GEWriteRTAR;
  int GEWriteRVER;
  // Steering GroverPrimaries(...)
  int GPWireFollowing;
  int GPNewWires;
};




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                    Inlined member functions                        //
//                                                                    //
////////////////////////////////////////////////////////////////////////

#include "grover/grover_inlines.hh"




////////////////////////////////////////////////////////////////////////
//                                                                    //
//                     Overloaded operators                           //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// (in)equality operators for the main classes.
// GroverParticles are 'equal' if they have the same parent RSEG, RTRA etc.
bool operator==(const GroverParticle&, const GroverParticle&);
bool operator!=(const GroverParticle&, const GroverParticle&);
// GroverVertices are 'equal' if all their particles are 'equal' (see above)
bool operator==(const GroverVertex&, const GroverVertex&);
bool operator!=(const GroverVertex&, const GroverVertex&);
// GroverWires are 'equal' if they have the same Number() (wnr in GTAR)
bool operator==(const GroverWire&, const GroverWire&);
bool operator!=(const GroverWire&, const GroverWire&);
// GroverCuts are 'equal' if all elements equal. This is what the compiler
// generates anyway.

// output streamers
ostream& operator<<(ostream&, GroverParticle&);
ostream& operator<<(ostream&, GroverVertex&);
ostream& operator<<(ostream&, GroverWire&);
// not defined for GroverCuts. Use GroverCuts::Dump(void) instead



////////////////////////////////////////////////////////////////////////
//                                                                    //
//                       Function prototypes                          //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// Filling to/from ARTE tables
extern int  GroverFillFromRSEG(GroverParticleList*);
extern int  GroverFillFromRTRA(GroverParticleList*);
extern int  GroverFillLikeRover(GroverParticleList*);
extern int  GroverFillLikeMD(GroverParticleList*);
extern int  GroverFillSmarter(GroverParticleList*);
extern int  GroverFillSmartNoVDSonly(GroverParticleList*);
extern int  GroverFillClonerm(GroverParticleList*);
extern int  GroverFillClonermNoVDSonly(GroverParticleList*);
extern int  GroverFillFromGTAR(GroverWireList*);
extern int  GroverCorrectWires(GroverWireList*);
extern int  GroverFillFromRTAR(GroverWireList*);
extern int  GroverFillFromRVER(GroverVertexList*);
extern int  GroverFillFromMVER(GroverVertexList*,
			       const int type=1);
#ifndef GROVER_WITHOUT_ARTE
extern int  GroverAddToRVER(const GroverVertexList*,
			    const int type = 1,  
			    const ArtePointer<TOPP> =0);
extern int  GroverAddToRVER(const GroverVertex*,
			    const int type = 1,  
			    const ArtePointer<TOPP> =0);
#endif

// Graphics
// - 2D
extern int  graph_ini(void);
extern void draw_targets(const GroverWireList*);
extern void draw_mc(const float z_plane);
extern void draw_track(const GroverParticle*,const int colour);
extern void draw_vertex(const GroverVertex*,const int colour);

// - 3D
extern int  GroverDraw3D(const GroverParticleList* Particles,
			 const GroverWireList*     Wires,
			 const GroverVertexList*   Primaries,
			 const GroverVertexList*   Secondaries,
			 const GroverVertexList*   MCPrimaries = 0,
			 const GroverVertexList*   MCSecondaries = 0,
			 const GroverParticleList* Specials = 0,
			 const GroverCuts*         usercuts = 0);
extern int  GroverDraw3D(void);

// Algorithms
extern int  GroverPrimaries(GroverParticleList*,
			    GroverWireList*,
			    const GroverCuts*,
			    GroverVertexList* primaries);
extern int  GroverPrimaryFinder(GroverParticleList*,
			    GroverWireList*,
			    const GroverCuts*,
			    GroverVertexList* primaries);
extern int  GroverEvent(void);                      
extern int  GroverSecondaries(GroverParticleList* all,
			      GroverWireList*,
			      const GroverCuts*,
			      GroverVertexList* input_primaries,
			      GroverVertexList* output_secondaries);
extern int  GroverUnbiasedSecondaries(GroverParticleList*,
				      const GroverWireList*,
				      const GroverCuts*,
				      const GroverVertexList* input_primaries,
				      GroverVertexList* output_secondaries);
extern int  GroverJpsiSec(GroverParticleList*,
			  const GroverWireList*,
			  const GroverCuts*,
			  GroverVertexList* primaries,
			  GroverVertexList* jpsis);
extern int  GroverJP(GroverVertex*,
		     GroverParticleList*,
		     GroverVertexList*,
		     const GroverCuts*);
extern int  GroverCharm(GroverParticleList*,
			const GroverWireList*,
			const GroverCuts*,
			GroverVertexList* primaries,
			GroverVertexList* charm);

// Utilities
extern int  GroverRefitRSEG(const GroverCuts* c = 0);
extern int  MoveToGlobal(GroverParticleList*);
extern int  MoveToGlobal(GroverVertexList*);
extern int  MoveToGlobal(GroverWireList*);
extern int  MoveToLocal(GroverParticleList*);
extern int  MoveToLocal(GroverVertexList*);
extern int  MoveToLocal(GroverWireList*);
#ifndef GROVER_WITHOUT_ARTE
extern ArtePointer<RSEG> RSEGCausedBy(const ArtePointer<MTRA>);
extern ArtePointer<MTRA> MTRAWhichCaused(const ArtePointer<RSEG>);
extern ArtePointer<RTRA> RTRACausedBy(const ArtePointer<MTRA>);   // not done!
extern ArtePointer<MTRA> MTRAWhichCaused(const ArtePointer<RTRA>);// not done!
extern int  GroverRefitRSEG(ArtePointer<RSEG>, const float momentum, 
			    const float smearing_sigma = 0.0);
extern int  GroverCatsExtrap(const ArtePointer<RSEG>,const float z0,
			     float* x,float* y,const float* z,
			     float* ax,float* ay,
			     const float* inverse_momentum,
			     float[]);
extern int  GroverCatsExtrap(const float z0,
			     float* x,float* y,const float* z,
			     float* ax,float* ay,
			     const float* inverse_momentum,
			     float[]);
extern int  GroverQuickFit(float& x,    float& y,  float& z,
			   float& dx,   float& dy, float& dz, 
			   float& chi2, float& mass,
			   const ArtePointer<RTRA> rtra1,   
			   const ArtePointer<RTRA> rtra2,
			   const ArtePointer<RTRA> rtra3=0, 
			   const ArtePointer<RTRA> rtra4=0, 
			   const ArtePointer<RTRA> rtra5=0);
extern int  GroverQuickFit(float& x,  float& y,  float& z,
			   float& dx, float& dy, float& dz,
			   float& chi2, float& mass,
			   const ArtePointer<RSEG> rseg1,   
			   const ArtePointer<RSEG> rseg2, 
			   const ArtePointer<RSEG> rseg3=0, 
			   const ArtePointer<RSEG> rseg4=0, 
			   const ArtePointer<RSEG> rseg5=0);
#endif 

extern int  GroverAddBest(GroverVertex*,GroverParticleList*,const int slow = 0);

// return px of GroverParticle
float GroverPx(GroverParticle* gp);
// return py of GroverParticle
float GroverPy(GroverParticle* gp);
// return pz of GroverParticle
float GroverPz(GroverParticle* gp);

// transform a GroverVertexList to VDS local coordinates
int GroverListToLocal(GroverVertexList *gvl);

// transform a GroverParticleList to VDS local coordinates
int GroverListToLocal(GroverParticleList *gpl);

// transform a GroverWireList to VDS local coordinates
int GroverListToLocal(GroverWireList *gwl);

// transform a GroverVertexList to global coordinates
int GroverListToGlobal(GroverVertexList *gvl);

//  transform a GroverParticleList to global coordinates
int GroverListToGlobal(GroverParticleList *gpl);

// transform a GroverWireList to global coordinates
int GroverListToGlobal(GroverWireList *gwl);

// Expert and obsolete/deprecated functions
extern int  GroverPrimEval(GroverVertexList* reconstructed,
			   GroverVertexList* optional_mc,GroverCuts*);
extern int  GroverEvent(const GroverCuts* optional_cuts);
extern int  PrimEvalInitPaw(void);
extern int  PrimEvalInitRoot(void);
extern int  GroverSecTest(void);
extern int  GroverSecFinder(GroverVertexList* mc,GroverVertexList* reco,
			    int type = 1, GroverCuts* c = 0);
extern int  GroverConstrainedFit(GroverVertex*, const float mass);
extern int  GeoFit(GroverVertex*);
extern void GroverSecondaryFit (GroverVertex*); 
extern int  GroverFindJpsis(GroverParticleList*,GroverCuts*,GroverVertexList*);
extern int  FakeWires(GroverWireList*, const int run_number);
extern bool ReasonableWires(const GroverWireList*, const int=1);
extern bool SortGPByDist(GroverParticle,GroverParticle);
extern bool RevSortGPByDist(GroverParticle,GroverParticle);
extern bool SortGPByChi2(GroverParticle,GroverParticle);
extern bool RevSortGPByChi2(GroverParticle,GroverParticle);
extern bool SortGPBySecWeight(GroverParticle,GroverParticle);
extern bool RevSortGPBySecWeight(GroverParticle,GroverParticle);
extern bool SortGPByPrimWeight(GroverParticle,GroverParticle);
extern bool RevSortGPByPrimWeight(GroverParticle,GroverParticle);
extern void three_dim_inv(double a[3][3]);
extern void four_dim_inv(double a[4][4]);
extern void five_dim_inv(double a[5][5]);
extern void six_dim_inv(double a[6][6]);
extern void seven_dim_inv(double a[7][7]);

// external functions
extern int  acLocalToGlobal(float* local, float* global, int subdetector);
extern int  acGlobalToLocal(float* local, float* global, int subdetector);



////////////////////////////////////////////////////////////////////////
//                                                                    //
//              Interface to the ARTE implementation                  //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// Wire positions for communication between GroverEvent() and 
// GroverPrimaries(...). Constructed in GroverPrimaries().
extern float wire_xpos[9],
             wire_ypos[9],
             wire_zpos[9],
             wire_xsize[9],
             wire_ysize[9],
	     wire_zsize[9];

// Root running flag. Instantiated in GroverDraw3D
extern bool               ggRootRunning;

// Global objects. Constructed in GroverEvent()
extern GroverParticleList ggParticles;
extern GroverWireList     ggWires;
extern GroverVertexList   ggPrimaries;
extern GroverVertexList   ggSecondaries;
extern GroverVertexList   ggMCPrimaries;
extern GroverVertexList   ggMCSecondaries;
extern GroverParticleList ggSpecialParticles;
extern GroverCuts         ggCuts;


// Average Beam_position
static const float ggBeamspotX =  0.300;
static const float ggBeamspotY = -0.050;



////////////////////////////////////////////////////////////////////////
//                                                                    //
//                   Miscellaneous stuff                              //
//                                                                    //
////////////////////////////////////////////////////////////////////////

// Colours for the HIGZ (2-D) graphics
const int Black        =  1,
          DarkGray     =  2,
          Blue         =  3,
          Green        =  4,
          Cyan         =  5,
          Red          =  6,
          Magenta      =  7,
          Brown        =  8,
          LightBlue    =  9,
          LightGreen   = 10,
          LightCyan    = 11,
          LightRed     = 12,
          LightMagenta = 13,
          Yellow       = 14,
          LightGray    = 15,
          White        =  0;

#endif

