//-----------------------------------------------------------
//
//  routine :      ITR_fill_OTDR_table
//
//                 fill OTDR table from ITR_Chanstat table
//
//  Y. Gorbunov 01/2002
//
//  inclusion into itrarte 05/02/2002   TZ
//------------------------------------------------------------

#include "arte/arte.hh"

#include "itrleda.h"
#include "ITR_arte.h"
#include "ITR_steer.h"
#include "grr.h"
#include "aframelib.h"

void ITR_fill_OTDR_table(){

  OTDR otd; 
  int NSLayers=0;
  char *routine = "ITR_fill_OTDR_table";

  aframe_mess(routine,GRR_SINFO,"Filling OTDR table.");  

  for(ArteTableIterator<GESL> igesl; igesl.valid(); igesl.next())
    if((*igesl)->cmp==3) NSLayers++;

  int deadCop[771];
    
  for ( LedaTableIterator<class ITR_Chamber> ch; ch.valid(); ch.next() ){
    LedaPointer<class ITR_nede> nedeIt = ch->tonede;

    for(int i=0;i<771;i++) deadCop[i]=1;

    if(1){

      int NumChip=0;
      int ShiftChip=0;

      for (LedaRelationIterator chipIt = ch->chip ;  
           chipIt.valid(); chipIt.next()) {
        
        LedaPointer<class ITR_Chip> theChip (chipIt);
        LedaPointer<class ITR_Chanstat> Chan = theChip->chanstat;

        if(NumChip==0) ShiftChip=0;
        if(NumChip==1) ShiftChip=128;
        if(NumChip==2) ShiftChip=256;
        if(NumChip==3) ShiftChip=384;
        if(NumChip==4) ShiftChip=512;
        if(NumChip==5) ShiftChip=640;
        
        for(int jj=(0+ShiftChip);jj<(128+ShiftChip);jj++)
          if(((int)Chan->status[(jj-ShiftChip)])==0){
            deadCop[jj]=0;
            deadCop[770]++;
        }
        NumChip++;
      }
    }

    if(deadCop[770]>1){
     
      int FirstWire=0,LastWire=0;


      for(int i=0;i<768;i++){ 

        if(FirstWire==0 && deadCop[i]<1 && deadCop[(i+1)]<1){
          FirstWire=i;
          int ij=i;
          ij++;
          while(deadCop[ij]<1){
            ij++;
          }
          LastWire=ij; 
          i=ij;
          LastWire--;

          
          float xFirst=XCorCalc(nedeIt->superl_db,nedeIt->lay_db,nedeIt->sec_db[0],FirstWire,NSLayers);
          float xSecon=XCorCalc(nedeIt->superl_db,nedeIt->lay_db,nedeIt->sec_db[0],LastWire,NSLayers);

          if(FirstWire<8 && LastWire<8) continue;

          if(FirstWire>759 && LastWire>759) continue;
        
          if(xFirst<xSecon){
            otd.xmin=xFirst;
            otd.xmax=xSecon;
          }
          if(xFirst>xSecon){
            otd.xmax=xFirst;
            otd.xmin=xSecon;
          }
          
          char sectorCH[5];
          if(nedeIt->sec_db[0]==1 || nedeIt->sec_db[1]==1) sprintf(sectorCH,"++");
          if(nedeIt->sec_db[0]==3 || nedeIt->sec_db[1]==3) sprintf(sectorCH,"+-");
          if(nedeIt->sec_db[0]==5 || nedeIt->sec_db[1]==5) sprintf(sectorCH,"--");
          if(nedeIt->sec_db[0]==7 || nedeIt->sec_db[1]==7) sprintf(sectorCH,"-+");
          int SlNum=0;
        
          if(NSLayers>7){
            if(nedeIt->superl_db==1)   SlNum=1;
            if(nedeIt->superl_db==2)   SlNum=3;
            if(nedeIt->superl_db==3)   SlNum=5;
            if(nedeIt->superl_db==4)   SlNum=6;
            if(nedeIt->superl_db==5)   SlNum=10;
            if(nedeIt->superl_db==6)   SlNum=11;
            if(nedeIt->superl_db==7)   SlNum=12;
            if(nedeIt->superl_db==8)   SlNum=13;
            if(nedeIt->superl_db==9)   SlNum=14;
            if(nedeIt->superl_db==10)  SlNum=15;        
          }
          
          if(NSLayers<=7){
            if(nedeIt->superl_db==1)   SlNum=1;
            if(nedeIt->superl_db==2)   SlNum=10;
            if(nedeIt->superl_db==3)   SlNum=11;
            if(nedeIt->superl_db==4)   SlNum=12;
            if(nedeIt->superl_db==5)   SlNum=13;
            if(nedeIt->superl_db==6)   SlNum=14;
            if(nedeIt->superl_db==7)   SlNum=15;
          }

          if(nedeIt->superl_db<5)
            sprintf(otd.hwc,"MS0%d%s%d",SlNum,sectorCH,nedeIt->lay_db); 
          else
            sprintf(otd.hwc,"MS%d%s%d",SlNum,sectorCH,nedeIt->lay_db);
          int flag=0;
          for ( ArteTableIterator<GEDE> gedeIt; gedeIt.valid(); gedeIt.next()){
            if(((*gedeIt)->cmp == 3) && ((*gedeIt)->gesl->nsl == nedeIt->superl_db) && 
               ((*gedeIt)->lay == nedeIt->lay_db) && 
               (((*gedeIt)->sec == nedeIt->sec_db[1])||((*gedeIt)->sec == nedeIt->sec_db[0]))){
              if(otd.xmin>=(*gedeIt)->sxmn && otd.xmax<=(*gedeIt)->sxmx)
                {
                  flag=1;
                  int gederow=(*gedeIt).index();
                  otd.gede =ArtePointer<GEDE>(gederow);
                  ArteTable<OTDR>::push_back( otd );
                }
            }
          }

          if(flag==0){
             float borders[2];
             borders[0]=otd.xmax;
             borders[1]=otd.xmin;
             for ( ArteTableIterator<GEDE> gedeIt; gedeIt.valid(); gedeIt.next()){
               if(((*gedeIt)->cmp == 3) && ((*gedeIt)->gesl->nsl == nedeIt->superl_db) && 
                  ((*gedeIt)->lay == nedeIt->lay_db) && 
                  (((*gedeIt)->sec == nedeIt->sec_db[1])||((*gedeIt)->sec == nedeIt->sec_db[0]))){

                 if(otd.xmin>=(*gedeIt)->sxmn &&  otd.xmax>=(*gedeIt)->sxmx){
                   otd.xmin=borders[1];
                   otd.xmax=(*gedeIt)->sxmx;
                   int gederow=(*gedeIt).index();
                   otd.gede=ArtePointer<GEDE>(gederow);
                   ArteTable<OTDR>::push_back( otd );
                 }
                 if(otd.xmax<=(*gedeIt)->sxmx && otd.xmin<=(*gedeIt)->sxmn){
                   otd.xmin=(*gedeIt)->sxmn;
                   otd.xmax=borders[0];
                   int gederow=(*gedeIt).index();
                   otd.gede=ArtePointer<GEDE>(gederow);
                   ArteTable<OTDR>::push_back( otd );
                 }
               }
             }
           }
           
         
           FirstWire=0; LastWire=0;
        }
        
      }
    }
  }

}

//-----------------------------
//
//
//-----------------------------
float XCorCalc(int superlay, int layer, int sector, int stripn , int Nslay){

  ArtePointer<GEDE> gede, gede1;
  int entsector,chambertype,gederow=0;
  entsector=sector;
  char *routine = "ITR_fill_OTDR_table";

  if(entsector==1 || entsector==2) sector=2;
  if(entsector==3 || entsector==4) sector=4;
  if(entsector==5 || entsector==6) sector=5;
  if(entsector==7 || entsector==8) sector=7;

  if(Nslay>7){
    if(superlay <= 8 ) 
      chambertype=2;
    else 
      chambertype=3;
  }
  else{
    if(superlay <= 5 ) 
      chambertype=2;
    else 
      chambertype=3;
  }
  
  for ( ArteTableIterator<GEDE> igede; igede.valid(); igede.next() ) {
   ArtePointer<GEDE> gede(*igede);
   ArtePointer<GESL> gesl(gede->gesl);
   if ((gede->cmp == 3) && (gesl->nsl == superlay)
       && (gede->lay == layer) && (gede->sec == sector))  
     gederow=gede.index();
  }
  
  if ( sector == 5 || sector  == 7) {
      gede =ArtePointer<GEDE>(gederow);
      gede1=ArtePointer<GEDE>(gederow+1);
    }
  else if (sector == 4 || sector == 2){
      gede1=ArtePointer<GEDE>(gederow);
      gede=ArtePointer<GEDE>(gederow+1);
    }                    
  
  int found=0;
  float xcoord,zcoord;
  xcoord=zcoord=0;
  float relpos=0;
  float gasgap = gede->zmax-gede->zmin;  
  zcoord    = gede->zfw[0]+gasgap/2;
  
  if (chambertype ==3 && (int) sector == 5 || (int) sector == 7 ) {
    // -x side Typ III 
    if (stripn>4 && stripn<763)
      {
        relpos = gede->nwi[0]+gede1->nwi[0]+4-stripn;
        xcoord=relpos*gede->pit + gede->xfw[0];
        found = 1;
      }    
  }  
  
  if ( chambertype ==3 && (int) sector == 4 || (int) sector == 2 ) {
    // +x side Typ III
    if (stripn>4 && stripn<763) {
      relpos = stripn-5;
      xcoord=relpos*gede->pit + gede->xfw[0];
      found = 1;
    }  
  }
  
  //chamber type
  if ( chambertype < 3 && ((int) sector == 5 || (int) sector == 7 )) {
    // -x side Typ I+II
    if(stripn<7) stripn=8;
    if(stripn>760) stripn=759;
    if (stripn>7 && stripn<760) {
      relpos = gede->nwi[0]+gede1->nwi[0]+7-stripn;
      xcoord = relpos*gede->pit + gede->xfw[0];
      found = 1;
    }
    
    
  }
  //chamber type
    if ( chambertype < 3 && ( sector == 4 || sector == 2 )) {
      // +x side Typ I+II
      if(stripn<7) stripn=8;
      if(stripn>760) stripn=759;
      if (stripn>7 && stripn<760) {
        relpos = stripn-8;
        xcoord = relpos*gede->pit + gede->xfw[0];
        found = 1;
      }
      
    }     
    if (!((int) sector== 4 || (int) sector== 2 || (int) sector== 5 || (int) sector== 7 ))
      aframe_mess(routine,GRR_SINFO,"Wrong sector !");

    return xcoord;
}

