*-- Author :    Hartwig Albrecht   19/10/95
*-- modified by Thomas Lohse       March 8, 1999
*-- modified by Wolfgang Gradl     October 10, 2000 
*-- ... allow selection of drell-yan process 
*-- ... (i.e. q \bar q \to \gamma^* \to l^+ l^- only)
*-- modified by Ullrich Schwanke   February 2, 2001
*-- ... allow selection of NLO processes to drell-yan 
*-- ... q\bar{q} -> gamma* g -> l+l- g
*-- ... q     g  -> gamma* q -> l+l- q
*-- modified by Olya Igonkina      March 8, 2001
*-- ... add CSM and COM processes for Charmonium production (psi, chi)
*-- ... (+) change PARP(176) for Psi(2S) radial function
*-- ... (+) change CKIN(5) - PT cut in PYTHIA - to 0.2 GeV instead of 1 GeV
*-- ... (+) changes are applied only to CSM and COM
*-- ... (+) change CKIN(5) - PT cut in PYTHIA - to 0.5 GeV instead of 0.2 GeV

      SUBROUTINE KALUND(TYPE)

************************************************************************
*
* Description: LUND command
*
* Called from: KUIP
*
* External references:
*
* Argument in/out description
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "arte/ArteFlag.fi"
#include "arte/ZIOCOM.fi"
#include "auto/TOPP.fi"
#include "pyth/PYSUBS.fi"
#include "pyth/PYPARS.fi"
#include "pyth/PYINT2.fi"
#include "arte/luopts.fi"
      INTEGER TYPE

      INTEGER L, TPNAM1
      integer mes_id_1, mes_id_2, mes_id_3, mes_id_4
      CHARACTER*12 CH

      save mes_id_1, mes_id_2, mes_id_3, mes_id_4
#include "arte/Arte.ff"
#include "auto/TOPP.ff"

C Lund initialization:

      IF (FIOMIX(5) .NE. 0.)  CALL MEERRC (mes_id_1,
     +  'IOMIX and LUND commands are mutually exclusive')
      IF (LUNDINIT .NE. 1)  THEN
        CALL ALTOPP
        CALL ALTDEC
        LUNDINIT = 1
      END IF

      FIOMIX(1) = 1.
      FIOMIX(2) = 0.
      FIOMIX(3) = 0.
      FIOMIX(4) = 0.
      FIOMIX(5) = 0.

      CALL KUGETC (CH, L)


      IF (CH(1:2) .EQ. 'CC')  THEN
        MSEL = 4
      ELSE IF (CH(1:2) .EQ. 'BB')  THEN
        MSEL = 5
      ELSE IF (CH(1:2) .EQ. 'IN')  THEN
        MSEL = 1
      ELSE IF (CH(1:2) .EQ. 'DY')  THEN
         MSEL = 11                        ! Z/gamma^* production
         MSUB(1)  = 1                     ! q qbar -> gamma/Z 
         MSTP(43) = 1                     ! only gamma^*, no Z
      ELSE IF (CH(1:5) .EQ. 'NLODY')  THEN
         MSEL = 0                         ! Z/gamma^* production
         MSUB(15) = 1                     ! q qbar -> gamma/Z g 
         MSUB(30) = 1                     ! q g    -> gamma/Z q 
         MSTP(43) = 1                     ! only gamma^*, no Z
      ELSE IF (CH(1:1) .EQ. 'J')   THEN
        MSEL = 0
        MSUB(86) = 1
        MSUB(87) = 1
        MSUB(88) = 1
        MSUB(89) = 1
      ELSE IF (CH(1:1) .EQ. 'U')   THEN
        MSEL = 0
        MSUB(86) = 1
        MSUB(87) = 1
        MSUB(88) = 1
        MSUB(89) = 1
        KFPR(86,1) = 553
        KFPR(87,1) = 10551
        KFPR(88,1) = 20553
        KFPR(89,1) = 555
      ELSE IF (CH(1:3) .EQ. 'CSM')   THEN  ! CSM

        MSEL = 0
        MSUB(86) = 1            ! g+g->J/psi+g
        MSUB(87) = 1            ! g+g->chi_c0+g
        MSUB(88) = 1            ! g+g->chi_c1+g
        MSUB(89) = 1            ! g+g->chi_c2+g
        MSUB(90) = 1            ! g+g->psi(2S)+g

        MSUB(104) = 1           ! g+g->chi_c0
        MSUB(105) = 1           ! g+g->chi_c2

        MSUB(106) = 1           ! g+q->chi_c0+g
        MSUB(107) = 1           ! g+q->chi_c1+g
        MSUB(108) = 1           ! g+q->chi_c2+g

        MSUB(125) = 1           ! q+q~->g+chi_c0
        MSUB(126) = 1           ! q+q~->g+chi_c1
        MSUB(127) = 1           ! q+q~->g+chi_c2

        PARP(176) = 0.53        ! radial function for Psi(2S)
        
*---------->
        CKIN(5)   = 0.5         ! threshold on PT transfered
*---------->

      ELSE IF (CH(1:3) .EQ. 'COM' )   THEN  ! COM

        MSEL = 0
        CALL PYINITCOM(2)  ! 2 is prefered set of parameters

        MSUB(86) = 1            ! g+g->J/psi+g
        MSUB(87) = 1            ! g+g->chi_c0+g
        MSUB(88) = 1            ! g+g->chi_c1+g
        MSUB(89) = 1            ! g+g->chi_c2+g
        MSUB(90) = 1            ! g+g->psi(2S)+g
        
        MSUB(104) = 1           ! g+g->chi_c0
        MSUB(105) = 1           ! g+g->chi_c2
        
        MSUB(106) = 1           ! g+q->chi_c0+g
        MSUB(107) = 1           ! g+q->chi_c1+g
        MSUB(108) = 1           ! g+q->chi_c2+g
        
        MSUB(125) = 1           ! q+q~->g+chi_c0
        MSUB(126) = 1           ! q+q~->g+chi_c1
        MSUB(127) = 1           ! q+q~->g+chi_c2

        MSUB(190) = 1           ! g+g->J/psi+g  (COM)
        MSUB(191) = 1           ! g+g->chi_c0+g (COM)
        MSUB(192) = 1           ! g+g->chi_c1+g (COM)
        MSUB(193) = 1           ! g+g->chi_c2+g (COM)
        MSUB(194) = 1           ! g+g->psi(2S)+g (COM)

        MSUB(119) = 1           ! g+g -> chi_c0 (COM)
        MSUB(120) = 1           ! g+g -> chi_c2 (COM)


        MSUB(132) = 1           ! g+q->chi_c0+g (COM)
        MSUB(133) = 1           ! g+q->chi_c1+g (COM)
        MSUB(134) = 1           ! g+q->chi_c2+g (COM)

        MSUB(135) = 1           ! q+q~->chi_c0+g (COM)
        MSUB(136) = 1           ! q+q~->chi_c1+g (COM)
        MSUB(137) = 1           ! q+q~->chi_c2+g (COM)

        PARP(176) = 0.53        ! radial function for Psi(2S)

*---------->
        CKIN(5)   = 0.5         ! threshold on PT transfered
*---------->


      ELSE
        MSEL = 1
        CALL MEERRC (mes_id_2,
     +   'LUND: phys. process must be one of cc,   
     +    bb, inel, Jpsi, Upsi, DY, NLODY')
      END IF

      CALL KUGETC (CH, L)
      IF (L .NE. 0 .AND. CH(1:2) .NE. 'IN')  THEN
        CALL MEERRC (mes_id_3,
     +    'LUND: 2nd phys. process must be inelastic')
        GO TO 90
      END IF

      CALL KUGETR (FIOMIX(2))
      IF (L .EQ. 0)  FIOMIX(2) = 0.

      LUNDRPART = 0
      call KUGETS (CH, L)
      IF (L .NE. 0)  THEN
        LUNDRPART = TPNAM1 (CH)
        IF (LUNDRPART .NE. 0)  THEN
          LUNDRPART = ITOPP_LUND(LUNDRPART)
        ELSE
          CALL MEERRC(mes_id_4, 'LUND: required particle is unknown')
        END IF
      END IF

 90   END

************************************************************************
* End of KALUND
************************************************************************
