*CMZ :  1.00/00 22/09/94  20.47.57  by  Fedor Ratnikov
*CMZ :  0.01/02 27/08/94  17.37.42  by  Dominik Ressing
*-- Author :    Dominik Ressing   23/08/94
************************************************************************
*
* L1GEO2S
* written by Dominik Ressing, 23/08/94 15.16.48
*
* This routines action:
* the two quadrangle RegA,RegB are overlapped.
* If the hit regions overlap the overlap-region is returned --
* else negative sizes of OvLap.
* The overlap Region has the z of the RegA. The
* stereo angle of OvLap is 0.
*
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* RegA        in          first region of type GeoInf
* RegB        in          second region of type GeoInf
* OvLap      out          overlap region of type GeoInf
*
************************************************************************
c
      SUBROUTINE L1GEO2S(RegA,RegB,OvLap)
c
      IMPLICIT NONE
c
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
c
      REAL RegA(L1GEOINFlen),RegB(L1GEOINFlen),OvLap(L1GEOINFlen)
c
      REAL    rx(4),ry(4),u,v,umin,umax,vmin,vmax,du,
     &        rd1,rd2,x01,x02,dx1,dx2,y01,y02,dy1,dy2
c
c
      IF ( RegA(L1GEOINFsin_al).eq.RegB(L1GEOINFsin_al) ) THEN
c        --- get distance of regions in tilded system of RegA
         CALL l1geort(RegB(L1GEOINFx0),RegB(L1GEOINFy0),
     &                RegA(L1GEOINFx0),RegA(L1GEOINFy0),
     &                RegA(L1GEOINFsin_al),u,v)
c        --- find the overlapping region in tilded system
         umin = MAX(0.,u)
         umax = MIN(RegA(L1GEOINFsx),u+RegB(L1GEOINFsx))
         vmin = MAX(0.,v)
         vmax = MIN(RegA(L1GEOINFsy),v+RegB(L1GEOINFsy))
c        --- do the two regions overlap ??
         IF ( umin.ge.umax .or. vmin.ge.vmax ) THEN
            OvLap(L1GEOINFx0) = -999999.
            OvLap(L1GEOINFsx) = -999999.
            OvLap(L1GEOINFy0) = -999999.
            OvLap(L1GEOINFsy) = -999999.
         ELSE
            OvLap(L1GEOINFsy) = vmax-vmin
            OvLap(L1GEOINFy0) = RegA(L1GEOINFy0)+vmin
            du = OvLap(L1GEOINFsy)*RegA(L1GEOINFsin_al)/
     &                                 RegA(L1GEOINFcos_al)
            IF ( du.lt.0 ) THEN
               OvLap(L1GEOINFx0) = RegA(L1GEOINFx0)+umin+du  ! start is smaller
               OvLap(L1GEOINFsx) = (umax-umin)-du            ! enlarge region
            ELSE
               OvLap(L1GEOINFx0) = RegA(L1GEOINFx0)+umin
               OvLap(L1GEOINFsx) = (umax-umin)+du
            END IF
         END IF
         OvLap(L1GEOINFsin_al) = RegA(L1GEOINFsin_al)
         OvLap(L1GEOINFcos_al) = RegA(L1GEOINFcos_al)
         OvLap(L1GEOINFz)  = RegA(L1GEOINFz)
         RETURN
      END IF
c
c     find intersection of the four border lines
c
      x01 = RegA(L1GEOINFx0)
      y01 = RegA(L1GEOINFy0)
      dx1 = RegA(L1GEOINFsin_al)
      dy1 = RegA(L1GEOINFcos_al)
      x02 = RegB(L1GEOINFx0)+RegB(L1GEOINFsx)
      y02 = RegB(L1GEOINFy0)
      dx2 = RegB(L1GEOINFsin_al)
      dy2 = RegB(L1GEOINFcos_al)
c     --- crossing of the outermost two borders (upper/lower point)
      CALL l1geolc(x01,y01,dx1,dy1,x02,y02,dx2,dy2,
     &     rx(1),ry(1),rd1,rd2)
c     --- crossing two smaller borders (left/right point )
      x02 = RegB(L1GEOINFx0)
      CALL l1geolc(x01,y01,dx1,dy1,x02,y02,dx2,dy2,
     &     rx(3),ry(3),rd1,rd2)
c     --- crossing of the innermost two borders (lower/upper point)
      x01 = x01 + RegA(L1GEOINFsx)
      CALL l1geolc(x01,y01,dx1,dy1,x02,y02,dx2,dy2,
     &     rx(2),ry(2),rd1,rd2)
c     --- crossing two larger borders (right/left point )
      x02 = x02 + RegB(L1GEOINFsx)
      CALL l1geolc(x01,y01,dx1,dy1,x02,y02,dx2,dy2,
     &     rx(4),ry(4),rd1,rd2)
c
      vmin = MIN(ry(1),ry(2))
      vmax = MAX(ry(1),ry(2))
c     --- limit to region sizes
      vmin = MAX(vmin,RegA(L1GEOINFy0),RegB(L1GEOINFy0))
      vmax = MIN(vmax,RegA(L1GEOINFy0)+RegA(L1GEOINFsy),
     &                RegB(L1GEOINFy0)+RegB(L1GEOINFsy))
c
      umin = MIN(rx(3),rx(4))  ! minimal maximal not takn height into account
      umax = MAX(rx(3),rx(4))
c
c
      x01 =RegA(L1GEOINFx0)
      CALL l1geolc(x01,y01,dx1,dy1,0.,vmin,1.,0.,    ! left of A with bottom
     &     rx(1),ry(1),rd1,rd2)
      CALL l1geolc(x01,y01,dx1,dy1,0.,vmax,1.,0.,    ! left of A with top
     &     rx(2),ry(2),rd1,rd2)
      x02 =RegB(L1GEOINFx0)
      CALL l1geolc(x02,y02,dx2,dy2,0.,vmin,1.,0.,    ! left of B with bottom
     &     rx(3),ry(3),rd1,rd2)
      CALL l1geolc(x02,y02,dx2,dy2,0.,vmax,1.,0.,    ! left of B with top
     &     rx(4),ry(4),rd1,rd2)
c
      umin = MAX(MIN(rx(1),rx(2)),MIN(rx(3),rx(4)),umin)
c
      x01 = RegA(L1GEOINFx0)+RegA(L1GEOINFsx)
      CALL l1geolc(x01,y01,dx1,dy1,0.,vmin,1.,0.,    ! right of A with bottom
     &     rx(1),ry(1),rd1,rd2)
      CALL l1geolc(x01,y01,dx1,dy1,0.,vmax,1.,0.,    ! right of A with top
     &     rx(2),ry(2),rd1,rd2)
      x02 = RegB(L1GEOINFx0)+RegB(L1GEOINFsx)
      CALL l1geolc(x02,y02,dx2,dy2,0.,vmin,1.,0.,    ! right of B with bottom
     &     rx(3),ry(3),rd1,rd2)
      CALL l1geolc(x02,y02,dx2,dy2,0.,vmax,1.,0.,    ! right of B with top
     &     rx(4),ry(4),rd1,rd2)
c
      umax = MIN(MAX(rx(1),rx(2)),MAX(rx(3),rx(4)),umax)
c
      IF ( umax.le.umin .or. vmax.le.vmin ) THEN
         OvLap(L1GEOINFx0) = -999999.
         OvLap(L1GEOINFsx) = -999999.
         OvLap(L1GEOINFy0) = -999999.
         OvLap(L1GEOINFsy) = -999999.
      ELSE
         OvLap(L1GEOINFx0) = umin
         OvLap(L1GEOINFsx) = umax - umin
         OvLap(L1GEOINFy0) = vmin
         OvLap(L1GEOINFsy) = vmax - vmin
      END IF
      OvLap(L1GEOINFsin_al) = 0.
      OvLap(L1GEOINFcos_al) = 1.
      OvLap(L1GEOINFz)  = RegA(L1GEOINFz)
c
      RETURN
c
      END
