*CMZ :  1.00/00 10/01/95  10.15.41  by  Dominik Ressing
*-- Author :    Dominik Ressing   27/07/94
************************************************************************
*
* L1POHIT
* written by Dominik Ressing, 27/07/94 17.18.03
*
* This routines action:
* returns hit numbers for track-hits. There may be up to max hits recorded.
*
* Arguments  kind         description
*          (in,update,out)
*-----------------------------------------------------------------------
* iDetSpe     in          detector specifier as defined in $l1detdf
* rPoint      in          point description as defined in $l1detdf
* MinHit      out         smallest number of found  hits ( 0 on error or not
*                                                          on detector plane )
* MaxHit      out         largest hit numberr found
*
************************************************************************
c
      SUBROUTINE L1POHIT(iDetSpe,rPoint,MinHit,MaxHit)
c
      IMPLICIT NONE
c
#include "gean/gcbank.fi"
#include "l1si/l1param.fi"
#include "l1si/l1geodf.fi"
#include "l1si/l1detdf.fi"
c
      INTEGER iDetSpe(L1DETSPElen),
     &        iDetInfo(L1DETINFlen),MaxHit,MinHit
      REAL    rPoint(L1POINTlen)
c
      INTEGER nHit,                  ! number of hits in a row (y=const)
     &        jDetGeo,               ! geometry of specified detector
     &        jCellGeo,              ! geometry of a detector cell
     &        jDetInfo               ! information about detector cells
      REAL    dz,                    ! z-distance of hit-point from detector plane
     &        xP,yP,                 ! track intersept with detector
     &        dx,dy,                 ! x,y in detector plane frame
     &        xx,xx1,xx2,            ! x-distance from origin
     &        rHit1,rHit2,           ! x-distance from origin in units of cells
     &        tol                    ! tolerance for cell calculus
c
      MinHit = 0 ! no hit in detector (so far)
      MaxHit = 0 ! no hit in detector (so far)
c
      CALL l1detif(iDetSpe,jDetGeo,jDetInfo,jCellGeo) ! get detector geometry
      if ( jDetGeo.le.0 ) GO TO 777                   ! no success
c
      dz = Q(jDetGeo+L1GEOINFz) - rPoint(L1POINTz0)  ! distance of detector from point
      xP = rPoint(L1POINTx0) + dz*rPoint(L1POINTdx)  ! track intersept with detector
      yP = rPoint(L1POINTy0) + dz*rPoint(L1POINTdy)  ! 
c
      tol = Q(jDetInfo+L1DETINFxspace)/100.   ! tolerance for calculations
      dx = xP - Q(jDetGeo+L1GEOINFx0)
      dy = yP - Q(jDetGeo+L1GEOINFy0)
      IF ( dy.lt.(-1.*tol) .or.
     &    (dy-Q(jDetGeo+L1GEOINFsy)).gt.tol ) RETURN ! y outside detector
      xx = dx - dy*Q(jDetGeo+L1GEOINFsin_al)/Q(jDetGeo+L1GEOINFcos_al)
      If (xx.lt.(-1.*tol) .or.
     &   (xx-Q(jDetGeo+L1GEOINFsx)).gt.tol ) RETURN ! x outside detector
c
      xx1 = xx - Q(jDetInfo+L1DETINFxlap)           ! take care of overlap
      xx2 = xx                                      ! central value is at lap/2.
c
      rHit1 = xx1 / Q(jDetInfo+L1DETINFxspace)           ! in units of driftcells
      MinHit = INT( rHit1 ) + IQ(jDetInfo+L1DETINFfirst) ! add hit offset
      rHit2 = xx2 / Q(jDetInfo+L1DETINFxspace)           ! in units of driftcells
      MaxHit = INT( rHit2 ) + IQ(jDetInfo+L1DETINFfirst) ! add hit offset
c
      nHit   = IQ(jDetInfo+L1DETINFnRow)           ! number of Hits in a row (y=const)
      MinHit = MAX(MinHit,IQ(jDetInfo+L1DETINFfirst))        ! limit HIt range
      MaxHit = MIN(MaxHit,nHit+IQ(jDetInfo+L1DETINFfirst)-1) ! maximal Hit
c
      IF ( MinHit.gt.MaxHit ) MaxHit = MinHit
c
      IF (IQ(jDetInfo+L1DETINFnCol).gt.1 ) THEN       ! pad-like detector
         MinHit = MinHit + INT(dy/Q(jDetInfo+L1DETINFyspace))*nHit
         MaxHit = MaxHit + INT(dy/Q(jDetInfo+L1DETINFyspace))*nHit
      END IF
c
      RETURN
c
 777  CONTINUE
c     --- error exit
      WRITE(L1LUNprn,900) iDetSpe(L1DETSPEdet),
     &                    iDetSpe(L1DETSPElayer),
     &                    iDetSpe(L1DETSPEsection),
     &                    MinHit,MaxHit
 900  FORMAT(1x,
     & 'L1POHIT ==> invalid detector iseq-det,layer,section,hits:'/
     & 10x,5i8)
c
      RETURN
c
      END
