//**********************************************************************/
// Created: 23.08.97,   Olya Igonkina   for   MARPLE package
//
// implementation of RcTraPoint class
//
//**********************************************************************/
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"

#include "marple/marple.hh"

RcTraPoint::RcTraPoint( const ArtePointer <RTRA>  rtraLink)
  : RcPoint(), fTraLink(rtraLink)  
{
  if( !(rtraLink != 0) || 
      rtraLink - *ArteTable<RTRA>::begin() > (int)ArteTable<RTRA>::size() )
    { 
      arteMsg << " MARPLE: RTRA entry number " << rtraLink 
	      << "does not exist" ;
      arteMsg.allp();
      arteMsg << "- empty object is created" ;
      arteMsg.allp();
      return;
    }
  setX(   rtraLink->xf  );
  setY(   rtraLink->yf  );
  setZ(   rtraLink->zf  );
  settx(  rtraLink->txf );
  setty(  rtraLink->tyf );
  setpe(  rtraLink->pf );
  setcov( rtraLink->cvf );

  checkAndCorrectCov();
}

// returns relation with MTRA 
const std::vector<ArtePointer<MTRA> > RcTraPoint::mtraList() const
{
  std::vector<ArtePointer<MTRA> > mList;
  if( ArteTable<MTRA>::size() == 0 ) return mList;
  for( ArteRelationIterator<MTRA>  rsMtra = link()->mtra.begin();
       rsMtra != link()->mtra.end();
       ++rsMtra )
    {
      mList.push_back(*rsMtra);
    }
  return mList;
}

// returns relation with HITB 
const std::vector<ArtePointer<HITB> > RcTraPoint::hitbList() const
{
  std::vector<ArtePointer<HITB> > hList;
  if( ArteTable<HITB>::size() == 0 ) return hList;
  for( ArteRelationIterator<HITB>  rsHitb = link()->hitb.begin();
       rsHitb != link()->hitb.end();
       ++rsHitb )
    {
      hList.push_back(*rsHitb);
    }
  return hList;
}

// update RTRA bank by RcTraPoint information
void RcTraPoint::updateARTE( int & iErr) const
{
  iErr = 0;
  ArtePointer<RTRA> rtra = link();

  static int idMess = 76576;
  arteMsg << "MARPLE:  updateARTE: RcTraPoint saved as a  RTRA";
  arteMsg.allp(idMess,1);
  
  rtra->xf  = x();
  rtra->yf  = y() ;
  rtra->zf  = z() ;
  rtra->txf = tx() ;
  rtra->tyf = ty();
  rtra->pf  = pe();
  for( int i=0; i<5; i++){
    for(int j=i; j<5; j++){
      rtra->cvf[c_matrix_pack[i][j]] = cov()[i][j]; 
      // start numbering from [0][0] as in struct RTRA
    }
  }
}

ostream & operator << (ostream & os, const RcTraPoint &rp)
{
  //os << static_cast<RcPoint&>(rp) ;
  const RcPoint * ptPoint = &rp;
  os << *ptPoint;
  os << "            RTRA link: " << rp.link() << endl ;
  return os ;
}
