//**********************************************************************/
// Created: 15.08.97  by   Olya Igonkina
//
// MARPLE initialization 
//
// fills the msGeslList with average Multiple Scattering for GESL 
//
// 9.10.01 - Add few empty superlayers in the middle 
//           used by RcPoint::prolongate to make small steps of rkmain. (OI)
//**********************************************************************/
#include <algo.h>

#include "marple/marple.hh"
#include "arte/ArteMessage.hh"
#include <gean/ZGESL.h>

std::vector<RcMsOfGesl> msGeslList;

//#include <algorithm>
bool comp_GESL_z (const RcMsOfGesl rms1, const RcMsOfGesl rms2)
{  return rms1.zcenter() < rms2.zcenter(); 
} 

MARPLETYPEOFMATCH maPresentMatch = MARPLENOMATCH;

HepDouble const MARPLE_INFINITY =  1048576;
HepDouble const MARPLE_NEGLIGIBLE =  1.e-30;

bool  MARPLE_MagnetON = 1; // by default magnet is ON
short MARPLE_DEBUG    = -1;   // by default MARPLE prints nothing on terminal 
short MARPLE_SOFT     = 0;     // use real cuts
short MARPLE_BKGR     = 0;     // allows to combine parts from different events
short MARPLE_MAKERSEG = 1;     //save match information into RSEG
float MARPLE_RISEID   = 50;   // RISE Id for momentum from match
float MARPLE_SEET     = 0.;  // threshold on ET of cluster for SVD-ECAL from rcevnt

// initialization itself

extern "C"
{
  void FORTRAN_ROUTINE( marpleinit)()
    {
      static int firstMarpleInit = 1;
      if( !firstMarpleInit )
	{
	  arteMsg << "MARPLE: marpleinit() is called twice";
	  arteMsg.allp();
	  return;
	}
      firstMarpleInit = 0;


      // initialization of Multiple Scattering array
      for( ArteTable<GESL>::iterator igesl = ArteTable<GESL>::begin();
	   igesl != ArteTable<GESL>::end(); ++igesl )
	{
	  if( (*igesl)->shp == 0 ) continue; // zero volume
	  RcMsOfGesl msEntry( *igesl);
	  msGeslList.push_back( msEntry);
	}
      sort(msGeslList.begin(),msGeslList.end(), comp_GESL_z); 
      
      // add fake superlayers if the magnet is empty 
      // to avoid large steps in rkmain
      // the code is not beautiful but it is called once per job only.

      
      float Zold = 0;
      msGeslList.reserve(msGeslList.size()+10); // maximum 10 new planes
      vector<RcMsOfGesl>::const_iterator imsend = msGeslList.end();
      for(vector<RcMsOfGesl>::iterator ims = msGeslList.begin();
	  ims != imsend; ++ims )
	{
	  if( ims->zcenter() < 200 ) {Zold = ims->zcenter(); continue;}
	  if( ims->zcenter() > 700 ) {Zold = ims->zcenter(); continue;} 
	  if(  ims->zcenter() - Zold > 80 )
	    {
	      // insert fake superlayers
	      int Nplanes = int( (ims->zcenter() - Zold)/50) ;
	      float dist = (ims->zcenter() - Zold) / (Nplanes+1);
	      for(int i = 1; i <= Nplanes; ++i )
		{
		  RcMsOfGesl msEntry(0,0,0,Zold + i*dist, 1);
		  msGeslList.push_back( msEntry);		  
		}
	    }
	  Zold = ims->zcenter();
	}
      // the code is not beautiful but it is called once per job only.
      sort(msGeslList.begin(),msGeslList.end(), comp_GESL_z); 
      

      if( MARPLE_DEBUG > 0 && MARPLE_DEBUG != 10 && MARPLE_DEBUG != 20 )
	{
	  for(vector<RcMsOfGesl>::iterator ims = msGeslList.begin();
	      ims != msGeslList.end(); ++ims )
	    cout << *ims;
	}
    }
  
}

int const  c_matrix_pack[5][5] = {
  {  0, 1, 3, 6,10},
  {  1, 2, 4, 7,11},
  {  3, 4, 5, 8,12},
  {  6, 7, 8, 9,13},
  { 10,11,12,13,14}
};

int const  fortran_matrix_pack[5][5] = {
  {  1, 2, 4, 7,11},
  {  2, 3, 5, 8,12},
  {  4, 5, 6, 9,13},
  {  7, 8, 9,10,14},
  { 11,12,13,14,15}
};

int const  c_rotation_matrix[3][3] = {
  {  0, 1, 2 },
  {  3, 4, 5 },
  {  6, 7, 8 }
};

int const  fortran_rotation_matrix[3][3] = {
  {  1, 2, 3 },
  {  4, 5, 6 },
  {  7, 8, 9 }
};

