
// Boris Fominykh 6.05.2002
// allignment correction with data base /MUON_NEW

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <math.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbAlignNew.h"

extern "C" {
  void MuNede2Gede();
  int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
}

extern "C" {
int MuAlignmentNew(int KTstatus,int major,int minor)
{
  MuNede2Gede();
  MuDbAlignNew Align;
  Align.Ini();
  Align.setMajor(major);
  Align.setMinor(minor);
  Align.setKTstatus(KTstatus);

// read alignment constants

  int ret=Align.Read();
  if(ret != 0) 
  {
    arteMsg << "MuAlignmentNew: alignment constants are not read, ret= " << ret << endl;
    arteMsg.error();
    return -1;
  }

// apply the corrections, loops over GESL, GEDE
  int par[4];  
  float dx = 0.;
  float dy = 0.;
  float dz = 0.;
  float xc = 0.;
  float yc = 0.;
  float angl = 0.;
  for ( ArteTableIterator<GESL> isl = ArteTable<GESL>::begin(); 
        isl != ArteTable<GESL>::end(); ++isl ) 
  {
    int cmp  = (*isl)->cmp;
// only muon superlayers
    if(cmp == 8 || cmp == 9) 
    {
      int msl  = (*isl)->nsl;  // # muon superlayer
      int iv1 = (*isl)->fpl;
      int iv2 = iv1 + (*isl)->npl - 1;
      if(iv2 >= iv1)
      {
// loops over muon gede

        // cout << "MuAlignmentNew: msl " << msl << " cmp " << cmp;
        // cout << " iv1 " << iv1 << " iv2 " << iv2 << endl;

        for ( int iv=iv1; iv<=iv2; iv++) 
        { 
          ArtePointer<GEDE> gede(iv);
// chamber name
          char name[10];
          strncpy(name,gede->hwc,9);
// decode the chamber name
          int type  = 0;
          int msld  = 0;
          int lay  = 0;
          int ud   = 0;
          int nx    = 0;
          int ncab  = 0;
          int ret=MuNameDecode(&name[0],&type,&msld,&lay,&ud,&nx,&ncab);
          if(ret == 0 && msl == msld) 
          {
            par[0] = msl; 
            par[1] = lay; 
            if(type == 4) par[1] = 4; 
            par[2] = ud; 
            par[3] = nx; 
            const MuAlignNew* cont = Align.GetPar(par);
            if(cont == NULL)
	    {
              arteMsg << "MuAlignmentNew: NO alignment parameters for msl " << par[0];
              arteMsg << " lay " << par[1] << " ud " << par[2];
              arteMsg << " nx " << par[3] << endl;
              arteMsg.warn();
              return -2;
            }

// get parameters
            dx   = cont->dx;
            dy   = cont->dy;
            dz   = cont->dz;
            xc   = cont->xc;
            yc   = cont->yc;
            angl = cont->angl;

            // cout << " MuAlignmentNew: " << name << " sl " << par[0] << " lay " << par[1];
            // cout << " ud " << par[2] << " nx " << par[3];
            // cout << " dx= " << dx << " dy= " << dy << " dz= " << dz;
            // cout << " xc " << xc << " yc " << yc << " angl " << angl << endl;

// apply corrections
// update the angle
            float anglch = atan2(gede->sin,gede->cos);
            float anglc= anglch + angl/57.29578;
            gede->sin = sin(anglc);
            gede->cos = cos(anglc);

// update the coordinates
            float dxc = dx*(gede->cos) - dy*(gede->sin);
            gede->xmin += dxc;
            gede->sxmn += dxc;
            gede->xmax += dxc;
            gede->sxmx += dxc;

            float dyc = dx*(gede->sin) + dy*(gede->cos);
            gede->ymin += dyc;
            gede->symn += dyc;
            gede->ymax += dyc;
            gede->symx += dyc;

            gede->zmin += dz;
            gede->zmax += dz;
          }
        }
      }
    }
  }  
  Align.Clear();

  if(KTstatus > 0)  return 1;
  else              return 2;
}
}


