
// Boris Fominykh  6.05.2002
// member functions of class MuDbEffPad

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <stdlib.h>
#include <fstream.h>
#include <iostream.h>
#include <iomanip.h>

#include "arte/arte.hh"
#include "arte/ArteMessage.hh"

#include "murec/MuDbEffPad.h"

extern "C" 
{
  int MuNameDecode(char*,int*,int*,int*,int*,int*,int*);
  int MuSetMajor(int,int);
  int MuSetMinor(int,int);
}

//** Constructor */
MuDbEffPad::MuDbEffPad()
  :  MuDbTable::MuDbTable()
{
}

//** Destructor */
MuDbEffPad::~MuDbEffPad()
{
  Clear();
}

//## Methods

//** Initialization */
void MuDbEffPad::Ini(int rormc)
{
  MuDbTable::setDBname("/MUON_NEW");

  RorMC = rormc;
  if(RorMC == 0)
  {
    MuDbTable::setName("MuonEffPad");
    MuDbTable::setTitle("MuonEffPad");
  }
  else
  {
    MuDbTable::setName("MuonGeoEffPad");
    MuDbTable::setTitle("MuonGeoEffPad");
  }

  MuDbTable::setDesc("UB:msl UB:ud UB:nx1 UB:nx2 UB:row1 UB:row2 UB:pad1 UB:pad2 F:eff");

  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
      for(int ud=0; ud<UdM; ud++)
      {
          Ncont[sl][ud] = 0;
      }  
  }  
  return;
}

void MuDbEffPad::Clear()
{
  Valid = -1;
  for(int sl=0; sl<SlM; sl++)
  {
      for(int ud=0; ud<UdM; ud++)
      {
          if(Ncont[sl][ud] > 0) Cont[sl][ud].clear();
          Ncont[sl][ud] = 0;
      }  
  }  
  return;
}


//** Read the table as it's defined in kumac*/
int MuDbEffPad::Read()
{
  int ret = 0;
  int check = 0;

// read the table from the file
// the case of measured efficiency
  if(RorMC == 0)
  {
    if(getenv("MUON_EFF_PAD") != NULL)
    {
      MuDbTable::setFile(getenv("MUON_EFF_PAD"));
      ret=MuDbEffPad::ReadFile();
      if(ret == 0) check=1;
    }
  }

// the case of MC efficiency
  if(RorMC == 1)
  {
    if(getenv("MUON_GEO_EFF_PAD") != NULL)
    {
      MuDbTable::setFile(getenv("MUON_GEO_EFF_PAD"));
      ret=MuDbEffPad::ReadFile();
      if(ret == 0) check=1;
    }
  }

  if(check == 0) ret=MuDbEffPad::ReadDB();

  return ret;
}

 
//** Read the table from DB*/
int MuDbEffPad::ReadDB()
{
  int ret=0;
  int nel=0;

// set major, minor
  int ipr = 0;
  int major = MuSetMajor(4+RorMC,ipr);
  int minor = MuSetMinor(4+RorMC,ipr);

  MuDbTable::setKTstatus(-2);
  MuDbTable::setMajor(major);
  MuDbTable::setMinor(minor);

// open DB
  ret=MuDbTable::Open(0);
  // cout << "MuDbEffPad::ReadDB: db is open, ret= " << ret << endl;
  if(ret > 0) return ret;

// fetch the table
  ret=MuDbTable::Fetch();
  // cout << "MuDbEffPad::ReadDB: db is fetched, ret= " << ret << " Size " << Size << endl;
  if(ret != 0)
  {
    MuDbTable::Close();
    return ret;
  }

// read the table fields 
  char* msl   = new char[Size];
  char* ud    = new char[Size];
  char* nx1   = new char[Size];
  char* nx2   = new char[Size];
  char* row1  = new char[Size];
  char* row2  = new char[Size];
  char* pad1  = new char[Size];
  char* pad2  = new char[Size];
  float* eff  = new float[Size];

  MuEffPad con;

  ret=31;
  Nf=SkipTableField(X,1);
  nel = GetTableField(X,TABLEubyte,Size,msl);
  // cout << "MuDbEffPad::ReadDB: field 1 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,2);
  nel = GetTableField(X,TABLEubyte,Size,ud);
  // cout << "MuDbEffPad::ReadDB: field 2 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,3);
  nel = GetTableField(X,TABLEubyte,Size,nx1);
  // cout << "MuDbEffPad::ReadDB: field 3 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,4);
  nel = GetTableField(X,TABLEubyte,Size,nx2);
  // cout << "MuDbEffPad::ReadDB: field 4 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,5);
  nel = GetTableField(X,TABLEubyte,Size,row1);
  // cout << "MuDbEffPad::ReadDB: field 5 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,6);
  nel = GetTableField(X,TABLEubyte,Size,row2);
  // cout << "MuDbEffPad::ReadDB: field 6 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,7);
  nel = GetTableField(X,TABLEubyte,Size,pad1);
  // cout << "MuDbEffPad::ReadDB: field 7 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,8);
  nel = GetTableField(X,TABLEubyte,Size,pad2);
  // cout << "MuDbEffPad::ReadDB: field 8 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  Nf=SkipTableField(X,9);
  nel = GetTableField(X,TABLEfloat,Size,eff);
  // cout << "MuDbEffPad::ReadDB: field 9 is read nel= " << nel << endl;
  if(nel <= 0) goto labelexit;

  // cout << "MuDbEffPad::ReadDB: table is read, Size= " << Size << endl; 

// fill set of containers
  for(int i=0; i<Size; i++)
  {
    if(*(msl+i) > 0 && *(msl+i) < 5 && *(ud+i) < 2)
    {
      con.IDnum = i;  
      con.sl    = *(msl+i);  
      con.ud    = *(ud+i);  
      con.nx1   = *(nx1+i);
      con.nx2   = *(nx2+i);
      con.r1    = *(row1+i);
      con.r2    = *(row2+i);
      con.pd1   = *(pad1+i);
      con.pd2   = *(pad2+i);
      con.key   = *(nx1+i) * 60 + *(row1+i) * 30 + *(pad1+i);
      con.eff   = *(eff+i);
      con.Valid = 1;

      // con.Print();

      int isl  = con.sl  - 3;
      int iud  = con.ud;
      Cont[isl][iud].insert(con);
      Ncont[isl][iud] += 1;
    }
  }
  Valid = 1;
  ret = 0;
  arteMsg << DBtable << " from " << DBname << " is initialized";
  arteMsg.warn();

labelexit:
  delete []msl;
  delete []ud;
  delete []nx1;
  delete []nx2;
  delete []row1;
  delete []row2;
  delete []pad1;
  delete []pad2;
  delete []eff;

// close DB;
  MuDbTable::Close();

  return ret;
}


//** Read the table from ASCII file*/
int MuDbEffPad::ReadFile()
{
  FILE *hl = fopen(FileName, "r");
  if(!hl) 
  {
    arteMsg << "MuDbEffPad::ReadFile - no input file " << FileName << endl;
    arteMsg.error();
    return 1;
  }

  char numb[11] = "0123456789";
  MuEffPad A;
  int count=0;

// loops over lines: max 5000 lines
  int nline=0;
  char line[121];
  for(int il=0; il<5000; il++) 
  {
// read the line
    if(fgets(line,sizeof(line),hl) == NULL) 
    {
      fclose(hl);
      goto goodexit;

      // if(feof(hl)) goto goodexit;
      // else 
      // { 
      //   arteMsg << "MuDbEffPad:::ReadFile -  error of file read";
      //   arteMsg.warn();
      //   return 2;
      // }
    }
    ++nline;

    // cout << nline << "  " << line << endl;

    char word[121];
    int  nx1  = 0;
    int  nx2  = 0;
    int  r1   = 0;
    int  r2   = 0;
    int  pd1  = 0;
    int  pd2  = 0;
    float eff = 0.;

// decord the line
    int nitem=sscanf(line,"%s nx %d %d row %d %d pads %d %d eff %f",
                           word,&nx1,&nx2,&r1,&r2,&pd1,&pd2,&eff);

// pass  commented cards
    if(word[0]=='#' || word[0]=='*' || word[0]=='/') continue;

// end of file
    if(!strcmp(word,"END")) 
    {
      fclose(hl);
      goto goodexit;
    }

// pass  blank lines
    if(nitem != 8) continue;

// control 'Mp' and '.' in the first word
    if(word[0] == 'M' && word[1] == 'p' && word[3] == '.') 
    {
      // cout << nline << " " << count << "  "; 
      // cout << word << "  nx1 " << nx1 << " nx2 " << nx2;
      // cout << " r1 " << r1 << " r2 " << r2;
      // cout << " pd1 " << pd1 << " pd2 " << pd2 << " eff " << eff << endl;

      A.Ini();

      int msl  = 0;
      int ud   = 0;

// decode msl
      for(int i=0; i<10; i++) 
      {
        if(word[2] == numb[i])
        {
          msl = i;
          break;
        }
      }

// decode ud
      if(msl > 2 && msl < 5)
      {
        for(int i=0; i<10; i++) 
        {
          if(word[4] == numb[i])
          {
            ud = i-1;
            break;
          }
        }
        if(ud >= 0 && ud < 2)
        {

   	  // cout << " msl " << msl << " ud " << ud;
          // cout << " nx1 " << nx1 << " nx2 " << nx2;
          // cout << " r1 " << r1 << " r2 " << r2;
          // cout << " pd1 " << pd1 << " pd2 " << pd2 << endl;

// fill container
          A.IDnum = count;
          count += 1; 
          A.sl   = msl;
          A.ud   = ud;
          A.nx1  = nx1;
          A.nx2  = nx2;
          A.r1   = r1;
          A.r2   = r2;
          A.pd1  = pd1;
          A.pd2  = pd2;
          A.key  = nx1*60 + r1*30 + pd1;

          A.eff  = eff;
          A.Valid = 1;

          // A.Print();

          Cont[msl-3][ud].insert(A);
          Ncont[msl-3][ud] += 1;
        } 
      } 
    } 
  } 
goodexit:
  if(count > 0) 
  {
    Valid = 1;
    arteMsg << DBtable << " initialized from file";
    arteMsg.warn();
    cout << FileName << endl;
    return 0;
  }
  else return 3;
}


//** Print the object content */
void MuDbEffPad::Print()
{
// check print control in kumac
  if(getenv("MUON_EFF_PAD_PRINT") == NULL) return;
  if(strcmp(getenv("MUON_EFF_PAD_PRINT"),"YES")) return;

  char fname[20];
  if(RorMC == 0) strncpy(fname,"MuEffPad.out",19);
  if(RorMC == 1) strncpy(fname,"MuGeoEffPad.out",19);

  // cout << "MuDbEffPad::Print is called, RorMC= " << RorMC;
  // cout << " file " << fname << endl;

  ofstream fout(fname);
  if (!fout) 
  {
    arteMsg << "MuDbEffPad::Print - can not open file " << fname << " RorMC= " << RorMC;
    arteMsg.warn();
    return;
  }

  fout << "  " << endl;
  fout << "DataBase: " << DBname << " Table: " << DBtable;
  fout << " Title: <" << Title << ">" << endl;
  fout << "Desc: <" << Desc << ">" << endl;
  fout << "Size= " << Size;
  fout << " Major " << Major << " Minor " << Minor << endl;

// loops over sl, ud, 
  for(int sl=0; sl<SlM; sl++)
  { 
    for(int ud=0; ud<UdM; ud++)
    { 
      fout << " " << endl;
      fout << " Sl " << sl+3 << " Ud " << ud;
      fout << " Ncontainers= " << Ncont[sl][ud] << endl;
      if(Ncont[sl][ud] > 0)
      {
        int count =0;
        set<MuEffPad>::iterator c1 = Cont[sl][ud].begin();
        set<MuEffPad>::iterator c2 = Cont[sl][ud].end();
        for(set<MuEffPad>::iterator i=c1; i != c2; i++)
	{
          count += 1;
          fout << "  " << count << "  IDnum " << (*i).IDnum << " sl " << (*i).sl;
          fout << " ud " << (*i).ud;
          fout << " nx " << (*i).nx1 << " " << (*i).nx2;
          fout << " row " << (*i).r1 << " " << (*i).r2;
          fout << " pads " << (*i).pd1 << " " << (*i).pd2;
          fout << " eff " << (*i).eff;
          fout << " Valid " << (*i).Valid << endl;
          fout << " " << endl;
        }
      }
    }
  }
  return;
}


// Get the efficiency */
const float MuDbEffPad::GetEff(int iv,int nrow,int npad)
{
  ArtePointer<GEDE> gede(iv);
// chamber name
  char name[10];
  strncpy(name,gede->hwc,9);
// decode the chamber name
  int type  = 0;
  int sl   = 0;
  int lay  = 0;
  int ud   = 0;
  int nx    = 0;
  int ncab  = 0;
  int ret=MuNameDecode(&name[0],&type,&sl,&lay,&ud,&nx,&ncab);
  if(ret != 0 || sl<3 || sl>4 || ud<0 || ud>1) 
  {
    arteMsg << "MuDbEffPad::GetPar - wrong request iv= " << iv;
    arteMsg.warn();
    return -1.;
  }

  // cout << "MuDbEffPad::GetEff: iv " << iv << " sl " << sl << " ud " << ud;
  // cout << " nx " << nx << " nrow " << nrow << " npad " << npad << endl;

  if(Ncont[sl-3][ud] > 0)
  {
    set<MuEffPad>::iterator c1 = Cont[sl-3][ud].begin();
    set<MuEffPad>::iterator c2 = Cont[sl-3][ud].end();
    for(set<MuEffPad>::iterator i=c1; i != c2; i++)
    {
      if(nx >= (*i).nx1 && nx <= (*i).nx2 && nrow >= (*i).r1 && nrow <= (*i).r2 &&
         npad >= (*i).pd1 && npad <= (*i).pd2)  return (*i).eff;
    }
  }
  arteMsg << "MuDbEffPad::GetEff - NO eff for gede " << iv << " nrow " << nrow;
  arteMsg << " npad " << npad << " sl " << sl << " ud " << ud << " nx " << nx;
  arteMsg.warn();
  return -1.;
}



