/****************************************************************************
* MuMCPadHit2Digi.C
* first author : Oxana Barsoukova
* purpose      : Infill of DMUT table for wires and DMUP for pads
* called from  : dgmuoo.F
* comment      : function mumcpadhit2digi(...) gets ipl - number of gede
*                plane and chid - number of wire in gede (total number is 16) 
*              : function mumcpad2hit2digi(...) gets also row number of
*                pads - ir
*****************************************************************************/

/********************************************
    Arrangement of connectors for pads

                    ^ y
		    |
        row1 row0   |   row1 row0
	---------   |   ---------
	| 1 | 3 |   |   | 3 | 1 |
	---------   |   ---------
	| 0 | 2 |   |   | 2 | 0 |
	---------   |   ---------
                    |
    <---------------+------------------
    x               |
        ---------   |   ---------
	| 0 | 2 |   |   | 2 | 0 |
	---------   |   ---------
	| 1 | 3 |   |   | 3 | 1 |
	---------   |   ---------
                    |

 ********************************************/

#include <stdio.h>
#include "etb.h"

#include "arte/arte.hh"
#include "arte/ArteTable.hh"
#include "arte/ArteMessage.hh"
#include "murec/MuDbPadArte.h"


extern MuDbPadArte   Pad;

extern "C"
{
  int acReadMuon(int);
  void mmdrop_(int &);
  void mmbook_(int &, const int &);
  int CheckEndian();
  void FillCharArr(LWord, char*);
}


extern "C" void mumcpadhit2digi_(int& ipl, int& chid)
{
  static int OnceCall = 1;
  static Word CratesNumberP = 0;
  static Word CratesIdP[30];
  
  LWord HexConst = 0x1;
  LWord mask = 0;

  char  mudata[4];
  LWord HeaderStr = 0;
  LWord NWord = 0;
  LWord* TableWordRef = 0;
  LWord TableWord = 0;
  LWord nbit = 0;
  LWord nbit1 = 0;

  if (OnceCall)
    {
      OnceCall = 0;
      
// check that the object is initialized
      if(Pad.Ivalid != 1) acReadMuon(0);
      if(Pad.Ivalid != 1) 
      {
        arteMsg << "MuMCPadHit2Digi: pads are not initialized";
        arteMsg.error();
        return;
      }

      Word CrateIdP_ = 0;
      
      for (int ii = 0; ii < 30; ii++)
	CratesIdP[ii] = CrateIdP_;
      
      for(Word sl=0; sl<Pad.PadGeoSl; sl++)
	for(Word udt=0; udt<Pad.PadGeoUpDown; udt++)
	  {
	    Word ud = Pad.PadGeoUpDown - 1 - udt; // ud=0 => ud=1
	    for(Word nx=0; nx<Pad.NchambP[sl][ud]; nx++) // NchambP - number of chambers
	      {
		int found = 0;
		
		CrateIdP_ = Pad.CrateP[sl][ud][nx]; // CrateP - crate's name
		
		for (int ii = 0; ii < 30; ii++)
		  if (CrateIdP_ != 0 && CrateIdP_ == CratesIdP[ii])
		    {
		      found = 1;
		      break;
		    }
		
		if(!found && CrateIdP_ != 0 && CrateIdP_ != CratesIdP[CratesNumberP])
		  {
		    CratesIdP[CratesNumberP] = CrateIdP_;
		    CratesNumberP++;
		  }
	      }
	  }
      
      //      mmdrop_(ArtePointer<DMUT>::getLink().getRef());

    } // end of OnceCall


  // Pad wires served by 6 crates, but for Tube wires it is 12 =>
  //                       reservation of DMUT size for 18 crates
  if (!ArteTable<DMUT>::booked())
    mmbook_(ArtePointer<DMUT>::getLink().getRef(), 18*34);

  // HeaderStr for Pads shifted on 12 Tube crates - they are first in DMUT
  for (LWord CrN = 0; CrN < CratesNumberP; CrN++)
    {
      HeaderStr = 2 + 34*(CrN + 12);
      FillCharArr(CratesIdP[CrN], mudata);
      ArtePointer<DMUT>(HeaderStr)->bits = mudata;
    }


  ArtePointer<GEDE> gede(ipl);
  // Switch to 0 and 1 sl numbers
  Word sl = gede->gesl->nsl - 3;
  Word channel = chid - 1;

  for(Word udt=0; udt<Pad.PadGeoUpDown; udt++)
    {
      Word ud = Pad.PadGeoUpDown - 1 - udt; // ud=0 => ud=1
      
      for(Word sd=0; sd<Pad.PadGeoSd; sd++)
	for(Word nx=0; nx<Pad.NchambP[sl][ud]; nx++)
	  if (ipl == int(Pad.PadGede[sl][ud][nx][sd]))
	    for (LWord CrN = 0; CrN < CratesNumberP; CrN++)
	      if (CratesIdP[CrN] == Pad.CrateP[sl][ud][nx])
		{
		  Word wire = (channel>=8) ? (channel-8) : channel;
		  
		  // Connectors of down modules put in order 0, 1 by X axis;
		  // Connectors of upper modules in order 1, 0
		  Word nc = 0;
		  if ((channel<= 7 && ud == 1) || (channel > 7 && ud == 0))
		    nc = 0;
		  if ((channel > 7 && ud == 1) || (channel <= 7 && ud == 0))
		    nc = 1;
		  
		  nbit1 = Pad.Nbit1P[sl][ud][nx][nc][sd];
		  int step = Pad.StepP[sl][ud][nx];
		  int astp = (step<0)? -step : step;
		  
		  // Wires in upper modules are swapped inside each connector
		  if (ud == 0) wire = 7 - wire;
		  nbit = nbit1 + astp*wire;
		  
		
		  LWord nwd = nbit/32;
		  LWord nbt1 = nbit - 32*nwd;
		  nbit = 32*nwd + 31 - nbt1;
		  
		  HeaderStr = 2 + 34*(CrN + 12);
		  NWord = HeaderStr + nbit/32 + 1; // Number of word
		  TableWordRef = (LWord*)(ArtePointer<DMUT>(NWord+1)->bits);
		  
		  if (CheckEndian() == 1)
		    TableWord = *TableWordRef;
		  else if (CheckEndian() == 2)
		    TableWord = ESP_SWAP_INT(*TableWordRef);
		  
		  Word nw = nbit/32;
		  Word nbtt = nbit - 32*nw;
		  Word nbyte = nbtt/8;
		  Word nbb = nbtt - 8*nbyte;
		  Word nb = 7 - nbb;
		  Word nbt = 8*nbyte+nb;
		  
		  mask = HexConst << nbt;
		  TableWord = TableWord | mask;
		  
		  FillCharArr(TableWord, mudata);
		  ArtePointer<DMUT>(NWord+1)->bits = mudata;
		}
    }
  return;
} 


extern "C" void mumcpad2hit2digi_(int& ipl, int& ir, int& chid)
{
  static int OnceCall = 1;
  static Word CratesNumberPad = 0;
  static Word CratesIdPad[30];
  
  LWord HexConst = 0x1;
  LWord mask = 0;

  char  mudata[4];
  LWord HeaderStr = 0;
  LWord NWord = 0;
  LWord* TableWordRef = 0;
  LWord TableWord = 0;
  LWord nbit = 0;
  LWord nbit1 = 0;

  if (OnceCall)
    {
      OnceCall = 0;
      
// check that the object is initialized
      if(Pad.Ivalid != 1) acReadMuon(0);
      if(Pad.Ivalid != 1) 
      {
        arteMsg << "MuMCPadHit2Digi: pads are not initialized";
        arteMsg.error();
        return;
      }

      Word CrateIdPad_ = 0;
      
      for (int ii = 0; ii < 30; ii++)
	CratesIdPad[ii] = CrateIdPad_;
      
      for(Word sl=0; sl<Pad.PadGeoSl; sl++)
	for(Word udt=0; udt<Pad.PadGeoUpDown; udt++)
	  {
	    Word ud = Pad.PadGeoUpDown - 1 - udt; // ud=0 => ud=1
	    for(Word nx=0; nx<Pad.NchambP[sl][ud]; nx++) // NchambP - number of chambers
	      {
		int found = 0;
		
		CrateIdPad_ = Pad.CratePP[sl][ud][nx]; // CratePP - crate's name for pads
		
		for (int ii = 0; ii < 30; ii++)
		  if (CrateIdPad_ != 0 && CrateIdPad_ == CratesIdPad[ii])
		    {
		      found = 1;
		      break;
		    }
		
		if(!found && CrateIdPad_ != 0 && CrateIdPad_ != CratesIdPad[CratesNumberPad])
		  {
		    CratesIdPad[CratesNumberPad] = CrateIdPad_;
		    CratesNumberPad++;
		  }
	      }
	  }
      
      mmdrop_(ArtePointer<DMUP>::getLink().getRef());

    } // end of OnceCall

  
  if (!ArteTable<DMUP>::booked())
    mmbook_(ArtePointer<DMUP>::getLink().getRef(), CratesNumberPad*34);
  
  for (LWord CrN = 0; CrN < CratesNumberPad; CrN++)
    {
      HeaderStr = 2 + 34*CrN;
      FillCharArr(CratesIdPad[CrN], mudata);
      ArtePointer<DMUP>(HeaderStr)->bits = mudata;
    }


  ArtePointer<GEDE> gede(ipl);
  // Switch to 0 and 1 sl numbers
  Word sl = gede->gesl->nsl - 3;
  Word irow = ir - 1;
  Word channel = chid - 1;


  for(Word udt=0; udt<Pad.PadGeoUpDown; udt++)
    {
      Word ud = Pad.PadGeoUpDown - 1 - udt; // ud=0 => ud=1
      
      for(Word sd=0; sd<Pad.PadGeoSd; sd++)
	for(Word nx=0; nx<Pad.NchambP[sl][ud]; nx++)
	  if (ipl == int(Pad.PadGede[sl][ud][nx][sd]))
	    for (LWord CrN = 0; CrN < CratesNumberPad; CrN++)
	      if (CratesIdPad[CrN] == Pad.CratePP[sl][ud][nx])
		{
		  channel = gede->nwi[2] - 1 - channel;

		  Word nc = 0;

		  // Chamber 99: A/C = 1/0
		  if (ud == 1 && nx == Pad.NchambP[sl][ud] - 1)
		    {
		      if (irow == 1)
			nc = 0;
		      if (irow == 0)
			nc = 1;
		    }
		  else
		    {
		      // Connectors have different number of pads, for instance: 15, 7 or 4
		      if ((channel <= (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 0 && irow == 0 && nx <= 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 0 && irow == 1 && nx > 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 1 && irow == 0 && nx <= 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 1 && irow == 1 && nx > 17))
			nc = 0;
		      if ((channel > (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 0 && irow == 0 && nx <= 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 0 && irow == 1 && nx > 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 1 && irow == 0 && nx <= 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][0] - 1) && ud == 1 && irow == 1 && nx > 17))
			nc = 1;
		      if ((channel <= (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 0 && irow == 1 && nx <= 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 0 && irow == 0 && nx > 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 1 && irow == 1 && nx <= 17) ||
			  (channel <= (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 1 && irow == 0 && nx > 17))
			nc = 2;
		      if ((channel > (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 0 && irow == 1 && nx <= 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 0 && irow == 0 && nx > 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 1 && irow == 1 && nx <= 17) ||
			  (channel > (Pad.NumbP[sl][ud][nx][2] - 1) && ud == 1 && irow == 0 && nx > 17))
			nc = 3;
		    }
		  

		  nbit1 = Pad.Nbit1PP[sl][ud][nx][nc];
		  
		  // To count pads inside each connector with zero
		  Word wire = channel;
		  if (nc == 1 && nx != 33)
		    wire = channel - Pad.NumbP[sl][ud][nx][0];
		  if (nc == 3)
		    wire = channel - Pad.NumbP[sl][ud][nx][2];
		  nbit = nbit1 + wire;
		  

		  HeaderStr = 2 + 34*CrN;
		  NWord = HeaderStr + nbit/32 + 1; // Number of word
		  TableWordRef = (LWord*)(ArtePointer<DMUP>(NWord+1)->bits);
		  
		  if (CheckEndian() == 1)
		    TableWord = *TableWordRef;
		  else if (CheckEndian() == 2)
		    TableWord = ESP_SWAP_INT(*TableWordRef);
		  
		  Word nw = nbit/32;
		  Word nbtt = nbit - 32*nw;
		  Word nbyte = nbtt/8;
		  Word nbb = nbtt - 8*nbyte;
		  Word nb = 7 - nbb;
		  Word nbt = 8*nbyte+nb;
		  
		  mask = HexConst << nbt;
		  TableWord = TableWord | mask;

		  FillCharArr(TableWord, mudata);
		  ArtePointer<DMUP>(NWord+1)->bits = mudata;
		}
    }
  return;
}





