/************************************************************************
* MuMCPixHit2Digi.C
* first author : Oxana Barsoukova
* purpose      : Infill of DMUX table
* called from  : DGMUOI.C
* comments     : function gets number of hitted read out channel
************************************************************************/

#include <stdio.h>
#include "etb.h"

#include "arte/arte.hh"
#include "arte/ArteTable.hh"
#include "arte/ArteMessage.hh"
#include "murec/MuDbPixelArte.h"

#include <netinet/in.h>

extern MuDbPixelArte Pix;
extern MuPixChamber  PixCh;


extern "C"
{
  int acReadMuon(int);
  void mmdrop_(int &);
  void mmbook_(int &, const int &);
}


extern "C" void FillCharArr(LWord int16, char out[])
{
  out[0]=(int16>>24)&0xFF;
  out[1]=(int16>>16)&0xFF;
  out[2]=(int16>>8)&0xFF;
  out[3]=int16&0xFF;
  
  return;
}

extern "C" int CheckEndian()
{
  int BigEndian = -1;
  LWord check_end_x = 0x01020304;
  LWord check_end_y = 0;
  
  // convert values between host and network byte order      
  check_end_y = htonl(check_end_x);
  
  if (check_end_y == check_end_x)
    BigEndian = 1; // Big Endian format
  else if (check_end_y == ESP_SWAP_INT(check_end_x))
    BigEndian = 2; // Little Endian format
  else
    BigEndian = 0;
  
  return BigEndian;
}


extern "C" void MuMCPixHit2Digi(int slay, int qu, int nx, int ny)
{
  static int OnceCall = 1;
  static Word CratesNumber = 0;
  static Word CratesId[10];


  LWord HexConst = 0x1;
  LWord mask = 0;

  char  mudata[4];
  LWord HeaderStr = 0;
  LWord NWord = 0;
  LWord* TableWordRef = 0;
  LWord TableWord = 0;
  LWord nbit = 0;
  LWord nbit1 = 0;

  if (OnceCall)
    {
      OnceCall = 0;

// check that the object is initialized
      if(Pix.Ivalid != 1) acReadMuon(0);
      if(Pix.Ivalid != 1) 
      {
	arteMsg << "MuMCPixHit2Digi: pixels are not initialized";
        arteMsg.error();
        return;
      }

      Word CrateId_ = 0;

      for (int ii = 0; ii < 10; ii++)
	CratesId[ii] = CrateId_;

      for(Word sl=0; sl<Pix.PixGeoSl; sl++)
	{
	  for(Word qu=0; qu<Pix.PixGeoQu; qu++)
	    {
	      for(Word nc=0; nc<Pix.PixNcon; nc++)
		{
		  int found = 0;
		  CrateId_ = Pix.CrateX[sl][qu][nc];

		  for (int ii = 0; ii<10; ii++)
		    if (CrateId_ != 0 && CrateId_ == CratesId[ii])
		      {
			found = 1;
			break;
		      }
		    
		  if(!found && CrateId_ != 0 && CrateId_ != CratesId[CratesNumber])
		    {
		      CratesId[CratesNumber] = CrateId_;
		      CratesNumber++;
		      break;
		    }
		}
	    }
	}

      mmdrop_(ArtePointer<DMUX>::getLink().getRef());
      
    } // end of OnceCall
  
  if (!ArteTable<DMUX>::booked())
    mmbook_(ArtePointer<DMUX>::getLink().getRef(), CratesNumber*34);
  
  for (LWord CrN = 0; CrN < CratesNumber; CrN++)
    {
      HeaderStr = 2 + 34*CrN;
      FillCharArr(CratesId[CrN], mudata);
      ArtePointer<DMUX>(HeaderStr)->bits = mudata;
    }

  
  for (Word nc=0; nc<Pix.PixNcon; nc++)
    for (Word nbt=0; nbt<PixCh.Nbit; nbt++)
      if ((PixCh.Pnx1[slay][nc][nbt] == nx+1) && (PixCh.Pny1[slay][nc][nbt] == ny+1))
	for (LWord CrN = 0; CrN < CratesNumber; CrN++)
	  if (CratesId[CrN] == Pix.CrateX[slay][qu][nc])
	    {
	      nbit1 = LWord(abs(Pix.Nbit1X[slay][qu][nc]));

	      nbit = nbit1 + (15 - nbt);
	      if (Pix.Nbit1X[slay][qu][nc] < 0)
		nbit = nbit1 - (15 - nbt);

	      /*	      
	      printf("\nMuMCPixHit2Digi.C: PixCh.Pnx1=%d, PixCh.Pny1=%d, sl=%d, qu=%d, nc=%d, crate=%X, nbit=%d, nbt=%d\n",
		     PixCh.Pnx1[slay][nc][nbt], PixCh.Pny1[slay][nc][nbt], slay, qu, nc, CratesId[CrN], nbit, nbt);
	      */
	      
	      HeaderStr = 2 + 34*CrN;
	      NWord = HeaderStr + nbit/32 + 1; // Number of word
	      TableWordRef = (LWord*)(ArtePointer<DMUX>(NWord+1)->bits);
	      
	      if (CheckEndian() == 1)
		TableWord = *TableWordRef;
	      else if (CheckEndian() == 2)
		TableWord = ESP_SWAP_INT(*TableWordRef);

	      Word nw = nbit/32;
	      Word nbtt = nbit - 32*nw;
	      Word nbyte = nbtt/8;
	      Word nbb = nbtt - 8*nbyte;
	      Word nb = 7 - nbb;
	      Word nbt = 8*nbyte+nb;
	      
	      mask = HexConst << nbt;
	      TableWord = TableWord | mask;
	      
	      FillCharArr(TableWord, mudata);
	      ArtePointer<DMUX>(NWord+1)->bits = mudata;
	    }
  
  return;
} 






