
// Boris Fominykh  10.02.2000
// initialization of muon detector reconstruction
// acReadMuonA.C - alignment corrections, is called from digi2hitb
// acReadMuon.C  - initialization of digi2hitb

// history:
//  30/11/00 - implemented the fatal exit in the case of error - B.Fominykh 
//   6/05/02 - /MUON_NEW + MC, acReadMuonA.C is separated  - B.Fominykh 
//   27/06/02 - bug correction in acReadMuonA.C            - B.Fominykh 

// program has to be called after read of first event
// (keytables and run number(time) are needed)
//
// input parameter KTstatus is not used
//
// return code      2  -  data updated from DB using the run number (/MUON_DB) or time (/MUON_NEW)
//                  1  -  data updated from DB using keytable
//                  0  -  no actions because data are uptodate 
//                 -1  -  data updated from ASCII file
//                 -2  -  error
//

#include <stdio.h>
#include <ctype.h>
#include <string.h>
#include <iostream.h>
#include <time.h>

#include "arte/arte.hh"
#include "farm/farm.hh"
#include "arte/ArteMessage.hh"
#include "database/keytable.h"

static int first1   = 0;
static int first2   = 0;
static int MUtableM = 0;
static int MUtab1M  = 0;
static int MUtab2M  = 0;
static int OldOrNew = 0;
static int OldOrNewA = 0;

extern "C" {
  void getrun_();
  int MuSetMajor(int,int);
  int MuSetMinor(int,int);
  void musteer_();
  int MuAlignment(int,int,char*);
  int MuAlignmentNew(int,int,int);
  int MuD2Hini(int,int,int,char*);

//  ================================
// initialization of hit preparation

int acReadMuon(int KTstatus) 
//  ================================
{
  // cout << "acReadMuon is called" << endl;
// initialization of steering parameter
  int mode=1;
  int mustpar=0;
  musteer_(&mode,&mustpar);

// set data base name
  char DBname[21];

// get the run number
  int run=0;
  getrun_(&run);

// real data 2000 - /MUON_DB
  if(ArteTable<MTRA>::size() <= 0 && run < 17942) strncpy(DBname,"/MUON_DB",9);
// real data >= 2002 or MC
  else strncpy(DBname,"/MUON_NEW",10);

// DBname defined in kumac has the highest priority
  if(getenv("MUON_DBNAME") != NULL) strncpy(DBname,getenv("MUON_DBNAME"),20);

// set OldOrNew
  if(!strncmp(DBname,"/MUON_DB",8))  OldOrNew=0;  
  if(!strncmp(DBname,"/MUON_NEW",9)) OldOrNew=1;  

  int retdh=-2;
// two existing ASCII files have the highest priority 
  if((getenv("MUON_HOTS") != NULL) &&
     (getenv("MUON_CONFIG")  != NULL)) 
  {

// initialize only one time
    if (first2 == 0)
    {
      first2 = 1;
      int ret=MuD2Hini(-3,0,0,DBname);
      if(ret == 2) retdh=-1;
      else         retdh=-2;

      if(retdh == -1) arteMsg << "acReadMuon: initialization from files done";
      else arteMsg << "acReadMuon: error in initialization from files";
      arteMsg.warn();
    }
// correction is not needed
    else retdh=0;
  }

// if there was no initialization from ASCII files, no time definition in kumac
// let's read DB using keytable

  int keymode = 0;
  if(getenv("MUON_HOTS_TIME") == NULL && getenv("MUON_HOTS_MAJOR") == NULL) keymode=1;
  // int ret=KeyPrintTables(0);
  if(retdh == -2 && keymode == 1 && ArteTable<MTRA>::size() <= 0) 
  {

// let's check that key server exists
    int MUserver = KeyFindServer(DBname);

    if(MUserver > 0)
    {
// check that both table are here
      int MUtab1 = KeyFindTable(MUserver,"MuonConfig");
      int MUtab2 = 0;
// mask tables have different names in /MUON_DB and /MUON_NEW
      if(OldOrNew == 0) MUtab2 = KeyFindTable(MUserver,"MuonHots");
      if(OldOrNew == 1) MUtab2 = KeyFindTable(MUserver,"MuonMask");

      if(MUtab1 > 0 && MUtab2 > 0) 
      {
// check that at least one of two tables has to be updated
        int MUnew1=KeyTableNew(MUtab1);
        int MUnew2=KeyTableNew(MUtab2);

        if(MUnew1 > 0 || MUnew2 > 0) 
        {
// make corrections only if at least one table is new
          if (MUtab1 != MUtab1M || MUtab2 != MUtab2M)
          {
            int ret=MuD2Hini(2,0,0,DBname);
            if(ret == 1) 
            {
              MUtab1M = MUtab1;
              MUtab2M = MUtab2;
              retdh=1;
            }
            else         retdh=-2;

            arteMsg << "acReadMuon: ini from " << DBname << " is done with keytable, ret= " << retdh;
            arteMsg.warn();
          }
// tables are the same - no action
           else retdh=0;
        }
// both tables were not updated - no actions
        else retdh=0;
      }
    }          
  }          

// use DB with fixed major for initialization if it was not done yet
  if(retdh == -2) 
  {
// get the major - a run (time) defined in kumac or the actual run (time) otherwise
    int ipr = 0;
    int major = run;
    if(OldOrNew == 1) major = MuSetMajor(0,ipr);
    int minor = 0;
    if(OldOrNew == 1) minor = MuSetMinor(0,ipr);

// do correction only one time
    if(first2 != major) 
    {
      first2=major;
      int ret=MuD2Hini(-2,major,minor,DBname);
      if(ret == 2) retdh=2;
      else         retdh=-2;
    }
    else retdh=0;
  }

// set the status of initialization
  if(retdh > -2) 
  {
    mode=1;
    mustpar=retdh+2;
    musteer_(&mode,&mustpar);
  }

// error exit
  if(retdh == -2) ARTE_exit(GRR_SFATAL,"acReadMuon: hit preparation constants are not read",2);

// nomal output
  return retdh;
}



// ========================
//    alignment corrections

int acReadMuonA() 
// ========================
{
// get the run number
  int run=0;
  getrun_(&run);

// check that alignment correction  not needed
  if (first1 != 0)
  {
    if(getenv("MUON_ALIGN") != NULL) return 0;
    if(first1 == run)  return 0;
  }

  char DBname[21];

// set the data base name

// real data 2000
  if(ArteTable<MTRA>::size() <= 0 && run < 17942) strncpy(DBname,"/MUON_DB",9);
// data >= 2002 or MC
  else  strncpy(DBname,"/MUON_NEW",10);

  if(!strncmp(DBname,"/MUON_DB",8))  OldOrNewA=0;  
  if(!strncmp(DBname,"/MUON_NEW",9)) OldOrNewA=1;  
  int retal=-2;

// ASCII file has the highest priority 
  if(getenv("MUON_ALIGN") != NULL)
  {
// make alignment corrections only one time using ASCII file
    if (first1 == 0)
    {
      first1 = 1;

      int ret=0;
      if(OldOrNewA == 0) ret=MuAlignment(-3,0,NULL);
      else  ret=MuAlignmentNew(-3,0,0);

      if(ret == 2) retal=-1;
      else         retal=-2;
    }
// correction is not needed
    else retal=0;
  }

// if correction is not done yet and there is no run (time) definition  in kumac
// let's use keytable
  int keymode = 0;
  if(getenv("MUON_ALIGN_TIME") == NULL && getenv("MUON_ALIGN_MAJOR") == NULL) keymode=1;
  // cout << "acReadMuon: keymode= " << keymode << endl;

  if(retal == -2 && keymode == 1)
  {
// check that the key server exists
    int MUserver = KeyFindServer(DBname);

    // int ret=KeyPrintTables(1);
    if(MUserver > 0)
    {
// check that the key server exists
// bug correction 27.06.02 by B.Fominykh
      int MUtable;
      if (OldOrNewA == 0) MUtable = KeyFindTable(MUserver,"MuonAlignment");
      if (OldOrNewA == 1) MUtable = KeyFindTable(MUserver,"MuonAlign");

      if(MUtable > 0) 
      {
// check that the table was updated
        int MUnew = KeyTableNew(MUtable);

        if(MUnew > 0) 
        {
// make corrections only one time
          if (MUtable != MUtableM)
          {
            int ret=0;
            if(OldOrNewA == 0) ret=MuAlignment(2,0,DBname);
            else ret=MuAlignmentNew(2,0,0);
            if(ret == 1) 
            {
              MUtableM = MUtable;
              retal=1;
            }
            else retal=-2;

            arteMsg << "acReadMuonA: alignment corr done with keytable, " << DBname << " ret= " << retal;
            arteMsg.warn();
          }
// table the same - no action
          else retal=0;
        }
// table was not updated - no action
        else retal=0;
      }
    }          
  }          

// use time(run) if correction was not done yet
  if(retal == -2) 
  {
// apply correction only one time per run
    if(first1 != run) 
    {
      first1 = run;

// get the major - a time from kumac or the actual time if it's not set in kumac
      int major = run;
      int minor = 0;
      int ret=0;
      if(OldOrNewA == 0) ret=MuAlignment(-2,major,DBname);
      if(OldOrNewA == 1)
      {
        int ipr = 0;
        major = MuSetMajor(1,ipr);
        minor = MuSetMinor(1,ipr);
        ret=MuAlignmentNew(-2,major,minor);
      }

      if(ret == 2) retal=2;
      else         retal=-2;
    }
    else retal=0;
  }

// error exit
  if(retal == -2) ARTE_exit(GRR_SFATAL,"acReadMuonA: alignment correction was not done",1);
// gut exit
  return retal;
}
}
