// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified: 29-Mar-99
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// Modifications:
// - Introduced list of active walls   14-Aug-00 Wouter Hulsbergen 
// - use of all gwals in the vds                 17/05/01 A. Spiridonov
// - Removed trd/hipt walls                      04/07/01 Wouter
// - Added routine getActiveVolumes (in .hh)
// - Increased mult.scat. fudge factor for main  20/07/01 Wouter 
//   tracker  
//   Check of evrc->alca in condUpdate()         28/01/02 A.Spiridonov
//   fudge  otr,itr,hi-pt                        01/03/02 A.Spiridonov 

#include <iostream>
#include <stdio.h>
#include <algo.h>
#include "arte/arte.hh"
#include "arte/ArteMessage.hh"
#include <arte/artefunc.h>
#include "rang/Geom.hh"
#include "rang/Gede.hh"
#include "rang/Gwal.hh"
#include "rang/sortzv.hh"
#include "gean/ZGESL.h"
#include "component.h" 

#define GWALL_SELECT
#undef GWALL_NOWALLS  
#undef GWALL_NOVDS

vector<rangerVolume*> rangerGeom::volumes;
vector<vector<rangerVolume*>::iterator> rangerGeom::vGede;
vector<vector<rangerVolume*>::iterator> rangerGeom::vGwal;

vector<rangerVolume*> rangerGeom::activeVolumes;
int rangerGeom::firstPlane=0;
vector<vector<rangerVolume*>::const_iterator> rangerGeom::planeToUpstrActive;

void rangerGeom::update() {
  reset();
  build();
  gsort();
  arteMsg << "rangerGeom::update: geometry table (re)built" 
	  << " with " << volumes.size() 
	  << " (" << activeVolumes.size() << ") volumes" << endl ;
  arteMsg.warn();
}

void rangerGeom::condUpdate() {
  static int DBMAJOR_RANGER,EVT_RANGER,RUN_RANGER,EXP_RANGER;
  bool need_update;
  need_update = false;

  ArtePointer<EVHD> evhd = *ArteTable<EVHD>::begin();
  if( evhd->evt == EVT_RANGER && evhd->run == RUN_RANGER &&
      evhd->exp == EXP_RANGER ) return;
      
  //if(evhd->dbmajor != DBMAJOR_RANGER) need_update = true;

  unsigned int nVol = ArteTable<GEDE>::nrow() + ArteTable<GWAL>::nrow();
  if (nVol != volumes.size()) need_update = true;

  int nrow = ArtePointer<EVRC>::nrow(); 
  ArtePointer<EVRC> evrc = ArtePointer<EVRC>(nrow);
  //  ArtePointer<EVRC> evrc = *ArteTable<EVRC>::begin();
  if(evrc->alca & COMP_M_TARG) need_update = true;
  if(evrc->alca & COMP_M_SVD) need_update = true;
  if(evrc->alca & COMP_M_ITR) need_update = true;
  if(evrc->alca & COMP_M_OTR) need_update = true;
  if(evrc->alca & COMP_M_RICH) need_update = true;
  if(evrc->alca & COMP_M_ECAL) need_update = true;
  if(evrc->alca & COMP_M_MUON) need_update = true;
  if(evrc->alca & COMP_M_HIPT) need_update = true;
  if(evrc->alca & COMP_M_TRD) need_update = true;
  if(evrc->alca & COMP_M_DAQ) need_update = true;

  EVT_RANGER = evhd->evt;
  RUN_RANGER = evhd->run;
  EXP_RANGER = evhd->exp;

  if(need_update) 
    { 
      update();                             // update Geom
      DBMAJOR_RANGER = evhd->dbmajor;
#ifdef GEOM_DEBUG
      cout << " Geom.C  =============== "
	   << " update for exp " << EXP_RANGER << " run " << RUN_RANGER
	   << " evt " << EVT_RANGER << " dbmajor " << DBMAJOR_RANGER 
	   << endl;
      cout << " Geom.C ++++++++++++++++++++"; 
      cout << " targ " << (evrc->alca & COMP_M_TARG) ;
      cout << " svd " << (evrc->alca & COMP_M_SVD) ;
      cout << " itr " << (evrc->alca & COMP_M_ITR) ;
      cout << " otr " << (evrc->alca & COMP_M_OTR) ;
      cout << " rich " << (evrc->alca & COMP_M_RICH) ;
      cout << " ecal " << (evrc->alca & COMP_M_ECAL) ;
      cout << " muon " << (evrc->alca & COMP_M_MUON) ;
      cout << " hipt " << (evrc->alca & COMP_M_HIPT) ;
      cout << " trd " << (evrc->alca & COMP_M_TRD) ;
      cout << " daq " << (evrc->alca & COMP_M_DAQ) ;
      cout << " evrc->alca " << evrc->alca;
      cout << endl;
      cout  << "\007\007\007\007\007\007\007\007\n";
#endif
    }
}

void rangerGeom::reset() {
  for (vector<rangerVolume*>::iterator vIt=volumes.begin();
       vIt != volumes.end(); ++vIt) delete (*vIt);
  volumes.erase(volumes.begin(),volumes.end());
  vGede.erase(vGede.begin(),vGede.end());
  vGwal.erase(vGwal.begin(),vGwal.end());
  activeVolumes.clear() ;
  planeToUpstrActive.clear() ;
}

void rangerGeom::build() {
  volumes.clear() ;
  volumes.reserve(ArteTable<GEDE>::nrow()+ArteTable<GWAL>::nrow());
  
  for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	gede != ArteTable<GEDE>::end(); ++gede ) {
    rangerGede* Gede = new rangerGede( gede ); // allocate new Gede
    rangerGeWa* GeWa = Gede;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );
    volumes.push_back(vol);

#ifdef FULL_VOL
    rangerVolume* volNew = vol;
    if (volNew->getGede() != 0)  
      { 
	ArtePointer<GEDE> gede = volNew->getGede(); 
	if(gede->cmp != 1) continue;
	cout << "rangerGeom:build()   volume "  
	     << " z " << volNew->zMean() << " ms " << volNew->ms() 
	     << " dedx " << volNew->dedx();        
	cout << " gede " << gede << " dedx " << gede->dedx 
	     << " cmp " << gede->cmp << endl;  
      } 
    if (volNew->getGwal() != 0)  
      { 
	ArtePointer<GWAL> gwal = volNew->getGwal(); 
	continue;
	cout << "rangerGeom:build()   volume "  
	     << " z " << volNew->zMean() << " ms " << volNew->ms() 
	     << " dedx " << volNew->dedx();        
	cout << " gwal " << gwal << " dedx " << gwal->dedx 
	     << " shp " << gwal->shp << endl;  
      } 
#endif

  }

  for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
	gwal != ArteTable<GWAL>::end(); ++gwal ) {
    rangerGwal* Gwal = new rangerGwal( gwal ); // allocate new Gwal
    rangerGeWa* GeWa = Gwal;
    rangerVolume* vol = rangerVolume::makeVolume( GeWa );

#ifdef FULL_VOL
    rangerVolume* volNew = vol;
    ArtePointer<GWAL> gwal = volNew->getGwal(); 
    //continue;
    cout << "rangerGeom:build()   volume "  
	 << " z " << volNew->zMean() << " ms " << volNew->ms() 
	 << " dedx " << volNew->dedx();        
    cout << " gwal " << gwal << " dedx " << gwal->dedx 
	 << " shp " << gwal->shp << endl;  
#endif

    volumes.push_back(vol);
  }
}

inline bool 
compareVolumesZ(const rangerVolume* lhs, const rangerVolume* rhs)
{
  double refzlhs = 1e-8*lhs->plNumber()+lhs->zMean() ; 
  double refzrhs = 1e-8*rhs->plNumber()+rhs->zMean() ; 
  return refzlhs<refzrhs ;
}

void rangerGeom::gsort() {

  // sort the volumes
  //sort(volumes.begin(),volumes.end(), compareCall( &rangerVolume::zMean ) );
  sort(volumes.begin(),volumes.end(),&compareVolumesZ) ;

  // initialize the vGede and vGwal pointers
  vGede.reserve(ArtePointer<GEDE>::nrow());
  for ( ArteTable<GEDE>::iterator gede = ArteTable<GEDE>::begin();
	gede != ArteTable<GEDE>::end(); ++gede ) {
      vGede.push_back(volumes.end());
  }
  vGwal.reserve(ArtePointer<GWAL>::nrow());
  for ( ArteTable<GWAL>::iterator gwal = ArteTable<GWAL>::begin();
	gwal != ArteTable<GWAL>::end(); ++gwal ) {
      vGwal.push_back(volumes.end());
  }
  for (vector<rangerVolume*>::iterator vi=volumes.begin();
       vi != volumes.end(); ++vi) {
    if ((*vi)->plNumber() > 0) {
      if ((*vi)->plNumber() > (int) ArtePointer<GEDE>::nrow()) {
	cout << "warning: gede::nrow=" << ArtePointer<GEDE>::nrow()
	     << " pln=" << (*vi)->plNumber() << endl;
      }
      vGede[(*vi)->plNumber()-1] = vi;
    }
    else {
      if ((*vi)->plNumber() < 0) {
	if (-(*vi)->plNumber() > (int) ArtePointer<GWAL>::nrow()) {
	  cout << "warning: gwal::nrow=" << ArtePointer<GWAL>::nrow()
	       << " pln=" << (*vi)->plNumber() << endl;
	}
	vGwal[-(*vi)->plNumber()-1] = vi;
      }
      else {
	cout << "plNumber zero!" << endl;
      }
    }
  }

  // create vector of active volumes
  activeVolumes.clear() ;
  // this reserve is necessary, otherwise pointers are not correctly
  // set. it is a bit a waste of space though ...
  activeVolumes.reserve(volumes.size()) ;
  // these are pointers to the first active volume with a lower z value
  planeToUpstrActive.clear() ;
  firstPlane = -ArteTable<GWAL>::size() ;
  int nplanes = ArteTable<GWAL>::size()+ArteTable<GEDE>::size()+1 ;
  planeToUpstrActive.resize(nplanes,activeVolumes.begin()) ;

  for(vector<rangerVolume*>::iterator vi=volumes.begin();
      vi != volumes.end(); ++vi) 
    {
      int plane = (*vi)->plNumber() ;
      
      // set pointer of this plane to first active volume with lower z
      if(!activeVolumes.empty())
	planeToUpstrActive[plane-firstPlane] = activeVolumes.end() - 1 ;
      else
	planeToUpstrActive[plane-firstPlane] = activeVolumes.begin() ;
      
      // here the code to de/activate volume
      bool active = true;
#ifdef GWALL_SELECT
      //float z = (*vi)->zMean() ;
      rangerGeWa* gewa = const_cast<rangerGeWa*> ((*vi)->getGeWa()) ;
      int cmp = 0 ;
      if( gewa->getGede().index() ) cmp = gewa->getGede()->gesl->cmp ;
      if( gewa->getGwal().index() ) cmp = gewa->getGwal()->gesl->cmp ;
      
      // remove all objects with very low ms:
      // (there are no gedes below 1.e-7)
      active = active && !(plane<0 && gewa->ms() < 1.e-8) ;
      
      // remove the g10 strips and end-pieces in the otr (they don't
      // need correction)
      active = active && !(cmp==LGESL_OTR && plane<0 && gewa->ms()<1.e-7) ;    
      // the remaining walls in the otr are coverplates and end-caps.
      
      // remove all walls in otr itr and replace by fudge factors
      if( cmp==LGESL_OTR ) 
	{
	  //const float otrfudge = 4. ;
	  const float otrfudge = 3.5 ;
	  const float otrdedx = 1.7;
	  if(ArteTable<GEDE>::nrow() > 3800 ||
	     (ArteTable<GEDE>::nrow() <= 3800 && (*vi)->zMean() > 450.) )
	    {	      
	      if(plane<0 ) active = false ;
	      else 
		{	
		  (*vi)->setMs((*vi)->ms()*otrfudge) ;
		  (*vi)->setDeDx((*vi)->dedx()*otrdedx) ;
		}
	    }
	}
      
      // remove all walls in itr and replace by fudge factors
      if( cmp==LGESL_ITR ) 
	{ 
	  //const float itrfudge = 3. ;
	  const float itrfudge = 2.5 ;
	  const float itrdedx = 1.8;
	  if(ArteTable<GEDE>::nrow() > 3800 ||
	     (ArteTable<GEDE>::nrow() <= 3800 && (*vi)->zMean() > 450.) )
	    {
	      if(plane<0) active = false ;
	      else 
		{
		  (*vi)->setMs((*vi)->ms()*itrfudge) ;
		  (*vi)->setDeDx((*vi)->dedx()*itrdedx) ;
		}
	    }
	}
      
      // remove trd and hi-pt volumes (saves about 10% CPU)
      
      if( cmp==LGESL_TRD ) active = false ;
      
      if(ArteTable<GEDE>::nrow() > 3800)                    // full geometry
	{
	  if(cmp==LGESL_PTC) active = false ;
	}
      //else                                                // reduced geometry
      // remove all walls in hi-pt and replace by fudge factors
      //if( cmp==LGESL_PTC ) 
      //{ 
      // const float ptcfudge = 2. ;
      //const float ptcdedx = 2.3;
      //if(plane<0) active = false ;
      //else 
      //  {
      //    (*vi)->setMs((*vi)->ms()*ptcfudge) ;
      //    (*vi)->setDeDx((*vi)->dedx()*ptcdedx) ;
      //  }
      //}

      //%%%%
#ifdef GWALL_NOVDS
      float z = (*vi)->zMean() ;
      const float vdsfudge = 1.2 ; 
      bool isvds = z<207. ;
      if(active && isvds) 
	{
	  (*vi)->setMs((*vi)->ms()*vdsfudge) ;
	  //if(plane>0) (*vi)->setMs((*vi)->ms()*vdsfudge) ;
	  //else        active = false ;
	}
#endif      
#endif 

      // too thick 
      if ((*vi)->dedx() > 0.003 && (*vi)->zMean() < 682. ) active = false;
      // too narrow
      if (plane < 0 &&  (*vi)->zMean() < 682. && 
	  ((gewa->getGwal()->xmax - gewa->getGwal()->xmin) < 0.1 || 
	   (gewa->getGwal()->ymax - gewa->getGwal()->ymin) < 0.1 )) 
	active = false; 
      // out of acceptance
      if( plane < 0 && gewa->getGwal()->shp == 1 && (*vi)->zMean() < 682.
	  && gewa->getGwal()->rot[1] == 0.)
	{
	  if((fabs((gewa->getGwal()->xmax / gewa->getGwal()->zmin)) > 0.25 && 
	      fabs((gewa->getGwal()->xmin / gewa->getGwal()->zmin)) > 0.25 &&
	      fabs(( 0.5* ( gewa->getGwal()->xmin 
			    + gewa->getGwal()->xmax ) 
		     / gewa->getGwal()->zmin)) > 0.25 ) ||
	     (fabs((gewa->getGwal()->ymax / gewa->getGwal()->zmin)) > 0.16 && 
	      fabs((gewa->getGwal()->ymin / gewa->getGwal()->zmin)) > 0.16 &&
	      fabs(( 0.5* ( gewa->getGwal()->ymin 
			    + gewa->getGwal()->ymax ) 
		     / gewa->getGwal()->zmin)) > 0.16 )
	     ) active = false;
	}

      if(active && (*vi)->zMean() < 207. && ArteTable<GEDE>::nrow() <= 3800 )
	  {
	    (*vi)->setMs((*vi)->ms()*1.2) ; 
	  }
      if(active && (*vi)->zMean() < 682. && plane < 0
	 && ArteTable<GEDE>::nrow() <= 3800 )
	  {
#ifdef FULL_DEBUG
	    if((gewa->getGwal()->xmax - gewa->getGwal()->xmin) > 0.1 &&
	       (gewa->getGwal()->ymax - gewa->getGwal()->ymin) > 0.1
	       && gewa->getGwal()->shp == 1 || true ) 
	      cout << " "  
		 << gewa->getGwal()->gesl->name
		 << " de " << (*vi)->dedx()
		 << " dx " << (gewa->getGwal()->xmax - gewa->getGwal()->xmin)
		 << " dy " << (gewa->getGwal()->ymax - gewa->getGwal()->ymin)
		 << endl;
#endif
	    (*vi)->setDeDx((*vi)->dedx() * 0.5);	        
	  }

      // if(plane<0 && (*vi)->zMean() < 450.
      //	    && active && (cmp==LGESL_ITR || cmp==LGESL_OTR))
      //cout << " gesl " << gewa->getGwal()->gesl 
      //    << gewa->getGwal()->gesl->name 
      //   << " ms " << (*vi)->ms() << " dedx " << (*vi)->dedx() << endl;
      
      // add volume to active list


      if(active) 
	{
	  activeVolumes.push_back(*vi) ;
	}
    }
}
