// 
//  Author:  Rainer Mankel                     Created:  24-Aug-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// Modified:   
// - moved many routines to State.C
//                                     14-Aug-00 Wouter Hulsbergen
//  remove printout from StateP::predict                  08.12.00  A. Spiridonov
//  #define TRANSCD                                       01/03/02 A.Spiridonov


#define TRANSCD
//#define FULL_PREDICT

#include <iostream>
#include <math.h>
#include "rang/StateP.hh"
#include "rang/zfpar.hh"
#include "rang/rkmain.hh"
#include "rang/utfeld.hh"
#include "rang/rktranscd.hh"
#include "arte/ArteMessage.hh"

//rangerStateP::rangerStateP() : rangerState(5) {} 

int rangerStateP::predict(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) const {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStateP::predict: zDestination=" << zDestination << endl;

  cout << "rangerStateP::predict: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

#ifdef FULL_PREDICT
  cout << "rangerStateP::predict: -z " << z() 
       << " v " <<  x() << " " << y() << " " << tx() << " " << ty()
       << " " << qP() 
    //<< " " << sqrt(cov()(5,5))/qP() 
       << endl;
#endif

  if (newState) {
    s = new rangerStateP();
    if (!s) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }
  if (! F) {
    F = new HepMatrix(5,5);
    if (!F) {
      cout << "rangerState::predict: allocation error" << endl; 
      exit(1);
    }
  }

  //if(s->vec()(5) == 0.) 
  //cout << "rangerStateP::predict: ZERO P s->vec()= " << s->vec() 
  // << " zState " << zState << " zDestination " << zDestination << endl;

#ifdef FULL
  cout << "rangerStateP::predict: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);

  float zmin;
  if (ArteTable<GEDE>::nrow() <= 3800) zmin = 156.;
  else zmin = zfpar_.zminMag;
  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       //|| (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || (zDestination<zmin && zState<zmin)
       || deltaZ < 0.001) {
    s->setZ( zDestination );      // linear transport
    s->setX( x() + (zDestination - zState) * tx() );
    s->setY( y() + (zDestination - zState) * ty() );
    s->setTx( tx() );
    s->setTy( ty() );
    s->setQp( qP() );
    s->setQpEst( qPEst() );

    for (int i=1; i<=5; i++) {
      for (int j=1; j<=5; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;

    s->cov() = c.similarity(*F);
        
#ifdef FULL_PREDICT
    float corxp = s->cov()(1,5);
    float sx = s->cov()(1,1); 
    float sp = s->cov()(5,5);  
    corxp /= sqrt(sx * sp);
    cout << "rangerStateP::predict: Lz " << s->z() 
	 << " z0 " << zState
      //<< " v " <<  s->x() << " " << s->y() << " " << s->tx() 
      // << " " << s->ty() 
	 << " qp " << s->qP() 
	 << " sx " << sqrt(sx)  << " sp " << sqrt(sp)
	 << " cxp " << corxp
	 << endl;
#endif

    if (fabs(s->x()) < 700);
    else {
      arteMsg << "StateP::predict: produced garbage (lin), vec="
	      << s->vec() << endl;
      //      cout <<  "StateP::predict: produced garbage (lin), vec="
      //      << s->vec() << endl;
      //double cc = cin.get();
      return -10;
    } 

  }
  else {
#ifdef TRANSCD
    //double fstep = fabs(zDestination - zState);
    //if (fstep<=0) fstep = 1;
    double pIn[5];
    pIn[0] = x();
    pIn[1] = y();
    pIn[2] = tx();
    pIn[3] = ty();
    //pIn[4] = qPExp;
    pIn[4] = qP();

#ifdef FULL 
    cout << "StateP::predict: pIn="
	 << pIn[0] << " "
	 << pIn[1] << " "
	 << pIn[2] << " "
	 << pIn[3] << " "
	 << pIn[4] 
	 << " transporting from " << zState
	 << " to " << zDestination << endl;
#endif

    if ( pIn[0] > -700 &&  pIn[0] < 700 && // check validity of rkmain input
	 pIn[1] > -700 &&  pIn[1] < 700 &&  
	 pIn[2] > -10 &&  pIn[2] < 10 &&  
	 pIn[3] > -10 &&  pIn[3] < 10 &&  
	 pIn[4] > -100 &&  pIn[4] < 100 
	 && pIn[4] != 0. ); 
    else {
      cout << "StateP::predict: debug invalid pIn="
	   << pIn[0] << " " << pIn[1] << " " << pIn[2] << " "
	   << pIn[3] << " " << pIn[4] 
	   << " transporting from " << zState
	   << " to " << zDestination << endl;
      return -10;
    }

    //double cOut[25];
    //double cIn[25];
    double rkd[25];
    //for (int i=0; i<=4; i++) {
    //  for (int j=0; j<=4; j++) {
    //	cIn[5*j+i] = c(i+1,j+1);
    // }
    //}
    double pOut[5];
    int ierror;

    double zOut = zDestination;
    double zIn = zState;
    //rktrans_(zIn,pIn,zOut,pOut,rkd,ierror);
    //rk5clip_(zIn,pIn,zOut,pOut,rkd,ierror);
    rktransf_(zIn,pIn,zOut,pOut,rkd,ierror);

    if (ierror == 0) 
      {
	
	for (int i=1; i<=5; i++) {
	  for (int j=1; j<=5; j++) {
	    (*F)(i,j) = rkd[5*(j-1)+i-1];
	  }
	}
	
	s->setZ( zDestination );
	s->setX( pOut[0] );
	s->setY( pOut[1] );
	s->setTx( pOut[2] );
	s->setTy( pOut[3] );
	s->setQp( qP() );  // only valid in case momentum is unchanged
	s->setQpEst( qPEst() );
	
	//for (int i=1; i<=5; i++) {
	// for (int j=1; j<=5; j++) {
	//   s->cov()(i,j) = cOut[5*(j-1)+i-1];
	// }
	//}
    
	s->cov() = c.similarity(*F);
		
#ifdef FULL_PREDICT 
	float corxp = s->cov()(1,5); 
	float sx = s->cov()(1,1);  
	float sp = s->cov()(5,5);   
	corxp /= sqrt(sx * sp); 
	
	cout << "rangerStateP::predict: Oz " << s->z() 
	     << " z0 " << zState
	  //	   << " v "  
	  // << s->x() << " " << s->y() << " " << s->tx() << " "
	  // << s->ty() 
	     << " qp " << s->qP() 
	  //   << " sx " << sqrt(sx)  << " sp " << sqrt(sp) 
	  //   << " cxp " << corxp 
	  //   << " -- cxp " << corxp 
	     << endl;
	//if( (zIn < 682. && zOut > 207.) ) 
	// {
	//   cout << " This is  s->cov() " << s->cov() << endl;
	// }
#endif
	
	//if (abs(s->x()) < 700);
	if (s->x() > -700 && s->x() < 700 );
	else {
	  cout << "StateP::predict: produced garbage (mag), vec="
	       << s->vec() << endl;
	  cout << " The input was " 
	       << pIn[0] << " "
	       << pIn[1] << " "
	       << pIn[2] << " "
	       << pIn[3] << " "
	       << pIn[4] 
	       << " transporting from " << zState
	       << " to " << zDestination << endl;
	  cout << " The output was " 
	       << pOut[0] << " "
	       << pOut[1] << " "
	       << pOut[2] << " "
	       << pOut[3] << endl;
	  //double cc = cin.get();
	  return -10;
	} 
      }
    
    //                                       RKMAIN

    else {
      cout << "state::predict: rkmain returns ierror=" << ierror << endl;
      return -1;
    }


#else
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;

#ifdef FULL 
    cout << "=i=i=i= rangerStateP::predict: pin,z= "<< pin[0]
	 << " " << pin[1] << " " << pin[2]
	 << " " << pin[3] << " " << pin[4] << " " << pin[5] << endl;
#endif

    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pout[6];
    double rkd[25];
    int ierror;
    rkmain_(icovar,zout,fstep,pin,prec,pout,rkd,ierror);

#ifdef FULL 
    cout << "=o=o=o= rangerStateP::predict: pout,z= "<< pout[0]
	 << " " << pout[1] << " " << pout[2]
	 << " " << pout[3] << " " << pout[4] << " " << pout[5] << endl;
#endif

    if (ierror == 0) {
      if (pout[0] != zDestination) 
	cout << "stateP::predict: zDest changed from "
	     << zDestination << " to " << pout[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
      }
      s->setZ( zDestination );
      s->setX( pout[1] + (*F)(1,5) * (qP() - qPExp) );
      s->setY( pout[2] + (*F)(2,5) * (qP() - qPExp) );
      s->setTx( pout[3] + (*F)(3,5) * (qP() - qPExp) );
      s->setTy( pout[4] + (*F)(4,5) * (qP() - qPExp) );
      s->setQp( qP() );  // only valid in case momentum is unchanged
      s->setQpEst( qPEst() );
    }
    else {
      cout << "state::predict: rkmain returns ierror=" << ierror << endl;
      return -1;
    }
    s->cov() = c.similarity(*F);
#endif

  }


#ifdef FULL
  cout << "rangerStateP::predict: after calculation, *F=" << *F << endl;

  cout << "rangerStateP::predict: the transported vector is"
       << s->vec() << endl;
#endif

  //s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateP::predict: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}




int rangerStateP::predictOld(const double& zDestination,
			 const double& qPExp,rangerState*& s,
			 HepMatrix*& F) {

  int newState     = (s == 0);  // denote if memory allocations exist
  int newTransport = (F == 0);   //   .    .    .        .         .

#ifdef FULL
  cout << "rangerStateP::predictOld: zDestination=" << zDestination << endl;

  cout << "rangerStateP::predictOld: targetState=" << s
       << " newState= " <<  newState << endl;
#endif

  if (newState) {
    s = new rangerStateP();
    if (!s) {
      cout << "rangerState::predictOld: allocation error" << endl; 
      exit(1);
    }
    F = new HepMatrix(5,5);
    if (!F) {
      cout << "rangerState::predictOld: allocation error" << endl; 
      exit(1);
    }
  }

#ifdef FULL
  cout << "rangerStateP::predictOld: after allocation: s->vec()=" << s->vec() 
       << endl << "       *F=" << *F << endl;
#endif

  double deltaZ = fabs(zDestination - zState);

  if ( (zDestination>zfpar_.zmaxMag && zState>zfpar_.zmaxMag)
       || (zDestination<zfpar_.zminMag && zState<zfpar_.zminMag)
       || deltaZ < 0.001) {
    s->setZ( zDestination );      // linear transport
    s->setX( x() + (zDestination - zState) * tx() );
    s->setY( y() + (zDestination - zState) * ty() );
    s->setTx( tx() );
    s->setTy( ty() );
    s->setQp( qP() );
    s->setQpEst( qPEst() );

    for (int i=1; i<=5; i++) {
      for (int j=1; j<=5; j++) {
	(*F)(i,j) = 0.;
      }
    }
    (*F)(1,1) = 1;
    (*F)(2,2) = 1;
    (*F)(3,3) = 1;
    (*F)(4,4) = 1;
    (*F)(5,5) = 1;
    (*F)(1,3) = zDestination - zState;
    (*F)(2,4) = zDestination - zState;
  }
  else {
    double zout = zDestination;
    double fstep = fabs(zDestination - zState);
    if (fstep<=0) fstep = 1;
    double pin[6];
    pin[0] = zState;
    pin[1] = x();
    pin[2] = y();
    pin[3] = tx();
    pin[4] = ty();
    pin[5] = qPExp;
    double prec[4];
    double dzTrunc = fabs(zDestination - zState);
    if (dzTrunc<1) dzTrunc = 1;

    prec[0] = 0.001 / dzTrunc;
    prec[1] = 0.001 / dzTrunc;
    prec[2] = 0.0001 / dzTrunc;
    prec[3] = 0.0001 / dzTrunc;

    int icovar = 1;
    //int icovar = 2;
    double pOut[6];
    double rkd[25];
    int ierror;

    if ( fabs(pin[1]) < 700 &&   // check validity of rkmain input
	 fabs(pin[2]) < 700 &&
	 fabs(pin[3]) < 10   &&
	 fabs(pin[4]) < 10   &&
	 fabs(pin[5]) < 100 );
    else {
      cout << "StateP::predictOld: invalid pin="
	   << pin[1] << " "
	   << pin[2] << " "
	   << pin[3] << " "
	   << pin[4] << " "
	   << pin[5] << endl;
      return -2;
    }


    rkmain_(icovar,zout,fstep,pin,prec,pOut,rkd,ierror);

    if (ierror == 0) {
      if (pOut[0] != zDestination) 
	cout << "state::predictOld: zDest changed from "
	     << zDestination << " to " << pOut[0] << endl;
      for (int i=1; i<=5; i++) {
	for (int j=1; j<=5; j++) {
	  (*F)(i,j) = rkd[5*(j-1)+i-1];
	}
      }
      s->setZ( zDestination );
      s->setX( pOut[1] + (*F)(1,5) * (qP() - qPExp) );
      s->setY( pOut[2] + (*F)(2,5) * (qP() - qPExp) );
      s->setTx( pOut[3] + (*F)(3,5) * (qP() - qPExp) );
      s->setTy( pOut[4] + (*F)(4,5) * (qP() - qPExp) );
      s->setQp( qP() );  // only valid in case momentum is unchanged
      s->setQpEst( qPEst() );
    }
    else {
      cout << "state::predictOld: rkmain returns ierror=" << ierror << endl;
      return -1;
    }
  }


#ifdef FULL
  cout << "rangerStateP::predictOld: after calculation, *F=" << *F << endl;

  cout << "rangerStateP::predictOld: the transported vector is"
       << s->vec() << endl;
#endif

  s->cov() = c.similarity(*F);

#ifdef FULL
  cout << "rangerStateP::predictOld: the transported cov is"
       << s->cov() << endl;
#endif

  if (1) {                // everything OK
    return 0;
  }
  else {
    cout << " This should not happen!!!" << endl;
    if (newState) delete s;
    if (newTransport) delete F;
    return -1000;
  }
}

/*
#ifdef FULL_DUBUG
    if( zIn  > 682. && zOut < 230. ) 
    //if ( false )
      {
	printf("rkd t1 %8f3 %8f3 %8f3 %8f3 %8f3  \n"
	       ,rkd[0],rkd[5],rkd[10],rkd[15],rkd[20]);  
	printf("rkd t2 %8f3 %8f3 %8f3 %8f3 %8f3  \n"
	       ,rkd[1],rkd[6],rkd[11],rkd[16],rkd[21]);  
	printf("rkd t3 %8f3 %8f3 %8f3 %8f3 %8f3  \n"
	       ,rkd[2],rkd[7],rkd[12],rkd[17],rkd[22]);  
	printf("rkd t4 %8f3 %8f3 %8f3 %8f3 %8f3  \n"
	       ,rkd[3],rkd[8],rkd[13],rkd[18],rkd[23]);  
	printf("rkd t5 %8f3 %8f3 %8f3 %8f3 %8f3  \n"
	       ,rkd[4],rkd[9],rkd[14],rkd[19],rkd[24]);  
      }
#endif

    if ( false )
      //if( zIn  > 682. && zOut < 230. ) 
      //%%%%
      {
	double pIn1[5],pOut1[5],rkd1[25];
	rk5clip_(zIn,pIn,zOut,pOut,rkd1,ierror);

	// numeric derivatives
	for(int i=0; i < 5; ++i){pIn1[i] = pIn[i];} pIn1[0] += 1.; 
	rk5clip_(zIn,pIn1,zOut,pOut1,rkd1,ierror);
	rkd[0] = (pOut[0] - pOut1[0]) / (pIn[0] - pIn1[0]);
	rkd[1] = (pOut[1] - pOut1[1]) / (pIn[0] - pIn1[0]);
	rkd[2] = (pOut[2] - pOut1[2]) / (pIn[0] - pIn1[0]);
	rkd[3] = (pOut[3] - pOut1[3]) / (pIn[0] - pIn1[0]);
	rkd[4] = (pOut[4] - pOut1[4]) / (pIn[0] - pIn1[0]);
	for(int i=0; i < 5; ++i){pIn1[i] = pIn[i];} pIn1[1] += 1.; 
	rk5clip_(zIn,pIn1,zOut,pOut1,rkd1,ierror);
	rkd[5] = (pOut[0] - pOut1[0]) / (pIn[1] - pIn1[1]);
	rkd[6] = (pOut[1] - pOut1[1]) / (pIn[1] - pIn1[1]);
	rkd[7] = (pOut[2] - pOut1[2]) / (pIn[1] - pIn1[1]);
	rkd[8] = (pOut[3] - pOut1[3]) / (pIn[1] - pIn1[1]);
	rkd[9] = (pOut[4] - pOut1[4]) / (pIn[1] - pIn1[1]);
	for(int i=0; i < 5; ++i){pIn1[i] = pIn[i];} pIn1[2] += 0.01; 
	rk5clip_(zIn,pIn1,zOut,pOut1,rkd1,ierror);
	rkd[10] = (pOut[0] - pOut1[0]) / (pIn[2] - pIn1[2]);
	rkd[11] = (pOut[1] - pOut1[1]) / (pIn[2] - pIn1[2]);
	rkd[12] = (pOut[2] - pOut1[2]) / (pIn[2] - pIn1[2]);
	rkd[13] = (pOut[3] - pOut1[3]) / (pIn[2] - pIn1[2]);
	rkd[14] = (pOut[4] - pOut1[4]) / (pIn[2] - pIn1[2]);
	for(int i=0; i < 5; ++i){pIn1[i] = pIn[i];} pIn1[3] += 0.01; 
	rk5clip_(zIn,pIn1,zOut,pOut1,rkd1,ierror);
	rkd[15] = (pOut[0] - pOut1[0]) / (pIn[3] - pIn1[3]);
	rkd[16] = (pOut[1] - pOut1[1]) / (pIn[3] - pIn1[3]);
	rkd[17] = (pOut[2] - pOut1[2]) / (pIn[3] - pIn1[3]);
	rkd[18] = (pOut[3] - pOut1[3]) / (pIn[3] - pIn1[3]);
	rkd[19] = (pOut[4] - pOut1[4]) / (pIn[3] - pIn1[3]);
	for(int i=0; i < 5; ++i){pIn1[i] = pIn[i];} pIn1[4] *= 0.9; 
	rk5clip_(zIn,pIn1,zOut,pOut1,rkd1,ierror);
	rkd[20] = (pOut[0] - pOut1[0]) / (pIn[4] - pIn1[4]);
	rkd[21] = (pOut[1] - pOut1[1]) / (pIn[4] - pIn1[4]);
	rkd[22] = (pOut[2] - pOut1[2]) / (pIn[4] - pIn1[4]);
	rkd[23] = (pOut[3] - pOut1[3]) / (pIn[4] - pIn1[4]);
	rkd[24] = (pOut[4] - pOut1[4]) / (pIn[4] - pIn1[4]);

      }
*/
