// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
//
// Modifications:
// - Slight modification in inclusion of header files
//                                      14-Aug-00 Wouter Hulsbergen
//   applyDeDxe( dedx() )                       01/02/03  A.Spiridonov

#undef FULL_PN

#include <iostream>
#include <math.h>
#include "arte/arte.hh"
#include "rang/Thick.hh"
#include "rang/zfpar.hh"
#include "rang/utfeld.hh"
#include "rang/RadEl.hh"
#include "rang/ParticleType.hh"
#include "rang/ScatThick.hh"

void rangerThick::processNoise(rangerParticleType pType) {

//   double sqxy = sqrt(1 + state->tx() * state->tx() 
// 		     + state->ty() * state->ty());
                                 // traversed distance in medium correction

  rangerScatThick scat( ms(), state->qPEst(),
			state->tx(),
			state->ty(),-dz());

  state->applyScat( scat );

  if (pType == ELECTRON) {
    rangerRadEl rad( radiationThickness(), state->qP() );
    state->applyRadEl( rad );
  }

  //state->applyDeDx( radiationThickness() );
  state->applyDeDxe( dedx() );

}


void rangerThick::predProcessNoise(rangerParticleType pType) {

//   double sqxy = sqrt(1 + predictedState->tx() * predictedState->tx() 
// 		     + predictedState->ty() * predictedState->ty());
                                 // traversed distance in medium correction
#ifdef FULL_PN
  cout << " rangerThick::predProcessNoise  ms " << ms() 
       << " dedex " << dedx() << " z " << predictedState->z() << endl;
#endif

  rangerScatThick scat( ms(), predictedState->qPEst(),
			predictedState->tx(),
			predictedState->ty(),-dz());

  predictedState->applyScat( scat );

  if (pType == ELECTRON) {
    rangerRadEl rad( radiationThickness(), predictedState->qP() );
    predictedState->applyRadEl( rad );
  }

  //predictedState->applyDeDx( radiationThickEff() );
  predictedState->applyDeDxe( dedx() );

#ifdef FULL
  cout << "rangerThick::processNoise: correction applied, cov matrix is now "
       << state->cov() << endl;
#endif
}
