// 
//  Author:  Rainer Mankel                     Created:  29-Sep-98
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 
#include <iostream.h>
#include <math.h>
#include "rang/VolEpi.hh"
#include "rang/Arrow.hh"

//-- Constructor
rangerVolEpi::rangerVolEpi(){};

//-- Constructor
rangerVolEpi::rangerVolEpi( rangerGeWa* gewa ) : rangerVolume( gewa ) 
{
  // calculate the reference coordinates in ACS and PCS
  float zP = 0.5*(Gewa->zmin() + Gewa->zmax()) ;
  float xP = 0.5*(Gewa->xmin() + Gewa->xmax()) ;
  float yP = 0.5*(Gewa->ymin() + Gewa->ymax()) ;
  rangerArrow a(zP,xP,yP,1,1) ;
  rotateArrow(a,-1) ;
  zA = a.z ;

  // for the rest I don't know
  xA1 = yA1 = -1.e5 ;
  xA2 = yA2 =  1.e5 ;
}


//-- Destructor
rangerVolEpi::~rangerVolEpi(){};

float rangerVolEpi::sLength(float z,float x,float y,float tx,float ty) const {

  rangerArrow a(z,x,y,tx,ty);
  rotateArrow(a);
  a.x -= (Gewa->sin() / Gewa->cos()) * a.y;  // see Hera-B 97-090 p 68

  float zP = 0.5 * (Gewa->zmin() + Gewa->zmax()); 
  float xP = a.x + (zP - a.z) * a.tx;  // coordinates in PCS
  float yP = a.y + (zP - a.z) * a.ty;  // see Hera-B 97-090 p 67
  if (xP < Gewa->xmin()) return 0;
  if (xP > Gewa->xmax()) return 0;
  if (yP < Gewa->ymin()) return 0;
  if (yP > Gewa->ymax()) return 0;

  return sqrt(1 + a.tx * a.tx + a.ty * a.ty);
}

