// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomObj
// basic ranger domain object

#ifndef RANGER_DOMOBJ_HH
#define RANGER_DOMOBJ_HH

#include "rang/Domain.hh"
class rangerXPro;

class rangerNavItem;
class rangerHL;
class rangerDomLay;
class rangerSeedTableau;
class rangerSeedSet;
class rangerDomSeg;

class rangerDomObj {

public:
  rangerDomObj();
  ~rangerDomObj();

  void appDef(rangerDomain* rd) { if (rd) defaults.push_back(rd); };
  void appDom(rangerDomain* rd) { if (rd) { domains.push_back(rd);
  rd->setNum( domains.size() );}
  };
  void appNav(rangerNavItem* ni) { if (ni) navItems.push_back(ni); };
  rangerHL* makeHL(rangerDomLay* dl); 
  void appDomSeg(rangerDomSeg* ds) { if (ds) domsegs.push_back(ds); };
  void appSeedTab(rangerSeedTableau* st) { if (st) seedTabs.push_back(st); };

  vector<rangerDomain*>& def() { return defaults; };
  vector<rangerDomain*>& dom() { return domains; };
  vector<rangerNavItem*>& nav() { return navItems; };
  vector<rangerHL*>& hls() { return hlists; };
  vector<rangerDomSeg*>& dseg() { return domsegs; };
  vector<rangerSeedTableau*>& stab() { return seedTabs; };
  int nSeedSets();
  rangerXPro* xpro() { return xPro;};

  vector<rangerDomain*>::iterator findDom(string dName);
  int addExclusionRange(ArtePointer<GEDE> ge,double umin,double umax,
			int subgl1=1,int subgl2=99);
  int clearExclusionRanges();
  void dump(const int level=1);

private:
  vector<rangerDomain*> defaults;
  vector<rangerDomain*> domains;
  vector<rangerNavItem*> navItems;
  vector<rangerHL*> hlists;
  vector<rangerDomSeg*> domsegs;
  vector<rangerDomWin*> navigWin;
  vector<rangerDomWin*> winList;
  vector<rangerSeedTableau*> seedTabs;
  rangerXPro* xPro;
  int updateLevel;
};

//rangerDomObj* rangerDO = 0;   // global DO pointer

#endif
