// 
//  Author:  Rainer Mankel                     Created:  07-Mar-99
//           Humboldt University Berlin        Modified:  
//  Email:   mankel@ifh.de                     HERA-B Collaboration
// 

// class rangerDomWin

#ifndef RANGER_DOMWIN_HH
#define RANGER_DOMWIN_HH

#include <vector.h>
#include <string.h>
#include "arte/arte.hh"
#include "rang/Domain.hh"
#include "rang/DomLay.hh"

class rangerNavItem;
class rangerProxyWin;

class rangerDomWin {
  
public:
  rangerDomWin(){};
  rangerDomWin(rangerDomain* rd,const int dCmp,const int dSec);
  ~rangerDomWin(){};

  void accum(const ArtePointer<GEDE> ge);
  void addDomLay(rangerDomLay* dl) { DomLays.push_back(dl); };
  void addEntrance(const int dir,rangerDomLay* dl);
  static void rotatePoint(float& x,float& y,float& z,
			       const ArtePointer<GEDE> ge,const int dir);
  static void xyzMinMax(float& xmin,float& xmax,
			     float& ymin,float& ymax,
			     float& zmin,float& zmax,
			     const ArtePointer<GEDE> ge);
  rangerDomain* dom() const { return Dom; };
  vector<rangerDomLay*>& dls() { return DomLays;};
  int sec() const { return Sec; };
  int cmp() const { return Cmp; };
  float xmin() const { return xMin; }; 
  float xmax() const { return xMax; }; 
  float ymin() const { return yMin; }; 
  float ymax() const { return yMax; }; 
  float zmin() const { return zMin; }; 
  float zmax() const { return zMax; }; 
  void updLsn();
  void setNI(rangerNavItem* ni) { navItem = ni; };
  rangerNavItem* getNI() const { return navItem; };
  rangerDomLay* firstDL(const int method,const int direction) const {
    return ((direction>0)? firstLay[method] : lastLay[method]); };
  rangerDomLay* entryDL(const int direction) {
    return ((direction>0)? entranceLay : exitLay); };
  vector<ArtePointer<GEDE> >& shad() { return shadows;};
  void addProxy(rangerProxyWin* wiP) { ProxyWins.push_back(wiP); };
  int setProxyLayers();
  vector<rangerProxyWin*>& proxywins() { return ProxyWins; };
  rangerDomLay* getDlByLsn(const int lsn,const int method);
  friend ostream& operator<<(ostream &os,const rangerDomWin &w);
  void dump(const int level=1);
private:
  rangerDomain* Dom;
  int Sec;
  int Cmp;
  int nShad;
  int maxLsn[3];
  rangerDomLay* firstLay[3];
  rangerDomLay* lastLay[3];
  rangerDomLay* entranceLay;
  rangerDomLay* exitLay;
  vector<ArtePointer<GEDE> > shadows;   // for debug only, thus redundancy
  vector<rangerDomLay*> DomLays;
  vector<rangerProxyWin*> ProxyWins;
  float xMin;
  float xMax;
  float yMin;
  float yMax;
  float zMin;
  float zMax;
  rangerNavItem* navItem;
};
#endif
