// 
//  Author:  Wouter Hulsbergen                 Created:  14-Aug-00
// 

// class rangerRichRing
// node for reconstructed ring in Rich

#ifndef rangerRichRing_HH
#define rangerRichRing_HH

#ifndef rangerHitBase_HH
#include "rang/HitBase.hh"
#endif

class rangerRichRing : public rangerHitBase {

public:
  // constructors, destructor, assignment
  rangerRichRing();
  rangerRichRing(ArtePointer<RSEG> rseg, rangerFitMode fm, const float radTh);
  virtual ~rangerRichRing(); 

  // declared in rangerNode
  virtual float dz() { return 0 ; }    // Thickness in z
  virtual int dim() const { return 2 ; } // Dimension of measurement

  //  virtual int plane(){ cout << "RichRing: Plane!!!" << endl; return 0; }
  int plane();

  virtual float ms() const { return (0.0136*0.0136)*fRadThick; }
  virtual float msPlain() const { return (0.0136*0.0136)*fRadThick; }
  virtual int hasHitb() { return 0; }
  virtual const ArtePointer<HITB> getHitb() const { return 0; } 
  
  // declared in rangerHitBase
  virtual void projectP(HepVector& resid,HepMatrix& HH,rangerStateP* s);
  virtual void projectPI(HepVector& resid,HepMatrix& HH,rangerStatePI* s);
  virtual void projectL(HepVector& resid,HepMatrix& HH,rangerStateL* s);
  virtual void projectLI(HepVector& resid,HepMatrix& HH,rangerStateLI* s);

private:
  ArtePointer<RSEG> fRseg ;  // Arte pointer to RSEG table
  float fRadThick;            // Radiation thickness of this node
} ;

#endif
