
#ifndef _RCFITFLAG_HH_
#define _RCFITFLAG_HH_

// *****************************************************************************
//
// source:    rcFitFlag.hh
//
// type:      header file
//
// created:   12 Dec 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Department of Physics
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// purpose:   define fit flags for RTRA tracks
//
// hints:     needs to keep compatibility to former manual assignments
//            RTRA.dd:
//
//            Dattrib FIT  I      0       'fit flag: D + 100 * E with ...'
//              Dtext                     'D = 0: reconstructed charged track'
//              Dtext                     '    1: reconstructed momentum to main vertex'
//              Dtext                     '    2: reconstructed short decay'
//              Dtext                     '    3: reconstructed long decay'
//              Dtext                     '    4: conversion (from tracks)'
//              Dtext                     '    5: space point (no angular measurements)'
//              Dtext                     '    6: neutral particle (from ECAL)'
//              Dtext                     '    7: match segment'
//              Dtext                     '    8: charged particle direction (from RICH)'
//              Dtext                     '    9: track seed (eg from TC area)'
//              Dtext                     '   10: conversion direction (from RICH)'
//              Dtext                     '   11: track fit without momentum'
//              Dtext                     'E: error flag from track or vertex fit'
//
// changes:
// 12 Dec 2000 (AG) creation
//
// *****************************************************************************

#define FIT_CHARGED     0 // reconstructed charged track
#define FIT_MAIN        1 // reconstructed momentum to main vertex
#define FIT_SHORT       2 // reconstructed short decay
#define FIT_LONG        3 // reconstructed long decay
#define FIT_CONVTRACKS  4 // conversion (from tracks)
#define FIT_SPACEPOINT  5 // space point (no angular measurements)
#define FIT_NEUTRAL     6 // neutral particle (from ECAL)
#define FIT_MATCH       7 // match segment
#define FIT_RICH        8 // charged particle direction (from RICH)
#define FIT_TCAREA      9 // track seed (eg from TC area)
#define FIT_CONVDIR    10 // conversion direction (from RICH)
#define FIT_NOMOMENTUM 11 // track fit without momentum

#endif
