
// *****************************************************************************
//
// source:    rcClassCAL1HT.C
//
// type:      source code
//
// created:   20 Jun 2000
//	
// author:    Dr. Andreas Gellrich
//            HERA-B Collaboration
//            Humboldt-University Berlin
//            Physics Department
//            Invalidenstr. 110
//            10115 Berlin
//            Germany
//            E-mail: Andreas.Gellrich@desy.de
//
// symbol:    rcClassCAL1HT
//
// called by: rcClassify
//
// purpose:   check for high-Et clusters
//
// hints:     do NOT set flagdst here!
//
// changes:
// 20 Jun 2000 (AG) creation
//  7 Jul 2000 (AG) loop over clusters
// 12 Jul 2000 (AG) set cut to 4 GeV
// 19 Jul 2000 (AG) polished code
// 13 Nov 2000 (AG) introduced cutCAL1HT which can be steered via kumac
//
// *****************************************************************************

// ARTE includes
#include "arte/arte.hh"
#include "farm/farm.hh"

// globals
float etCAL1HT  = 0;
float cutCAL1HT = 4.0; // cut on cluster transverse energy sum

//
// function code
//
int
rcClassCAL1HT()
{
  // show up
  ARTE_mark( "rcClassCAL1HT 0", 0 );
  
  // inits
  int rc   = 0;
  etCAL1HT = 0;
  
  // loop over ECAL clusters
  for( ArteTableIterator<RCCL> rccl; rccl.valid(); rccl.next() ){
    
    // transverse energy    
    float et = 
      rccl->esum *
      sqrt( rccl->x * rccl->x + rccl->y * rccl->y ) /
      sqrt( rccl->x * rccl->x + rccl->y * rccl->y + rccl->z * rccl->z );
    
    // find highest Et cluster
    if( et > etCAL1HT ) etCAL1HT = et; 
  }
  //cout << "rcClassCAL1HT: max Et=" << etCAL1HT << endl;
  
  // check transverse energy of cluster
  if( etCAL1HT > cutCAL1HT ) rc = 1;

  // show up
  ARTE_mark( "rcClassCAL1HT 1", 0 );
  
  return rc;
}
