
// **********************************************************************
//
// Routine:      rcInventRTRA.C
// Author:       C.Borgmeier, D.Samtleben
//
// Description:  select and store RTRA tracks
//
// Modification:
// 03 Jan 2001 (AG) removed 99
// 14 Sep 2001 (IR) reset to zero rtra entries during the initialization
// **********************************************************************

#include <iostream>
#include <set>
#include <math.h>

#include <arte/arte.hh>
#include <arte/RSEG_cmp.hh>

void fillRTRAfromMatch( ArtePointer<RSEG> vdsEcal,
			ArtePointer<RSEG> vdsRich )
{


  ArtePointer<RSEG> vds     = 0;
  ArtePointer<RSEG> rich    = 0;
  ArtePointer<RCCL> cluster = 0;

  if( vdsRich != 0 ) {
    for( ArteRelationIterator<RSEG> vdsIt = vdsRich->rend.begin();
	 vdsIt != vdsRich->rend.end();
	 ++vdsIt ) {
      if( ((*vdsIt)->cmp & Rsegc::vxd) != 0 )  vds = *vdsIt;
      if( ((*vdsIt)->cmp & Rsegc::rich) != 0 ) rich = *vdsIt;
    }
  }
  // override VDS segment if VDS-ECAL match exists
  if( vdsEcal != 0 ) {
    for( ArteRelationIterator<RSEG> vdsIt = vdsEcal->rend.begin();
	 vdsIt != vdsEcal->rend.end();
	 ++vdsIt ) {
      if( ((*vdsIt)->cmp & Rsegc::vxd) != 0 )  vds = *vdsIt;
    }

    ArteRelationIterator<RCCL> rcclIt = vdsEcal->rccl.begin();
    cluster = *rcclIt;
  }
  
  RTRA rtra;
  
  // empty variabales
memset((void*) &rtra,0,sizeof(RTRA));
/*
  rtra.hin = 0;
  rtra.hou = 0;
  rtra.hri = 0;
  rtra.htr = 0;
  rtra.hmu = 0;
  
  rtra.flr = 0;
  rtra.flt = 0;
  rtra.fle = 0;
  rtra.flm = 0;
  
  rtra.lmmu = 0;
  rtra.lmh = 0;

  rtra.lte = 0;
  rtra.lth = 0;

  rtra.lee = 0;
  rtra.leh = 0;
  rtra.lemi= 0;
  
  rtra.lre = 0.;
  rtra.lrmu= 0.;
  rtra.lrpi= 0.;
  rtra.lrk = 0.;
  rtra.lrp = 0.;
*/

  // --Track parameters--------------------------------

  rtra.xf  = vds->xf;
  rtra.yf  = vds->yf;
  rtra.zf  = vds->zf;
  rtra.txf = vds->txf;
  rtra.tyf = vds->tyf;
      
  const int index[5][5] = { {0, 1,  3, 6,10},
			    {1, 2,  4, 7,11},
			    {3, 4,  5, 8,12},
			    {6, 7,  8, 9,13},
			    {10,11,12,13,14}};
  
  for (int i=0; i<=3; i++){for (int j=0; j<=3; j++)
    rtra.cvf[index[i][j]] = vds->cvf[index[i][j]];};

  for (int i=0; i<=4; i++){
    rtra.cvf[index[i][4]] = 0.;
    rtra.cvf[index[4][i]] = 0.;
    //rtra.cvf[index[4][4]] = 10000.;
  };
  
  if( vdsEcal!=0 ) {
    rtra.pf  = vdsEcal->pf;
    rtra.chi = vdsEcal->chi;
  } else {
    rtra.pf  = vdsRich->pf;
    rtra.chi = vdsRich->chi;
  }
  
  rtra.cvf[index[4][4]] = (rtra.pf!=0)? (.04*.04)*(rtra.pf*rtra.pf): 10000;
  
  rtra.hsi = int(vds->time+vds->dtim);   // number of vds hits
  rtra.hca = (cluster!=0)? int(cluster->wid): 0; // cluster width
  
  ArtePointer<RTRA> Prtra = ArteTable<RTRA>::push_back( rtra );
  
  createRelation( Prtra, vds, &RTRA::rseg );

  //===== assign all OTR-tracks associated to the VDS-segment by a match  


      for( ArteRelationIterator<RSEG> vdsOtrSeg = vds->rbeg.begin();
	   vdsOtrSeg != vds->rbeg.end();
	   ++vdsOtrSeg ) {
        ArtePointer<RSEG> otr_seg = 0;

	if( ((*vdsOtrSeg)->cmp & ( Rsegc::vxd | Rsegc::patt ))
	    == ( Rsegc::vxd | Rsegc::patt ) ) {
	  
	  if( (*vdsOtrSeg)->rend.begin()==(*vdsOtrSeg)->rend.end() ) {
	    std::cerr << "no OTR seg for vdsotr " << *vdsOtrSeg << endl;
	  }
	    else{
	  ArteRelationIterator<RSEG> OtrSegIt = (*vdsOtrSeg)->rend.begin();
	  for(; OtrSegIt != (*vdsOtrSeg)->rend.end(); ++OtrSegIt ) {
	    if((*OtrSegIt)->cmp & Rsegc::patt){
         	  otr_seg = (*OtrSegIt);
                  break;
	    }
	}
	    }

	if(otr_seg!=0){
         createRelation( Prtra, otr_seg, &RTRA::rseg );
	}
      }
      }

  if( vdsEcal != 0 ) createRelation( Prtra, vdsEcal, &RTRA::rseg );
  if( vdsRich != 0 ) createRelation( Prtra, vdsRich, &RTRA::rseg );
  if( cluster != 0 ) createRelation( Prtra, cluster, &RTRA::rccl );
  if( rich != 0 )    createRelation( Prtra, rich,    &RTRA::rseg );


  Prtra->fit = 0;
}


void rcInventRTRA()
{

  std::set<ArtePointer<RSEG> > matches;

  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end();
       ++rseg ) {

    //
    // Find VDS+ECAL segments
    // and select VDS+RICH segment, which fits best.
    //
    if( ((*rseg)->cmp & ( Rsegc::vxd | Rsegc::ecal ) )
	== (Rsegc::vxd | Rsegc::ecal) ) {
      if( (*rseg)->rbeg.begin() != (*rseg)->rbeg.end() ) {
 	std::cerr << "there is rbeg for rseg " << *rseg << endl;
      }

      if( (*rseg)->rend.begin() == (*rseg)->rend.end() ) {
 	std::cerr << "no rend for rseg " << *rseg << endl;
      }
      ArtePointer<RSEG> svdSeg = *(*rseg)->rend.begin();
      
      if( (*rseg)->rccl.begin() == (*rseg)->rccl.end() ) {
 	std::cerr << "no rccl for rseg " << *rseg << endl;
      }
      ArtePointer<RCCL> clus   = *(*rseg)->rccl.begin();
      
      float rDiffSoFar = .007;
      ArtePointer<RSEG> vdsRichSoFar = 0;
      float vetx
	= ( clus->x - svdSeg->xf - (450-svdSeg->zf)* svdSeg->txf ) / 900;
      float vety
	= ( clus->y - svdSeg->yf - (-svdSeg->zf) * svdSeg->tyf ) / 1350;
     
      for( ArteRelationIterator<RSEG> vdsRichSeg = svdSeg->rbeg.begin();
	   vdsRichSeg != svdSeg->rbeg.end();
	   ++vdsRichSeg ) {
	if( ((*vdsRichSeg)->cmp & ( Rsegc::vxd | Rsegc::rich ))
	    == ( Rsegc::vxd | Rsegc::rich ) ) {
	  
	  ArteRelationIterator<RSEG> richSeg = (*vdsRichSeg)->rend.begin();
	  for( ; richSeg != (*vdsRichSeg)->rend.end(); ++richSeg ) {
	    if( ((*richSeg)->cmp & ( Rsegc::vxd | Rsegc::rich ))
		== ( Rsegc::rich ) ) {
	      float rDiff =  sqrt
		( ((*richSeg)->txf - vetx) * ((*richSeg)->txf - vetx)
		  + ((*richSeg)->tyf - vety) * ((*richSeg)->tyf - vety) );
	      if( rDiff < rDiffSoFar ) {
		rDiffSoFar = rDiff;
		vdsRichSoFar = *vdsRichSeg;
	      }
	      break;
	    }
	  }
	  if( richSeg==(*vdsRichSeg)->rend.end() ) {
	    std::cerr << "no RICH seg for vdsrich " << *vdsRichSeg << endl;
	  }
	}
      }
      matches.insert( vdsRichSoFar );
      fillRTRAfromMatch( *rseg,  vdsRichSoFar );
    }
  }
  for( ArteTable<RSEG>::iterator rseg = ArteTable<RSEG>::begin();
       rseg != ArteTable<RSEG>::end();
       ++rseg ) {
    if( ((*rseg)->cmp & ( Rsegc::vxd | Rsegc::rich ) )
	== (Rsegc::vxd | Rsegc::rich) 
	&& matches.find( *rseg ) == matches.end() ) {
      fillRTRAfromMatch( 0, *rseg );      
    }
  }
}
