#ifndef __PHOTONDETECTORENTRYWINDOW_H__
#define __PHOTONDETECTORENTRYWINDOW_H__

#include "rich/ricutil.h"
#include "rich/dielectric.h"
// ------------------------------------------------------------------------
/// description of the photon tracing through the photon detector entry window
/**
  Description of the photon tracing through the photon detector entry window
  using a rectangle for the description of the surface inside the vessel.
  A thickness is assigned to the window and a dielectric material (Plexi).
 */
class PhotonDetectorEntryWindow {
int splitted;
DiElectric *mat;
Rectangle *uw,*lw;
double    uwthickness,lwthickness;

public:
/// returns 1, if the photon has an intersection point with the window
  int intersection(Photon* pp);
/// returns 1, if no losses occure during tracing the photon to the window
  int trace(Photon* pp);
  int traceonly(Photon* pp);
/// tracing to find the ring center
  int upper_trace(StraightLine* t);
  int lower_trace(StraightLine* t);
/// returns the dielectric material, describing the window material (see also class DiElectric)
  inline const DiElectric& Material()             const { return *mat; }
/// set the dielectric material, describing the window material
  inline       void        Material(DiElectric* m)      { mat=m; }
// returns the rectangle, describing the surface of the upper window
  inline const Rectangle&  UpperWindow()          const { return *uw; }
// set the rectangle describing the surface off the upper window
  inline       void        UpperWindow(const Rectangle& r)    { 
    uw=new Rectangle(r); 
    }
// returns the rectangle, describing the surface of the lower window 
  inline const Rectangle&  LowerWindow()          const { return *lw; }
// set the rectangle describing the surface off the lower window
  inline       void        LowerWindow(const Rectangle& r)    { lw=new Rectangle(r); }
/// returns the thickness of the upper window
  inline       double      UpperWindowThickness() const { return uwthickness; }
/// set the thickness of the upper window
  inline       double&     UpperWindowThickness()       { return uwthickness; }
/// returns the thickness of the lower window
  inline       double      LowerWindowThickness() const { return lwthickness; }
/// set the thickness of the lower window
  inline       double&     LowerWindowThickness()       { return lwthickness; }
/// print the contents of the class to an ASCII file.
  void print(FILE* fp=stdout);
/// constructor reading from an ASCII-FILE produced using 'print'.
  PhotonDetectorEntryWindow(ScriptFile&);
/// destructor
  ~PhotonDetectorEntryWindow();
// constructor from Arte-Table
  PhotonDetectorEntryWindow(ArtePointer<CPDW> a);
  };

#endif

