/////////////////////////////////////////////////////
//
//  Modified:
//    26 May, 2000 (DD) Error warning if db access using max. revision
//    21 Feb, 2001 (DD) track direction from RICH MIMP
//    17 Apr, 2001 (DD) MC track direction with all MIMP photons and ring radius
//                      from RICH added
//
/////////////////////////////////////////////////////
#include "rich/rich.h"

extern "C" {
#include "table.h"
}

extern "C" {
void badrich_();
void ricini_();
}


// -------------------------------------------------------------------------
// --- Functions for RICH event processing
// --- a) initrich_() 
// --- b) exitrich_()
// --- c) dric2hitb_();
// --- d) printrich_()



float getRadius(int run_no, int verb) {

#ifdef USE_ARTE
  static int err_cnt=0;
  if (err_cnt<10) {
    ARTE_error(GRR_SWARNING," $ RICH: getRadius():  You are using an obsolete access to RICH database!!",-1);
    err_cnt++;
  }
#endif

  static int   oldRunno=-1;
  static float oldRadius=0;

  if (oldRunno==run_no) {
    if (verb>1) fprintf(stdout," $$$ RICH Warning: db not contacted, returning old radius %f\n",oldRadius);
    return oldRadius;
  }

  if (ArteTable<MTRA>::nrow()) {
    oldRadius=.0526*sqrt(RichRun::photon_fraction);
    oldRunno=run_no;
    if (verb>1) fprintf(stdout," $$$ RICH Warning: Returning MC radius %f\n",oldRadius);    
    return oldRadius;
  }

  if (run_no<1) {
    if (verb>0) fprintf(stdout," $$$ RICH Warning: Invalid run number %d, returning old radius %f\n",run_no,oldRadius);
    return oldRadius;
  }

  char dbsite[256];
  dbsite[0]=0;
  char *getdbsite=getenv("RICH_RECODB");
  if (!getdbsite) {
    fprintf(stdout," $$$ RICH Warning: RICH_RECODB not defined\n");
    return 0;
  }
  strcpy(dbsite,getdbsite);

  Tables x=OpenTables(dbsite,0);
  if (!x) {
    fprintf(stdout," $$$ RICH Warning: Cannot open database %s \n",dbsite);
    return 0;
  }

  int tx=BookTable(x,"RICH_RADIUS",0,0);
  if (!tx) {
    fprintf(stdout," $$$ RICH Warning: Cannot find a table with RICH radii\n");
    return 0;
  }

  TableStamp(x,run_no,TABLEmaxrev);
  if (!FetchTable(x,tx,TABLElt)) { 
    fprintf(stdout," $$$ RICH Warning: Cannot fatch a table for run %d \n",run_no); 
    return 0;
  }

  SkipTableField(x,1);
  int   major=TableMajor(x);
  int   minor=TableMinor(x);
  float radius,temperature,pressure,time;
  GetTableField(x,TABLEfloat,10000,&pressure);
  GetTableField(x,TABLEfloat,10000,&temperature);
  GetTableField(x,TABLEfloat,10000,&radius);
  GetTableField(x,TABLEfloat,10000,&time);
  if (verb>0) {
    printf(" major minor x TABLEint %d %d %d %d \n",major,minor,(int)x,TABLEint);
    printf("RUN %d:   P=%f  T=%f  r=%f  t=%f\n",run_no,pressure,temperature,radius,time);
  }
  if (verb>1) PrintTables(x,1);

  CloseTables(x);
  oldRunno=run_no;
  oldRadius=radius*1e-3;

  return oldRadius;
}


void initrich_() {

  if (RICH_DAQ) return;

  cout<<"$ RICH initialization: "<<endl;

#ifndef NO_ARTE
  // --- For MC use gean/ricini.C
  if (ArteTable<MTRA>::nrow()) {
    cout<<" $$$ RICH Warning:  initrich: RICH initialization for MC, now calling ricinit_() ..."<<endl;
    ricini_();
    RichRun::type=4;
    RICH_find_magnet();
    RICH_find_gede();
    printrich_();
    // -- booking RITH table
    RITH rith;
    rith.rad0=0.0526*sqrt(RichRun::photon_fraction);
    rith.par[0]=25;
    rith.par[1]=52.6*sqrt(RichRun::photon_fraction);
    rith.par[2]=-1;
    rith.par[3]=20000;
    rith.par[4]=5;
    ArteTable<RITH>::push_back(rith);
    return;
  }
#endif

  // --- Select one of three possible options for RICH initialization:
  // --- a) database, 
  // --- b) custom config files, 
  // --- c) default config files.
  //

  RichRun::type=0;
  
  char* RICHconfigdir= getenv("RICH_CONFIGDIR");
  char* RICHdatabase = getenv("RICH_INSTALLDB");
  char* RICHherab    = getenv("HB");
  char* RICHhbroot   = getenv("HBROOT");
  char *datapath=0;
  char *messageString=0;
  if (RICHdatabase) {
    RichRun::type=1;
  }
  else if (RICHconfigdir) {
    datapath=new char[256];
    datapath[0]=0;
    strcpy(datapath,RICHconfigdir);
    if (datapath[strlen(datapath)-1]!='/') strcat(datapath,"/");
    RichRun::type=2;
  }
  else if (RICHhbroot) {
    datapath=new char[256];
    datapath[0]=0;
    strcpy(datapath,RICHhbroot);
    strcat(datapath,"/GEO/RICH");
    if (datapath[strlen(datapath)-1]!='/') strcat(datapath,"/");
    RichRun::type=3;
  }
  else {
    fprintf(stdout," $$$ RICH: initrich(): initialization source not defined \n");
    fprintf(stdout," $$$                 : cannot initialize RICH classes\n");
    messageString = new char[200];
    messageString[0] = 0;
    sprintf(messageString, " $$$ RICH: initrich(): initialization source not defined \n");
    sprintf(messageString, " $$$                 : cannot initialize RICH classes\n");
    grr_error(GRR_SFATAL,messageString,1);
    exitrich_();
    //Exit(1);
  }


  // --- initialize keytables
  DAQ_INITIALIZATION::RichKeyInit();

  // --- initialize basic classes
  RICH_DAQ = new RICHDAQ(datapath);

  // --- find RICH gede number
  if (RichRun::gede==0) { 
    for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin(); 
	 igede!=ArteTable<GEDE>::end(); ++igede) {
      if ((*igede)->cmp == 5) {
	RichRun::gede=(*igede);
	break;
      }
    }
  }

#ifndef NO_ARTE
  // --- find magnet status
  ArtePointer<EVHD> evhd(1);
  if (evhd->run>0) { // DATA
    if (evhd->date>980000 && evhd->date<990408) RichRun::magnet=0;
    else RichRun::magnet=1;
  }

  // -- remove hot channels if not db initialization
  if (RichRun::type!=1) badrich_();
#endif

  printrich_();
}


void exitrich_() {
  if (!RICH_DAQ) return;
  if (RICH_DAQ)       { delete RICH_DAQ;       RICH_DAQ=0; }
  if (RICH_Vessel)    { delete RICH_Vessel;    RICH_Vessel=0; }
  if (RICH_Detector)  { delete RICH_Detector;  RICH_Detector=0; }
  if (RICH_LambdaPhi) { delete RICH_LambdaPhi; RICH_LambdaPhi=0; }
}


void printrich_() {

  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;
  cout<<"$ RICH initialization ";
  switch (RichRun::type) {
  case 1: cout<<"from database"<<endl; break;
  case 2: cout<<"from custom datafiles"<<endl; break;
  case 3: cout<<"from default datafiles"<<endl; break;
  case 4: cout<<"for MC from datafiles"<<endl; break;
  default: cout<<"not yet done"<<endl; return; break;
  };
  cout<<"$ Classes initialized:";
  if (RICH_Vessel)    cout<<" RICH_Vessel";
  if (RICH_Detector)  cout<<" RICH_Detector";
  if (RICH_LambdaPhi) cout<<" RICH_LambdaPhi";
  if (RICH_DAQ)       cout<<" RICH_DAQ";
  cout<<endl;
  cout<<"$ Run conditions:";
  if (RichRun::magnet) cout<<" magnet ON ";
  else cout<<" magnet OFF ";
  cout<<endl;
  cout<<"$ Bad channels ";
  switch (RichRun::bad_channels) {
  case 1: cout<<"removed from current DAQ file"<<endl; break;
  case 2: cout<<"removed from text file"<<endl; break;
  case 3: ;
  case 4: cout<<"removed from database, reference run = "<<RichRun::bad_channels_reference<<endl; break;
  case 5: cout<<"NOT removed - forced by user"<<endl; break;
  default: cout<<"NOT removed"<<endl;
  }
  if (RichRun::type==1) DAQ_INITIALIZATION::PrintUsedTables();
  cout<<"$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$$"<<endl;

}

void dric2hitb_() {
  if  (ArteTable<MTRA>::nrow()) return;
  if  (!RICH_DAQ) initrich_();
  RICH_DAQ->fillHITBfromDRIC();
}

void hidric_(int n) {
  n++; // dummy n
  dric2hitb_();
}


void RICH_find_magnet(){

  // -- find magnet status
  for (ArteTable<GESL>::iterator igesl=ArteTable<GESL>::begin();
       igesl!=ArteTable<GESL>::end(); ++igesl) {
    if ((*igesl)->cmp==12) {
      RichRun::magnet=(*igesl)->mfl;
      break;
    }
  } 
}

void RICH_find_gede() {
  //  -- find rich gede pointer
  for (ArteTable<GEDE>::iterator igede=ArteTable<GEDE>::begin();
       igede!=ArteTable<GEDE>::end(); ++igede) {
    if ((*igede)->cmp==5) {
      RichRun::gede=(*igede);
      break;
    }
  }
}





//
//  find MC track direction from a MIMP
//
Hep3Vector getTrackDir(ArtePointer<MIMP> mimp, double &theta) {

  if (!RICH_DAQ) initrich_();

  // --- get photon from MIMP
  Vec3 ph_dir(  mimp->tx,  mimp->ty,   mimp->p);
  double energy=norm(ph_dir);
  ph_dir /= energy;	
  Vec3 ph_pol = FromPolar(1,mimp->time,mimp->q);


  // --- find Cherenkov angle	
  double lambda           = energy2lambda(energy);
  double refraction_index = RICH_Vessel->Radiator().RefractiveIndex(lambda);
  ArtePointer<MTRA> mtra  = mimp->mtra;
  double P=sqrt(mtra->px*mtra->px+mtra->py*mtra->py+mtra->pz*mtra->pz);
  double mass=mtra->topp->mass;
  double beta=P/sqrt(P*P+mass*mass);
  theta = (beta*refraction_index>1) ? acos(1/(beta*refraction_index)) : 0;
  

  // --- find rotation angle, axis
  double rotation_angle = -theta;
  Vec3   rotation_axis  = normalize(cross(ph_dir, ph_pol));


  // --- make rotation using Hep functions
  Hep3Vector n(ph_dir.x(),ph_dir.y(),ph_dir.z());
  Hep3Vector p(ph_pol.x(),ph_pol.y(),ph_pol.z());
  Hep3Vector t(rotation_axis.x(), rotation_axis.y(), rotation_axis.z());
  Hep3Vector new_n = n;
  new_n.rotate(rotation_angle, t);

  return new_n;
}



//
//  find MC ring from a MTRA
//
void mcRICH(ArtePointer<MTRA> t, float &tx, float &ty, float &rad) {

  if (!RICH_DAQ) initrich_();

  tx=ty=rad=0;

  int n0=RichRun::gede->mim1.index();
  int n1=RichRun::gede->miml.index()+1;
  if (n1<=n0) return;

  Hep3Vector dir;
  rad=0;
  int nh=0;
  for (int i=n0; i<n1; i++) {
    ArtePointer<MIMP> mimp(i);
    if (mimp->mtra!=t) continue;
    double            theta;
    dir += getTrackDir(mimp,theta);
    rad += theta;
    nh++;
  }
  if (!nh) return;
  tx=dir.x()/dir.z();
  ty=dir.y()/dir.z();
  rad/=nh;
}
