// Implementation file for the HCUT class library
// =========================================================
// Author: Martin Braeuer 
//
// NOTE: - This should run with the old and new gede versions
//       - One can cut on Helixes or regions (2do)
//
// Modification history:
// MBr  Feb.99    : Initial ideas of MBr and Stefan Scharein
// MBr  17.02.2000: Class Version 0.9
// MBr  01.03.2000: Class Version 0.9 final
// GS   02.07.2001: Class Version 1.0 (added new contsr HCUT_class(char *state, int dummy)                                    
// MBr    .  .    :
//                                  
// =========================================================

#include <stdio.h>
#include <stdlib.h>
#include  <string.h>
#include  <math.h>
#include  <malloc.h>
#include  <iostream.h>

#include "silarte/Class_HCUT.h"


#define H_WORKING 1
#define H_LOW     0

#define HCUT_FILE 1
#define HCUT_DB   0

// output an std error:
void HCUT_class::fError_out(const char *s) {
  fprintf(stderr,"VDS/HCUT: '%s'\n",s);
}


/* ==================================================================
 * == HCUT_class constructor : std constructor                     ==
 * == needs : ---                                                  ==
 * == Result : Failure message because of missing HOLMES pointer   ==
 * ==                                                              ==
 * ================================================================== 
 */
HCUT_class::HCUT_class() {
  fError_out("Default constructor call failed!");
  return;
}

HCUT_class::HCUT_class(char file[]) {
  FILE   *infile;
  char   line[80], *token,outs[80];
  int    usid,sideid;
  int    chip[10];

  iFoundOne = 0;
  printf("Class_HCUT init: bad VDS Helix map read in from file %s\n",file);
  fReadFile(file);
  return;
}

HCUT_class::HCUT_class(char *state, int dummy) {
//  printf("address received by HCUT = %p\n", state);
  iFoundOne = 0;
  printf("Class_HCUT init: bad VDS Helix map read in from DBS\n");
  // reset array to 'working':
  fClear();
  for (int i=0; i<127; i++) {
    for (int j=0; j<10; j++) {
      hstate[i][j] = *state;
//      printf("usid=%i; chip=%i; status=%i\n", i, j, (int) hstate[i][j]);
      state++;
    }
  }
  for (int i=0; i<127; i++) {
    for (int j=0; j<10; j++) {
      if (!hstate[i][j]) {
	iFoundOne = 1;
	return;
      }
    }
  }
  return;
}
 
/* ==================================================================
 * == fReadFile :  Read a simple text file with the bad chips      ==
 * == needs  : --                                                  ==
 * == pars   : char filenam[] Name of file                         ==
 * == return : 1 if ok and at least one found.                     ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fReadFile(char filenam[]) {
  FILE   *infile;
  char   line[80], *token,outs[80];
  int    usid,sideid;
  int    chip[10];

  // check file:
  if( (infile = fopen(filenam,"r")) == NULL) {
    fError_out("Filename invalid");
    return(0);
  }
  
  // reset array to 'working':
  fClear();

  // Cutted on NO chip:
  iFoundOne = 0;
  sideid = 0;
  for (int ch=0;ch<10;chip[ch++]=-10);
  int nsides =0;
  while( fgets(line, 80, infile) ) {
    if( (token = strtok(line," \n")) == NULL) continue;
    if (*token == '#') continue;
    usid= atoi(token); token = strtok(NULL," \n"); // read side == first

    if(token == NULL) continue; // nothing m,ore ??
    sideid= atoi(token); token = strtok(NULL," \n"); // read side == first
    //printf("2s=%3d  %d \n",usid,sideid);

    if ((usid<0) || (usid>127)) {// wrond USID
      sprintf(outs,"ERROR USID wrong (0..127) is %3d, side=%d.",usid,sideid);
      fError_out(outs);
      continue;
    }
    // catch for pure txt lines:
 
    ++nsides;
    for (int ch=0;ch<10;++ch) {
      if ((token == NULL) || (*token == '#')) break;
      chip[ch]  = atoi(token);token = strtok(NULL," \n");
      
      //printf("s=%3d  %d %d\n",usid,sideid,chip[ch]);
      if((chip[ch]<0) || (chip[ch]>9)) {
	sprintf(outs,"ERROR side %3d (%d) chip number in textfile not (0..9) is %d.",
		usid,sideid,chip[ch]);
	fError_out(outs);
	continue;
      }
      if(hstate[usid][chip[ch]] != H_WORKING) {
	sprintf(outs,"Consistency problem: chip %d of side=%3d (%d)  already flagged!",
		chip[ch],usid,sideid);
	fError_out(outs);
	continue;
      }
      iFoundOne = 1;
      hstate[usid][chip[ch]] = H_LOW;
    }
  }
  printf("Class_HCUT init: Found %d sides with  'properties'\n",nsides);
  return(iFoundOne);
}

/* ==================================================================
 * == fCutSide :  check if hit is on a doubtfull                   ==
 * == needs  : Class inited                                        ==
 * == pars   : sl=superlayer, sec=sector, lay=layer (1..count)     ==
 * == return : 0 if hit to be rejected, 1 else                     ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fCutSide(int sl,int sec,int lay, float cog) { 
//  if (!iFoundOne) return(1);
  int usid = (lay-1)+(4*(sec-1))+(16*(sl-1)); 

  // check:
  if (hstate[usid][fHelixid(usid % 4,cog)] == H_WORKING) 
    return(1);
  else
    return(0); 
}


/* ==================================================================
 * == fCutSide :  check if hit is on a doubtfull chip              ==
 * == needs  : Class inited                                        ==
 * == pars   : ARTE ID == 100*sl (1..7) +10*sect (1..4) + lay      ==
 * == return : 0 if hit to be rejected, 1 else                     ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fCutSide(int side, float cog) { 
//  if (!iFoundOne) return(1);
  int psid,unsin;
  if ((side<0) || (side>61000)) return(0); // hits bad ANYHOW if its ID out of range!!
  if (side>127)
    if (side < 20000)
      unsin = (side%10) + 4*((side%1000)/300) + 16*(  int(side/10)%10); 
    else 
      unsin = (side%10) + 4*((side%1000)/300) + 16*(1+int(side/10000)); 	   
  else 
    unsin = side;
//  printf("si=%d %d   %8.3f c%d p%d s%d\n",side,unsin,cog,fHelixid(unsin % 4,cog),unsin % 4,hstate[unsin][fHelixid(unsin % 4,cog)]);
  // check:
  if (hstate[unsin][fHelixid(unsin % 4,cog)] == H_WORKING) 
    return(1);
  else
    return(0); 
}
int HCUT_class::fCutSide(int side, int iChip) { 
  if (side > 127 || side < 0 || iChip > 9 || iChip <0)
    return(0);
  if (hstate[side][iChip] == H_WORKING) 
    return(1);
  else
    return(0);  
}

/* ==================================================================
 * == fDumpTable :  print out the internal Table                   ==
 * == needs  : Class inited                                        ==
 * == pars   : --                                                  ==
 * == return : 1                                                   ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fDumpTable() { 
  int ok=0,j;
  printf("State Table:\n");
  if (!iFoundOne) {
    printf("NO cuts!\n");
    return(1);
  }
  for (int i=0;i<127;++i) {
    ok=1;
    for(j=0;j<10;++j)
      if (hstate[i][j] != H_WORKING) 
	ok=0;
    if (ok==0) {
      printf("%3d : %d : ",i,fUnsinSide(i));
      for(j=0;j<10;++j)
	if (hstate[i][j] != H_WORKING) 
	  printf("%d  ",j);
      printf("\n");
    }
  }
  printf("\n");
  return(1); 
}

/* ==================================================================
 * == fCutARTE :  check if hit is on a doubtfull chip              ==
 * == needs  : Class inited                                        ==
 * == pars   : ARTE ID == 100*sl (1..7) +10*sect (1..4) + lay      ==
 * == return : 0 if hit to be rejected, 1 else                     ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fCutARTE(int arte, float cog) { 
  if (!iFoundOne) return(1);
  if ((arte<0) || (arte>844)) return(0); // hits bad ANYHOW if its ID out of range!!
  int usid = 16*((arte/100)-1) + 4*(((arte%100)/10)-1) + ((arte % 10)-1);
  
  // check:
  if (hstate[usid][fHelixid(usid,cog)] == H_WORKING) 
    return(1);
  else
    return(0); 
}



/* ==================================================================
 * == fClear : Clear al HELIX states internally to H_WORKING       ==
 * ==                                                              ==
 * ================================================================== 
 */
void HCUT_class::fClear() {
  // reset array to 'working':
  for(int i=0;i<128;++i)
    for(int j=0;j<10;++j)
      hstate[i][j] = H_WORKING;
}

/* ==================================================================
 * == fUnsinSide :  Calculate side no out of 7bit ID of side       ==
 * == needs  : --                                                  ==
 * == pars   : The USID (0..127)                                   ==
 * == return : side ID                                             ==
 * ==                                                              ==
 * ================================================================== 
 */
int HCUT_class::fUnsinSide(int unsin) {
  if ((unsin<0) || (unsin>127)) return(0);
  int lay = unsin%4; // layer
  int sec = 300*((unsin/4)%4); // sector
  int sla = unsin/16; // superlayer
  int cap = (sla < 3) ? 10*sla : 0;
  sla = (sla>3)? sla-1 : 1;
  return(10000*sla + sec + sla + lay);
}













