// ========================================================
// Modification history:
//
// 01.07.2001 (G. Sciacca) If requested, handles VDS bad Helix masking by calling fillHITBfromSPARSEVTmask();
//
// ========================================================
#include <stdio.h>
#include <iostream.h>
#include <iomanip.h>
#include <strings.h>
#include <stdlib.h>

// some Arte stuff
#include "arte/arte.hh"
#include "farm/farm.hh"
#include "silarte/silarte.hh"
#include "silarte/EVinfo.hh"

// prototype declarations
#include "aframelib.h"
#include "vdsleda.h"
#include "silraw.h"
#include "marvin.h"
// #include "vdsrhp.h"

// return codes
#include "grr.h"

// sparse event definition
#include "event.h"
#include "etb.h"

//-----------------------------------------------------------------------------

int SvdEvnt()
{
  int  rc;

  // size and location of sparsified event
#define SEVT_INIT_SIZE    0x7FFF     // size of event buffers 
  static char sparsevt[SEVT_INIT_SIZE];
  Marvin_Sevt_t *sevt;

  char *routine = "SvdEvnt";
  static bool  firsttime  = true;

  //
  // No SVD Digitization in case of MC  
  //
  if ( CheckMC() )
    return GRR_OK;

  //
  // REAL DATA
  //

  // get event pointer
  EVENT_HDR *event = ARTE_get_event();
  if ( !event ) {
    aframe_mess(routine,GRR_SERROR,"Could not find event!");
    return GRR_FISHY;
  }      

  // T.Glebe: the stuff from here moved to SvdStartup()

  // ---------------------
  // Normal event handling
  // ---------------------

  // Call silraw - check for correct format handling first
  if ( silraw_iol.rawdata_type.value != 0 ) {
    aframe_mess(routine,GRR_SERROR,
		"Silraw raw data type not EVENT format - check IOL parameter!");
    return GRR_FISHY;
  }
  rc = silraw_event( (char*) event, 0 );
  if ( rc != GRR_OK )
    return GRR_FISHY;
  
  // Do cluster finding
  sevt = marvin_fetch_sevt();
  if ( !sevt ) {
    aframe_mess(routine,GRR_SERROR,
		"no Marvin_Sevt_t available (marvin not initialized?)");
    return GRR_FISHY;
  }
  sevt->data = sparsevt; 
  sevt->size = SEVT_INIT_SIZE;
  sevt->next = sevt->data; 
  marvin_event();

  // Bad Helix chip masking OFF (default)?
  char defaultmasking[6];
  sprintf(defaultmasking,"OFF");
  char * masking;
  if (getenv("VDS_MASKING") == NULL){
    masking = defaultmasking;
  } else {
    masking = getenv("VDS_MASKING");
  };

  // Fill HITB table
  if (!strcmp(masking, "OFF")) {
    if(firsttime == true) {
      printf("\n");
      aframe_mess(routine,GRR_SINFO,
                "VDS masking switched off: no masking of bad Helix chips will take place\n");
      firsttime = false;
    }
    fillHITBfromSPARSEVT();
  }
  else {
    fillHITBfromSPARSEVTmask();
  }
#ifdef XXX
  // Call RHP monitoring
  rc = vdsrhp_event( event );
  if ( rc != GRR_OK )
    return GRR_FISHY;
#endif

  return GRR_OK;

}
