#include <stdio.h>
#include <stdlib.h>
#include <signal.h>
#include <string.h>
#include <sys/stat.h>

#include "trigger_format.h"
#include "l2trigtab.h"
#include "rpm.h"
#include "grr.h"
#include "errno.h"

extern char Key[20];
extern int errno;
extern int  nArg;
extern char **vArg;

static int MakeST(int ** l2st,int * length, int major, int minor);

  /*****************
   *
   * L2STFormat:
   * Function called by the trigger table formater handler
   * to generate the symbol formatted table for the SLP.
   * Options to produce ASCii and/or BINARY file dumps
   * are available via compilation flags DUMP_ASCII/BINARY.
   * The DUMP_NODE flag should be used ONLY online !
   *
   ***/

int L2STFormat(int ** l2st,int * length, int major, int minor)
{
  int *l2formst;
  int size;
  char mess[200];


  if(MakeST(&l2formst,&size, major, minor)!=0){
     sprintf(mess,"L2STFormat: MakefST failed !\n");
     grr_error(GRR_SFATAL,mess,1);
     return 1;
  }

  *l2st = l2formst;
  *length = size;

  /*printf("L2STFormat: size of symbol table %d \n", *length);*/

  return 0;
}


static int MakeST(int ** l2st,int * length, int major, int minor) {

  L2SYMBOL *theSym;
  int nEntries;
  int rc;
  L2FORMTHEAD *header;
  char *wrPtr;
  int flag = 0;
  int dump = 0;
  int i;
  char mess[200];
  FILE *fdout;
  char fName[100];
  char ModeName[10];
  
  // TERESA: the key has to be different in case of running offline (0 online)

  rc = L2TS_ReadFromDB("/SLT_SETUP", 0, &theSym, &nEntries, major, minor);

  *l2st = (int*) malloc(sizeof(L2FORMTHEAD)+nEntries*sizeof(L2SYMBOL));
  *length = sizeof(L2FORMTHEAD)+nEntries*sizeof(L2SYMBOL);

  if(l2st != NULL){
     header = (L2FORMTHEAD *) *l2st;
     header->nentries = nEntries;
     wrPtr = (char*) (header + 1);
     memcpy((L2SYMBOL*)wrPtr, theSym, sizeof(L2SYMBOL)*nEntries);
     flag = 0;
  }
  else{
    sprintf(mess,"L2STFormat/MakeST: 
                  couldn't allocate space for symbol table \n"); 
    grr_error(GRR_SFATAL,mess,0);
    flag = 1;
    return flag;
  }
  
  /* decode the SltDumpMode to be used for the L2tabs */
  for( i=1; i<nArg; i++ ){
    if( !strncmp( vArg[i], "-sltdumpmode:", 13 ) ){
      strcpy(ModeName, vArg[i]+13);
      sprintf(mess,"L2FormArte/MakeST: sltdumpmode %s\n",ModeName);
      grr_error(GRR_SINFO,mess,0); 
    } 
  }
  if (!strncmp(ModeName,"ASCII",5)){
  
    for( i=1; i<nArg; i++ ){
      if( !strncmp( vArg[i], "-sltdumpdir:", 12 ) ){
	dump++;
	strcpy(fName, vArg[i]+12);
	sprintf(mess,"\nL2STFormat/MakeST: dumping the L2 Symbol Table (l2st.txt) to %s\n",fName);
	grr_error(GRR_SINFO,mess,0); 
      } 
    }
    if (dump == 0) {
      sprintf(mess,"\nL2FormArte/Write_BuffMapAscii: Couldn't read address of sltdumpdir directory\n");
      grr_error(GRR_SERROR,mess,0);
    } 
    strcat(fName,"l2st.txt_");
    strcat(fName,Key);
    umask(0111);

    /*    open file for symbol table trigger information: ---------------- */
    fdout = fopen(fName, "w");
    if( fdout == NULL) {
      sprintf(mess,"L2FormArte/MakeST: unable to open %s\n",fName);
      grr_error(GRR_SERROR,mess,1);
      sprintf(mess,"L2FormArte/MakeST: error %s\n",strerror(errno));
      grr_error(GRR_SERROR,mess,1);
      flag = 1;
    } else {
      rc = L2TS_WriteToFile(fName, theSym, nEntries);
      
      if(rc != GRR_OK) {
	sprintf(mess,"L2FormArte/L2STFormat: Problems to write ASCii file (l2st.txt)");
	grr_error(GRR_SERROR,mess,0);
      } else {
	sprintf(mess,"L2FormArte/L2STFormat: ASCii file (l2st.txt ) generated");
	grr_error(GRR_SINFO,mess,0);
      }  
      
      fclose(fdout);
      flag = 0;
    }
  } else {
    sprintf(mess,"L2FormArte/MakeST: NO SLT symbol table will be dumped !!!\n");
    grr_error(GRR_SINFO,mess,0);
  }
  return flag;
  
}
 



