//*****************************************************************************
//
// source:    matchrtra.C
//
// type:      source code
//
// created:   14 Nov 2000
//      
// author:    Brian Petersen
//            E-mail: aagaard@nbi.dk
//
// symbol:    MatchSltRTRA
//
// called by: matchslt
//
// purpose:   Match SLT track with RTRA
//
// hints:
//
// changes:
// 14 Nov 2000 (BP) Creation
// 29 Nov 2000 (AG) added function name to printout
// 30 Jan 2001 (BP) Check if track is accepted
// 05 Jun 2001 (BP) Only match with long tracks
//
//*****************************************************************************
#include <iostream.h>
#include "arte/arte.hh"
#include "arte/RSEG_cmp.hh"
#include "sltarte/matchslt.hh"

#include "sltinfo.h"

#define POW(x) ((x)*(x))
#define OTRRESOLUTION 0.08      //Given by the SLT tracks.

 /* trk is trackindex from SLT_TRIGGER */

void MatchSltRTRA(int trk,ArtePointer<RTRA> &track,float &xchi,float &ychi,float &xrchi,float &yrchi){

  SLT_SILITRACK sili;
  SLT_SILITRACKCOV cov;
  SLT_REFITTRACK refit;
  SLT_Track info;

  track=0;
  xchi=ychi=999999.;
  xrchi=yrchi=999999.;

  if (SLTTrackInfo(trk,&info)!=SLT_OK) {
     cout<<"MatchSltRTRA: Error getting track info for track";
     cout<<trk<<endl;
     return;
  }
  if (info.status!=SLTTRACK_ACCEPT) return;

  if (SLTSiliTrackInfo(trk,&sili)!=SLT_OK) {
    cout<<"MatchSltRTRA: Error getting L2sili track for track";
    cout<<trk<<endl;
    return;
  }
  if (SLTRefitTrackInfo(trk,&refit)!=SLT_OK) {
    cout<<"MatchSltRTRA: Error getting Refit track for track";
    return;
  }
  if (SLTSiliTrackCovInfo(trk,&cov)!=SLT_OK) {
    cout<<"MatchSltRTRA: Error getting covariance matrix for track";
    cout<<trk<<endl;
    return;
  }
  for( ArteTable<RTRA>::iterator rtra = ArteTable<RTRA>::begin();
       rtra != ArteTable<RTRA>::end(); ++rtra ) {

    if ((*rtra)->csvd>00&&(*rtra)->cmain>0 ) { // Track in the VDS and OTR/ITR?
      double cix,ciy,rcix,rciy;
      float z=(*rtra)->zf;
      float x=sili.bx+sili.tx*z;
      float y=sili.by+sili.ty*z;
	
      float etx=cov.xcov[0];
      float ety=cov.ycov[0];
      float ex=cov.xcov[0]*z*z-2*z*cov.xcov[1]+cov.xcov[2];
      float ey=cov.ycov[0]*z*z-2*z*cov.ycov[1]+cov.ycov[2];
	
      cix  = POW(sili.tx-(*rtra)->txf)/(etx+(*rtra)->cvf[5]);
      cix += POW(x-(*rtra)->xf)/(ex+(*rtra)->cvf[0]);
      ciy  = POW(sili.ty-(*rtra)->tyf)/(ety+(*rtra)->cvf[9]);
      ciy += POW(y-(*rtra)->yf)/(ey+(*rtra)->cvf[2]);


      
      ArtePointer<RPNT> rpnt=0;
      z=1500;
      // Find RPNT closest to PC1,
      for(ArteRelationIterator<RPNT> itr=(*rtra)->rpnt.begin();
	  itr != (*rtra)->rpnt.end(); ++itr ) {
	if ((*itr)->zf<z&&(*itr)->zf>680) {
	  rpnt=*itr;
	  z=rpnt->zf;
	}
      }
      z=(rpnt)->zf;
      x=refit.bx+refit.tx*z;
      y=refit.by+refit.ty*z;
      
      rcix =  POW(refit.tx-(rpnt)->txf)/POW(OTRRESOLUTION/70);
      rcix += POW(x-(rpnt)->xf)/POW(OTRRESOLUTION);
      rciy =  POW(refit.ty-(rpnt)->tyf)/POW(OTRRESOLUTION/70*12);
      rciy += POW(y-(rpnt)->yf)/POW(OTRRESOLUTION*12);
      
      if (cix+ciy+rcix+rciy<xchi+ychi+xrchi+yrchi) {
	xchi=cix;
	ychi=ciy;
	xrchi=rcix;
	yrchi=rciy;
	track=*rtra;
	}
    }
  }
}
