//*****************************************************************************
//
// source:    matchslt.C
//
// type:      source code
//
// created:   14 Nov 2000
//
// author:    Brian Petersen
//            E-mail: aagaard@nbi.dk
//
// symbol:    MatchSLT
//
// called by: 
//
// purpose:   Evaluate SLT triggers and return tracks triggered on
//
// hints:     
//
// changes:
// 14 Nov 2000 (BP) Creation
// 29 Nov 2000 (AG) added function name to printout
//
//*****************************************************************************
#include <iostream.h>

#include "arte/arte.hh"
#include "sltinfo.h"
#include "sltarte/matchslt.hh"

#define VDSCUT 55.0
#define OTRCUT 55.0

float MatchSLT(ArtePointer<RTRA> &track1,ArtePointer<RTRA> &track2,TrigType &type){
  track1=0;
  track2=0;
  float best=100000000.;

  type=NO_SLT_TRIGGER;
  if (SLTReadDSLT()!=SLT_OK) {
    //cout<<"MatchSLT: No SLT information found"<<endl;
    return -1.0;
  }
  else {  
    SLT_STATUS stat;
    SLT_TRIGGER trig;

    if (SLTStatus(&stat)!=SLT_OK) 
      return -1.0;
    if (!(stat.status&SLTEVENT_ACCEPT)) 
      return -1.0;
    
    type=GHOST;

    float chix[2],chiy[2];
    float rchix[2],rchiy[2];
    TrigType typ=SINGLE_E;

    for(int i=0;i<stat.ntriggers;i++) {
      if (SLTTrigInfo(i,&trig)!=SLT_OK) {
        cout<<"MatchSLT: Error getting info on trigger number "<<i<<endl;
        continue;
      }
      if (trig.status==SLTTRIG_REJECT) 
        continue;
      
      ArtePointer<RTRA> rtrk[2];
      float totalchi=0;
      SLT_SEEDTRACK seed;
      for(int j=0;j<trig.ntracks;j++) {
	MatchSltRTRA(trig.trackindex[j],rtrk[j],chix[j],chiy[j],rchix[j],rchiy[j]);
	if ((chix[j]<VDSCUT||chiy[j]<VDSCUT)&&rchix[j]<OTRCUT&&rchiy[j]<OTRCUT) {
	  totalchi+=chix[j]+chiy[j]+rchix[j]+rchiy[j];

	  if( SLTSeedTrackInfo( trig.trackindex[0], &seed) != SLT_OK) {
	    totalchi=100000000.;
	    continue;
	  }
	  if (seed.trackID==EPLUS||seed.trackID==EMINUS) {
	    if (j==0) {
	    	if (chix[j]<VDSCUT&&chiy[j]<VDSCUT)
	          typ=SINGLE_E;
		else
		  typ=SINGLE_E_LOOSE;
	    } else {
	      if (chix[j]<VDSCUT&&chiy[j]<VDSCUT&&typ==SINGLE_E)
	        typ=PAIR_E;
	      else
	      	typ=PAIR_E_LOOSE;
	    }
	  }
	  if (seed.trackID==MUPLUS||seed.trackID==MUMINUS) {
	    if (j==0) {
               if (chix[j]<VDSCUT&&chiy[j]<VDSCUT)
	         typ=SINGLE_MU;
               else
		 typ=SINGLE_MU_LOOSE;
	    } else {
              if (chix[j]<VDSCUT&&chiy[j]<VDSCUT&&typ==SINGLE_MU)
                typ=PAIR_MU;
              else
                typ=PAIR_MU_LOOSE;
	    }
          }
	}
	else // Track was not matched to my satisfaction
	  totalchi=200000000.;
      }

      if (totalchi<best) {
	best=totalchi;
	track1=rtrk[0];
	type=typ;
	if (trig.ntracks>1)
	  track2=rtrk[1];
      }
    }
  }
  if (best<100000000.)
    return best;
  else
    return -1.0;
}
