// *****************************************************************************
//
// source:    L3_Origin.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include <list.h>
#include <stdio.h>

#include "L3_Parameter.hh"
#include "L3_SLP.hh"
#include "L3_Vtx.hh"
#include "L3_Origin.hh"
#include "L3_Impact.hh"

#ifdef L3_ARTE_FRAME
#include "L3_Arte.hh"
#endif

vector<L3_Track>::iterator L3_Origin::BestMatch(L3_Track &tofind,vector<L3_Track> &tr,float *dist)
{
  
  float dummy;
  if(dist==NULL) dist = &dummy;
  vector<L3_Track>::iterator result = tr.end();
  *dist = -1;
  for(vector<L3_Track>::iterator i=tr.begin();i!=tr.end();i++)
    {
      float newdist = TrackDist(*i,tofind);
      if( *dist<0 || newdist<*dist )
	{
	  *dist = newdist;
	  result = i;
	}
    }
  return result;
}

const int ntmin = 3;        //at least three tracks on a primary vertex
const float scale = 0.024;   //scale is 240 microns

class TrkInfo {
public:
  TrkInfo(L3_Track* a,float longi,float verti) : t_track(a),t_longi(longi),t_verti(verti) {}
  TrkInfo() : t_track(NULL),t_longi(0.0),t_verti(0.0) {}    
  TrkInfo(const TrkInfo &a){ operator=(a); }
  TrkInfo& operator = (const TrkInfo &a)
    {
      t_track = a.t_track;
      t_longi = a.t_longi;
      t_verti = a.t_verti;
      return *this;
    }
  float longi() const { return t_longi; }
  float verti() const { return t_verti; }
  L3_Track* track() const { return t_track; }
private:
  L3_Track* t_track;
  float t_longi,t_verti;
};

ostream& operator<<(ostream &o,const TrkInfo &a)
{
  //o << a.longi() << ',' << a.verti() << '[';
  o.form("(%.0f,%.0f)",a.longi()*1E4,a.verti()*1E4);
  o << '[';
  if(a.track()==NULL) o << "NULL"; else o << a.track()->nr();
  o << ']';
  return o;
}

class CL_lessLongi : public binary_function< TrkInfo, TrkInfo, bool > {
public:
  inline
  bool operator() (const TrkInfo& R , const TrkInfo& S) const {
    return ( R.longi() < S.longi() ); 
  }
};

class Cluster {
public:
  Cluster(){}
  Cluster(int p) : target(p) {}
  Cluster(const Cluster &a){ operator=(a); }
  void add(const TrkInfo &a){ data.push_back(a); }
  unsigned int Ntracks() const { return data.size(); }
  Cluster& operator=(const Cluster& a)
    {
      data = a.data;
      target = a.target;
      return *this;
    }
  vector<TrkInfo> getUniqueTracks(const Cluster &a) const {  
    vector<TrkInfo> result;
    for(vector<TrkInfo>::const_iterator i = data.begin();i!=data.end();i++) 
      {
	L3_Track *tr1 = i->track(); 
	int exists = 0;
	for(vector<TrkInfo>::const_iterator ia = a.data.begin();ia!=a.data.end();ia++) 	
	  {
	    L3_Track *tr2 = ia->track();
	    if( (void*)tr1 == (void*)tr2 ){ exists=1; break; }
	  }
	if(!exists) result.push_back( *i );
      }
    return result;
  }
  vector<TrkInfo> data;  
  int target;
};

class CL_greater : public binary_function< Cluster, Cluster, bool > {
public:
  inline
  bool operator() (const Cluster& R , const Cluster& S) const {
    return ( R.Ntracks() > S.Ntracks() ); 
  }
};

ostream& operator<<(ostream &o,const Cluster &a)
{
  o << L3_Geometry::getTarget(a.target).name() << ' ' << a.data.size() << " : ";
  for(vector<TrkInfo>::const_iterator i = a.data.begin();i!=a.data.end();i++) 
    {
      o << *i << ' ';
    }
  o << endl;
  return o;
}

void DoCluster(vector<TrkInfo>& List,int start,int end,vector<Cluster> &result,int target)
{
  //cout << "DoCluster " << start << " .. " << end << endl;
  float maxDiff = (List[end].longi() - List[start].longi() + scale ) * 10000;
  float diff = maxDiff;
  int von = 0;
  for(int k=start;k<end;k++)
    {
      float d = List[k+1].longi() - List[k].longi();
      if( d<diff )
	{
	  von = k;
	  diff = d;
	}
    }
  if( diff<scale )
    {
      int bis = von+1;
      do {
       float low = maxDiff;
       float high = maxDiff;
       if( von-1>=start )
	 {
	   low = List[von].longi()-List[von-1].longi();
	   //cout << "low " << low << endl;
	 }
       if( bis+1<=end ) 
	 {
	   high = List[bis+1].longi()-List[bis].longi();
	   //cout << "high " << high << endl;
	 }
       //cout << "low=" << low << ",  high=" << high << ",  diff=" << diff << endl;
       if( low<=high ){
	 if(diff + low < scale ){
	   diff += low;
	   von--;
	 } else 	   
	   break;	 
       } else {
	 if(diff + high < scale ){
	   diff += high;
	   bis++;
	 } else break;
       }       
      } while(1);
      if( bis-von+1 >=ntmin )
	{
	  Cluster c(target);
	  //cout << "Cluster ";
	  for(int h=von;h<=bis;h++)
	    {
	      //cout << h << '=' << List[h].longi() << ' ';
	      c.add(List[h]);
	    }
	  //cout << endl;	  
	  result.push_back(c);
	}
      if( start < von-1 ) DoCluster(List,start,von-1,result,target);
      if( bis+1 < end   ) DoCluster(List,bis+1,end,result,target);
    }
}
 
void L3_Origin::MYreco(vector<L3_Track> &tr)
{
  if( tr.size() < 1 ) return;
  int N = L3_Geometry::nTarget();
  vector<Cluster> clusters;

  for(int i=0;i<N;i++) //loop over all wires 
    {      
      vector<TrkInfo> cand_list;
      const L3_Target &targ = L3_Geometry::getTarget(i);
      if(L3_Parameter::Info()) cout << "Target " << targ.name() << ' ' << targ.z() << ' ' << endl;
      for(vector<L3_Track>::iterator tra = tr.begin();tra!=tr.end();tra++)      
	{	  	  
	  tra->kalTransport(targ.z());
	  HepVector d(3);
	  d[0] = tra->x(); 
	  d[1] = tra->y();
	  d[2] = targ.z();
	  targ.distance(d);
	  float d0 = fabs(d[0]);
	  float d1 = fabs(d[1]);
	  int take = 0;
	  float v=0.0,h=0.0;
	  if( targ.dx() > targ.dy() ){  //horizontal wire
	    if( d0<targ.dx() && d1<L3_Parameter::SearchBandSize() ){ take = 1; h=d[0]; v=d[1]; }
	  } else { 
	    if( d1<targ.dy() && d0<L3_Parameter::SearchBandSize() ){ take = 1; h=d[1]; v=d[0]; }
	  }
	  if(take)
	    {
	      if(L3_Parameter::Info())
		cout << " " << tra->nr() << ' ' << d[0] << ' ' << d[1] << ' ' << h << endl;
	      //cout << " " << tra->MinDistanceToTarget(targ).mag() << " away from target" << endl;
	      cand_list.push_back( TrkInfo(&(*tra),h,v) );
	    }	
	}
      //do the clustering on this wire
      sort(cand_list.begin(),cand_list.end(),CL_lessLongi());      
      int n = cand_list.size();
      if(n-1>0) DoCluster(cand_list,0,n-1,clusters,i);
    }  
  //sort cluster according to their size, so large clusters are more likely to rule out small clusters
  sort(clusters.begin(),clusters.end(),CL_greater());
  if( L3_Parameter::Info() ){
    cout << "*** candidates ***" << endl;
    for(vector<Cluster>::iterator i = clusters.begin();i!=clusters.end();i++)
      {
	cout << *i;
      }  
  }
  //look for clusters which share too many tracks  
  {
    int N = clusters.size();
    for(int i=0;i<N-1;)
      {
	for(int k=N-1;k>i;)
	  {
	    if( clusters[i].target == clusters[k].target )  //do not compare clusters on the same wire
	      {
		k--;
		if( k<=i ){ i++; k=i; }
		continue;
	      }
	    vector<TrkInfo> List1 = clusters[i].getUniqueTracks(clusters[k]);
	    vector<TrkInfo> List2 = clusters[k].getUniqueTracks(clusters[i]);
	    int take1=1,take2=1;	    
	    if( List1.size() < clusters[i].Ntracks() )  //not all tracks unique
	      {
		vector<Cluster> res;
		if(List1.size()>1) DoCluster(List1,0,List1.size()-1,res,-1);
		if( res.size()<1 ) take1=0;
	      }
	    if( List2.size() < clusters[k].Ntracks() )
	      {
		vector<Cluster> res;
		if( List2.size()>1 ) DoCluster(List2,0,List2.size()-1,res,-1);
		if( res.size()<1 ) take2=0;
	      }
	    if( take1==0 && take2==0 )
	      {
		//if both cannot survive we take the one with more tracks
		//(take both if same number of tracks)
		//MAYBE this is NOT A GOOD IDEA for several IA in the detector - to be studied
		if( clusters[i].Ntracks() > clusters[k].Ntracks() )
		  take1=1;
		else if( clusters[k].Ntracks() > clusters[i].Ntracks() )
		  take2=1;	     
	      }	
	    //cout << "take1=" << take1 << ", take2=" << take2 << endl;
	    if( take1==take2 ){  //take both or both failed	      
	      k--;
	      if( k<=i ){ 
		i++;  //next outer
		k=i;  //leave inner loop
	      }
	    } else if(take1){	 
	      clusters.erase(clusters.begin()+k);
	      N--;
	      //k remains as it is
	      break;
	    } else if(take2){	 
	      clusters.erase(clusters.begin()+i);
	      N--;
	      //i remains as it is
	      break;//leave loop
	    } else {
	      cout << "LOGICAL MISTAKE" << endl;
	    }
	  }
      }
  }  
  if(L3_Parameter::Info()) cout << "*** found vertices ***" << endl;
  for(vector<Cluster>::iterator i=clusters.begin();i!=clusters.end();i++) 
    {
      const int tindx = i->target;  //target index
      const L3_Target& target = L3_Geometry::getTarget(tindx);
      if(L3_Parameter::Info()) cout << *i;
      PCovMatrix m;
      for(int x=0;x<3;x++)
	{
	  m.a[x][x] = 4*vtwire_.cw[tindx].a[x][x];
	  for(int y=0;y<x;y++)
	    {
	      m.a[x][y] = m.a[y][x] = 0.0;
	    }
	}
      Hep3Vector guess = target.get3center();
      L3_Vtx vtx(guess,m);
      for(vector<TrkInfo>::iterator ii = i->data.begin();ii!=i->data.end();ii++)
	{
	  vtx.add( *(ii->track()) );
	}	    
      vtkalm_.smooth[ vtx.Nr()-1 ] = 1;
      if( vtx.fit() == 0 ) 
	{
	  if(L3_Parameter::Info()) 
	    cout << "vertex with " << vtx.nTrack() << " tracks is (before smoothing) " << vtx.vertex() << endl;
	  vtx.smooth(14);		
	  if( vtx.nTrack() >= ntmin ) {
	    if(L3_Parameter::Info()) cout << "vertex with " << vtx.nTrack() << " tracks is (after  smoothing) " << vtx.vertex() << endl;
	    primVertexListNew.push_back( L3_Vertex(vtx) );
	  } else {
	    if(L3_Parameter::Info()) cout << "Vertex deleted" << endl;
	  }
	}	    
    }
}

#ifdef KL
static float LUT(float angle){
  static float a[]={ 0.62 , 0.4677 , 0.2724 , 0.1591 , 0.07723 , 0.07723};
  const int bins = sizeof(a)/sizeof(a[0]);
  const float dbin = (0.3-0.0)/float(bins);
  int b = int(angle/dbin);
  if( b<0 ) b=0; else if(b>=bins) b=bins-1;
  return a[b];
}
#endif

class L3_Wrapper {
public:
  static int isUsed(int i){ return used[i-1]; }
  static void markUsed(int i){ used[i-1]=1; }
  static void reset(){ for(int i=0;i<ntrtot;i++) used[i]=0; }
private:
  static int used[ntrtot];
};
int L3_Wrapper::used[ntrtot];

int L3_Origin::SLTvertex(vector<L3_Track> &l3tracks,L3_Track *q,L3_Vertex& primary)
{
  // cut constants
  const float c_dca = 0.1;
  const float c_dcaedca = 18;
  const float c_angle = 0.25;
  const float c_deta = 2.4;
  const float c_chi2 = 10;
  const float c_downstream = 2;
  const float c_imp = 0.3;
  const float c_impeimp = 40;
  int count = 0; //count # of secondary vertices found
  Hep3Vector primVertex = primary.vertex();
  //const L3_Target& target = L3_Geometry::ClosestTarget( primVertex );
  if(L3_Parameter::Info()) cout << "L2 track " << q->nr() << endl;
  for(vector<L3_Track>::iterator q3 = l3tracks.begin();q3!=l3tracks.end();q3++)      
    {
      if( L3_Wrapper::isUsed( q3->nr() ) ) 
	continue;
      else
	L3_Wrapper::markUsed( q3->nr() );
      float dca,dcaedca;
      {
	L3_Impact imp(*q,*q3);
	dca = imp.mag();
	dcaedca = dca/sqrt( imp.variance() );
      }
      float angle = q->AngleWithTrack(*q3);
      float imp,impeimp;
      {
	L3_Impact dist(*q3,primary);
	imp = dist.mag();
	impeimp = imp/sqrt( dist.variance() );
      }
      float deta = fabs(q->eta()-q3->eta());
      if( deta>M_PI ) deta = 2*M_PI - deta;
      L3_Vtx vtx;
      vtx.add(*q3); 
      vtx.add(*q);
      if( vtx.fit()!=0 ) continue;
      float chi2 = vtx.chi2();
      float downstream = (vtx.z()-primary.z())/vtx.dz();
      int take = 0;
      if( dca<c_dca ) take |= 1;
      if( dcaedca<c_dcaedca ) take |= 2;
      if( angle<c_angle ) take |= 4;
      if( deta<c_deta ) take |= 8;
      if( imp<c_imp ) take |= 16;
      if( impeimp<c_impeimp ) take |= 32;
      if( chi2<c_chi2 ) take |= 64;
      if( downstream>c_downstream ) take |= 128;
      if( L3_Parameter::Info() ){
	cout << q3-l3tracks.begin()+1 << ' ';
	char c = (take & 1) ? '+' : '-';
	cout << dca << c << ' ';
	c = (take & 2) ? '+' : '-';
	cout << dcaedca << c << ' ';
	c = (take & 4) ? '+' : '-';
	cout << angle << c << ' ';
	c = (take & 8) ? '+' : '-';
	cout << deta << c << ' ';
	c = (take & 16) ? '+' : '-';
	cout << imp << c << ' ';
	c = (take & 32) ? '+' : '-';
	cout << impeimp << c << ' ';
	c = (take & 64) ? '+' : '-';
	cout << chi2 << c << ' ';
	c = (take & 128) ? '+' : '-';
	cout << downstream << c << ' ';
	cout << vtx.dz()*1E4 << endl;
      }
      if( take==255 ){
	vtx.associate(*q);
	addTracks(vtx,l3tracks);
	secVertexList.push_back( L3_Vertex(vtx) );		  	    
	if(L3_Parameter::Info())
	  {
	    float d = primary.z();
	    cout << q3->nr() << " added to list " << vtx.vertex() << ' ' << d << endl;
	  }	
	count++;
      }
    }
  return count;
}

void L3_Origin::SLTreco(vector<L3_Track> &l3tracks,vector<L3_Track> &l2tracks)
{
  int n;
  const float CUT = 0.2*0.2; //2 mm distance
  for(vector<L3_Track>::iterator q = l2tracks.begin();q!=l2tracks.end();q++)
    {
      L3_Wrapper::reset(); //reset for each lepton
      for(vector<L3_Vertex>::iterator prim=primVertexList.begin();prim!=primVertexList.end();prim++)
	{
	  //check whether trigger lepton comes close to primary vertex
	  Hep3Vector primVertex = prim->vertex();
	  const L3_Target& target = L3_Geometry::ClosestTarget( primVertex );
	  primVertex = q->MinDistanceToTarget( target );
	  if( primVertex.mag2() < CUT )
	    {
	      n = SLTvertex(l3tracks,&(*q),*prim);
	      if( L3_Parameter::Info() ) cout << n << " 2-track vertices found on " << target.name() << endl;
	    }
	}
    }
  if( L3_Parameter::Info() ) cout << secVertexList.size() << " 2-track vertices found" << endl;
#if 0
  vector<L3_Vertex>::iterator max=candList.end();
  for(vector<L3_Vertex>::iterator j=candList.begin();j!=candList.end();j++){
    if( max==candList.end() )
      max = j;
    else {
      if( j->z() > max->z() ) max=j;
    }
  }
  if(max!=candList.end()) secVertexList.push_back( *max );		    
#endif
}


#ifdef L3_ARTE_FRAME
void showPrimVertices()
{
  int n=1;
  for (ArteTable<MVER>::iterator mi=ArteTable<MVER>::begin(); mi!=ArteTable<MVER>::end(); mi++ ) {    
    ArtePointer<MVER>& m = *mi;
    if (m->flag == 1) {
      const char *name = L3_Geometry::ClosestTarget(Hep3Vector(m->x,m->y,m->z)).name();
      printf("%i\t%.4f\t%.4f\t%.4f , %s\n",n,m->x,m->y,m->z,name);
      n++;
    }
  }
}
#endif

//we get a copy of the track array here
void L3_Origin::reconstruct(vector<L3_Track>& l3tracks, L3_Timer& Timer) {
#ifdef L3_ARTE_FRAME
  if(ArteTable<MVER>::booked() && L3_Parameter::Info()) showPrimVertices();
#endif
  Timer.start_Vertex();  
  vector<L3_Track> l2tracks = L3_SLP::getL2Tracks();
  if(L3_Parameter::Info()) cout << "# of tracks from L2: " << l2tracks.size() << endl;
  if(L3_Parameter::Info()) cout << "# of tracks from L3: " << l3tracks.size() << endl;    
  {
    int nextID = l3tracks.size() + 1;    
    int extra = 0;
    for(vector<L3_Track>::iterator i = l2tracks.begin();i!=l2tracks.end();i++)
      {
	i->kalTransport(0);  //vt uses z=0 as reference plane
	float d;     
	vector<L3_Track>::iterator p = BestMatch(*i,l3tracks,&d);
	if( p!=l3tracks.end() && d<0.001 ){ //good match
	  //cout << "Match for SLT track " << d << endl;
	  //cout << "SLT-track:" << endl << *i << endl;
	  //cout << "TLT-track:" << endl << *p << endl;
	  i->setnr( p->nr() );
	  l3tracks.erase( p );
	} else {
	  //no match for this l2 tracks -> give it the next number
	  i->setnr( nextID++ );
	  extra++;
	}
      }  
    if( l3tracks.size()+l2tracks.size() > (unsigned int)ntrtot ){
      int max = ntrtot;
      for(vector<L3_Track>::iterator i=l2tracks.begin();i!=l2tracks.end();i++){
	if( i->nr() > max ) i->setnr(max--);
      }
      for(vector<L3_Track>::iterator i=l3tracks.begin();i!=l3tracks.end();){
	if( i->nr() > max )
	  l3tracks.erase(i);
	else
	  i++;
      }
    }
  }
  L3_Vtx::NewEvent();
  for(vector<L3_Track>::iterator q = l3tracks.begin();q!=l3tracks.end();q++){  
    L3_Vtx::toVt(*q);
  }  
  for(vector<L3_Track>::iterator q = l2tracks.begin();q!=l2tracks.end();q++){
    L3_Vtx::toVt(*q);
  }
  VTreco(l3tracks,l2tracks);
#if 0
  {
    vector<L3_Track> all = l3tracks;
    for(vector<L3_Track>::iterator h=l2tracks.begin();h!=l2tracks.end();h++){
      all.push_back(*h);
    }
    MYreco(all);
  }
#endif
  if(primVertexList.size()>0) SLTreco(l3tracks,l2tracks);    
  Timer.stop_Vertex();
}

void showVertex(L3_Vtx &vtx)
{
  //display on screen
  printf("%i\t%.4f\t%.4f\t%.4f\t, %i tracks={ ",vtx.Nr(),vtx.x(),vtx.y(),vtx.z(),vtx.nTrack());
  for(int i=0;i<vtx.nTrack();i++)
    {
      int trnr = vtx.TrackNr(i);
      cout << trnr << ' ';
    }
  cout << '}' << endl;
}

void L3_Origin::addTracks(L3_Vtx &vtx,vector<L3_Track> &tr)
{
  vector<int> trackNr = vtx.OutgoingTracks();
  for(vector<L3_Track>::const_iterator tit = tr.begin();tit!=tr.end();tit++)
    {
      int tracknr = tit->nr();
      for(vector<int>::const_iterator iit = trackNr.begin();iit!=trackNr.end();iit++)
	{
	  if( tracknr == *iit )
	    {
	      vtx.associate(*tit);
	      break; // next track
	    }
	}
    }
}

void L3_Origin::VTreco(vector<L3_Track> &l3tracks,vector<L3_Track> &l2tracks)
{
  int status = 42;
  //call vt's reconstruction routine
  {
    //vtreco_(status);
    //if(status!=0) cout << "vtreco_ returns " << status << endl;
  }
  {
    int Bindex = 0; //no B track found (B -> Jpsi/K)
    vtprim_(Bindex,status);
    if(status!=0 && L3_Parameter::Debug()) cout << "vtprim_ returns " << status << endl;
  }

  //fill results
  //primary vertices
  int n = ivprim_.nvprim;
  if(L3_Parameter::Info()) cout << n << " PRIMARY vertices found" << endl;
  for(int i=0;i<n;i++)
    {      
      int index = ivprim_.ivprim[i];
      L3_Vtx vtx(index);      
      addTracks(vtx,l3tracks);
      addTracks(vtx,l2tracks);
      if(L3_Parameter::Info()) showVertex(vtx);
      L3_Vertex primary(vtx);
      primVertexList.push_back( primary );
      int targ = -1;
      float d = 1E14;
      Hep3Vector prim = primary.vertex();
      for(int k=0;k<L3_Geometry::nTarget();k++){
	const L3_Target& t = L3_Geometry::getTarget(k);
	float newd = t.MinDistanceToPoint(prim);
	if( newd < d ){
	  targ = k;
	  d = newd;
	}
      }
      if( targ>=0 )
	{
	  wireAssign[targ].push_back(i);
	}
    }
#if 0
  //secondary vertices
  n = ivsec_.nvsec;
  if(L3_Parameter::Info()) cout << n << " SECONDARY vertices found" << endl;
  for(int i=0;i<n;i++) 
    {
      int index = ivsec_.ivsec[i];
      L3_Vtx vtx(index);
      addTracks(vtx,l3tracks);
      addTracks(vtx,l2tracks);
      if(L3_Parameter::Info()) showVertex(vtx);
      secVertexList.push_back( L3_Vertex(vtx) );
    }
#endif
}






