// *****************************************************************************
//
// source:    L3_SLP.C
//
// type:      source code
//
// created:   06 Dec 1999
//
// author:    Stefan Scharein, Ullrich Schwanke
//            HERA-B Collaboration
//            DESY Zeuthen
//            Platanenallee 6
//            D-15738 Zeuthen
//            Germany
//            email: scharein@ifh.de, schwanke@ifh.de
//
// symbol:    
//
// changes:
//
// *****************************************************************************
#include "L3_SLP.hh"
//#ifdef L3_SLP_FRAME
//#define _BOOL
//#include "l2event.h"
//#undef _BOOL
//#endif

#include "L3_Trigger.hh"
#include "L3_Geometry.hh"

int L3_SLP::l2eventnr;
vector<L3_Track> L3_SLP::l2tracks;

void L3_SLP::init() {
  l2eventnr=0;
}

void L3_SLP::fill() {
  l2eventnr++;
}

void L3_SLP::clear() {
  l2tracks.clear();
}

void L3_SLP::setL2Tracks(vector<L3_Track> &v)
{
  l2tracks = v;
}

vector<L3_Track> L3_SLP::getL2Tracks() {
  return l2tracks;
}

int L3_SLP::getL2EventNr(){
  return l2eventnr;
}

void L3_SLP::setL2EventNr(int newnr){
  l2eventnr = newnr;
}

// ********************************************************************
//
// source:    PrintSltInfo.C
//
// type:      Example which shows SLT information
//
// author:    Brian Petersen
//
// needed:    link with $HBROOT/SLT/slt/Rev0500/$(BINTYPE)/libsltinfo.a
//            sltinfo.h is in $HBROOT/SLT/slt/Rev0500/include
//
// ********************************************************************
#if defined(L3_ARTE_FRAME) && defined(L3_SLT_INFO)
extern "C" {
#include "sltinfo.h"
}
#include "L3_Histogram.hh"
#include "farm/farm.hh"

void L3_SLP::PrintSltInfo()
{
   cout << "flt/slt/fcs: " << faDDAQflt_status() << ' ';
   cout << faDDAQslt_status() << ' ' << faDDAQFCS_tcode() << endl;   

   static L3_Tuple *over = NULL;
   if(over==NULL) over=L3_Histogram::BookNT("nt7001",7001);

   static L3_Tuple *tra = NULL;
   if(tra==NULL) tra=L3_Histogram::BookNT("nt7002",7002);

   cout<<"*** Slt Info:***"<<endl;
   if (SLTReadDSLT()!=SLT_OK) {
     cout <<"No SLT information found"<<endl;
   }
   else {
     SLT_STATUS stat;
     SLT_TRIGGER trig;
     SLT_TRACK track;
     SLT_SEEDTRACK seed;
     SLT_SILITRACK sili;
     if (SLTStatus(&stat)!=SLT_OK) {
       cout <<"No status information"<<endl;
       return;
     }
     over->column("stat",stat.status);
     if (stat.status==SLTEVENT_REJECT) {
       cout <<"Event rejected"<<endl;
       over->dumpData();
       return;
     }
     if (stat.status==SLTEVENT_BYPASS) {
       cout <<"Event was a FLTE BYPASS event!"<<endl;
       over->dumpData();
       return;
     }
     over->column("ntra",stat.ntracks);
     over->column("ntrig",stat.ntriggers);
     over->dumpData();
     cout<<"Event accepted with "<<stat.ntracks<<" tracks in ";
     cout<<stat.ntriggers<<" trigger pairs"<<endl;
     cout<<"-----Triggers:-------"<<endl;
     vector<int> tracks;
     for(int i=0;i<stat.ntriggers;i++) {
       if (SLTTrigInfo(i,&trig)!=SLT_OK) {
	 cout<<"Error getting info on trigger number "<<i<<endl;
	 continue;
       }
       if (trig.status==SLTTRIG_REJECT) {
	 cout<<"Trigger "<<i<<" rejected"<<endl;
	 continue;
       }
       cout<<"Trigger "<<i<<" accepted"<<endl;
       for(int j=0;j<trig.ntracks;j++) {
	 int log = 0;
	 if( find(tracks.begin(),tracks.end(),trig.trackindex[j]) == tracks.end() ){
	   //track with this number not yet present
	   tracks.push_back(trig.trackindex[j]);
	   log = 1;
	 } else 
	   cout << "again track " << trig.trackindex[j] << endl;
	 if (SLTTrackInfo(trig.trackindex[j],&track)!=SLT_OK) {
	   cout<<"Error get info on track "<<trig.trackindex[j]<<endl;
	   continue;
	 }
	 if(log) tra->column("stat",track.status);
	 switch(track.status) {
	 case SLTTRACK_ERROR:
	   cout<<"Error in data of track "<<j<<endl;
	   break;
	 case SLTTRACK_ACCEPT:
	   cout<<"Track "<<j<<" accepted:"<<endl;
	   break;
	 case SLTTRACK_REJECT_SILI:
	   cout<<"Track "<<j<<" not accepted L2Sili:"<<endl;
	   break;
	 case SLTTRACK_REJECT_OTHER:
	   cout<<"Track "<<j<<" rejected for unknown reason:"<<endl;
	   break;
	 }
	 if(track.status!=SLTTRACK_ERROR) {
	   if (SLTSeedTrackInfo(trig.trackindex[j],&seed)!=SLT_OK) {
	     cout<<"Error getting seed track for track"<<trig.trackindex[j];
	     cout<<endl;
	     continue;
	   }
	   cout<<"Seed energy:"<<seed.energy<<endl;
	   cout<<"Seed x,y,z:"<<seed.x2<<" "<<seed.y2<<" "<<seed.z2<<endl;
	   if( log ){
	     tra->column("energy",seed.energy);
	     tra->column("pxpz",seed.pxpz);
	     tra->column("charge",seed.charge);
	     tra->column("id",seed.trackID);
	     tra->column("x1",seed.x1);
	     tra->column("y1",seed.y1);
	     tra->column("z1",seed.z1);
	     tra->column("x2",seed.x2);
	     tra->column("y2",seed.y2);
	     tra->column("z2",seed.z2);
	   }
	   
	   if(track.status==SLTTRACK_ACCEPT) {
	     if (SLTSiliTrackInfo(trig.trackindex[j],&sili)!=SLT_OK) {
	       cout<<"Error getting L2sili track for track";
	       cout<<trig.trackindex[j]<<endl;
	       continue;
	     }
	     cout<<"slope,intersect,# hits and chisq in x: ";
	     cout<<sili.tx<<" "<<sili.bx<<" "<<sili.nhitx<<" "<<sili.chix<<endl;
	     cout<<"slope,intersect,# hits and chisq in y: ";
	     cout<<sili.ty<<" "<<sili.by<<" "<<sili.nhity<<" "<<sili.chiy<<endl;
	     if(log){
	       tra->column("pxpz",sili.pxpz);
	       tra->column("tx",sili.tx);
	       tra->column("bx",sili.bx);
	       tra->column("ty",sili.ty);
	       tra->column("by",sili.by);
	       tra->column("nhitx",sili.nhitx);
	       tra->column("nhity",sili.nhity);
	       tra->column("chix",sili.chix);
	       tra->column("chiy",sili.chiy);
	       constructTrack(sili.tx,sili.ty,sili.bx,sili.by,sili.pxpz,sili.nhitx,sili.nhity);
	     }
	   }
	 }
	 if(log) tra->dumpData();
       }
     }

   }
}
#else
void L3_SLP::PrintSltInfo(){
  cout << "L3_SLP::PrintSltInfo: no information available" << endl;
}
#endif

class residual {
public:
  residual(vector<L3_Hitb>::iterator a,float r) : hit(*a),resid(r) {}
  L3_Hitb hit;
  float resid;
  residual& operator = (const residual& a){
    hit = a.hit;
    resid = a.resid;
    return *this;
  }
}; 


class res_smaller : public binary_function< residual, residual, bool > {
public:
  inline
  bool operator() (const residual& R , const residual& S) const {
    return ( R.resid < S.resid ); 
  }
};

void L3_SLP::constructTrack(float tx,float ty,float bx,float by,float energy,int nhitx,int nhity){
  cout << "Track tx/ty/bx/by/e/nx/ny " << tx << ' ' << ty << ' ' << bx << ' ' << by << ' ';
  cout << energy << ' ' << nhitx << ' ' << nhity << endl; 
  extern L3_Trigger TLT;
  L3_Event e = TLT.getEvent();
  const L3_Silicon& sili = e.silicon();
  vector<L3_Hitb> hits;
  vector<residual> hitsx;
  vector<residual> hitsy;
  for(int sl=0;sl<8;sl++){
    int nAdded = 0;
    for(int sec=0;sec<4 && nAdded<4;sec++){
      for(int pl=0;pl<4;pl++){
	const L3_Plane &plane = L3_Geometry::getPlane(sl,sec,pl);
	if( plane.l3nsl() == - 1 ) continue;
	float x = bx + plane.zfw(0)*tx;
	//if( x<plane.sxmn() || x>plane.sxmx() ) continue;
	float y = by + plane.zfw(0)*ty;
	//if( y<plane.symn() || y>plane.symx() ) continue;
	cout << "Track is hitting sl/sec/pl: " << sl+1 << '/' << sec+1 << '/' << pl+1 << endl;
	float u = x*plane.cos() - y*plane.sin();
	vector<L3_Hitb> h = sili.Hitbs(sl,sec,pl);
	float du = 10.0;
	vector<L3_Hitb>::iterator best = h.end();
	for(vector<L3_Hitb>::iterator i=h.begin();i!=h.end();i++){
	  float newdu = fabs(i->x()-u);
	  if( newdu<du ){
	    du = newdu;
	    best = i;
	  }
	}
	if( best!=h.end() && du<10.0 ){
	  cout << "Best match is " << du << endl;
	  if( fabs(plane.cos())>0.5 )
	    hitsx.push_back( residual(best,du) );
	  else
	    hitsy.push_back( residual(best,du) );
	  nAdded++;
	}
      }
    }
  }
  {
    sort(hitsx.begin(),hitsx.end(),res_smaller());
    int diff = hitsx.size() - nhitx;
    if( diff>0 ) hitsx.erase(hitsx.begin()+nhitx,hitsx.end());    
    for(vector<residual>::iterator k=hitsx.begin();k!=hitsx.end();k++){
      hits.push_back(k->hit);
      cout << k->resid << ' ';
    }
    cout << endl;
  }
  {
    sort(hitsy.begin(),hitsy.end(),res_smaller());
    int diff = hitsy.size() - nhity;
    if( diff>0 ) hitsy.erase(hitsy.begin()+nhity,hitsy.end());    
    for(vector<residual>::iterator k=hitsy.begin();k!=hitsy.end();k++){
      hits.push_back(k->hit);
      cout << k->resid << ' ';
    }
    cout << endl;
  }
  cout << "Track with " << hits.size() << " hits" << endl;
  if( hits.size() > 4 ){
    cout << "Track tx/ty/bx/by/e/nx/ny " << tx << ' ' << ty << ' ' << bx << ' ' << by << ' ';
    cout << energy << ' ' << nhitx << ' ' << nhity << endl; 
    L3_Track tra( 1, bx,by,tx,ty,0.0,hits,energy);    
    tra.kalTransport(0.0);
    cout << "Track tx/ty/bx/by/e/nx/ny " << tra.mx() << ' ' << tra.my() << ' ' << tra.x() << ' ' << tra.y() << ' ';
    cout << energy << ' ' << hitsx.size() << ' ' << hitsy.size() << endl;
  }
}














