*CMZ :          01/10/96  19.12.58  by  T. Oest
*-- Author :    T. Oest   11/04/96

      SUBROUTINE V0XZ(SOLUT, TRK1, TRK2, ZINT, SZINT, IRET)

************************************************************************
*
* Description: This routine calculates the point of intersection for two
*              tracks in (x,z)
*
* Called from:
*
* External references:
*
* Argument in/out description:
*    input
*       SOLUT   : >=0 solution with larger z value
*                 <0  solution with lower z value
*       TRK1    : first track in MREC
*       TRK2    : first track in MREC
*
*    output
*       ZINT    : point of intersection
*       SZINT   : error on ZINT
*       IRET    : < 0 ERROR
*               : = 1 NO SECOND SOLUTION
*
************************************************************************

      IMPLICIT NONE

#include "arte/Arte.fi"
#include "auto/EV.fi"

      INTEGER           IRET
      INTEGER           TRK1, TRK2, TR(2)
      INTEGER           I, J
      DOUBLE PRECISION  ZINT, SZINT
      REAL*4            SOLUT

      LOGICAL           SCATT
      DOUBLE PRECISION  X(2), Y(2), Z(2), TX(2), TY(2), RHO(2), ARHO(2)
      DOUBLE PRECISION  XC(2), ZC(2), DELTA, PINV(2), DELTAZ
      DOUBLE PRECISION  DCIRCL, S1, S2, FAC1, L1, LS, ZOLD, ZOLD2
      DOUBLE PRECISION  PTRK1(5), PTRK2(5), VTRK1(5,5), VTRK2(5,5)
      REAL*4            R(3), FIELD(3), BF(2)
      INTEGER           IERR1, IERR2, ITERA

      DOUBLE PRECISION  EC

#include "arte/Arte.ff"
#include "auto/EV.ff"


      DATA   EC     / 2.99792458D-4 /, SCATT /.FALSE./
************************************************************************
      IRET   = 1
      ITERA  = 0
C-----------------------------------------------------------------------
C  fill in track parameter
C-----------------------------------------------------------------------
      TR(1)  = TRK1
      TR(2)  = TRK2

      DO I=1,2
       J          = TR(I)
       PINV(I)    = DBLE( FRTRA_PF(J) ) * EC
       TX(I)      = DBLE( FRTRA_TXF(J) )
       TY(I)      = DBLE( FRTRA_TYF(J) )
       X(I)       = DBLE( FRTRA_XF(J) )
       Y(I)       = DBLE( FRTRA_YF(J) )
       Z(I)       = DBLE( FRTRA_ZF(J) )
      ENDDO


C-----------------------------------------------------------------------
C  LOOP
C-----------------------------------------------------------------------
 10   CONTINUE
c       do i=1,2
c       PRINT*,' TRACK ',I
c       PRINT*,' XYZ   ',SNGL(X(I)),SNGL(Y(I)),SNGL(Z(I))
c       PRINT*,' T,PINV ',SNGL(TX(I)),SNGL(TY(I)),SNGL(PINV(I))
c       print*,''
c       enddo
C-----------------------------------------------------------------------
C  RHO
C-----------------------------------------------------------------------
      DO I=1,2
       R(1) = SNGL(X(I))
       R(2) = SNGL(Y(I))
       R(3) = SNGL(Z(I))
       CALL UTFELD(R, FIELD)
       BF(I) = FIELD(2)
c       print*,'ut field',z(i),x(i),y(i),bf(i)
       IF (ABS(FIELD(2)) .GE. 1.E-6) THEN
        RHO(I) = 1. / (PINV(I) * FIELD(2))
       ELSE
        RHO(I) = 1. / (PINV(I) * SIGN(1.E-6, FIELD(2)))
       ENDIF
       ARHO(I) = ABS(RHO(I))
      ENDDO

C-----------------------------------------------------------------------
C  no field??
C-----------------------------------------------------------------------
      IF (ABS(BF(1)).LE.1.E-3 .OR. ABS(BF(2)).LE.1.E-3) THEN
       IF (TX(1).EQ.TX(2)) THEN
        IRET = -1
        GOTO 9000
       ENDIF

       ZINT = (X(2)-X(1)+Z(1)*TX(1)-Z(2)*TX(2))/(TX(1)-TX(2))

c       print*,' no field : zint ',zint
      ELSE
C-----------------------------------------------------------------------
C  center of circles
C-----------------------------------------------------------------------
      IRET = 0

      DO I=1,2
       XC(I) = X(I) - RHO(I)         / SQRT(1.+TX(I)**2)
       ZC(I) = Z(I) + RHO(I) * TX(I) / SQRT(1.+TX(I)**2)
      ENDDO
C-----------------------------------------------------------------------
C  distance between center of circles
C-----------------------------------------------------------------------

      DCIRCL = SQRT( (XC(1)-XC(2))**2 + (ZC(1)-ZC(2))**2)

C-----------------------------------------------------------------------
C  calculate z coordinates for the point(s) of closest approach
C     (for each z coordinate there are two points on a helix,
C      it is assumed that tracks doesn't curl)
C-----------------------------------------------------------------------

c      print*,' dircl, arho1, arho2 ',sngl(dcircl), sngl(rho(1))
c     +             ,sngl(arho(2))
      IF (DCIRCL .GE. ARHO(1)+ARHO(2)) THEN
C-----------------------------------------------------------------------
C  CASE 1 :    ONE POINT OF CLOSEST APPROACH
C-----------------------------------------------------------------------
       IRET = 2

       ZINT = ZC(1) + ARHO(1) / DCIRCL * (ZC(2) - ZC(1))

c       print*,''
c       print*,' one solution '
c       print*,' zt1,zt2 ',zt1,zt2


      ELSE
C-----------------------------------------------------------------------
C  CASE 2 :    TWO POINTS OF INTERSECTION
C-----------------------------------------------------------------------
c       print*,''
c       print*,' two solutions '

       IRET   = 0

       L1 = 0.5*(DCIRCL + (ARHO(1)+ARHO(2))*(ARHO(1)-ARHO(2))/DCIRCL)
       LS = SQRT(MAX(ARHO(1)**2 - L1**2,0.0D0))

       IF (SOLUT.GE.0.0) THEN
        ZINT = ZC(1) + (L1*(ZC(2)-ZC(1)) + LS*(XC(2)-XC(1)))/DCIRCL
       ELSE
        ZINT = ZC(1) + (L1*(ZC(2)-ZC(1)) - LS*(XC(2)-XC(1)))/DCIRCL
       ENDIF
c       print*,' zt1,zt2 ',zt1,zt2


      ENDIF

      ENDIF
C-----------------------------------------------------------------------
C check for convergence
C-----------------------------------------------------------------------
       IF (ITERA.GE.2) THEN
        IF (ABS(ZOLD2-ZINT).LT.0.05*ABS(ZOLD2-ZOLD)) THEN
         ZINT = 0.5 *(ZOLD2+ZOLD)
        ENDIF
       ENDIF
C-----------------------------------------------------------------------
C now track the particles to znew
C-----------------------------------------------------------------------
       CALL V0SWIM (SCATT, TR(1), ZINT, PTRK1, VTRK1, IERR1)
       CALL V0SWIM (SCATT, TR(2), ZINT, PTRK2, VTRK2, IERR2)

       IF (IERR1.NE.0 .OR. IERR2.NE.0) THEN
        IRET  = -1
        GOTO 9000
       ENDIF



c test
c       DTX   = PTRK2(3)- PTRK1(3)
C       SZINT = (VTRK1(1,1)+VTRK2(1,1))/DTX**2
C     +  + 2.*ZINT*(VTRK1(1,3)*PTRK2(3)-VTRK2(1,3)*PTRK1(3))/DTX**3
C     +  + ZINT**2*(VTRK1(3,3)*PTRK2(3)**2+VTRK2(3,3)*PTRK1(3)**2)/DTX**4
C       SZINT = SQRT(SZINT)
c        SZINT = SQRT(VTRK1(1,1)+VTRK2(1,1))/ABS(DTX)
c       PRINT*,' TEST SZINT ',SZINT
c end test

       SZINT  = SQRT(VTRK1(1,1)/PTRK1(3)**2  +
     +               VTRK2(1,1)/PTRK2(3)**2)
c       PRINT*,'      SZINT ',iret,sngl(ZINT),sngl(SZINT)

      IF (ITERA.GE.10) THEN
       IRET = -3
       GOTO 9000
      ENDIF

      DELTA  = ABS((PTRK1(1)-PTRK2(1))/SQRT(VTRK1(1,1)+VTRK2(1,1)))
      DELTAZ = ABS(ZINT-ZOLD)/SZINT
c      PRINT*,' X1,2 DELTA ',SNGL(PTRK1(1)),SNGL(PTRK2(1)),SNGL(DELTA)

       X(1)       = PTRK1(1)
       Y(1)       = PTRK1(2)
       TX(1)      = PTRK1(3)
       TY(1)      = PTRK1(4)
       X(2)       = PTRK2(1)
       Y(2)       = PTRK2(2)
       TX(2)      = PTRK2(3)
       TY(2)      = PTRK2(4)
       Z(1)       = ZINT
       Z(2)       = ZINT

      IF ( ITERA.EQ.0 .OR. 
     +    (DELTA.GT.0.5 .AND. DELTAZ.GT.0.1)) THEN
       ITERA = ITERA + 1
       ZOLD2 = ZOLD
       ZOLD  = ZINT
       GOTO 10
      ENDIF



 9000 END

************************************************************************
* End of V0XZ
************************************************************************
