#ifndef __VTDISTANCE
#define __VTDISTANCE
// *****************************************************************************
//
// source:
//
// type:      source code
//
// created:   21. Aug 2000
//
// author:    Thorsten Glebe
//            HERA-B Collaboration
//            Max-Planck-Institut fuer Kernphysik
//            Saupfercheckweg 1
//            69117 Heidelberg
//            Germany
//            E-mail: T.Glebe@mpi-hd.mpg.de
//
// Description: an (incomplete) collection of distance functions
//
// changes:
// 21 Aug 2000 (TG) creation
// 04 Sep 2000 (TG) SGNdistance(const Track&, const Vertex&) function added
// 06 Sep 2000 (TG) SGNdistance(const Track&, double x, double y, double z) added
// 14 Sep 2000 (TG) distance() functions returning chi2 renamed to distanceChi2()
// 26 Okt 2000 (TG) #ifndef NO_ARTE added (for integration into CLUE)
//
// *****************************************************************************

template <class T> class ArtePointer;
#ifndef NO_ARTE
struct GTAR;
struct RVER;
struct MVER;
#endif // NO_ARTE
#ifdef USE_ROOT
class RecoVertex;
#endif
class VertexIf;

namespace VERTEX {
  class Track;
  class Vertex;

  /** @name --- Distance functions --- */
#ifndef NO_ARTE
  /// spatial distance betw. RVER \& MVER
  double distance(const ArtePointer<RVER>& rver , const ArtePointer<MVER>& mver);
  // spatial distance betw. RVER \& MVER
  inline double distance(const ArtePointer<MVER>& mver,
			 const ArtePointer<RVER>& rver) {
    return distance(rver,mver); 
  }

  /// spatial distance between RVER and GTAR
  double distance(const ArtePointer<RVER>& rver, const ArtePointer<GTAR>& gtar);
  // spatial distance between RVER and GTAR
  inline double distance(const ArtePointer<GTAR>& gtar,
			 const ArtePointer<RVER>& rver) {
    return distance(rver, gtar);
  }

  /// spatial distance between Vertex and GTAR
  double distance(const Vertex& rver, const ArtePointer<GTAR>& gtar);
  // spatial distance between Vertex and GTAR
  inline double distance(const ArtePointer<GTAR>& gtar,
			 const Vertex& rver) {
    return distance(rver, gtar);
  }

  /// spatial distance of track to wire
  double distance(const Track& t, const ArtePointer<GTAR>& gtar);
#endif // NO_ARTE

  /// $\chi^2$ distance track - track, $ndf = 1$
  double distanceChi2(Track& t1, Track& t2);

  /// spatial distance track - vertex
  double distance(const Track& t, const Vertex& v);
  // spatial distance track - vertex
  inline double distance(const Vertex& v, const Track& t) { return distance(t,v); }

  /// signed spatial distance track - vertex
  double SGNdistance(const Track& t, const Vertex& v);
  // signed spatial distance track - vertex
  inline double SGNdistance(const Vertex& v, const Track& t) {
    return SGNdistance(t,v);
  }

  /// signed spatial distance track - space point
  double SGNdistance(const Track& t, double xv, double yv, double zv);

  /// $\chi^2$ distance between two vertices, $ndf = 3$
  double distanceChi2(const Vertex& v1, const Vertex& v2);

  /// $\chi2$ distance vertex - space point, $ndf = 3$
  double distanceChi2(const Vertex& v, double x, double y, double z);

#ifdef USE_ROOT
  /// signed spatial distance track - RecoVertex
  double SGNdistance(const Track& t, const ArtePointer<RecoVertex>& ap);
#endif
  /// signed spatial distance track - VertexIf
  double SGNdistance(const Track& t, const VertexIf& ap);
} // end of namespace VERTEX
#endif
