#include <map>
#include <iostream>
#include "wax/RTManager.hh"
#include "wax/RTRelation.hh"
#include "wax/DecodeManager.hh"
#include "Otr/OtrI.h"

void SetRt(const waxRTRelation* rt, int sl,int cell) {
  // routine written by Reinhard

  float tmin=(float)rt->GetTMin();
  float tmax=(float)rt->GetTMax();
  float tbin=1;
  float t;
  float d[256];
  float dsig[256];
  //printf("tmin sl=%d cell=%d %f %f\n",sl,cell,tmin,tmax);fflush(stdout);

  waxRTRelation::RTResult rtr ;
  int k=0;
  for (t=tmin;t<=tmax;t+=tbin) {
    rtr = rt->GetRTResultFromT(t) ;
    d   [k]=rtr.fR;
    dsig[k]=rtr.fSigmaR;
    k++;
  }
  OtrRtSet(sl,cell,0,tmin,tmax,k,d,dsig);
}

void waxCopyRTToOtrI()
{
  OtrRtInitialize();
  const waxRTManager* rtm = waxRTManager::GetMe() ;
  // first retrieve the mapping:
  typedef map<int, const waxRTRelation*> SulToRTMap ;
  SulToRTMap map05mm ;
  SulToRTMap map10mm ;
  for(ArteTable<GEDE>::iterator gedeIt = ArteTable<GEDE>::begin() ;
      gedeIt != ArteTable<GEDE>::end(); gedeIt++) 
    if(gedeIt->cmp==2 && gedeIt->nsl>0 && gedeIt->sec>0) {
      char slname[4] ;
      strncpy(slname,gedeIt->hwc,3);
      slname[3]='\0' ;
      int sulindex = OtrISuperLayerFromName(slname) ;
      const waxRTRelation* rtr =
	rtm->GetRTRelationFromGedeIndex((*gedeIt).index()) ;
      if(gedeIt->pit<1) map05mm[sulindex] = rtr ;
      else              map10mm[sulindex] = rtr ;
    }
  // now set the otri rtrelations
  int nrt=0;
  for(SulToRTMap::iterator mapIt = map05mm.begin() ;
      mapIt != map05mm.end() ; mapIt++, nrt++)
    SetRt(mapIt->second,mapIt->first,5) ;

  for(SulToRTMap::iterator mapIt = map10mm.begin() ;
      mapIt != map10mm.end() ; mapIt++, nrt++)
    SetRt(mapIt->second,mapIt->first,10) ;
  
  //if( map10mm.find(5) == map10mm.end() )
  // SetRt(map10mm.find(6)->second,5,10) ;
  
  cout << "* Copied " << nrt << " rtrelations to OtrI." << endl ;
}  

void waxCopyTZeroToOtrI()
{   
  const waxDecodeManager* dm = waxDecodeManager::GetMe() ;
  int nt0=0;
  for(waxDecodeManager::ConstAsdIterator asdIt = dm->GetAsds() ;
      *asdIt != 0 ; asdIt++) 
    for(const waxDecodeChannel* channel = (*asdIt)->GetChannels() ;
        channel != (*asdIt)->GetChannels()+16 ; channel++) {
      int ci = OtrIFromFed ((*asdIt)->GetFedIndex(),channel->GetFedChannelIndex()) ;
      if(!OtrIError() && ci!=-1) {
	OtrT0Put(ci,channel->GetTZeroCount()) ;
	//OtrT0SigmaPut(ci,channel->GetTZeroErrCount()) ;
	nt0++;
      }
    }
  cout << "* Copied " << nt0 << " tzeros to OtrI." << endl ;
}

void waxCopyTZeroFromOtrI()
{   
  const waxDecodeManager* dm = waxDecodeManager::GetMe() ;
  int nt0=0;
  for(waxDecodeManager::ConstAsdIterator asdIt = dm->GetAsds() ;
      *asdIt != 0 ; asdIt++) 
    for(const waxDecodeChannel* channel = (*asdIt)->GetChannels() ;
        channel != (*asdIt)->GetChannels()+16 ; channel++) {
      int ci = OtrIFromFed ((*asdIt)->GetFedIndex(),channel->GetFedChannelIndex()) ;
      if(!OtrIError() && ci!=-1) {
	const_cast<waxDecodeChannel*>(channel)->
	  SetTZero(OtrT0Get(ci),OtrT0Sigma(ci)) ;
	nt0++;
      }
    }
  cout << "* Copied " << nt0 << " tzeros from OtrI." << endl ;
}

void waxCopyStatusFromOtrI()
{   
  const waxDecodeManager* dm = waxDecodeManager::GetMe() ;
  int ntot(0),ngood(0);
  for(waxDecodeManager::ConstAsdIterator asdIt = dm->GetAsds() ;
      *asdIt != 0 ; asdIt++) 
    for(const waxDecodeChannel* channel = (*asdIt)->GetChannels() ;
        channel != (*asdIt)->GetChannels()+16 ; channel++) {
      int ci = OtrIFromFed ((*asdIt)->GetFedIndex(),channel->GetFedChannelIndex()) ;
      unsigned char mask = 1;
      if(!OtrIError() && ci!=-1 && OtrIIsOk(ci)) mask=0;
      const_cast<waxDecodeChannel*>(channel)->SetStatus(mask) ;
      if(mask==0) ngood++ ;
      ntot++;
    }
  cout << "* Copied masks from OtrI: tot/good= " << ntot << " / " << ngood << endl ;
}

void waxCopyToOtrI()
{
  cout << "***********************************************************************" << endl
       << " * You are calling `waxToOtrI'. It copies the wax t0 and rt-relations * "<< endl
       << " * that you have specified in your kumac into the otri datastructures.* "<< endl
       << " * This routine comes with no warranty :)   (WH)                      * "<< endl ;
  
  OtrIInitialize();
  waxCopyRTToOtrI() ;
  waxCopyTZeroToOtrI() ;
  cout << "***********************************************************************" << endl ;
}
